import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import numpy as np
from scipy.integrate import odeint
from mpmath import *
mp.dps = 25; mp.pretty = True

def Om(a):
	Oma=om*a**(-3)
	return Oma

def ODE(a):
	ODEa=(1-om)*a**(-3*(1+w))
	return ODEa

def Omp(a):
	Ompa=(-3*om)/a**4
	return Ompa

def ODEp(a):
	ODEpa=-3*a**(-1-3*(1 + w))*(1-om)*(1+w)
	return ODEpa
	
def H(a):
	Ha=sqrt(Om(a)+ODE(a))
	return Ha

def Hp(a):
	Hpa=(ODEp(a) + Omp(a))/(2*sqrt(ODE(a) + Om(a)))
	return Hpa
	
	
def systemODE(y, a, om,w,cs2,k):
	dm, Vm, dDE, VDE, Phi = y
	dyda = [-((9*a**2*(k**2*(Vm*(1 + w) + a*(3*Phi*(1 + w) + dDE*(2 + 3*w))*H(a)) + VDE*(-k**2 + 3*a**2*H(a)*((2 + 3*w)*H(a) + a*Hp(a))))*ODE(a) + 9*a**3*H(a)*(3*a**2*VDE*H(a)*ODEp(a) + k**2*(a*dDE*ODEp(a) + 2*dm*Om(a) + 3*Phi*Om(a) + a*dm*Omp(a))) + Vm*(2*k**4 + 27*a**4*H(a)*(2*H(a)*Om(a) + a*Hp(a)*Om(a) + a*H(a)*Omp(a))))/(a**2*k**2*H(a)*(2*k**2 + 9*a**2*((1 + w)*ODE(a) + Om(a))))),(-(a*Vm) + (k**2*Phi)/H(a))/a**2,-((VDE*(2*k**4 + 9*a**2*(3*a**3*(1 + w)*H(a)*Hp(a)*ODE(a) + k**2*Om(a) + H(a)**2*(3*a**2*(2 + 3*cs2)*(1 + w)*ODE(a) + 3*a**3*(1 + w)*ODEp(a) + (cs2 - w)*(2*k**2 + 9*a**2*Om(a))))) + 3*a*(3*a*(a*k**2*(3*dDE*(cs2 - w) + (2*dm + 3*Phi)*(1 + w))*H(a) - Vm*(1 + w)*(k**2 - 3*a**2*H(a)*(2*H(a) + a*Hp(a))))*Om(a) + H(a)*(dDE*k**2*(2*k**2*(cs2 - w) + 3*a**2*(1 + w)*((2 + 3*cs2)*ODE(a) + a*ODEp(a))) + 3*a**2*(1 + w)*(3*k**2*Phi*(1 + w)*ODE(a) + a*(dm*k**2 + 3*a*Vm*H(a))*Omp(a)))))/(a**2*k**2*H(a)*(2*k**2 + 9*a**2*((1 + w)*ODE(a) + Om(a))))),(k**2*(cs2*dDE + Phi*(1 + w)) + a*(-1 + 3*cs2)*VDE*H(a))/(a**2*H(a)),(-3*((a*k**2*(3*Phi*(1 + w) + dDE*(2 + 3*w))*H(a) + VDE*(-k**2 + 3*a**2*H(a)*((2 + 3*w)*H(a) + a*Hp(a))))*ODE(a) + (a*k**2*(2*dm + 3*Phi)*H(a) + Vm*(-k**2 + 3*a**2*H(a)*(2*H(a) + a*Hp(a))))*Om(a) + a**2*H(a)*((dDE*k**2 + 3*a*VDE*H(a))*ODEp(a) + (dm*k**2 + 3*a*Vm*H(a))*Omp(a))))/(k**2*H(a)*(2*k**2 + 9*a**2*((1 + w)*ODE(a) + Om(a))))]
	return dyda

#Cosmological parameters 

#Matter
om=0.3
ode=1-om

#Dark Energy
w=-0.8
cs2=0.1

#Initial conditions (k is in units of H_0)
aini=10**(-5)
d0=1
k=200
	
dmini=d0*(aini+3*om/k**2)
Vmini=-d0*sqrt(om)*aini**(0.5)
dDEini=d0*(1+w)*(aini/(1-3*w)+3*om/k**2)
VDEini=-d0*(1+w)*sqrt(om)*aini**(0.5)
Phiini=-(1.5)*d0*om/k**2

# ODE solver parameters
abserr = 1.0e-10
relerr = 1.0e-10
y0 = [dmini, Vmini, dDEini, VDEini, Phiini]
#a = np.linspace(aini, 1, 1000)
a = 10**np.linspace(-5.0, 0, 1000)

# Some print statements for debugging purposes
print('Om={0}'.format(om))
print('w={0}'.format(w))
print('cs2={0}'.format(cs2))
print('k={0}'.format(k))
print('aini={0}'.format(aini))
print('H(aini)={0}'.format(H(aini)))
print('dmini={0}'.format(dmini))
print('Vmini={0}'.format(Vmini))
print('dDEini={0}'.format(dDEini))
print('VDEini={0}'.format(VDEini))
print('Phiini={0}'.format(Phiini))
print('Om(aini)={0}'.format(Om(aini)))

# Solve the ODE
sol = odeint(systemODE, y0, a , args=(om,w,cs2,k),atol=abserr, rtol=relerr,h0=10**(-10))

# This makes the plot and saves it as a PDF
fig=plt.figure()
plt.loglog(a, abs(sol[:, 0]), 'r', label=r'$ \delta_{m}(a)$')
plt.loglog(a, abs(sol[:, 1]), 'g', label=r'$ V_m(a)$')
plt.loglog(a, abs(sol[:, 2]), 'b', label=r'$ \delta_{DE}(a)$')
plt.loglog(a, abs(sol[:, 3]), 'c', label=r'$ V_{DE}(a)$')
plt.loglog(a, abs(sol[:, 4]), 'y', label=r'$ \Phi(a)$')
plt.legend(loc='best')
plt.title(r'Evolution of perturbation variables, see also 0909.0007')
plt.xlabel('a')
plt.ylabel('f(a)')
plt.grid()
#plt.show()
fig.savefig('ODE_python.pdf', bbox_inches='tight')