(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 11.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[   1046278,      19548]
NotebookOptionsPosition[   1035513,      19383]
NotebookOutlinePosition[   1035881,      19399]
CellTagsIndexPosition[   1035838,      19396]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData["Quit"], "Input",
 CellChangeTimes->{{3.7964571836902575`*^9, 
  3.796457184259124*^9}},ExpressionUUID->"db04ee06-7e83-4916-8dfc-\
1e570623902e"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Case", " ", "1"}], ":", " ", 
   RowBox[{"Matter", " ", "+", " ", 
    RowBox[{"cosmological", " ", "constant", " ", "\[CapitalLambda]"}]}]}], 
  " ", "*)"}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356254487086363`*^9, 3.735625464138838*^9}, {
   3.7356259272085533`*^9, 3.7356259374289474`*^9}, 
   3.7674234968330965`*^9},ExpressionUUID->"498c69b4-d2ae-45f2-9c4f-\
86d2f808d835"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"Friedmann", " ", "equation"}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"H", "[", "a_", "]"}], ":=", 
   SqrtBox[
    RowBox[{
     RowBox[{"om", " ", 
      SuperscriptBox["a", 
       RowBox[{"-", "3"}]]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "om"}], ")"}], 
      SuperscriptBox["a", 
       RowBox[{
        RowBox[{"-", "3"}], 
        RowBox[{"(", 
         RowBox[{"1", "+", "w"}], ")"}]}]]}]}]]}]}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356253342187953`*^9, 3.735625344922727*^9}, 
   3.735625464138838*^9},ExpressionUUID->"a5c31e34-7d0c-406f-8961-\
05747b8d6600"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"Cosmological", " ", "parameters"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"om", "=", 
     RowBox[{"3", "/", "10"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"w", "=", 
     RowBox[{"-", "1"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"k", "=", "200"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"ai", "=", 
     SuperscriptBox["10", 
      RowBox[{"-", "5"}]]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"\[Delta]0", "=", "1"}], ";"}]}]}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356244440484858`*^9, 3.7356244615491915`*^9}, {
   3.7356245730495615`*^9, 3.7356246242495565`*^9}, {3.7356249005791054`*^9, 
   3.7356249050494165`*^9}, {3.7356252737181115`*^9, 3.7356253801189585`*^9}, 
   3.735625464138838*^9, 
   3.7356262997380915`*^9},ExpressionUUID->"c8d3a0bc-cd93-4e23-a8eb-\
6763f8e81225"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"The", " ", "initial", " ", "conditions"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Delta]man", "[", "a_", "]"}], ":=", 
     RowBox[{"\[Delta]0", 
      RowBox[{"(", 
       RowBox[{"a", "+", 
        RowBox[{"3", " ", 
         FractionBox["om", 
          RowBox[{"k", "^", "2"}]]}]}], ")"}]}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Vman", "[", "a_", "]"}], ":=", 
    RowBox[{
     RowBox[{"-", "\[Delta]0"}], 
     SqrtBox["om"], 
     SuperscriptBox["a", 
      RowBox[{"1", "/", "2"}]]}]}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"\[Phi]an", "[", "a_", "]"}], ":=", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["3", "2"]}], "\[Delta]0", " ", 
     RowBox[{"om", "/", 
      RowBox[{"k", "^", "2"}]}]}]}]}]}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356246272481565`*^9, 3.735624689719591*^9}, {
   3.735624872839239*^9, 3.7356248730495434`*^9}, 3.735625464138838*^9, {
   3.796457073315317*^9, 
   3.7964570801737065`*^9}},ExpressionUUID->"a4666d80-c540-4409-b18e-\
84415c6df473"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"This", " ", "solves", " ", "the", " ", "ODEs"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"sol", "=", 
    RowBox[{"NDSolve", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[Delta]m", "'"}], "[", "a", "]"}], "\[Equal]", 
         RowBox[{
          FractionBox[
           RowBox[{"-", 
            RowBox[{"Vm", "[", "a", "]"}]}], 
           RowBox[{
            RowBox[{"H", "[", "a", "]"}], " ", 
            RowBox[{"a", "^", "2"}]}]], "+", 
          RowBox[{"3", 
           RowBox[{
            RowBox[{"\[Phi]", "'"}], "[", "a", "]"}]}]}]}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"Vm", "'"}], "[", "a", "]"}], "\[Equal]", 
         RowBox[{
          FractionBox[
           RowBox[{"-", 
            RowBox[{"Vm", "[", "a", "]"}]}], "a"], "+", 
          RowBox[{
           FractionBox[
            RowBox[{"k", "^", "2"}], 
            RowBox[{
             RowBox[{"H", "[", "a", "]"}], " ", 
             RowBox[{"a", "^", "2"}]}]], 
           RowBox[{"\[Phi]", "[", "a", "]"}]}]}]}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"k", "^", "2"}], " ", 
          RowBox[{"\[Phi]", "[", "a", "]"}]}], "\[Equal]", 
         RowBox[{
          RowBox[{"-", 
           FractionBox[
            RowBox[{"3", "om"}], 
            RowBox[{"2", "a"}]]}], 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"\[Delta]m", "[", "a", "]"}], "+", 
            RowBox[{"3", "a", " ", 
             FractionBox[
              RowBox[{"H", "[", "a", "]"}], 
              RowBox[{"k", "^", "2"}]], " ", 
             RowBox[{"Vm", "[", "a", "]"}]}]}], ")"}]}]}], ",", 
        RowBox[{
         RowBox[{"\[Delta]m", "[", "ai", "]"}], "==", 
         RowBox[{"\[Delta]man", "[", "ai", "]"}]}], ",", 
        RowBox[{
         RowBox[{"Vm", "[", "ai", "]"}], "==", 
         RowBox[{"Vman", "[", "ai", "]"}]}], ",", 
        RowBox[{
         RowBox[{"\[Phi]", "[", "ai", "]"}], "\[Equal]", 
         RowBox[{"\[Phi]an", "[", "ai", "]"}]}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\[Delta]m", ",", "Vm", ",", "\[Phi]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"a", ",", "ai", ",", "1"}], "}"}]}], "]"}]}], ";"}]}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356244739603434`*^9, 3.7356246148394423`*^9}, {
   3.7356246945093637`*^9, 3.7356247027992535`*^9}, {3.735624811109524*^9, 
   3.735624865839184*^9}, 3.735625464138838*^9, 3.7673472294954605`*^9, {
   3.7964570837990227`*^9, 
   3.796457092637639*^9}},ExpressionUUID->"919a3c43-bbf1-4a02-96b5-\
64dc585f085c"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"This", " ", "makes", " ", "the", " ", "plots"}], " ", 
  "*)"}]], "Input",
 CellChangeTimes->{{3.796457107055458*^9, 
  3.7964571088509345`*^9}},ExpressionUUID->"9d05f989-b7e9-4bc7-a032-\
53bc21e8e4e7"],

Cell[BoxData[
 RowBox[{"LogLinearPlot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       FractionBox[
        RowBox[{"om", " ", 
         SuperscriptBox["a", 
          RowBox[{"-", "3"}]]}], 
        RowBox[{
         RowBox[{"H", "[", "a", "]"}], "^", "2"}]], ",", 
       FractionBox[
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "-", "om"}], ")"}], 
         SuperscriptBox["a", 
          RowBox[{
           RowBox[{"-", "3"}], 
           RowBox[{"(", 
            RowBox[{"1", "+", "w"}], ")"}]}]]}], 
        RowBox[{
         RowBox[{"H", "[", "a", "]"}], "^", "2"}]]}], "}"}], "//", "Abs"}], "//",
     "Evaluate"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "ai", ",", "1"}], "}"}], ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameStyle", "\[Rule]", "Black"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<a\>\"", ",", "\"\<\[CapitalOmega](a)\>\""}], "}"}]}], ",", 
   RowBox[{"BaseStyle", "\[Rule]", 
    RowBox[{"FontSize", "\[Rule]", "12"}]}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", "Blue", "}"}], ",", 
      RowBox[{"{", "Black", "}"}]}], "}"}]}], ",", 
   RowBox[{"PlotLegends", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
     "\"\<\!\(\*SubscriptBox[\(\[CapitalOmega]\), \(m\)]\)(a)\>\"", ",", 
      "\"\<\!\(\*SubscriptBox[\(\[CapitalOmega]\), \(DE\)]\)(a)\>\""}], 
     "}"}]}]}], "]"}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.735625107069229*^9, 3.7356251838925276`*^9}, 
   3.735625464138838*^9, {3.7964571230934024`*^9, 
   3.796457133132511*^9}},ExpressionUUID->"21338be5-35ce-4e6c-926a-\
18b7ac170390"],

Cell[BoxData[
 TemplateBox[{GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          RGBColor[0, 0, 1]], 
         LineBox[CompressedData["
1:eJwt13c8VX8YB/BrZK/rXnukSELJSNupZKQiomRVdpEkDSpJkUooIzKSqIwk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          "]]}, Annotation[#, "Charting`Private`Tag$6724#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          GrayLevel[0]], 
         LineBox[CompressedData["
1:eJwVlnc0198fx+299/bhY1NmCO/7NopCtggZWX1VsrMqlEgqGUVGlJBREhnh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          "]]}, Annotation[#, "Charting`Private`Tag$6724#2"]& ]}}, {}, {}}, {
    Ticks -> {
       Charting`ScaledTicks[{Log, Exp}], Automatic}, 
     GridLines -> {None, None}, FrameTicks -> {{Automatic, Automatic}, {
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, DisplayFunction -> 
     Identity, DisplayFunction -> Identity, Ticks -> {Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , Automatic}, 
     AxesOrigin -> NCache[{-Log[100000], 0}, {-11.512925464970229`, 0}], 
     FrameTicks -> {{Automatic, 
        Charting`ScaledFrameTicks[{Identity, Identity}]}, {Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, GridLines -> {None, None}, 
     DisplayFunction -> Identity, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None, 
       "ClippingRange" -> {{{-11.512925230012566`, -2.3495766255041283`*^-7}, \
{2.333334978037541*^-15, 
        0.9999999999999976}}, {{-11.512925230012566`, \
-2.3495766255041283`*^-7}, {2.333334978037541*^-15, 0.9999999999999976}}}}, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, 
     BaseStyle -> (FontSize -> 12), 
     CoordinatesToolOptions -> {"DisplayFunction" -> ({
         Exp[
          Part[#, 1]], 
         Part[#, 2]}& ), "CopiedValueFunction" -> ({
         Exp[
          Part[#, 1]], 
         Part[#, 2]}& )}, DisplayFunction :> Identity, 
     Frame -> {{True, True}, {True, True}}, FrameLabel -> {{
        FormBox["\"\[CapitalOmega](a)\"", TraditionalForm], None}, {
        FormBox["\"a\"", TraditionalForm], None}}, FrameStyle -> GrayLevel[0],
      FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, PlotRange -> 
     NCache[{{-Log[100000], 0}, {0., 
        0.9999999999999976}}, {{-11.512925464970229`, 0}, {0., 
        0.9999999999999976}}], PlotRangeClipping -> True, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}],FormBox[
    FormBox[
     TemplateBox[{
      "\"\\!\\(\\*SubscriptBox[\\(\[CapitalOmega]\\), \\(m\\)]\\)(a)\"", 
       "\"\\!\\(\\*SubscriptBox[\\(\[CapitalOmega]\\), \\(DE\\)]\\)(a)\""}, 
      "LineLegend", DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    RGBColor[0, 0, 1]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    RGBColor[0, 0, 1]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    GrayLevel[0]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    GrayLevel[0]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0, 0, 1], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> RGBColor[0., 0., 0.6666666666666666], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "RGBColor[0, 0, 1]"], Appearance -> None, BaseStyle -> {},
                     BaselinePosition -> Baseline, DefaultBaseStyle -> {}, 
                    ButtonFunction :> With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0, 0, 1]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0, 0, 1], Editable -> False, Selectable -> 
                    False]}], "]"}], ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], "GrayLevel[0]"], 
                    Appearance -> None, BaseStyle -> {}, BaselinePosition -> 
                    Baseline, DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False]}], 
                 "]"}]}], "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356251594029627`*^9, 3.7356251846031313`*^9}, {
   3.7356252952428603`*^9, 3.73562531597219*^9}, 3.7356253878385553`*^9, 
   3.735625464138838*^9, 3.7356276353308363`*^9, 3.7673472386499577`*^9, {
   3.7964571117905283`*^9, 
   3.7964571339936757`*^9}},ExpressionUUID->"0b1e3621-7972-4050-bd28-\
4b3b42bc4dc9"],

Cell[BoxData[
 RowBox[{"LogLogPlot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Delta]m", "[", "a", "]"}], "/.", 
        RowBox[{"sol", "[", 
         RowBox[{"[", "1", "]"}], "]"}]}], ",", 
       RowBox[{"\[Delta]man", "[", "a", "]"}]}], "}"}], "//", "Abs"}], "//", 
    "Evaluate"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "ai", ",", "1"}], "}"}], ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameStyle", "\[Rule]", "Black"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
     "\"\<a\>\"", ",", 
      "\"\<\!\(\*SubscriptBox[\(\[Delta]\), \(m\)]\)(a)\>\""}], "}"}]}], ",", 
   RowBox[{"BaseStyle", "\[Rule]", 
    RowBox[{"FontSize", "\[Rule]", "12"}]}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", "Black", "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"Dashed", ",", "Black"}], "}"}]}], "}"}]}], ",", 
   RowBox[{"PlotLegends", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<Numerical\>\"", ",", "\"\<Analytical\>\""}], "}"}]}]}], 
  "]"}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356249752743273`*^9, 3.735624983659234*^9}, {
   3.735625027689151*^9, 3.7356250891291304`*^9}, 3.735625464138838*^9, {
   3.7964571413011923`*^9, 
   3.7964571414915004`*^9}},ExpressionUUID->"08f42a6c-dfef-4a69-af6b-\
cf8b9b64e082"],

Cell[BoxData[
 TemplateBox[{GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          GrayLevel[0]], 
         LineBox[CompressedData["
1:eJwVx3k0FAgcwPExJCs34xzmMIZkm3UstvOVN4laj0HkqE1Em5tSjl6HcnRQ
SSTM2Nwkoty/H/LURjNuxm3lauORIoq1f3zf531pHgEcLyKBQLDf7H/XPZGe
Ia6L3cVSUTqFDAyRjOPwxXTR2Ld8PbmAgdMldtcIorrY8tvjXRKb37E6Puax
wUSf840ZM3kMzEncwtNdYSJ321x3bg4DbWqtKaUzTJzINxYoZDEwXbFb43Ur
E6XXrNb/SmHgnsYZpdn7TBScY5s9i2bgZdKN2+EkJgqTdX83cmDgreO/rrtw
ddDZLdvPhMrAsA/+KTE/6+Dtt+xvTZPa6O2tXrnynIEsw6Iv5jxtpAuF7VV7
GChPiT9ZeVwbxw9KB0G9Nsq6525336KNVz6/MZe30MZalJExKaLjTFLqahmf
jgcfOCRN2tPRyWDwQCqHjtslqQTTeRomlhtf5w/RcDX64izrDg1zo5U+2Z2j
YVvqmZqh7TRssvYeVJ2nYkBMGFGmhooFIvxmVgQV7XPuzFjYUdHA3e6s8hoF
Tzu3wMdKCuIhlRQvMQoGhasnSAq0cFZ+a42DlhZmPht0O7qkiVpvhdOtDE28
lJZf8FJUE6dP9Ul+2E1Gz8yP9FAaGWsmq8xu7tdAhya1LW36Gmi9VlwBluq4
QggUCTZTx8CFd7qpf6hhr/2FjHv2apjZ2fZ0zUsVwx721712VcXTj76R1iNU
8Jrj35yBEBWcmDXrS0lWRvnIyE/KicoYoO+SXZtOwuhlU1teCgkTjuScuVKh
hNCtuupXqoRVzj1GbVWKaKTc2pL+UhEzv6jaljUq4NNcjypDVECN0BunzAbl
MdwjaZu5UB55w91R7HE5JEZJmVaMymG4I5c8tSCLPPJi6Z/zsrhDunokRE8G
RSt2NlxkyuD8k7RKBXMpZAnY92JNpXBkXnBpwUcSX80SiPXekjiu/t52PFgC
p60CbxI2VfiJIqeYIY5BE82+EeniWKP8MCi0QwyjumPJhpte+N4xPTVCxMZr
dxX0Rol4ItvzcJqMCPLfu+cVyopg6dHIiAaDdQiNc3pkYrQOKXcuFRTbrMJZ
64m1XJdV6HXQKlB78RWW8gV1Mu++wp5WTnf+20Xo90xMPLm2CIsP3LNC2HOw
X2lYNEt8HuykC+jHyFOw8zr7VsfNaVhu27EkUjIMRMf4PG3JcdDPXrAUPT8M
xx7x9bw+j4Fb5Ks0sd3DMLchNlk2OAaN+hYWW1uGgL2/yTGkZAzuxrjclxoZ
hO88P16c4xgwDsT9oiI7ANs63atVn44Cp3zS1yCgF4yYLqTDycNQ9Jj34RhL
AL0miXkBMf0w6reiYTzEh4gfqYtn3frBJ6fYt9KVD9Umw07xhv0QpqIq0+T8
Hgoz0vh7B/sg+du/tn2cVtjdaDgUZNwH3XXJPUTLN2DH0YqnzPQA59DsmBML
oWFVvpIQ3AXlP/Z5VbsCuHIcGkaPdEGTf754s3M9SPhyG6Z1umDc9rKVkFML
n8UyGwKFnUAj6fHFLCtBYmmvzSl2J2SkRwiPs56D79CuEy30DnDVbF1+4/wM
JihZJXob7RC+wd01zCmC3J1CrbKBdqhqskYJyzww1Zq32fuwHUyPZrxzZ3GB
qXSVayLZDgVuk+RJzhPw8bdK6JkSwAw1MUvOMgXquC2Xuc0COJP/T7EXKxGU
6wYOFV0VgPWAujfVMhZKC0UU5k4KoOW0f/B51hVYzjFtctongNi+62tJrEAI
jkLODFkAAouGUFJcQn3Xi+JA9jIf/gM0PrbX
          "]]}, Annotation[#, "Charting`Private`Tag$7299#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          Dashing[{Small, Small}], 
          GrayLevel[0]], 
         LineBox[CompressedData["
1:eJwVzX1YCwgcB/BeprRH761Nb2trFRpzVMvpOIeikppoldzp3YNIOWfpLorL
XR5zyKKX1dHLrBTl2bTr96PS0tiK0U2FaKvO6cml1yu3++P7fJ7v958vLe4Q
J9HIwMBgmz7/O5+A9BITT3yZvyXL/SYD04lnOUqCJ0p564cLRAwcuhV+ysDY
E6vE07Om+t49M/Am7rMH+mT5k4arGFjBX1DmOeWBXJnQtrKCgaGyIGr9sAda
CsPcbcoZWGyrdmxVeGBWwteK3wUM9H8wbDfymwcGP+icqM1l4I+k0/k8kgfe
gKSrqyIY+GuUz3y00B2zcyLk3q4MPDaYKvh5uTs2O37BbtW6YXKyg2SqTv/D
iMz3K3NDukbTJfVnYNnRi6o7UW448I15GjS74STxbfueBW6Y/Y/cz3qjflfk
UrzFdBy+VDhzW0nHUk9mqnYHHSOZvRsKOXT0Ffxg5jtKQ37D6hxlHw0p65lT
rHM0rMy1+zt8Pw2XGu7V9C2lYUtQci9l1BWfbTs/bt7kiiJDZRsr0xUTuN3y
TeGuyIwN32c/S0Vl//GUvyRUxACyIJFARWpGhiVR5YIj1qZNES4umGvOkAeP
O6NLh2ZIwXBGsl2j8V1jZxza20McXOuE9V91xqfTnLBJK2WfWe+I6vxrhxTL
HDFotqYRAh3wQw6TncZ2wMNjnZ6F3y3GlDaphr9jMZY+fXx9NpGCL0J0pq0x
FIy/Mk2azyTjXNovNzXpZHw3wu4RFNjjAt2VdfZ8ezy0LPqGrJiEfKq3RCgg
4fngiqTsRjskbBqPPFhvh1Lu81WPpbboI+8gF9+1xdJPlLDbD2ywjRA/tRJt
0DHj9F52rzXGGdGYbI01lvWrszYPWOGKUi2t4bUV8nYKnXRjluhXUKBJHrVE
L/N7r9KXWOAjmTA3w8MCR4uuSWz8FuHTxoa4k76L8NWo6vhYChHnd4sj65OJ
OODwJGzgyEI03UJs1aUtRBszqpVtiQnuCHNoDS42wSb7y2kZ3QQ0zV/jre4i
4Pf/dg/pXhnh/hKezX69e24kbLlmYYi2XadVbL31IScy7zPn4dOj0E/uegXn
jotqQmfAJ1eRt1rviwgX0eI7EyDW3WNJbk+Av4Kjru74CCunA36K1fvxYmx5
+uYPcHVuu8vMpg8Qbi6i73LSgWikfHbcUQeTj73GDW/1A++Zcd6+2n4QXy0b
3MVSwcmtod/2rVDB64NTjqv7lFCbUkfY0KuElIqaA5IYJZidVWxtj1bCMTLF
ooX7BOb4tK6nkU+gYPp9WA9HAe1/Rj17H64A9R8Fz40C5fBivqfGJUAOnICR
N5EshMqkc3dOrkBomFuXeC8GwGxuSmIZDdCSWm3Sxm2GpEK6mVNkMwyE/bhV
w5FB6PSe697hMqCRligJgRLQOokfJm6WQElxpiaKVQcnLljJ2pfXQYyzYlLO
rYU1ra+1lTtrgfdZ+GU/RwynZAc4uF0M0pYgXBhYBfKiCerYxirwDSnpjGUJ
gafyusxhCkG0W+uk5RRBwsh2tTqkCIZd+eVWgQIwf2fLM9kggKTqtzWJLD5Y
j5Lrk5fyIeilQ7JrYB50DMnPrFybB+3xqUeOsrJB6uZVesEtG/J6cmYvsQ7D
8odtuwbJh0G18X4G6ez55jYy3bSaO7nuP9IQfjg=
          "]]}, Annotation[#, "Charting`Private`Tag$7299#2"]& ]}}, {}, {}}, {
    Ticks -> {
       Charting`ScaledTicks[{Log, Exp}], 
       Charting`ScaledTicks[{Log, Exp}]}, GridLines -> {None, None}, 
     FrameTicks -> {{
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}, {
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, DisplayFunction -> 
     Identity, DisplayFunction -> Identity, Ticks -> {Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& }, AxesOrigin -> 
     NCache[{-Log[
         100000], -10.334270396333912`}, {-11.512925464970229`, \
-10.334270396333912`}], FrameTicks -> {{Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}, {Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, GridLines -> {None, None}, 
     DisplayFunction -> Identity, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None, 
       "ClippingRange" -> {{{-11.512925230012566`, -2.3495766255041283`*^-7}, \
{-10.334270396333912`, 
        0.000022264794502756058`}}, {{-11.512925230012566`, \
-2.3495766255041283`*^-7}, {-10.334270396333912`, 
        0.000022264794502756058`}}}}, DisplayFunction -> Identity, 
     AspectRatio -> NCache[GoldenRatio^(-1), 0.6180339887498948], 
     Axes -> {True, True}, AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, 
     BaseStyle -> (FontSize -> 12), 
     CoordinatesToolOptions -> {"DisplayFunction" -> ({
         Exp[
          Part[#, 1]], 
         Exp[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Exp[
          Part[#, 1]], 
         Exp[
          Part[#, 2]]}& )}, DisplayFunction :> Identity, 
     Frame -> {{True, True}, {True, True}}, FrameLabel -> {{
        FormBox[
        "\"\\!\\(\\*SubscriptBox[\\(\[Delta]\\), \\(m\\)]\\)(a)\"", 
         TraditionalForm], None}, {
        FormBox["\"a\"", TraditionalForm], None}}, FrameStyle -> GrayLevel[0],
      FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, PlotRange -> 
     NCache[{{-Log[100000], 0}, {-10.334270396333912`, 
        0.000022264794502756058`}}, {{-11.512925464970229`, 
         0}, {-10.334270396333912`, 0.000022264794502756058`}}], 
     PlotRangeClipping -> True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}],FormBox[
    FormBox[
     TemplateBox[{"\"Numerical\"", "\"Analytical\""}, "LineLegend", 
      DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    GrayLevel[0]], {
                    LineBox[{{0, 10}, {40, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    GrayLevel[0]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {40, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    Dashing[{Small, Small}], 
                    GrayLevel[0]], {
                    LineBox[{{0, 10}, {40, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    Dashing[{Small, Small}], 
                    GrayLevel[0]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {40, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], "GrayLevel[0]"], 
                    Appearance -> None, BaseStyle -> {}, BaselinePosition -> 
                    Baseline, DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False]}], 
                 "]"}], ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   RowBox[{"Dashing", "[", 
                    RowBox[{"{", 
                    RowBox[{"Small", ",", "Small"}], "}"}], "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], "GrayLevel[0]"], 
                    Appearance -> None, BaseStyle -> {}, BaselinePosition -> 
                    Baseline, DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False]}], 
                 "]"}]}], "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{
  3.7356249842490606`*^9, {3.7356250498492017`*^9, 3.735625093137724*^9}, {
   3.735625304988657*^9, 3.735625318403673*^9}, 3.7356253898930063`*^9, 
   3.735625464138838*^9, 3.735627635531335*^9, 3.7673472414463377`*^9, {
   3.7964571150091295`*^9, 
   3.7964571422680693`*^9}},ExpressionUUID->"e6624f99-8e03-4b55-93c6-\
77105a611c98"],

Cell[BoxData[
 RowBox[{"LogLogPlot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"Vm", "[", "a", "]"}], "/.", 
        RowBox[{"sol", "[", 
         RowBox[{"[", "1", "]"}], "]"}]}], ",", 
       RowBox[{"Vman", "[", "a", "]"}]}], "}"}], "//", "Abs"}], "//", 
    "Evaluate"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "ai", ",", "1"}], "}"}], ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameStyle", "\[Rule]", "Black"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
     "\"\<a\>\"", ",", "\"\<\!\(\*SubscriptBox[\(V\), \(m\)]\)(a)\>\""}], 
     "}"}]}], ",", 
   RowBox[{"BaseStyle", "\[Rule]", 
    RowBox[{"FontSize", "\[Rule]", "12"}]}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", "Black", "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"Dashed", ",", "Black"}], "}"}]}], "}"}]}], ",", 
   RowBox[{"PlotLegends", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<Numerical\>\"", ",", "\"\<Analytical\>\""}], "}"}]}]}], 
  "]"}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356249293691154`*^9, 3.7356249982486596`*^9}, {
   3.7356252311484785`*^9, 3.7356252421689615`*^9}, 3.735625464138838*^9, {
   3.7964571461146317`*^9, 
   3.796457147058689*^9}},ExpressionUUID->"21f67940-086a-4d0f-84eb-\
90366fe82074"],

Cell[BoxData[
 TemplateBox[{GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          GrayLevel[0]], 
         LineBox[CompressedData["
1:eJwVk3k41Gsbx5Ei+84wtiE6UeM0xxbJWAvHMipbWhTqIFLSCdWbtSgOEUcM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          "]]}, Annotation[#, "Charting`Private`Tag$8278#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          Dashing[{Small, Small}], 
          GrayLevel[0]], 
         LineBox[CompressedData["
1:eJwV0H04EwgcwHFks3bI29gYRjV6uXZSXo5wKotUjDI5jpjlqSh0O6Sj99ej
wnmJEJI4KvWQ6vcrvVDWpBOJOnsxWx0nndiyrvvj+3z+/9ptS+LwdLS0tDZ8
7X81cWhfRnRAMY3ra5BEwxTycY5I1wHDgJHstYuGo43BB7TmOGBXnKIiaQcN
e1Ti4W1fmHizMV27h0/DmlxChcM0E0+sLcWCaBpuvB1ge1XBxOV7xF4MDg1L
TXutHnQxMbtjp4fzShp63leYKc8ykSHIXhGupuJ+yuFT6RQmbntCr1dkUvFk
+ErN1vKFmGQotH5CoqJAllh49NuFGFFMbJnKt0A+37JlumkBrqdf6+PTLNB+
YOB5q+cCJOYkTzhWm6PY12AP3J2Pjz5V5S5zMMesyQ4349XzMa42mpt1hYKK
vCLVNZE9Cv2dLO3dKBi2dPCHIo49egynxJDazDC32fmgaMgOUxOeVnr5meGl
Q2Z/B++ww5Pu9fr3XphiewB/kDrOwEtfNOqjYaZYpy16yMpg4G8342VlEhNc
GhmcYK62xfRYn1AVzwTRz6KQp2uLa7gX31UojVFprNcWamODr1ZZZRYJjNGm
c2C0a4E1ptAIeb0qIxyN6SfLPOjYL5tTHXPECNtGWl2PeFuhb22qx3KSEQao
G24A2xJNeEMfS6nzcPfEU4eir9/dItTxAidDvPBCWKXmUZG/albyysUAY3+f
oWgyLHCJ9qLNg4H6KFW69hcWmGNDKN9CwP8GkxZvrb5dSkHXvOPS4kQy5qyv
ic+6YYZhN3qro47MxVbuy+XCVlPMvk9IvnKChBf+pQZdu2+C/J/W5eSe00Or
1MMxroPGKBlqLqc0ELHiTW/mWrERJqwj3Jl/nYDpm8vp8ol5eDjvV+HzTl1c
YnDrbYqjIUbt1mdRZXNw/HxJi4mbPvq4ZZbI3+vg2/HutIntZGwZtvi8haCD
YstnQeJkEroLCvibyNpoMtfWyLSMiMFjXo/ltlrYZp6/J7VHFwOVk+PfpWrg
5889o/K3OqjX7cSvyv4MUdVx60oMtXHsl/ZHYU0quBq4L+PeUg0oZuie1i3T
UHg6ra5howoii0tCTDunoC/Upo52fQpmZ3eF+Ks/gmcXp/dy5wdodCclnCFO
wodzkZUpa8fgtNtUPM15AoIN6uy30OXQ0WBiWPV8DD4Jl3zUbnwDZcUdae7j
SqgvrpBtYXWD0IB5c+MmOfy1a9rKeUgEg82e0bbecthe07CzJUIEYxEh5H+W
yUFgQTVs5z4Do7qsqDOGciiYeR/Uz+mCUL/XxD+FI9B7p+ClDrsDBvfnhoUH
jgDHTzkcxkJQjqumY/1l0DzrxbsVAaApML64wl0G7YmXiQ+5d8HAy3EDYZEM
xEH7/Qc4t4F1anNFDUkGdhRHkS67BVIcm/wVj6VQVpoxEM5qAlU0rzjRTwoR
1l2fOrh/gD5p3xpvFymkfyn//g2nHqwbz47NY0qhtT0ASexa8J4F36sEKbgE
lj2NZJXDgSLLd5MPJFD34wh9hHMe8nyc8h80S0DByK00YhdCtZztnV8lgfjL
kgYeKxcerdx7zuWQBAJeW/IZ7GPQN3hylV6qBB7HJibvZWWB4mClvC9WAsf6
D6rzWLthZnHrmdoQCXSvvpdKOZ5zl9wj8khbLYH/AJUZgDg=
          "]]}, Annotation[#, "Charting`Private`Tag$8278#2"]& ]}}, {}, {}}, {
    Ticks -> {
       Charting`ScaledTicks[{Log, Exp}], 
       Charting`ScaledTicks[{Log, Exp}]}, GridLines -> {None, None}, 
     FrameTicks -> {{
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}, {
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, DisplayFunction -> 
     Identity, DisplayFunction -> Identity, Ticks -> {Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& }, AxesOrigin -> 
     NCache[{-Log[
         100000], -6.358449017169272}, {-11.512925464970229`, \
-6.358449017169272}], FrameTicks -> {{Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}, {Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, GridLines -> {None, None}, 
     DisplayFunction -> Identity, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None, 
       "ClippingRange" -> {{{-11.512925230012566`, -2.3495766255041283`*^-7}, \
{-6.358449017169272, -0.6019865196417993}}, {{-11.512925230012566`, \
-2.3495766255041283`*^-7}, {-6.358449017169272, -0.6019865196417993}}}}, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, 
     BaseStyle -> (FontSize -> 12), 
     CoordinatesToolOptions -> {"DisplayFunction" -> ({
         Exp[
          Part[#, 1]], 
         Exp[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Exp[
          Part[#, 1]], 
         Exp[
          Part[#, 2]]}& )}, DisplayFunction :> Identity, 
     Frame -> {{True, True}, {True, True}}, FrameLabel -> {{
        FormBox[
        "\"\\!\\(\\*SubscriptBox[\\(V\\), \\(m\\)]\\)(a)\"", TraditionalForm],
         None}, {
        FormBox["\"a\"", TraditionalForm], None}}, FrameStyle -> GrayLevel[0],
      FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, PlotRange -> 
     NCache[{{-Log[100000], 0}, {-6.358449017169272, 
        0.}}, {{-11.512925464970229`, 0}, {-6.358449017169272, 0.}}], 
     PlotRangeClipping -> True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}],FormBox[
    FormBox[
     TemplateBox[{"\"Numerical\"", "\"Analytical\""}, "LineLegend", 
      DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    GrayLevel[0]], {
                    LineBox[{{0, 10}, {40, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    GrayLevel[0]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {40, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    Dashing[{Small, Small}], 
                    GrayLevel[0]], {
                    LineBox[{{0, 10}, {40, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    Dashing[{Small, Small}], 
                    GrayLevel[0]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {40, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], "GrayLevel[0]"], 
                    Appearance -> None, BaseStyle -> {}, BaselinePosition -> 
                    Baseline, DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False]}], 
                 "]"}], ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   RowBox[{"Dashing", "[", 
                    RowBox[{"{", 
                    RowBox[{"Small", ",", "Small"}], "}"}], "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], "GrayLevel[0]"], 
                    Appearance -> None, BaseStyle -> {}, BaselinePosition -> 
                    Baseline, DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False]}], 
                 "]"}]}], "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356249565389457`*^9, 3.735624965148364*^9}, 
   3.735624999728712*^9, 3.7356252432630863`*^9, 3.7356253920277452`*^9, 
   3.735625464138838*^9, 3.7356276358321695`*^9, 3.767347243410343*^9, 
   3.7964571491503963`*^9},ExpressionUUID->"670b7099-45df-468c-af38-\
ec6607c2f691"],

Cell[BoxData[
 RowBox[{"LogLogPlot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Phi]", "[", "a", "]"}], "/.", 
        RowBox[{"sol", "[", 
         RowBox[{"[", "1", "]"}], "]"}]}], ",", 
       RowBox[{"\[Phi]an", "[", "a", "]"}]}], "}"}], "//", "Abs"}], "//", 
    "Evaluate"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "ai", ",", "1"}], "}"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", "All"}], ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameStyle", "\[Rule]", "Black"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<a\>\"", ",", "\"\<\[Phi](a)\>\""}], "}"}]}], ",", 
   RowBox[{"BaseStyle", "\[Rule]", 
    RowBox[{"FontSize", "\[Rule]", "12"}]}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", "Black", "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"Dashed", ",", "Black"}], "}"}]}], "}"}]}], ",", 
   RowBox[{"PlotLegends", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<Numerical\>\"", ",", "\"\<Analytical\>\""}], "}"}]}]}], 
  "]"}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.735625006189026*^9, 3.7356250182190866`*^9}, {
   3.7356252371093397`*^9, 3.73562526005897*^9}, 3.735625464138838*^9, {
   3.796457153025648*^9, 
   3.7964571542405243`*^9}},ExpressionUUID->"dd904323-0776-4741-a290-\
a878d65db428"],

Cell[BoxData[
 TemplateBox[{GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          GrayLevel[0]], 
         LineBox[CompressedData["
1:eJwknHc8lV8YwM2kjJJy3Xtxt5LSQLJXZaVIUSIj0kCkKJKEFiVJpIQkKU2i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          "]]}, Annotation[#, "Charting`Private`Tag$9220#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          Dashing[{Small, Small}], 
          GrayLevel[0]], 
         LineBox[CompressedData["
1:eJxTTMoPSmViYGAwAWIQ/S/lgNI8NvUDpSHHpANOqR2YK3xV+sgZNTjf5tBL
kVeTEPw60daeKlEEvzvS9F/UAlU4v/xp3ox2XQQ/PV1qx48NKnC+0q1bF3fa
IPiPnHgL9+9ThvMbPp+wEHRG8F9Omflr03klOD9c547jzCAEf8IW4+bzdxXh
/OUtIm8DsxH8w17pdyTeK8D5qxjPH9WvRvB1YgMzxX7Lw/kH3MRnpLIg+K8E
2XeHyMnB+XInb704oyIL579IvMH11FoGzt/9bKd5m700nO/1e+3W/e5ScH7B
x9PqMxMk4fz5l88u+Z0qAecnT/8p+q9aHM5/8sr8xoxpYnB+vlbU0j1zReH8
fu9laQ1bReD8nRHXjM7uFEaY/1UiYNMhIThfuqQ10fyOIJy/8N7VWtdHAnB+
VegCmecf+eF8bd5d94s1+OD893Nm7xCy4IHz77+/UPkxgwsRn1LnAh4VccD5
QpzyAsLz2BDhIza1sOQSC5xf9ufSi+f3meD8uKUpHrP5GOH8jT411Qd1/u2H
8Wf0Vq5a6/cLzr8eIrdKcvM3ON/mTNDVlSc/wfmfJscuKnZ9B+cH8q5SCpN5
Dud/P6v9hXH9PTh/zayFT8P0L8D5QW6vHobrH4DzLzgfLBHt7N8H4wMA+LiY
Vw==
          "]]}, Annotation[#, "Charting`Private`Tag$9220#2"]& ]}}, {}, {}}, {
    Ticks -> {
       Charting`ScaledTicks[{Log, Exp}], 
       Charting`ScaledTicks[{Log, Exp}]}, GridLines -> {None, None}, 
     FrameTicks -> {{
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}, {
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, DisplayFunction -> 
     Identity, DisplayFunction -> Identity, Ticks -> {Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& }, 
     AxesOrigin -> {-11.512925230012566`, -11.644914273033567`}, 
     FrameTicks -> {{Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}, {Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, GridLines -> {None, None}, 
     DisplayFunction -> Identity, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None, 
       "ClippingRange" -> {{{-11.512925230012566`, -2.3495766255041283`*^-7}, \
{-11.644914273033567`, -11.394964482877636`}}, {{-11.512925230012566`, \
-2.3495766255041283`*^-7}, {-11.644914273033567`, -11.394964482877636`}}}}, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, -11.644914273033567`}, 
     BaseStyle -> (FontSize -> 12), 
     CoordinatesToolOptions -> {"DisplayFunction" -> ({
         Exp[
          Part[#, 1]], 
         Exp[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Exp[
          Part[#, 1]], 
         Exp[
          Part[#, 2]]}& )}, DisplayFunction :> Identity, 
     Frame -> {{True, True}, {True, True}}, FrameLabel -> {{
        FormBox["\"\[Phi](a)\"", TraditionalForm], None}, {
        FormBox["\"a\"", TraditionalForm], None}}, FrameStyle -> GrayLevel[0],
      FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, 
     PlotRange -> {All, All}, PlotRangeClipping -> True, 
     PlotRangePadding -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     Ticks -> {Automatic, Automatic}}],FormBox[
    FormBox[
     TemplateBox[{"\"Numerical\"", "\"Analytical\""}, "LineLegend", 
      DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    GrayLevel[0]], {
                    LineBox[{{0, 10}, {40, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    GrayLevel[0]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {40, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    Dashing[{Small, Small}], 
                    GrayLevel[0]], {
                    LineBox[{{0, 10}, {40, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    Dashing[{Small, Small}], 
                    GrayLevel[0]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {40, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], "GrayLevel[0]"], 
                    Appearance -> None, BaseStyle -> {}, BaselinePosition -> 
                    Baseline, DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False]}], 
                 "]"}], ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   RowBox[{"Dashing", "[", 
                    RowBox[{"{", 
                    RowBox[{"Small", ",", "Small"}], "}"}], "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], "GrayLevel[0]"], 
                    Appearance -> None, BaseStyle -> {}, BaselinePosition -> 
                    Baseline, DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False]}], 
                 "]"}]}], "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.735625009548967*^9, 3.735625018868743*^9}, 
   3.7356252611290593`*^9, 3.735625393768678*^9, 3.735625464138838*^9, 
   3.7356276363082056`*^9, 3.767347245460272*^9, 
   3.7964571551770973`*^9},ExpressionUUID->"310bb450-bd56-4478-bced-\
f05fb9863795"],

Cell[BoxData[
 RowBox[{"(*", " ", "end", " ", "*)"}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.735625441878965*^9, 
  3.735625464138838*^9}},ExpressionUUID->"0e338e55-3212-424f-9561-\
904e25c89922"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Case", " ", "2"}], ":", " ", 
   RowBox[{"Matter", " ", "+", " ", 
    RowBox[{"Dark", " ", "energy"}]}]}], " ", "*)"}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356254487086363`*^9, 3.735625464138838*^9}, {
  3.735625508308816*^9, 
  3.7356255123786583`*^9}},ExpressionUUID->"4d77c56b-40e7-4487-be08-\
e4ce8255ee8b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"Friedmann", " ", "equation"}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"H", "[", "a_", "]"}], ":=", 
   SqrtBox[
    RowBox[{
     RowBox[{"om", " ", 
      SuperscriptBox["a", 
       RowBox[{"-", "3"}]]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "om"}], ")"}], 
      SuperscriptBox["a", 
       RowBox[{
        RowBox[{"-", "3"}], 
        RowBox[{"(", 
         RowBox[{"1", "+", "w"}], ")"}]}]]}]}]]}]}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356253342187953`*^9, 3.735625344922727*^9}, 
   3.735625464138838*^9},ExpressionUUID->"31065b41-3987-4eed-b984-\
f19a31348596"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"Cosmological", " ", "parameters"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"om", "=", 
     RowBox[{"3", "/", "10"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"w", "=", 
     RowBox[{
      RowBox[{"-", "8"}], "/", "10"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"cs2", "=", 
     RowBox[{"1", "/", "10"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"k", "=", "200"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"ai", "=", 
     SuperscriptBox["10", 
      RowBox[{"-", "5"}]]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"\[Delta]0", "=", "1"}], ";"}]}]}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356244440484858`*^9, 3.7356244615491915`*^9}, {
   3.7356245730495615`*^9, 3.7356246242495565`*^9}, {3.7356249005791054`*^9, 
   3.7356249050494165`*^9}, {3.7356252737181115`*^9, 3.7356253801189585`*^9}, 
   3.735625464138838*^9, {3.735625523818483*^9, 3.7356255385981607`*^9}, 
   3.7356263046485615`*^9},ExpressionUUID->"0cf06a0a-d8f8-4ddf-8ce7-\
64d59c2a8f73"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"The", " ", "inital", " ", "conditions"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Delta]man", "[", "a_", "]"}], ":=", 
     RowBox[{"\[Delta]0", 
      RowBox[{"(", 
       RowBox[{"a", "+", 
        RowBox[{"3", " ", 
         FractionBox["om", 
          RowBox[{"k", "^", "2"}]]}]}], ")"}]}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Vman", "[", "a_", "]"}], ":=", 
    RowBox[{
     RowBox[{"-", "\[Delta]0"}], 
     SqrtBox["om"], 
     SuperscriptBox["a", 
      RowBox[{"1", "/", "2"}]]}]}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"\[Phi]an", "[", "a_", "]"}], ":=", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["3", "2"]}], "\[Delta]0", " ", 
     RowBox[{"om", "/", 
      RowBox[{"k", "^", "2"}]}]}]}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"\[Delta]DEan", "[", "a_", "]"}], ":=", 
    RowBox[{"\[Delta]0", 
     RowBox[{"(", 
      RowBox[{"1", "+", "w"}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       FractionBox["a", 
        RowBox[{"1", "-", 
         RowBox[{"3", "w"}]}]], "+", 
       RowBox[{"3", 
        FractionBox["om", 
         RowBox[{"k", "^", "2"}]]}]}], ")"}]}]}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"VDEan", "[", "a_", "]"}], ":=", 
    RowBox[{
     RowBox[{"-", "\[Delta]0"}], " ", 
     RowBox[{"(", 
      RowBox[{"1", "+", "w"}], ")"}], 
     SqrtBox["om"], 
     SuperscriptBox["a", 
      RowBox[{"1", "/", "2"}]]}]}]}]}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356246272481565`*^9, 3.735624689719591*^9}, {
   3.735624872839239*^9, 3.7356248730495434`*^9}, 3.735625464138838*^9, {
   3.735625551077653*^9, 3.7356256094488688`*^9}, {3.796457171218781*^9, 
   3.796457180632489*^9}},ExpressionUUID->"795a662f-48fb-4e99-ab6a-\
2adacc1d85ef"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"This", " ", "solves", " ", "the", " ", "ODEs"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"sol", "=", 
    RowBox[{"NDSolve", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[Delta]m", "'"}], "[", "a", "]"}], "\[Equal]", 
         RowBox[{
          FractionBox[
           RowBox[{"-", 
            RowBox[{"Vm", "[", "a", "]"}]}], 
           RowBox[{
            RowBox[{"H", "[", "a", "]"}], " ", 
            RowBox[{"a", "^", "2"}]}]], "+", 
          RowBox[{"3", 
           RowBox[{
            RowBox[{"\[Phi]", "'"}], "[", "a", "]"}]}]}]}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"Vm", "'"}], "[", "a", "]"}], "\[Equal]", 
         RowBox[{
          FractionBox[
           RowBox[{"-", 
            RowBox[{"Vm", "[", "a", "]"}]}], "a"], "+", 
          RowBox[{
           FractionBox[
            RowBox[{"k", "^", "2"}], 
            RowBox[{
             RowBox[{"H", "[", "a", "]"}], " ", 
             RowBox[{"a", "^", "2"}]}]], 
           RowBox[{"\[Phi]", "[", "a", "]"}]}]}]}], ",", 
        "\[IndentingNewLine]", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Delta]DE", "'"}], "[", "a", "]"}], "\[Equal]", 
         RowBox[{
          RowBox[{
           RowBox[{"-", 
            FractionBox[
             RowBox[{"VDE", "[", "a", "]"}], 
             RowBox[{
              RowBox[{"H", "[", "a", "]"}], 
              RowBox[{"a", "^", "2"}]}]]}], 
           RowBox[{"(", 
            RowBox[{"1", "+", 
             FractionBox[
              RowBox[{"9", 
               RowBox[{"a", "^", "2"}], 
               RowBox[{
                RowBox[{"H", "[", "a", "]"}], "^", "2"}], 
               RowBox[{"(", 
                RowBox[{"cs2", "-", "w"}], ")"}]}], 
              RowBox[{"k", "^", "2"}]]}], ")"}]}], "-", 
          RowBox[{
           FractionBox["3", "a"], 
           RowBox[{"(", 
            RowBox[{"cs2", "-", "w"}], ")"}], 
           RowBox[{"\[Delta]DE", "[", "a", "]"}]}], "+", 
          RowBox[{"3", 
           RowBox[{"(", 
            RowBox[{"1", "+", "w"}], ")"}], 
           RowBox[{
            RowBox[{"\[Phi]", "'"}], "[", "a", "]"}]}]}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"VDE", "'"}], "[", "a", "]"}], "\[Equal]", 
         RowBox[{
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{"1", "-", 
              RowBox[{"3", "cs2"}]}], ")"}]}], 
           FractionBox[
            RowBox[{"VDE", "[", "a", "]"}], "a"]}], "+", 
          RowBox[{
           FractionBox[
            RowBox[{
             RowBox[{"k", "^", "2"}], " ", "cs2"}], 
            RowBox[{
             RowBox[{"H", "[", "a", "]"}], " ", 
             RowBox[{"a", "^", "2"}]}]], 
           RowBox[{"\[Delta]DE", "[", "a", "]"}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"1", "+", "w"}], ")"}], 
           FractionBox[
            RowBox[{"k", "^", "2"}], 
            RowBox[{
             RowBox[{"H", "[", "a", "]"}], " ", 
             RowBox[{"a", "^", "2"}]}]], 
           RowBox[{"\[Phi]", "[", "a", "]"}]}]}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"k", "^", "2"}], " ", 
          RowBox[{"\[Phi]", "[", "a", "]"}]}], "\[Equal]", 
         RowBox[{
          RowBox[{"-", 
           FractionBox["3", "2"]}], 
          RowBox[{"a", "^", "2"}], 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"om", " ", 
             SuperscriptBox["a", 
              RowBox[{"-", "3"}]], 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"\[Delta]m", "[", "a", "]"}], "+", 
               RowBox[{"3", "a", " ", 
                FractionBox[
                 RowBox[{"H", "[", "a", "]"}], 
                 RowBox[{"k", "^", "2"}]], " ", 
                RowBox[{"Vm", "[", "a", "]"}]}]}], ")"}]}], "+", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1", "-", "om"}], ")"}], 
             SuperscriptBox["a", 
              RowBox[{
               RowBox[{"-", "3"}], 
               RowBox[{"(", 
                RowBox[{"1", "+", "w"}], ")"}]}]], 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"\[Delta]DE", "[", "a", "]"}], "+", 
               RowBox[{"3", "a", " ", 
                FractionBox[
                 RowBox[{"H", "[", "a", "]"}], 
                 RowBox[{"k", "^", "2"}]], " ", 
                RowBox[{"VDE", "[", "a", "]"}]}]}], ")"}]}]}], ")"}]}]}], ",", 
        RowBox[{
         RowBox[{"\[Delta]m", "[", "ai", "]"}], "==", 
         RowBox[{"\[Delta]man", "[", "ai", "]"}]}], ",", 
        RowBox[{
         RowBox[{"Vm", "[", "ai", "]"}], "==", 
         RowBox[{"Vman", "[", "ai", "]"}]}], ",", 
        RowBox[{
         RowBox[{"\[Delta]DE", "[", "ai", "]"}], "==", 
         RowBox[{"\[Delta]DEan", "[", "ai", "]"}]}], ",", 
        RowBox[{
         RowBox[{"VDE", "[", "ai", "]"}], "==", 
         RowBox[{"VDEan", "[", "ai", "]"}]}], ",", 
        RowBox[{
         RowBox[{"\[Phi]", "[", "ai", "]"}], "\[Equal]", 
         RowBox[{"\[Phi]an", "[", "ai", "]"}]}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
       "\[Delta]m", ",", "Vm", ",", "\[Delta]DE", ",", "VDE", ",", "\[Phi]"}],
        "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"a", ",", "ai", ",", "1"}], "}"}]}], "]"}]}], ";"}]}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356244739603434`*^9, 3.7356246148394423`*^9}, {
   3.7356246945093637`*^9, 3.7356247027992535`*^9}, {3.735624811109524*^9, 
   3.735624865839184*^9}, 3.735625464138838*^9, {3.735625644888775*^9, 
   3.735625825578323*^9}, {3.735629445958326*^9, 3.7356294534987936`*^9}, {
   3.735629577347864*^9, 3.735629604707694*^9}, 3.7673472729113216`*^9, {
   3.796457193762419*^9, 
   3.796457204151884*^9}},ExpressionUUID->"98a8ab14-440c-40e3-a07e-\
a33c2ab3a583"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"The", " ", "plots"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.796457214633644*^9, 
  3.7964572152337356`*^9}},ExpressionUUID->"782bdd20-5acf-43fe-ac43-\
5bff278d2bf3"],

Cell[BoxData[
 RowBox[{"LogLinearPlot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       FractionBox[
        RowBox[{"om", " ", 
         SuperscriptBox["a", 
          RowBox[{"-", "3"}]]}], 
        RowBox[{
         RowBox[{"H", "[", "a", "]"}], "^", "2"}]], ",", 
       FractionBox[
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "-", "om"}], ")"}], 
         SuperscriptBox["a", 
          RowBox[{
           RowBox[{"-", "3"}], 
           RowBox[{"(", 
            RowBox[{"1", "+", "w"}], ")"}]}]]}], 
        RowBox[{
         RowBox[{"H", "[", "a", "]"}], "^", "2"}]]}], "}"}], "//", "Abs"}], "//",
     "Evaluate"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "ai", ",", "1"}], "}"}], ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameStyle", "\[Rule]", "Black"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<a\>\"", ",", "\"\<\[CapitalOmega](a)\>\""}], "}"}]}], ",", 
   RowBox[{"BaseStyle", "\[Rule]", 
    RowBox[{"FontSize", "\[Rule]", "12"}]}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", "Blue", "}"}], ",", 
      RowBox[{"{", "Black", "}"}]}], "}"}]}], ",", 
   RowBox[{"PlotLegends", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
     "\"\<\!\(\*SubscriptBox[\(\[CapitalOmega]\), \(m\)]\)(a)\>\"", ",", 
      "\"\<\!\(\*SubscriptBox[\(\[CapitalOmega]\), \(DE\)]\)(a)\>\""}], 
     "}"}]}]}], "]"}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.735625107069229*^9, 3.7356251838925276`*^9}, 
   3.735625464138838*^9, {3.796457218882472*^9, 
   3.7964572261331882`*^9}},ExpressionUUID->"b8e631e1-f5f8-4d2b-9f67-\
7edc15242a64"],

Cell[BoxData[
 TemplateBox[{GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          RGBColor[0, 0, 1]], 
         LineBox[CompressedData["
1:eJwl13c81d8bAPBLtoxrXltWkhIKlXxoUCpJVISEUEpmSYuMr0qRURRSaVnJ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          "]]}, Annotation[#, "Charting`Private`Tag$3786#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          GrayLevel[0]], 
         LineBox[CompressedData["
1:eJwVVnc81d8fRlb23uNy3ZuMkFLR5xwaFMoIWdlJWlailFFUyCYiISmSIrLd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          "]]}, Annotation[#, "Charting`Private`Tag$3786#2"]& ]}}, {}, {}}, {
    Ticks -> {
       Charting`ScaledTicks[{Log, Exp}], Automatic}, 
     GridLines -> {None, None}, FrameTicks -> {{Automatic, Automatic}, {
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, DisplayFunction -> 
     Identity, DisplayFunction -> Identity, Ticks -> {Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , Automatic}, 
     AxesOrigin -> NCache[{-Log[100000], 0}, {-11.512925464970229`, 0}], 
     FrameTicks -> {{Automatic, 
        Charting`ScaledFrameTicks[{Identity, Identity}]}, {Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, GridLines -> {None, None}, 
     DisplayFunction -> Identity, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None, 
       "ClippingRange" -> {{{-11.512925230012566`, -2.3495766255041283`*^-7}, \
{2.333334649091166*^-12, 
        0.9999999999976665}}, {{-11.512925230012566`, \
-2.3495766255041283`*^-7}, {2.333334649091166*^-12, 0.9999999999976665}}}}, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, 
     BaseStyle -> (FontSize -> 12), 
     CoordinatesToolOptions -> {"DisplayFunction" -> ({
         Exp[
          Part[#, 1]], 
         Part[#, 2]}& ), "CopiedValueFunction" -> ({
         Exp[
          Part[#, 1]], 
         Part[#, 2]}& )}, DisplayFunction :> Identity, 
     Frame -> {{True, True}, {True, True}}, FrameLabel -> {{
        FormBox["\"\[CapitalOmega](a)\"", TraditionalForm], None}, {
        FormBox["\"a\"", TraditionalForm], None}}, FrameStyle -> GrayLevel[0],
      FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, PlotRange -> 
     NCache[{{-Log[100000], 0}, {0., 
        0.9999999999976665}}, {{-11.512925464970229`, 0}, {0., 
        0.9999999999976665}}], PlotRangeClipping -> True, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}],FormBox[
    FormBox[
     TemplateBox[{
      "\"\\!\\(\\*SubscriptBox[\\(\[CapitalOmega]\\), \\(m\\)]\\)(a)\"", 
       "\"\\!\\(\\*SubscriptBox[\\(\[CapitalOmega]\\), \\(DE\\)]\\)(a)\""}, 
      "LineLegend", DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    RGBColor[0, 0, 1]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    RGBColor[0, 0, 1]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    GrayLevel[0]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    GrayLevel[0]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0, 0, 1], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> RGBColor[0., 0., 0.6666666666666666], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "RGBColor[0, 0, 1]"], Appearance -> None, BaseStyle -> {},
                     BaselinePosition -> Baseline, DefaultBaseStyle -> {}, 
                    ButtonFunction :> With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0, 0, 1]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0, 0, 1], Editable -> False, Selectable -> 
                    False]}], "]"}], ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], "GrayLevel[0]"], 
                    Appearance -> None, BaseStyle -> {}, BaselinePosition -> 
                    Baseline, DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False]}], 
                 "]"}]}], "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356251594029627`*^9, 3.7356251846031313`*^9}, {
   3.7356252952428603`*^9, 3.73562531597219*^9}, 3.7356253878385553`*^9, 
   3.735625464138838*^9, 3.7356258302783213`*^9, 3.7356276431516705`*^9, {
   3.7356294598625746`*^9, 3.7356294755922065`*^9}, 3.735629609736992*^9, 
   3.735632826910898*^9, 3.767347279235149*^9, 
   3.796457227255734*^9},ExpressionUUID->"eccf32bc-a4fd-4f66-ac56-\
caec39ea2640"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LogLogPlot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\[Delta]m", "[", "a", "]"}], ",", 
        RowBox[{"Vm", "[", "a", "]"}], ",", 
        RowBox[{"\[Delta]DE", "[", "a", "]"}], ",", 
        RowBox[{"VDE", "[", "a", "]"}], ",", 
        RowBox[{"\[Phi]", "[", "a", "]"}]}], "}"}], "/.", 
      RowBox[{"sol", "[", 
       RowBox[{"[", "1", "]"}], "]"}]}], "//", "Abs"}], "//", "Evaluate"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "ai", ",", "1"}], "}"}], ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameStyle", "\[Rule]", "Black"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<a\>\"", ",", "\"\<\>\""}], "}"}]}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Red", ",", 
      RowBox[{"Darker", "[", "Green", "]"}], ",", "Blue", ",", "Cyan", ",", 
      RowBox[{"Darker", "[", "Yellow", "]"}]}], "}"}]}], ",", 
   RowBox[{"BaseStyle", "\[Rule]", 
    RowBox[{"FontSize", "\[Rule]", "12"}]}], ",", 
   RowBox[{"PlotLegends", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
     "\"\<\!\(\*SubscriptBox[\(\[Delta]\), \(m\)]\)(a)\>\"", ",", 
      "\"\<\!\(\*SubscriptBox[\(V\), \(m\)]\)(a)\>\"", ",", 
      "\"\<\!\(\*SubscriptBox[\(\[Delta]\), \(DE\)]\)(a)\>\"", ",", 
      "\"\<\!\(\*SubscriptBox[\(V\), \(DE\)]\)(a)\>\"", ",", 
      "\"\<\[Phi](a)\>\""}], "}"}]}], ",", 
   RowBox[{"AspectRatio", "\[Rule]", "0.8"}], ",", 
   RowBox[{"GridLines", "\[Rule]", "Automatic"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.735632737035317*^9, 3.735632905180521*^9}, 
   3.7356349191112084`*^9, {3.796457235685342*^9, 
   3.7964572368544064`*^9}},ExpressionUUID->"eb9ec060-d0b2-4925-bf7d-\
83bfffdfa688"],

Cell[BoxData[
 TemplateBox[{GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          RGBColor[1, 0, 0]], 
         LineBox[CompressedData["
1:eJwVx3k0FAgcwHHHpPLcZtyGMZOR482W53jSlkUTea0ZNjSoRLQVQnm5kmql
wnRpWjnfss5NUjti/X6u0jNiZZg37pmctTuUnNHaP77v876UkCh2mIKcnJzP
Zv+7EYpm+Up0LKhUSd5RScNY5Qx2N4GOc1mvU+5U0HD6CStNTpGO2XRJnNLm
965KxkO+meNY4lfL2TIalnK3FNGXzXHAl08pL6Xh4UZPk6cz5hhy24NMKqZh
nrbQsE1gjimn97uU8mjo3DJDnL1rjncqWLk112iYQrp+O4FkjsS7jt62vjS8
FWC3cbRwBxbTqyh2pjSMn4jkpdvswEHu8fXWSSqGhxvwl2toyEkRVzsWUdFM
LP673pmGLZFKY4IAKkp+UD0PTVSUDdqnh26hYurnDkdNVyqqOQWEOVWZ4cz9
R6u13WaYXqtXJvUxQz/rIZdHbDO0YP08ZyejILfO9mr3MAVvXmqaZ2RS8Pdr
xH9YZyjYcxAEwzsp2OoZPqQnM8U1n64J1QZTrJDvbmckmmJaZhK4sUzROoh1
WmfNBOP6X7A+8E0QD+jywggmOJUZQlXuIeOs5tYGXzIZyeu7Bg8tGCP5jXha
QDNGz540hxeKxjh9QqQ8sccII4V+WbEUI2yYrHf4ZZ8hFtkwrgksDdFzrfo5
MA1QFM10Oe9ggNHznfRHx/Wx+17nENdHHwvedf22FqaH+pfztrVx9PDkwxXS
RqIudqwU8sWxuvh+1kHEy9HBpiKzEzpcHYyyPFrSmEfCLzNOikU8EmYfKj2V
+pyIrtw37889JWK9f//urnptHHxla5T/QhsLvuh517ZooadYJNiNWmgYd/2E
w5Am+kMdy0msiUUjwmR3iQa63Wpuqx/TwISfCo2m5tWxlunhkShTRyvVl6Ox
FmpIZCxXPDBXQ9njXL6WowoGkaJzGu1VcFTWc2k+QhmXGMf0dDeVGLz1lsRs
w/VXWmP5m2ptN9HQzlfCM1JG+41NG3QenI/rJaAyPZmw8I6AF7/2Tk+NKuBe
LTd/B6kCBpeEHsxVk8f7PeSwZqI8PvVKSmy23gDRXE5qmesG8DIvVVQfXoVF
mxjr9/GrMOBLrtB/tgjZTKOdaSuL4CxgC8vffAKvgB97Prh/hk/3gopj3f+F
pS8WA9ETMmCpVpgdMZqCWBdDCwnMwFKX1YL8kxEIDlF9xqmUgmXJPFPxwghk
ZuYy1XKkEJj0Zy5hzwg0XAma60yVQoulq+vW18OQH9VXEHFEClnpR++qjA5B
q7VzkIWCFGguGd/pqg8Cf7eJfwdHAuy6ybPWUQNwOVnOD3THoerXookjjB5w
/hrYOR8+DGPnlg1th7shajziB/V9wxBRWn2Wz+mGpvy2/V46wxCvq6fW6v8W
Uve6bNDahyBn5aO3iC0AO96nbDZtCIR/5fQrMDvA7fTNgZJpMbAPzI77MRBs
z9arBGWIoG79+7CXHIA9XrKHCaEiaI0sV2r3b4K2UIUL/H0ikHineIjZjZBA
lFo9WBwACsmim8DkQ+Su1Ce9YQOQn5coDmDUQNKCmnuwZz9wjAVLHf5/AJXA
tb9C74eEb4VOI+wqqNHNjm8j9EN9qyduY5ZB6DHOSCUIwd4rvzOIUQgXe4M/
rtsLoSJw0miS/Rim+TrbqSQhzJhyizWYPLilaMOJ+NwHp8ql1WEMLgS2pmb6
1PSB56BBuCnzBrwV2oauZPXB65ORMRcYqaASYxQE5/rghujq2n1GNNQuS2JK
vPqgx7U5jpSR3fSSIyqotOqD/wAor7d6
          "]]}, Annotation[#, "Charting`Private`Tag$5376#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          RGBColor[0, 
           NCache[
            Rational[2, 3], 0.6666666666666666], 0]], 
         LineBox[CompressedData["
1:eJwVlHk81VsXxhGRjOWYE3JRlJCUXHNk5qgohUQUUYZkrIgMSVeJkvnmxqUo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          "]]}, Annotation[#, "Charting`Private`Tag$5376#2"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          RGBColor[0, 0, 1]], 
         LineBox[CompressedData["
1:eJwd23c4lm0bAHBk7/XYK3qQGRWKVG+yQ3ZI2SoJaaFCJSVRVFZIkqRQSNZ9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          "]]}, Annotation[#, "Charting`Private`Tag$5376#3"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          RGBColor[0, 1, 1]], 
         LineBox[CompressedData["
1:eJwV13c8VW8cB3ClQWWF4h73nnM5MkuRkFIZoZAZkgpp22SLpFQqKylkhFBK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          "]]}, Annotation[#, "Charting`Private`Tag$5376#4"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          RGBColor[
           NCache[
            Rational[2, 3], 0.6666666666666666], 
           NCache[
            Rational[2, 3], 0.6666666666666666], 0]], 
         LineBox[CompressedData["
1:eJwknHk4VV8Xx0UahEpluO48iTRJKJIpEhVSpAhFkkwlZKiUkiJFfkJCSNJM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          "]]}, Annotation[#, "Charting`Private`Tag$5376#5"]& ]}}, {}, {}}, {
    Ticks -> {
       Charting`ScaledTicks[{Log, Exp}], 
       Charting`ScaledTicks[{Log, Exp}]}, GridLines -> {
       Charting`ScaledTickValues[{Log, Exp}], 
       Charting`ScaledTickValues[{Log, Exp}]}, FrameTicks -> {{
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}, {
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, DisplayFunction -> 
     Identity, DisplayFunction -> Identity, Ticks -> {Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& }, AxesOrigin -> 
     NCache[{-Log[
         100000], -12.188579460163682`}, {-11.512925464970229`, \
-12.188579460163682`}], FrameTicks -> {{Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}, {Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, 
     GridLines -> {{-11.512925464970229`, -9.210340371976182, \
-4.605170185988091, -2.3025850929940455`, 
      0.}, {-11.512925464970229`, -9.210340371976182, -4.605170185988091, \
-2.3025850929940455`, 0.}}, DisplayFunction -> Identity, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None, 
       "ClippingRange" -> {{{-11.512925230012566`, -2.3495766255041283`*^-7}, \
{-12.188579460163682`, -0.31516543016317267`}}, {{-11.512925230012566`, \
-2.3495766255041283`*^-7}, {-12.188579460163682`, -0.31516543016317267`}}}}, 
     DisplayFunction -> Identity, AspectRatio -> 0.8, Axes -> {True, True}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, 
     BaseStyle -> (FontSize -> 12), 
     CoordinatesToolOptions -> {"DisplayFunction" -> ({
         Exp[
          Part[#, 1]], 
         Exp[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Exp[
          Part[#, 1]], 
         Exp[
          Part[#, 2]]}& )}, DisplayFunction :> Identity, 
     Frame -> {{True, True}, {True, True}}, FrameLabel -> {{
        FormBox["\"\"", TraditionalForm], None}, {
        FormBox["\"a\"", TraditionalForm], None}}, FrameStyle -> GrayLevel[0],
      FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {Automatic, Automatic}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, PlotRange -> 
     NCache[{{-Log[100000], 0}, {-12.188579460163682`, 
        0.}}, {{-11.512925464970229`, 0}, {-12.188579460163682`, 0.}}], 
     PlotRangeClipping -> True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}],FormBox[
    FormBox[
     TemplateBox[{
      "\"\\!\\(\\*SubscriptBox[\\(\[Delta]\\), \\(m\\)]\\)(a)\"", 
       "\"\\!\\(\\*SubscriptBox[\\(V\\), \\(m\\)]\\)(a)\"", 
       "\"\\!\\(\\*SubscriptBox[\\(\[Delta]\\), \\(DE\\)]\\)(a)\"", 
       "\"\\!\\(\\*SubscriptBox[\\(V\\), \\(DE\\)]\\)(a)\"", "\"\[Phi](a)\""},
       "LineLegend", DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    RGBColor[1, 0, 0]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    RGBColor[1, 0, 0]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    RGBColor[0, 
                    NCache[
                    Rational[2, 3], 0.6666666666666666], 0]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    RGBColor[0, 
                    NCache[
                    Rational[2, 3], 0.6666666666666666], 0]], {}}}, 
                    AspectRatio -> Full, ImageSize -> {20, 10}, 
                    PlotRangePadding -> None, ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    RGBColor[0, 0, 1]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    RGBColor[0, 0, 1]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    RGBColor[0, 1, 1]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    RGBColor[0, 1, 1]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #4}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    RGBColor[
                    NCache[
                    Rational[2, 3], 0.6666666666666666], 
                    NCache[
                    Rational[2, 3], 0.6666666666666666], 0]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    RGBColor[
                    NCache[
                    Rational[2, 3], 0.6666666666666666], 
                    NCache[
                    Rational[2, 3], 0.6666666666666666], 0]], {}}}, 
                    AspectRatio -> Full, ImageSize -> {20, 10}, 
                    PlotRangePadding -> None, ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #5}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[1, 0, 0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> RGBColor[0.6666666666666666, 0., 0.], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "RGBColor[1, 0, 0]"], Appearance -> None, BaseStyle -> {},
                     BaselinePosition -> Baseline, DefaultBaseStyle -> {}, 
                    ButtonFunction :> With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[1, 0, 0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[1, 0, 0], Editable -> False, Selectable -> 
                    False]}], "]"}], ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0, 
                    Rational[2, 3], 0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> RGBColor[0., 0.4444444444444444, 0.], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "RGBColor[0, 2/3, 0]"], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = RGBColor[0, 
                    Rational[2, 3], 0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0, 
                    Rational[2, 3], 0], Editable -> False, Selectable -> 
                    False]}], "]"}], ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0, 0, 1], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> RGBColor[0., 0., 0.6666666666666666], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "RGBColor[0, 0, 1]"], Appearance -> None, BaseStyle -> {},
                     BaselinePosition -> Baseline, DefaultBaseStyle -> {}, 
                    ButtonFunction :> With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0, 0, 1]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0, 0, 1], Editable -> False, Selectable -> 
                    False]}], "]"}], ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0, 1, 1], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[0., 0.6666666666666666, 0.6666666666666666], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "RGBColor[0, 1, 1]"], Appearance -> None, BaseStyle -> {},
                     BaselinePosition -> Baseline, DefaultBaseStyle -> {}, 
                    ButtonFunction :> With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0, 1, 1]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0, 1, 1], Editable -> False, Selectable -> 
                    False]}], "]"}], ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[
                    Rational[2, 3], 
                    Rational[2, 3], 0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[0.4444444444444444, 0.4444444444444444, 0.], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "RGBColor[2/3, 2/3, 0]"], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = RGBColor[
                    Rational[2, 3], 
                    Rational[2, 3], 0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[
                    Rational[2, 3], 
                    Rational[2, 3], 0], Editable -> False, Selectable -> 
                    False]}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2, ",", #3, ",", #4, ",", #5}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{{3.7356328125508842`*^9, 3.735632905990855*^9}, 
   3.735634920472043*^9, 3.7673472830302134`*^9, 
   3.796457238333595*^9},ExpressionUUID->"51708043-2c36-494b-b2ee-\
5819cf103c2f"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LogLogPlot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Delta]m", "[", "a", "]"}], "/.", 
        RowBox[{"sol", "[", 
         RowBox[{"[", "1", "]"}], "]"}]}], ",", 
       RowBox[{"\[Delta]man", "[", "a", "]"}]}], "}"}], "//", "Abs"}], "//", 
    "Evaluate"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "ai", ",", "1"}], "}"}], ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameStyle", "\[Rule]", "Black"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
     "\"\<a\>\"", ",", 
      "\"\<\!\(\*SubscriptBox[\(\[Delta]\), \(m\)]\)(a)\>\""}], "}"}]}], ",", 
   RowBox[{"BaseStyle", "\[Rule]", 
    RowBox[{"FontSize", "\[Rule]", "12"}]}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", "Black", "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"Dashed", ",", "Black"}], "}"}]}], "}"}]}], ",", 
   RowBox[{"PlotLegends", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<Numerical\>\"", ",", "\"\<Analytical\>\""}], "}"}]}]}], 
  "]"}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356249752743273`*^9, 3.735624983659234*^9}, {
   3.735625027689151*^9, 3.7356250891291304`*^9}, 3.735625464138838*^9, {
   3.7964572450599484`*^9, 
   3.796457246451383*^9}},ExpressionUUID->"439b4876-4c8e-47f2-b562-\
4a4f2f24b447"],

Cell[BoxData[
 TemplateBox[{GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          GrayLevel[0]], 
         LineBox[CompressedData["
1:eJwVx3k0FAgcwHHHpPLcZtyGMZOR482W53jSlkUTea0ZNjSoRLQVQnm5kmql
wnRpWjnfss5NUjti/X6u0jNiZZg37pmctTuUnNHaP77v876UkCh2mIKcnJzP
Zv+7EYpm+Up0LKhUSd5RScNY5Qx2N4GOc1mvU+5U0HD6CStNTpGO2XRJnNLm
965KxkO+meNY4lfL2TIalnK3FNGXzXHAl08pL6Xh4UZPk6cz5hhy24NMKqZh
nrbQsE1gjimn97uU8mjo3DJDnL1rjncqWLk112iYQrp+O4FkjsS7jt62vjS8
FWC3cbRwBxbTqyh2pjSMn4jkpdvswEHu8fXWSSqGhxvwl2toyEkRVzsWUdFM
LP673pmGLZFKY4IAKkp+UD0PTVSUDdqnh26hYurnDkdNVyqqOQWEOVWZ4cz9
R6u13WaYXqtXJvUxQz/rIZdHbDO0YP08ZyejILfO9mr3MAVvXmqaZ2RS8Pdr
xH9YZyjYcxAEwzsp2OoZPqQnM8U1n64J1QZTrJDvbmckmmJaZhK4sUzROoh1
WmfNBOP6X7A+8E0QD+jywggmOJUZQlXuIeOs5tYGXzIZyeu7Bg8tGCP5jXha
QDNGz540hxeKxjh9QqQ8sccII4V+WbEUI2yYrHf4ZZ8hFtkwrgksDdFzrfo5
MA1QFM10Oe9ggNHznfRHx/Wx+17nENdHHwvedf22FqaH+pfztrVx9PDkwxXS
RqIudqwU8sWxuvh+1kHEy9HBpiKzEzpcHYyyPFrSmEfCLzNOikU8EmYfKj2V
+pyIrtw37889JWK9f//urnptHHxla5T/QhsLvuh517ZooadYJNiNWmgYd/2E
w5Am+kMdy0msiUUjwmR3iQa63Wpuqx/TwISfCo2m5tWxlunhkShTRyvVl6Ox
FmpIZCxXPDBXQ9njXL6WowoGkaJzGu1VcFTWc2k+QhmXGMf0dDeVGLz1lsRs
w/VXWmP5m2ptN9HQzlfCM1JG+41NG3QenI/rJaAyPZmw8I6AF7/2Tk+NKuBe
LTd/B6kCBpeEHsxVk8f7PeSwZqI8PvVKSmy23gDRXE5qmesG8DIvVVQfXoVF
mxjr9/GrMOBLrtB/tgjZTKOdaSuL4CxgC8vffAKvgB97Prh/hk/3gopj3f+F
pS8WA9ETMmCpVpgdMZqCWBdDCwnMwFKX1YL8kxEIDlF9xqmUgmXJPFPxwghk
ZuYy1XKkEJj0Zy5hzwg0XAma60yVQoulq+vW18OQH9VXEHFEClnpR++qjA5B
q7VzkIWCFGguGd/pqg8Cf7eJfwdHAuy6ybPWUQNwOVnOD3THoerXookjjB5w
/hrYOR8+DGPnlg1th7shajziB/V9wxBRWn2Wz+mGpvy2/V46wxCvq6fW6v8W
Uve6bNDahyBn5aO3iC0AO96nbDZtCIR/5fQrMDvA7fTNgZJpMbAPzI77MRBs
z9arBGWIoG79+7CXHIA9XrKHCaEiaI0sV2r3b4K2UIUL/H0ikHineIjZjZBA
lFo9WBwACsmim8DkQ+Su1Ce9YQOQn5coDmDUQNKCmnuwZz9wjAVLHf5/AJXA
tb9C74eEb4VOI+wqqNHNjm8j9EN9qyduY5ZB6DHOSCUIwd4rvzOIUQgXe4M/
rtsLoSJw0miS/Rim+TrbqSQhzJhyizWYPLilaMOJ+NwHp8ql1WEMLgS2pmb6
1PSB56BBuCnzBrwV2oauZPXB65ORMRcYqaASYxQE5/rghujq2n1GNNQuS2JK
vPqgx7U5jpSR3fSSIyqotOqD/wAor7d6
          "]]}, Annotation[#, "Charting`Private`Tag$8239#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          Dashing[{Small, Small}], 
          GrayLevel[0]], 
         LineBox[CompressedData["
1:eJwVzX1YCwgcB/BeprRH761Nb2trFRpzVMvpOIeikppoldzp3YNIOWfpLorL
XR5zyKKX1dHLrBTl2bTr96PS0tiK0U2FaKvO6cml1yu3++P7fJ7v958vLe4Q
J9HIwMBgmz7/O5+A9BITT3yZvyXL/SYD04lnOUqCJ0p564cLRAwcuhV+ysDY
E6vE07Om+t49M/Am7rMH+mT5k4arGFjBX1DmOeWBXJnQtrKCgaGyIGr9sAda
CsPcbcoZWGyrdmxVeGBWwteK3wUM9H8wbDfymwcGP+icqM1l4I+k0/k8kgfe
gKSrqyIY+GuUz3y00B2zcyLk3q4MPDaYKvh5uTs2O37BbtW6YXKyg2SqTv/D
iMz3K3NDukbTJfVnYNnRi6o7UW448I15GjS74STxbfueBW6Y/Y/cz3qjflfk
UrzFdBy+VDhzW0nHUk9mqnYHHSOZvRsKOXT0Ffxg5jtKQ37D6hxlHw0p65lT
rHM0rMy1+zt8Pw2XGu7V9C2lYUtQci9l1BWfbTs/bt7kiiJDZRsr0xUTuN3y
TeGuyIwN32c/S0Vl//GUvyRUxACyIJFARWpGhiVR5YIj1qZNES4umGvOkAeP
O6NLh2ZIwXBGsl2j8V1jZxza20McXOuE9V91xqfTnLBJK2WfWe+I6vxrhxTL
HDFotqYRAh3wQw6TncZ2wMNjnZ6F3y3GlDaphr9jMZY+fXx9NpGCL0J0pq0x
FIy/Mk2azyTjXNovNzXpZHw3wu4RFNjjAt2VdfZ8ezy0LPqGrJiEfKq3RCgg
4fngiqTsRjskbBqPPFhvh1Lu81WPpbboI+8gF9+1xdJPlLDbD2ywjRA/tRJt
0DHj9F52rzXGGdGYbI01lvWrszYPWOGKUi2t4bUV8nYKnXRjluhXUKBJHrVE
L/N7r9KXWOAjmTA3w8MCR4uuSWz8FuHTxoa4k76L8NWo6vhYChHnd4sj65OJ
OODwJGzgyEI03UJs1aUtRBszqpVtiQnuCHNoDS42wSb7y2kZ3QQ0zV/jre4i
4Pf/dg/pXhnh/hKezX69e24kbLlmYYi2XadVbL31IScy7zPn4dOj0E/uegXn
jotqQmfAJ1eRt1rviwgX0eI7EyDW3WNJbk+Av4Kjru74CCunA36K1fvxYmx5
+uYPcHVuu8vMpg8Qbi6i73LSgWikfHbcUQeTj73GDW/1A++Zcd6+2n4QXy0b
3MVSwcmtod/2rVDB64NTjqv7lFCbUkfY0KuElIqaA5IYJZidVWxtj1bCMTLF
ooX7BOb4tK6nkU+gYPp9WA9HAe1/Rj17H64A9R8Fz40C5fBivqfGJUAOnICR
N5EshMqkc3dOrkBomFuXeC8GwGxuSmIZDdCSWm3Sxm2GpEK6mVNkMwyE/bhV
w5FB6PSe697hMqCRligJgRLQOokfJm6WQElxpiaKVQcnLljJ2pfXQYyzYlLO
rYU1ra+1lTtrgfdZ+GU/RwynZAc4uF0M0pYgXBhYBfKiCerYxirwDSnpjGUJ
gafyusxhCkG0W+uk5RRBwsh2tTqkCIZd+eVWgQIwf2fLM9kggKTqtzWJLD5Y
j5Lrk5fyIeilQ7JrYB50DMnPrFybB+3xqUeOsrJB6uZVesEtG/J6cmYvsQ7D
8odtuwbJh0G18X4G6ez55jYy3bSaO7nuP9IQfjg=
          "]]}, Annotation[#, "Charting`Private`Tag$8239#2"]& ]}}, {}, {}}, {
    Ticks -> {
       Charting`ScaledTicks[{Log, Exp}], 
       Charting`ScaledTicks[{Log, Exp}]}, GridLines -> {None, None}, 
     FrameTicks -> {{
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}, {
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, DisplayFunction -> 
     Identity, DisplayFunction -> Identity, Ticks -> {Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& }, AxesOrigin -> 
     NCache[{-Log[
         100000], -10.334270396333912`}, {-11.512925464970229`, \
-10.334270396333912`}], FrameTicks -> {{Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}, {Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, GridLines -> {None, None}, 
     DisplayFunction -> Identity, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None, 
       "ClippingRange" -> {{{-11.512925230012566`, -2.3495766255041283`*^-7}, \
{-10.334270396333912`, 
        0.000022264794502756058`}}, {{-11.512925230012566`, \
-2.3495766255041283`*^-7}, {-10.334270396333912`, 
        0.000022264794502756058`}}}}, DisplayFunction -> Identity, 
     AspectRatio -> NCache[GoldenRatio^(-1), 0.6180339887498948], 
     Axes -> {True, True}, AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, 
     BaseStyle -> (FontSize -> 12), 
     CoordinatesToolOptions -> {"DisplayFunction" -> ({
         Exp[
          Part[#, 1]], 
         Exp[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Exp[
          Part[#, 1]], 
         Exp[
          Part[#, 2]]}& )}, DisplayFunction :> Identity, 
     Frame -> {{True, True}, {True, True}}, FrameLabel -> {{
        FormBox[
        "\"\\!\\(\\*SubscriptBox[\\(\[Delta]\\), \\(m\\)]\\)(a)\"", 
         TraditionalForm], None}, {
        FormBox["\"a\"", TraditionalForm], None}}, FrameStyle -> GrayLevel[0],
      FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, PlotRange -> 
     NCache[{{-Log[100000], 0}, {-10.334270396333912`, 
        0.000022264794502756058`}}, {{-11.512925464970229`, 
         0}, {-10.334270396333912`, 0.000022264794502756058`}}], 
     PlotRangeClipping -> True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}],FormBox[
    FormBox[
     TemplateBox[{"\"Numerical\"", "\"Analytical\""}, "LineLegend", 
      DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    GrayLevel[0]], {
                    LineBox[{{0, 10}, {40, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    GrayLevel[0]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {40, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    Dashing[{Small, Small}], 
                    GrayLevel[0]], {
                    LineBox[{{0, 10}, {40, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    Dashing[{Small, Small}], 
                    GrayLevel[0]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {40, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], "GrayLevel[0]"], 
                    Appearance -> None, BaseStyle -> {}, BaselinePosition -> 
                    Baseline, DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False]}], 
                 "]"}], ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   RowBox[{"Dashing", "[", 
                    RowBox[{"{", 
                    RowBox[{"Small", ",", "Small"}], "}"}], "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], "GrayLevel[0]"], 
                    Appearance -> None, BaseStyle -> {}, BaselinePosition -> 
                    Baseline, DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False]}], 
                 "]"}]}], "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{
  3.7356249842490606`*^9, {3.7356250498492017`*^9, 3.735625093137724*^9}, {
   3.735625304988657*^9, 3.735625318403673*^9}, 3.7356253898930063`*^9, 
   3.735625464138838*^9, 3.7356258357628703`*^9, 3.735627643352227*^9, {
   3.735629461682068*^9, 3.7356294769578686`*^9}, 3.7356296113577585`*^9, 
   3.735632927820633*^9, 3.767347288315137*^9, 
   3.796457250453369*^9},ExpressionUUID->"2b1592d9-b47a-45cf-a12c-\
c66ef5e4015c"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"LogLogPlot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"Vm", "[", "a", "]"}], "/.", 
        RowBox[{"sol", "[", 
         RowBox[{"[", "1", "]"}], "]"}]}], ",", 
       RowBox[{"Vman", "[", "a", "]"}]}], "}"}], "//", "Abs"}], "//", 
    "Evaluate"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "ai", ",", "1"}], "}"}], ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameStyle", "\[Rule]", "Black"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
     "\"\<a\>\"", ",", "\"\<\!\(\*SubscriptBox[\(V\), \(m\)]\)(a)\>\""}], 
     "}"}]}], ",", 
   RowBox[{"BaseStyle", "\[Rule]", 
    RowBox[{"FontSize", "\[Rule]", "12"}]}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", "Black", "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"Dashed", ",", "Black"}], "}"}]}], "}"}]}], ",", 
   RowBox[{"PlotLegends", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<Numerical\>\"", ",", "\"\<Analytical\>\""}], "}"}]}]}], 
  "]"}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356249293691154`*^9, 3.7356249982486596`*^9}, {
   3.7356252311484785`*^9, 3.7356252421689615`*^9}, 3.735625464138838*^9, 
   3.7964572537449512`*^9},ExpressionUUID->"38b88036-0f64-4859-9faa-\
e4dd24191c06"],

Cell[BoxData[
 TemplateBox[{GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          GrayLevel[0]], 
         LineBox[CompressedData["
1:eJwVlHk81VsXxhGRjOWYE3JRlJCUXHNk5qgohUQUUYZkrIgMSVeJkvnmxqUo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          "]]}, Annotation[#, "Charting`Private`Tag$10020#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          Dashing[{Small, Small}], 
          GrayLevel[0]], 
         LineBox[CompressedData["
1:eJwV0H04EwgcwHFks3bI29gYRjV6uXZSXo5wKotUjDI5jpjlqSh0O6Sj99ej
wnmJEJI4KvWQ6vcrvVDWpBOJOnsxWx0nndiyrvvj+3z+/9ptS+LwdLS0tDZ8
7X81cWhfRnRAMY3ra5BEwxTycY5I1wHDgJHstYuGo43BB7TmOGBXnKIiaQcN
e1Ti4W1fmHizMV27h0/DmlxChcM0E0+sLcWCaBpuvB1ge1XBxOV7xF4MDg1L
TXutHnQxMbtjp4fzShp63leYKc8ykSHIXhGupuJ+yuFT6RQmbntCr1dkUvFk
+ErN1vKFmGQotH5CoqJAllh49NuFGFFMbJnKt0A+37JlumkBrqdf6+PTLNB+
YOB5q+cCJOYkTzhWm6PY12AP3J2Pjz5V5S5zMMesyQ4349XzMa42mpt1hYKK
vCLVNZE9Cv2dLO3dKBi2dPCHIo49egynxJDazDC32fmgaMgOUxOeVnr5meGl
Q2Z/B++ww5Pu9fr3XphiewB/kDrOwEtfNOqjYaZYpy16yMpg4G8342VlEhNc
GhmcYK62xfRYn1AVzwTRz6KQp2uLa7gX31UojVFprNcWamODr1ZZZRYJjNGm
c2C0a4E1ptAIeb0qIxyN6SfLPOjYL5tTHXPECNtGWl2PeFuhb22qx3KSEQao
G24A2xJNeEMfS6nzcPfEU4eir9/dItTxAidDvPBCWKXmUZG/albyysUAY3+f
oWgyLHCJ9qLNg4H6KFW69hcWmGNDKN9CwP8GkxZvrb5dSkHXvOPS4kQy5qyv
ic+6YYZhN3qro47MxVbuy+XCVlPMvk9IvnKChBf+pQZdu2+C/J/W5eSe00Or
1MMxroPGKBlqLqc0ELHiTW/mWrERJqwj3Jl/nYDpm8vp8ol5eDjvV+HzTl1c
YnDrbYqjIUbt1mdRZXNw/HxJi4mbPvq4ZZbI3+vg2/HutIntZGwZtvi8haCD
YstnQeJkEroLCvibyNpoMtfWyLSMiMFjXo/ltlrYZp6/J7VHFwOVk+PfpWrg
5889o/K3OqjX7cSvyv4MUdVx60oMtXHsl/ZHYU0quBq4L+PeUg0oZuie1i3T
UHg6ra5howoii0tCTDunoC/Upo52fQpmZ3eF+Ks/gmcXp/dy5wdodCclnCFO
wodzkZUpa8fgtNtUPM15AoIN6uy30OXQ0WBiWPV8DD4Jl3zUbnwDZcUdae7j
SqgvrpBtYXWD0IB5c+MmOfy1a9rKeUgEg82e0bbecthe07CzJUIEYxEh5H+W
yUFgQTVs5z4Do7qsqDOGciiYeR/Uz+mCUL/XxD+FI9B7p+ClDrsDBvfnhoUH
jgDHTzkcxkJQjqumY/1l0DzrxbsVAaApML64wl0G7YmXiQ+5d8HAy3EDYZEM
xEH7/Qc4t4F1anNFDUkGdhRHkS67BVIcm/wVj6VQVpoxEM5qAlU0rzjRTwoR
1l2fOrh/gD5p3xpvFymkfyn//g2nHqwbz47NY0qhtT0ASexa8J4F36sEKbgE
lj2NZJXDgSLLd5MPJFD34wh9hHMe8nyc8h80S0DByK00YhdCtZztnV8lgfjL
kgYeKxcerdx7zuWQBAJeW/IZ7GPQN3hylV6qBB7HJibvZWWB4mClvC9WAsf6
D6rzWLthZnHrmdoQCXSvvpdKOZ5zl9wj8khbLYH/AJUZgDg=
          "]]}, Annotation[#, "Charting`Private`Tag$10020#2"]& ]}}, {}, {}}, {
    Ticks -> {
       Charting`ScaledTicks[{Log, Exp}], 
       Charting`ScaledTicks[{Log, Exp}]}, GridLines -> {None, None}, 
     FrameTicks -> {{
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}, {
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, DisplayFunction -> 
     Identity, DisplayFunction -> Identity, Ticks -> {Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& }, AxesOrigin -> 
     NCache[{-Log[
         100000], -6.358449017169272}, {-11.512925464970229`, \
-6.358449017169272}], FrameTicks -> {{Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}, {Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, GridLines -> {None, None}, 
     DisplayFunction -> Identity, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None, 
       "ClippingRange" -> {{{-11.512925230012566`, -2.3495766255041283`*^-7}, \
{-6.358449017169272, -0.6019865196417993}}, {{-11.512925230012566`, \
-2.3495766255041283`*^-7}, {-6.358449017169272, -0.6019865196417993}}}}, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, 
     BaseStyle -> (FontSize -> 12), 
     CoordinatesToolOptions -> {"DisplayFunction" -> ({
         Exp[
          Part[#, 1]], 
         Exp[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Exp[
          Part[#, 1]], 
         Exp[
          Part[#, 2]]}& )}, DisplayFunction :> Identity, 
     Frame -> {{True, True}, {True, True}}, FrameLabel -> {{
        FormBox[
        "\"\\!\\(\\*SubscriptBox[\\(V\\), \\(m\\)]\\)(a)\"", TraditionalForm],
         None}, {
        FormBox["\"a\"", TraditionalForm], None}}, FrameStyle -> GrayLevel[0],
      FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, PlotRange -> 
     NCache[{{-Log[100000], 0}, {-6.358449017169272, 
        0.}}, {{-11.512925464970229`, 0}, {-6.358449017169272, 0.}}], 
     PlotRangeClipping -> True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}],FormBox[
    FormBox[
     TemplateBox[{"\"Numerical\"", "\"Analytical\""}, "LineLegend", 
      DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    GrayLevel[0]], {
                    LineBox[{{0, 10}, {40, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    GrayLevel[0]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {40, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    Dashing[{Small, Small}], 
                    GrayLevel[0]], {
                    LineBox[{{0, 10}, {40, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    Dashing[{Small, Small}], 
                    GrayLevel[0]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {40, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], "GrayLevel[0]"], 
                    Appearance -> None, BaseStyle -> {}, BaselinePosition -> 
                    Baseline, DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False]}], 
                 "]"}], ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   RowBox[{"Dashing", "[", 
                    RowBox[{"{", 
                    RowBox[{"Small", ",", "Small"}], "}"}], "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], "GrayLevel[0]"], 
                    Appearance -> None, BaseStyle -> {}, BaselinePosition -> 
                    Baseline, DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False]}], 
                 "]"}]}], "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356249565389457`*^9, 3.735624965148364*^9}, 
   3.735624999728712*^9, 3.7356252432630863`*^9, 3.7356253920277452`*^9, 
   3.735625464138838*^9, 3.7356258379814944`*^9, 3.735627643637348*^9, 
   3.7356294798181543`*^9, 3.735629613117354*^9, 3.7356329295147743`*^9, 
   3.767347290045441*^9, 
   3.7964572550776477`*^9},ExpressionUUID->"7dd88b16-f17f-44de-922b-\
53a6a19a6f3a"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LogLogPlot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Delta]DE", "[", "a", "]"}], "/.", 
        RowBox[{"sol", "[", 
         RowBox[{"[", "1", "]"}], "]"}]}], ",", 
       RowBox[{"\[Delta]DEan", "[", "a", "]"}], ",", 
       RowBox[{
        FractionBox["3", "2"], 
        RowBox[{"(", 
         RowBox[{"1", "+", "w"}], ")"}], 
        FractionBox["om", 
         RowBox[{"cs2", " ", 
          RowBox[{"k", "^", "2"}]}]], "\[Delta]0"}]}], "}"}], "//", "Abs"}], "//",
     "Evaluate"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "ai", ",", "1"}], "}"}], ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameStyle", "\[Rule]", "Black"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
     "\"\<a\>\"", ",", 
      "\"\<\!\(\*SubscriptBox[\(\[Delta]\), \(DE\)]\)(a)\>\""}], "}"}]}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"DotDashed", ",", "Black"}], "}"}], ",", 
      RowBox[{"{", "Red", "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"Dashed", ",", "Blue"}], "}"}]}], "}"}]}], ",", 
   RowBox[{"BaseStyle", "\[Rule]", 
    RowBox[{"FontSize", "\[Rule]", "12"}]}], ",", 
   RowBox[{"PlotLegends", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
     "\"\<Numerical\>\"", ",", 
      "\"\<Analytical (k<<aH/\!\(\*SubscriptBox[\(c\), \(s\)]\))\>\"", ",", 
      "\"\<Analytical (k>>aH/\!\(\*SubscriptBox[\(c\), \(s\)]\))\>\""}], 
     "}"}]}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      SuperscriptBox["10", 
       RowBox[{"-", "6"}]], ",", 
      SuperscriptBox["10", 
       RowBox[{"-", "4"}]]}], "}"}]}]}], "]"}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356249752743273`*^9, 3.735624983659234*^9}, {
   3.735625027689151*^9, 3.7356250891291304`*^9}, 3.735625464138838*^9, {
   3.735625845909025*^9, 3.7356258510977726`*^9}, {3.7356259805786643`*^9, 
   3.735626188438608*^9}, 
   3.7964572574551373`*^9},ExpressionUUID->"a8597ec2-9743-44d1-96c1-\
848199c96884"],

Cell[BoxData[
 TemplateBox[{GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          Dashing[{0, Small, Small, Small}], 
          GrayLevel[0]], 
         LineBox[CompressedData["
1:eJwd23c4lm0bAHBk7/XYK3qQGRWKVG+yQ3ZI2SoJaaFCJSVRVFZIkqRQSNZ9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          "]]}, Annotation[#, "Charting`Private`Tag$10964#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          RGBColor[1, 0, 0]], 
         LineBox[CompressedData["
1:eJwVz31MU2cUBvAr42M4JFZUhiCl7b3vfclgJiijUQQU2AQ2lSplTHBYZKRB
ZAszbIOoG1hDaqHCHNRhJ2ruHIkCdYyPEngioC5Da00GrCkL4viq32NMAlvd
3R8nT345J3lyZJpiVb4HwzDR4vyf7oOQm715ZFss5YZcipLlVSqbJw9ZFT+V
KXqmJf0r5hUeWksnKxd9b3HivuYlQUZnZ9hPH1IIRq8mfoHA0j+70rmfYmdP
qrRtluCQvr1DmkNxLuDX4IEhAvXvhSknsihir8+udtUSaP95KFxPp6jacEv9
l5GgXP3gjzLRw41Cg7uaIP9J9PQm0R+X5oUE6AlKXElKYTfFpYgxRWwFwdno
Yh/dLgq/+rtR1SUEeUmPNm1+j2KssGN31F6CHczWppIdFEfXnDj1xRqCY/YQ
pSyOono4zV8IIIi4v++lbSuFuX6V0S4huESDqo6K7gs6X0f9CW6ryx44YimY
UOvZYW+Ck8UjLcYtFF+S55ejFjioqi/onikpKmNyBl0ODitTJujxjRT6rGj3
B+c5xO2/s2qEp+iON7fWmDlMHZAmlIp2cT55A40cDL2Z4YGiU/8cvRlp4uBO
ePammlC8qi8/zdRymK9xethZCl1PPytUcEibrrjaJRP7paq05x9xePFZcog6
mKJ08nDDyUgOPr7jtddWiP/WPc4+84Z4b5K+iBGt23ZIdjGcw8IPI0yPH8U3
57TNvRyHTAXj2f8aRXvGwZ759RzevZo9MORLMTeYNZ7nL+7jrN42L4piIYkm
PGXRnJKba/iXR0HBus6FVhbL9kj8Zh7ykDsc9q5YFsNeNknbDR4T21d80ter
gL2opeO7eh7H524pJYkKTB8+8nekhsfs16ZFi02OG+5vzUI4j8wI5zaTSo7t
yXOVvpMExh83VtjGZNh8ufWXRDPB95WrH6cXyvDEc9wwoSLoTy1wvv40DPpP
NSrHIofmZbbBDWVhMOzqvqm5yCEiJ127dkmKpcXPTUXxHPB2YEO+pxSJt08r
B0dZuCQ+1r2hoXiL1AfWHWER+rNjZohdj6J7vwl3vFnMHBhdPrklBPucZ+4+
alTAOtUVo4sPhlPbFTDPK5C6dKW97511qHm/uTv5mhwek8eaTG1BWJvRbm0r
lOM/Ykyb3A==
          "]]}, Annotation[#, "Charting`Private`Tag$10964#2"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          Dashing[{Small, Small}], 
          RGBColor[0, 0, 1]], 
         LineBox[CompressedData["
1:eJxTTMoPSmViYGAwAWIQ/S/lgNI8NvUD1zZce52drnpgrvBV6SNn1OB8m0Mv
RV5NQvDrRFt7qkQR/O5I039RC1Th/PKneTPadRH89HSpHT82qMD5SrduXdxp
g+A/cuIt3L9PGc5v+HzCQtAZwX85ZeavTeeV4PxwnTuOM4MQ/AlbjJvP31WE
85e3iLwNzEbwD3ul35F4rwDnr2I8f1S/GsHXiQ3MFPstD+cfcBOfkcqC4L8S
ZN8dIicH58udvPXijIosnP8i8QbXU2sZOH/3s53mbfbScL7X77Vb97tLwfkF
H0+rz0yQhPPnXz675HeqBJyfPP2n6L9qcTj/ySvzGzOmicH5+VpRS/fMFYXz
+72XpTVsFYHzd0ZcMzq7Uxhh/leJgE2HhOB86ZLWRPM7gnD+wntXa10fCcD5
VaELZJ5/5IfztXl33S/W4IPz38+ZvUPIggfOv//+QuXHDC5EfEqdC3hUxAHn
C3HKCwjPY0OEj9jUwpJLLHB+2Z9LL57fZ4Lz45ameMzmY4TzN/rUVB/U+bcf
xp/RW7lqrd8vOP96iNwqyc3f4HybM0FXV578BOd/mhy7qNj1HZwfyLtKKUzm
OZz//az2F8b19+D8NbMWPg3TvwDnB7m9ehiufwDOv+B8sES0s38fjA8A8Szd
Zw==
          "]]}, Annotation[#, "Charting`Private`Tag$10964#3"]& ]}}, {}, {}}, {
    Ticks -> {
       Charting`ScaledTicks[{Log, Exp}], 
       Charting`ScaledTicks[{Log, Exp}]}, GridLines -> {None, None}, 
     FrameTicks -> {{
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}, {
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, DisplayFunction -> 
     Identity, DisplayFunction -> Identity, Ticks -> {Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& }, AxesOrigin -> 
     NCache[{-11.512925230012566`, -
        Log[1000000]}, {-11.512925230012566`, -13.815510557964274`}], 
     FrameTicks -> {{Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}, {Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, GridLines -> {None, None}, 
     DisplayFunction -> Identity, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {0, 0}}, PlotRangeClipping -> True, ImagePadding -> 
     All, DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None, 
       "ClippingRange" -> {{{-11.512925230012566`, -2.3495766255041283`*^-7}, \
{-12.188579460919918`, -2.8331370819218815`}}, {{-11.512925230012566`, \
-2.3495766255041283`*^-7}, {-12.188579460919918`, -9.210340371976184}}}}, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, -13.815510557964274`}, 
     BaseStyle -> (FontSize -> 12), 
     CoordinatesToolOptions -> {"DisplayFunction" -> ({
         Exp[
          Part[#, 1]], 
         Exp[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Exp[
          Part[#, 1]], 
         Exp[
          Part[#, 2]]}& )}, DisplayFunction :> Identity, 
     Frame -> {{True, True}, {True, True}}, FrameLabel -> {{
        FormBox[
        "\"\\!\\(\\*SubscriptBox[\\(\[Delta]\\), \\(DE\\)]\\)(a)\"", 
         TraditionalForm], None}, {
        FormBox["\"a\"", TraditionalForm], None}}, FrameStyle -> GrayLevel[0],
      FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, PlotRange -> 
     NCache[{{-11.512925230012566`, 
        0.}, {-Log[1000000], -Log[10000]}}, {{-11.512925230012566`, 
       0.}, {-13.815510557964274`, -9.210340371976184}}], PlotRangeClipping -> 
     True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {Automatic, Automatic}}, 
     Ticks -> {Automatic, Automatic}}],FormBox[
    FormBox[
     TemplateBox[{
      "\"Numerical\"", 
       "\"Analytical (k<<aH/\\!\\(\\*SubscriptBox[\\(c\\), \\(s\\)]\\))\"", 
       "\"Analytical (k>>aH/\\!\\(\\*SubscriptBox[\\(c\\), \\(s\\)]\\))\""}, 
      "LineLegend", DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    Dashing[{0, Small, Small, Small}], 
                    GrayLevel[0]], {
                    LineBox[{{0, 10}, {40, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    Dashing[{0, Small, Small, Small}], 
                    GrayLevel[0]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {40, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    RGBColor[1, 0, 0]], {
                    LineBox[{{0, 10}, {40, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    RGBColor[1, 0, 0]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {40, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    Dashing[{Small, Small}], 
                    RGBColor[0, 0, 1]], {
                    LineBox[{{0, 10}, {40, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    Dashing[{Small, Small}], 
                    RGBColor[0, 0, 1]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {40, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   RowBox[{"Dashing", "[", 
                    RowBox[{"{", 
                    RowBox[{"0", ",", "Small", ",", "Small", ",", "Small"}], 
                    "}"}], "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], "GrayLevel[0]"], 
                    Appearance -> None, BaseStyle -> {}, BaselinePosition -> 
                    Baseline, DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False]}], 
                 "]"}], ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[1, 0, 0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> RGBColor[0.6666666666666666, 0., 0.], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "RGBColor[1, 0, 0]"], Appearance -> None, BaseStyle -> {},
                     BaselinePosition -> Baseline, DefaultBaseStyle -> {}, 
                    ButtonFunction :> With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[1, 0, 0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[1, 0, 0], Editable -> False, Selectable -> 
                    False]}], "]"}], ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   RowBox[{"Dashing", "[", 
                    RowBox[{"{", 
                    RowBox[{"Small", ",", "Small"}], "}"}], "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0, 0, 1], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> RGBColor[0., 0., 0.6666666666666666], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "RGBColor[0, 0, 1]"], Appearance -> None, BaseStyle -> {},
                     BaselinePosition -> Baseline, DefaultBaseStyle -> {}, 
                    ButtonFunction :> With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0, 0, 1]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0, 0, 1], Editable -> False, Selectable -> 
                    False]}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2, ",", #3}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.735625852432808*^9, 3.7356260137527685`*^9, 3.7356260438821464`*^9, 
   3.735626074758394*^9, {3.7356261215685005`*^9, 3.7356261308481293`*^9}, {
   3.7356261648183613`*^9, 3.735626189108024*^9}, 3.735627644153694*^9, 
   3.7356294819878173`*^9, 3.73562961449796*^9, 3.7356329308610945`*^9, 
   3.7673472935305233`*^9, 
   3.7964572587734127`*^9},ExpressionUUID->"879edab1-d2b3-4796-bff3-\
9c3a69711f0b"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LogLogPlot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"VDE", "[", "a", "]"}], "/.", 
        RowBox[{"sol", "[", 
         RowBox[{"[", "1", "]"}], "]"}]}], ",", 
       RowBox[{"VDEan", "[", "a", "]"}]}], "}"}], "//", "Abs"}], "//", 
    "Evaluate"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "ai", ",", "1"}], "}"}], ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameStyle", "\[Rule]", "Black"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
     "\"\<a\>\"", ",", "\"\<\!\(\*SubscriptBox[\(V\), \(DE\)]\)(a)\>\""}], 
     "}"}]}], ",", 
   RowBox[{"BaseStyle", "\[Rule]", 
    RowBox[{"FontSize", "\[Rule]", "12"}]}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", "Black", "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"Dashed", ",", "Black"}], "}"}]}], "}"}]}], ",", 
   RowBox[{"PlotLegends", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<Numerical\>\"", ",", "\"\<Analytical\>\""}], "}"}]}]}], 
  "]"}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356249293691154`*^9, 3.7356249982486596`*^9}, {
   3.7356252311484785`*^9, 3.7356252421689615`*^9}, 3.735625464138838*^9, {
   3.735625865929031*^9, 3.7356258696735373`*^9}, 
   3.796457261447672*^9},ExpressionUUID->"66b428f3-0340-4546-9ef8-\
cf8da7b8b746"],

Cell[BoxData[
 TemplateBox[{GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          GrayLevel[0]], 
         LineBox[CompressedData["
1:eJwV13c8VW8cB3ClQWWF4h73nnM5MkuRkFIZoZAZkgpp22SLpFQqKylkhFBK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          "]]}, Annotation[#, "Charting`Private`Tag$11858#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          Dashing[{Small, Small}], 
          GrayLevel[0]], 
         LineBox[CompressedData["
1:eJwVxX04FHYAB/C7y1tE3u6cu3PHJVrotnpKLb2qrMvE6cVLtkVCL15beUvK
2lgzNi9deQl5XxcpPa6j31dS19PpELKL1JXXzaRXnWLbH5/nYxMYIQqmUSiU
r//z/zP7wC/UsYfguDesBniI0U8TKbXsUcrnvPTo52GkxusUZY49LOUveMmP
eejUqJ8FztphjtnRE+pHPJRnahfbT9nhUaV4bUU7Dx6NQt6VUTuc6Opv/PwW
DwVm3ezbCjs8cAiVupbx4HJr1HzsdzscViXWhR3kIYl++pd4uh1qnJiLNeNc
nPFdPuNXtBCNUfI8VSwXxwbDxT85LUTlJ+oKXR0uQkJYDVO1thCfkOyMTbMC
X6XqkLrYwvvV4YhVdCuoNxpGkZsLYOZfZLTuLAfJr+UrTVwX4IrBnp5MNgej
2ec0dUo+WBJH8dJKNnY79m04J+IjdVPEANOBjcxry1KU/TZoVtyx3n6dhYof
zMe9DtqgN7/i5w5XFlqEIX3MCWu8CdTEie9aopqqbBUkWOOxZWBQjbclHAO8
whjTPNy9s7pTv4cJbLEQB2vx8JusMKzOj4kxE13ZDi4XdsWM6aphC3DvqUYU
tlZAIsV06IAFRvb26g+u5mDh1ll+9DsGZENS5x/XsZFhECndGMuAcFpST9xY
6KEfiP32Ix2Rk/ftz31niVHjuC8UqXRceNhWOh3MBIWmiThuQEfQ2Q/0mQQL
qJ9nVCblmuPFmHOvOJeBi4Y1cxULzBGx2K+ssYCOMKf1LwMqzJCxrXx/cr05
ptYMta9bZgapT8/SNqkZ0jfNrNhfb4oLb5medbdMQcm60Nq7yhTsI6f3OveZ
YG3nTo+8OyYoftJ9fLPaGCd1k/aUbDVB/M4izvDkfGQ6yfZ96DKGg+GNgZhF
RkjMO5mTH2iMify8BtOV8/DnTMxZvbb5GJhoj5sM1YdXpdHUsUEjqFkPPNXR
eni4Jj0ocswQpnN5xmaFOvjr2vIWPYohZIycqCOdWiCylzGrWPNw9GPnyPAA
DQWXMwSvrQ3wTdm+r/KMqDBJ8RlwX6mPK+6JCc2OM+RXV+3zApe5EKfHVUs8
NESedbusepMeHu3gVltefUfErZKy1gBduChE3VX3XpGo5/G1YUE6eJUVUBKz
+R/yVB1VI4nThpdhNX8XZ5jMi06YTcvWwvs2hzfUmidkw3io30TeHFw6Xzy4
S9BODIXJVW+u0vD08BR7Wb+SdLzW7hqpoSG0XHKowV9JcgrOzPT9QcMxC6ZR
i88Dwp3M9W69SEPuh789e0UKIhBLPuVk0dDdlNtDc5MTzyGVl3MMDaItY892
C0ByTi3XxC2l4dqntcE3/AnxdZTZhi+hoSW8SqfV5yax6lm/PXAxDWrPpK0q
USMp+8y9VMinwYa+SKnl1kCutwd6sE1pKCxIUPkKakkvN7OkcZIKfyvFe7nP
ZVIgp7fVjlMRP1v05RPRJbI3Ou996SgV0hYh9NwqyWhruXv6MypWuBfeDxAU
Ec2hpncBHVRU7xniDInyCWG42ojaqBi1ziwxdhOTFMi3bblHxf6q55JgQSYx
MO8qWtJMhfAxK8TaLZUom3zv85uouBsUHv29IJlkhwy8ZUipSO1Nmc4WRBIf
k2Brg3oq2l2bj9DTMm5yZGPC2Voq/gXY7HOt
          "]]}, Annotation[#, "Charting`Private`Tag$11858#2"]& ]}}, {}, {}}, {
    Ticks -> {
       Charting`ScaledTicks[{Log, Exp}], 
       Charting`ScaledTicks[{Log, Exp}]}, GridLines -> {None, None}, 
     FrameTicks -> {{
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}, {
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, DisplayFunction -> 
     Identity, DisplayFunction -> Identity, Ticks -> {Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& }, AxesOrigin -> 
     NCache[{-Log[
         100000], -10.287495220286774`}, {-11.512925464970229`, \
-10.287495220286774`}], FrameTicks -> {{Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}, {Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, GridLines -> {None, None}, 
     DisplayFunction -> Identity, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None, 
       "ClippingRange" -> {{{-11.512925230012566`, -2.3495766255041283`*^-7}, \
{-10.287495220286774`, -2.2114244320758996`}}, {{-11.512925230012566`, \
-2.3495766255041283`*^-7}, {-10.287495220286774`, -2.2114244320758996`}}}}, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, -10.287495220286774`}, 
     BaseStyle -> (FontSize -> 12), 
     CoordinatesToolOptions -> {"DisplayFunction" -> ({
         Exp[
          Part[#, 1]], 
         Exp[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Exp[
          Part[#, 1]], 
         Exp[
          Part[#, 2]]}& )}, DisplayFunction :> Identity, 
     Frame -> {{True, True}, {True, True}}, FrameLabel -> {{
        FormBox[
        "\"\\!\\(\\*SubscriptBox[\\(V\\), \\(DE\\)]\\)(a)\"", 
         TraditionalForm], None}, {
        FormBox["\"a\"", TraditionalForm], None}}, FrameStyle -> GrayLevel[0],
      FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, PlotRange -> 
     NCache[{{-Log[100000], 
         0}, {-10.287495220286774`, -2.2114244320758996`}}, \
{{-11.512925464970229`, 0}, {-10.287495220286774`, -2.2114244320758996`}}], 
     PlotRangeClipping -> True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}],FormBox[
    FormBox[
     TemplateBox[{"\"Numerical\"", "\"Analytical\""}, "LineLegend", 
      DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    GrayLevel[0]], {
                    LineBox[{{0, 10}, {40, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    GrayLevel[0]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {40, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    Dashing[{Small, Small}], 
                    GrayLevel[0]], {
                    LineBox[{{0, 10}, {40, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    Dashing[{Small, Small}], 
                    GrayLevel[0]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {40, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], "GrayLevel[0]"], 
                    Appearance -> None, BaseStyle -> {}, BaselinePosition -> 
                    Baseline, DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False]}], 
                 "]"}], ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   RowBox[{"Dashing", "[", 
                    RowBox[{"{", 
                    RowBox[{"Small", ",", "Small"}], "}"}], "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], "GrayLevel[0]"], 
                    Appearance -> None, BaseStyle -> {}, BaselinePosition -> 
                    Baseline, DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False]}], 
                 "]"}]}], "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.73562587043311*^9, 3.735627644396616*^9, 
  3.7356294853817005`*^9, 3.7356296171318803`*^9, 3.7356329327657976`*^9, 
  3.796457263050271*^9},ExpressionUUID->"00b1706d-6929-469e-b0df-\
c9b7313bfd0a"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"LogLogPlot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Phi]", "[", "a", "]"}], "/.", 
        RowBox[{"sol", "[", 
         RowBox[{"[", "1", "]"}], "]"}]}], ",", 
       RowBox[{"\[Phi]an", "[", "a", "]"}]}], "}"}], "//", "Abs"}], "//", 
    "Evaluate"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "ai", ",", "1"}], "}"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", "All"}], ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameStyle", "\[Rule]", "Black"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<a\>\"", ",", "\"\<\[Phi](a)\>\""}], "}"}]}], ",", 
   RowBox[{"BaseStyle", "\[Rule]", 
    RowBox[{"FontSize", "\[Rule]", "12"}]}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", "Black", "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"Dashed", ",", "Black"}], "}"}]}], "}"}]}], ",", 
   RowBox[{"PlotLegends", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<Numerical\>\"", ",", "\"\<Analytical\>\""}], "}"}]}]}], 
  "]"}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.735625006189026*^9, 3.7356250182190866`*^9}, {
   3.7356252371093397`*^9, 3.73562526005897*^9}, 3.735625464138838*^9, 
   3.796457267281381*^9},ExpressionUUID->"3b4c2933-646a-459c-9374-\
ea97c5a820ff"],

Cell[BoxData[
 TemplateBox[{GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          GrayLevel[0]], 
         LineBox[CompressedData["
1:eJwknHk4VV8Xx0UahEpluO48iTRJKJIpEhVSpAhFkkwlZKiUkiJFfkJCSNJM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          "]]}, Annotation[#, "Charting`Private`Tag$12769#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          Dashing[{Small, Small}], 
          GrayLevel[0]], 
         LineBox[CompressedData["
1:eJxTTMoPSmViYGAwAWIQ/S/lgNI8NvUDpSHHpANOqR2YK3xV+sgZNTjf5tBL
kVeTEPw60daeKlEEvzvS9F/UAlU4v/xp3ox2XQQ/PV1qx48NKnC+0q1bF3fa
IPiPnHgL9+9ThvMbPp+wEHRG8F9Omflr03klOD9c547jzCAEf8IW4+bzdxXh
/OUtIm8DsxH8w17pdyTeK8D5qxjPH9WvRvB1YgMzxX7Lw/kH3MRnpLIg+K8E
2XeHyMnB+XInb704oyIL579IvMH11FoGzt/9bKd5m700nO/1e+3W/e5ScH7B
x9PqMxMk4fz5l88u+Z0qAecnT/8p+q9aHM5/8sr8xoxpYnB+vlbU0j1zReH8
fu9laQ1bReD8nRHXjM7uFEaY/1UiYNMhIThfuqQ10fyOIJy/8N7VWtdHAnB+
VegCmecf+eF8bd5d94s1+OD893Nm7xCy4IHz77+/UPkxgwsRn1LnAh4VccD5
QpzyAsLz2BDhIza1sOQSC5xf9ufSi+f3meD8uKUpHrP5GOH8jT411Qd1/u2H
8Wf0Vq5a6/cLzr8eIrdKcvM3ON/mTNDVlSc/wfmfJscuKnZ9B+cH8q5SCpN5
Dud/P6v9hXH9PTh/zayFT8P0L8D5QW6vHobrH4DzLzgfLBHt7N8H4wMA+LiY
Vw==
          "]]}, Annotation[#, "Charting`Private`Tag$12769#2"]& ]}}, {}, {}}, {
    Ticks -> {
       Charting`ScaledTicks[{Log, Exp}], 
       Charting`ScaledTicks[{Log, Exp}]}, GridLines -> {None, None}, 
     FrameTicks -> {{
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}, {
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, DisplayFunction -> 
     Identity, DisplayFunction -> Identity, Ticks -> {Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& }, 
     AxesOrigin -> {-11.512925230012566`, -11.710293264538642`}, 
     FrameTicks -> {{Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}, {Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, GridLines -> {None, None}, 
     DisplayFunction -> Identity, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None, 
       "ClippingRange" -> {{{-11.512925230012566`, -2.3495766255041283`*^-7}, \
{-11.710293264538642`, -11.3947291099947}}, {{-11.512925230012566`, \
-2.3495766255041283`*^-7}, {-11.710293264538642`, -11.3947291099947}}}}, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, -11.710293264538642`}, 
     BaseStyle -> (FontSize -> 12), 
     CoordinatesToolOptions -> {"DisplayFunction" -> ({
         Exp[
          Part[#, 1]], 
         Exp[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Exp[
          Part[#, 1]], 
         Exp[
          Part[#, 2]]}& )}, DisplayFunction :> Identity, 
     Frame -> {{True, True}, {True, True}}, FrameLabel -> {{
        FormBox["\"\[Phi](a)\"", TraditionalForm], None}, {
        FormBox["\"a\"", TraditionalForm], None}}, FrameStyle -> GrayLevel[0],
      FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, 
     PlotRange -> {All, All}, PlotRangeClipping -> True, 
     PlotRangePadding -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     Ticks -> {Automatic, Automatic}}],FormBox[
    FormBox[
     TemplateBox[{"\"Numerical\"", "\"Analytical\""}, "LineLegend", 
      DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    GrayLevel[0]], {
                    LineBox[{{0, 10}, {40, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    GrayLevel[0]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {40, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    Dashing[{Small, Small}], 
                    GrayLevel[0]], {
                    LineBox[{{0, 10}, {40, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    Dashing[{Small, Small}], 
                    GrayLevel[0]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {40, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], "GrayLevel[0]"], 
                    Appearance -> None, BaseStyle -> {}, BaselinePosition -> 
                    Baseline, DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False]}], 
                 "]"}], ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   RowBox[{"Dashing", "[", 
                    RowBox[{"{", 
                    RowBox[{"Small", ",", "Small"}], "}"}], "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], "GrayLevel[0]"], 
                    Appearance -> None, BaseStyle -> {}, BaselinePosition -> 
                    Baseline, DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False]}], 
                 "]"}]}], "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.735625009548967*^9, 3.735625018868743*^9}, 
   3.7356252611290593`*^9, 3.735625393768678*^9, 3.735625464138838*^9, 
   3.735625879008547*^9, 3.7356276446967278`*^9, 3.7356294872579947`*^9, 
   3.735629619387619*^9, 3.7356329347456656`*^9, 
   3.796457268487754*^9},ExpressionUUID->"39346b79-da29-47da-9ff7-\
75e7d184cac1"],

Cell[BoxData[
 RowBox[{"(*", " ", "end", " ", "*)"}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.735625441878965*^9, 3.735625464138838*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"Case", " ", "3"}], ":", " ", 
    RowBox[{"Matter", " ", "+", " ", "Radiation"}]}], " ", "//", " ", 
   RowBox[{"k", " ", "evolution"}]}], " ", "*)"}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356254487086363`*^9, 3.735625464138838*^9}, {
   3.735625508308816*^9, 3.7356255123786583`*^9}, {3.7368546415020256`*^9, 
   3.736854646924579*^9}, 3.736855204917408*^9, {3.766999036665384*^9, 
   3.7669990400885253`*^9}, {3.767004044047413*^9, 3.767004045491911*^9}, 
   3.7670049286730413`*^9, 
   3.7670150490767164`*^9},ExpressionUUID->"b9d6d53d-bcad-4fa6-88a7-\
34e7adfb795b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"Cosmological", " ", "parameters"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalOmega]\[Gamma]0", "=", 
     RowBox[{
      FractionBox["415", "100"], "*", 
      RowBox[{
       SuperscriptBox["10", 
        RowBox[{"-", "5"}]], "/", 
       RowBox[{"h", "^", "2"}]}]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"h", "=", 
     RowBox[{"7", "/", "10"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"om", "=", 
     RowBox[{"1", "-", "\[CapitalOmega]\[Gamma]0"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"ai", "=", 
     SuperscriptBox["10", 
      RowBox[{"-", "8"}]]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"ap", "=", 
     RowBox[{"10", " ", "ai"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"H0", "=", "1"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"w\[Gamma]", "=", 
     RowBox[{"1", "/", "3"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"cs2\[Gamma]", "=", 
     RowBox[{"1", "/", "3"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"aeq", "=", 
     RowBox[{"\[CapitalOmega]\[Gamma]0", "/", "om"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{"Friedman", " ", "equation"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"H", "[", "a_", "]"}], ":=", 
    SqrtBox[
     RowBox[{
      RowBox[{"om", " ", 
       SuperscriptBox["a", 
        RowBox[{"-", "3"}]]}], "+", 
      RowBox[{"\[CapitalOmega]\[Gamma]0", " ", 
       SuperscriptBox["a", 
        RowBox[{
         RowBox[{"-", "3"}], 
         RowBox[{"(", 
          RowBox[{"1", "+", "w\[Gamma]"}], ")"}]}]]}]}]]}]}]}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356253342187953`*^9, 3.735625344922727*^9}, 
   3.735625464138838*^9, {3.7670002343494787`*^9, 3.767000240007764*^9}, 
   3.7670008362373605`*^9, {3.7670019254231896`*^9, 3.7670019257496815`*^9}, 
   3.767002010116867*^9, 3.767003150769675*^9, {3.7670044141212025`*^9, 
   3.767004446241784*^9}, 3.767004943746189*^9, {3.7670050080960464`*^9, 
   3.7670050243241205`*^9}, {3.7670052183047867`*^9, 
   3.7670052221352024`*^9}, {3.7670701651352663`*^9, 
   3.767070208317212*^9}},ExpressionUUID->"20a3f001-faa0-4fdb-b1b7-\
bc88125104f5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"The", " ", "inital", " ", "conditions"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]0", "[", 
      RowBox[{"a_", ",", "k_"}], "]"}], ":=", 
     RowBox[{
      RowBox[{"-", "1"}], "/", 
      SuperscriptBox["k", 
       RowBox[{"3", "/", "2"}]]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"\[Delta]\[Gamma]0", "[", 
      RowBox[{"a_", ",", "k_"}], "]"}], ":=", 
     RowBox[{
      RowBox[{"-", "2"}], 
      RowBox[{"\[Phi]0", "[", 
       RowBox[{"a", ",", "k"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"\[Delta]c0", "[", 
      RowBox[{"a_", ",", "k_"}], "]"}], ":=", 
     RowBox[{
      FractionBox["3", "4"], 
      RowBox[{"\[Delta]\[Gamma]0", "[", 
       RowBox[{"a", ",", "k"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"Vm0", "[", 
      RowBox[{"a_", ",", "k_"}], "]"}], ":=", 
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"k", "^", "2"}], " ", 
      FractionBox["ai", 
       RowBox[{"H0", 
        SqrtBox["\[CapitalOmega]\[Gamma]0"], " "}]], " ", 
      RowBox[{"\[Phi]0", "[", 
       RowBox[{"a", ",", "k"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"V\[Gamma]0", "[", 
      RowBox[{"a_", ",", "k_"}], "]"}], ":=", 
     RowBox[{"Vm0", "[", 
      RowBox[{"a", ",", "k"}], "]"}]}], ";"}]}]}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356246272481565`*^9, 3.735624689719591*^9}, {
   3.735624872839239*^9, 3.7356248730495434`*^9}, 3.735625464138838*^9, {
   3.735625551077653*^9, 3.7356256094488688`*^9}, {3.736854660690071*^9, 
   3.7368546715821238`*^9}, {3.73685504106439*^9, 3.7368550813497915`*^9}, 
   3.76699984139952*^9, {3.7669999090055523`*^9, 3.767000052173439*^9}, {
   3.7670002989900303`*^9, 3.7670003119008837`*^9}, {3.767000804122465*^9, 
   3.7670008110914736`*^9}, 3.76700086483064*^9, 3.767001208940386*^9, {
   3.7670017011887536`*^9, 3.7670017105864887`*^9}, 3.76700176727139*^9, 
   3.7670018401406918`*^9, {3.7670032129104495`*^9, 3.7670032340102596`*^9}, {
   3.767003616150666*^9, 3.767003616994241*^9}, {3.7670036522042994`*^9, 
   3.7670036793012247`*^9}, {3.7670044530416827`*^9, 
   3.7670044836567087`*^9}, {3.796457286861802*^9, 
   3.796457292939658*^9}},ExpressionUUID->"2c08f95f-7b86-41bf-8ffc-\
b2a9d8f1b9b9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "This", " ", "solves", " ", "the", " ", "ODEs", " ", "as", " ", "a", " ", 
    "function", " ", "ok", " ", "the", " ", "wavenumber", " ", "k"}], " ", 
   "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"sol", "[", "k_", "]"}], ":=", 
    RowBox[{
     RowBox[{"sol", "[", "k", "]"}], "=", 
     RowBox[{"NDSolve", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[Delta]m", "'"}], "[", "a", "]"}], "\[Equal]", 
          RowBox[{
           FractionBox[
            RowBox[{"-", 
             RowBox[{"Vm", "[", "a", "]"}]}], 
            RowBox[{
             RowBox[{"H", "[", "a", "]"}], " ", 
             RowBox[{"a", "^", "2"}]}]], "+", 
           RowBox[{"3", 
            RowBox[{
             RowBox[{"\[Phi]", "'"}], "[", "a", "]"}]}]}]}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"Vm", "'"}], "[", "a", "]"}], "\[Equal]", 
          RowBox[{
           FractionBox[
            RowBox[{"-", 
             RowBox[{"Vm", "[", "a", "]"}]}], "a"], "+", 
           RowBox[{
            FractionBox[
             RowBox[{"k", "^", "2"}], 
             RowBox[{
              RowBox[{"H", "[", "a", "]"}], " ", 
              RowBox[{"a", "^", "2"}]}]], 
            RowBox[{"\[Phi]", "[", "a", "]"}]}]}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"\[Delta]\[Gamma]", "'"}], "[", "a", "]"}], "\[Equal]", 
          RowBox[{
           RowBox[{"-", 
            FractionBox[
             RowBox[{"V\[Gamma]", "[", "a", "]"}], 
             RowBox[{
              RowBox[{"H", "[", "a", "]"}], 
              RowBox[{"a", "^", "2"}]}]]}], "-", 
           RowBox[{
            FractionBox["3", "a"], 
            RowBox[{"(", 
             RowBox[{"cs2\[Gamma]", "-", "w\[Gamma]"}], ")"}], 
            RowBox[{"\[Delta]\[Gamma]", "[", "a", "]"}]}], "+", 
           RowBox[{"3", 
            RowBox[{"(", 
             RowBox[{"1", "+", "w\[Gamma]"}], ")"}], 
            RowBox[{
             RowBox[{"\[Phi]", "'"}], "[", "a", "]"}]}]}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"V\[Gamma]", "'"}], "[", "a", "]"}], "\[Equal]", 
          RowBox[{
           RowBox[{
            RowBox[{"-", 
             RowBox[{"(", 
              RowBox[{"1", "-", 
               RowBox[{"3", "w\[Gamma]"}]}], ")"}]}], 
            FractionBox[
             RowBox[{"V\[Gamma]", "[", "a", "]"}], "a"]}], "+", 
           RowBox[{
            FractionBox[
             RowBox[{
              RowBox[{"k", "^", "2"}], " ", "cs2\[Gamma]"}], 
             RowBox[{
              RowBox[{"H", "[", "a", "]"}], " ", 
              RowBox[{"a", "^", "2"}]}]], 
            RowBox[{"\[Delta]\[Gamma]", "[", "a", "]"}]}], "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"1", "+", "w\[Gamma]"}], ")"}], 
            FractionBox[
             RowBox[{"k", "^", "2"}], 
             RowBox[{
              RowBox[{"H", "[", "a", "]"}], " ", 
              RowBox[{"a", "^", "2"}]}]], 
            RowBox[{"\[Phi]", "[", "a", "]"}]}]}]}], ",", 
         RowBox[{
          RowBox[{"\[Phi]", "[", "a", "]"}], "==", 
          RowBox[{
           RowBox[{"-", 
            FractionBox["3", "2"]}], 
           FractionBox[
            RowBox[{"a", "^", "2"}], 
            RowBox[{"k", "^", "2"}]], 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"om", " ", 
              SuperscriptBox["a", 
               RowBox[{"-", "3"}]], 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"\[Delta]m", "[", "a", "]"}], "+", 
                RowBox[{"3", "a", " ", 
                 FractionBox[
                  RowBox[{"H", "[", "a", "]"}], 
                  RowBox[{"k", "^", "2"}]], " ", 
                 RowBox[{"Vm", "[", "a", "]"}]}]}], ")"}]}], "+", 
             RowBox[{"\[CapitalOmega]\[Gamma]0", " ", 
              SuperscriptBox["a", 
               RowBox[{
                RowBox[{"-", "3"}], 
                RowBox[{"(", 
                 RowBox[{"1", "+", "w\[Gamma]"}], ")"}]}]], 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"\[Delta]\[Gamma]", "[", "a", "]"}], "+", 
                RowBox[{"3", "a", " ", 
                 FractionBox[
                  RowBox[{"H", "[", "a", "]"}], 
                  RowBox[{"k", "^", "2"}]], " ", 
                 RowBox[{"V\[Gamma]", "[", "a", "]"}]}]}], ")"}]}]}], 
            ")"}]}]}], ",", 
         RowBox[{
          RowBox[{"\[Delta]m", "[", "ai", "]"}], "==", 
          RowBox[{"\[Delta]c0", "[", 
           RowBox[{"ai", ",", "k"}], "]"}]}], ",", 
         RowBox[{
          RowBox[{"Vm", "[", "ai", "]"}], "==", 
          RowBox[{"Vm0", "[", 
           RowBox[{"ai", ",", "k"}], "]"}]}], ",", 
         RowBox[{
          RowBox[{"\[Delta]\[Gamma]", "[", "ai", "]"}], "==", 
          RowBox[{"\[Delta]\[Gamma]0", "[", 
           RowBox[{"ai", ",", "k"}], "]"}]}], ",", 
         RowBox[{
          RowBox[{"V\[Gamma]", "[", "ai", "]"}], "==", 
          RowBox[{"V\[Gamma]0", "[", 
           RowBox[{"ai", ",", "k"}], "]"}]}], ",", 
         RowBox[{
          RowBox[{"\[Phi]", "[", "ai", "]"}], "==", 
          RowBox[{"\[Phi]0", "[", 
           RowBox[{"ai", ",", "k"}], "]"}]}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
        "\[Delta]m", ",", "Vm", ",", "\[Delta]\[Gamma]", ",", "V\[Gamma]", 
         ",", "\[Phi]"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"a", ",", "ai", ",", "1"}], "}"}], ",", 
       RowBox[{"MaxStepSize", "\[Rule]", 
        SuperscriptBox["10", 
         RowBox[{"-", "5"}]]}], ",", 
       RowBox[{"MaxStepFraction", "\[Rule]", 
        SuperscriptBox["10", 
         RowBox[{"-", "5"}]]}], ",", 
       RowBox[{"StartingStepSize", "\[Rule]", 
        SuperscriptBox["10", 
         RowBox[{"-", "5"}]]}], ",", 
       RowBox[{"MaxSteps", "\[Rule]", 
        RowBox[{"10", "^", "6"}]}], ",", 
       RowBox[{"Method", "\[Rule]", "Automatic"}]}], "]"}]}]}], 
   ";"}]}]], "Input",
 CellChangeTimes->{{3.767000062606375*^9, 3.76700007587354*^9}, {
   3.767000117818261*^9, 3.7670002200253425`*^9}, {3.767000334646226*^9, 
   3.7670003497741113`*^9}, {3.7670006449247065`*^9, 
   3.7670006680052977`*^9}, {3.7670007068011217`*^9, 3.767000728449688*^9}, {
   3.7670010196211257`*^9, 3.7670010490387807`*^9}, {3.7670014088407974`*^9, 
   3.7670014276572733`*^9}, {3.7670018032029686`*^9, 3.767001808313428*^9}, {
   3.7670033800456734`*^9, 3.767003386616849*^9}, 3.7670034819158373`*^9, {
   3.7670037198004446`*^9, 3.767003750469631*^9}, {3.767004056704776*^9, 
   3.7670041193844557`*^9}, {3.767004219094764*^9, 3.7670043887432117`*^9}, {
   3.767004491866829*^9, 3.7670045185115004`*^9}, {3.7964572964429846`*^9, 
   3.7964573083378787`*^9}},ExpressionUUID->"00b28196-73b5-4b8e-9d0c-\
4337b6e55de1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "The", " ", "values", " ", "of", " ", "k", " ", "we", " ", "consider"}], 
   " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"kv", "=", 
    RowBox[{"3000", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"3", "/", "100"}], ",", 
       RowBox[{"1", "/", "10"}], ",", "2"}], "}"}]}]}], ";"}]}]], "Input",
 CellChangeTimes->{{3.7670045515777073`*^9, 3.767004592519874*^9}, {
  3.796457316485325*^9, 
  3.796457323992377*^9}},ExpressionUUID->"11b3cfd6-2567-4c43-b205-\
79b3ea5bdcf5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "The", " ", "solution", " ", "for", " ", "the", " ", "values", " ", "of", 
    " ", "k"}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"Off", "[", 
     RowBox[{"NDSolve", "::", "ivres"}], "]"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"sol003", "=", 
     RowBox[{"sol", "[", 
      RowBox[{"kv", "[", 
       RowBox[{"[", "1", "]"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"sol01", "=", 
     RowBox[{"sol", "[", 
      RowBox[{"kv", "[", 
       RowBox[{"[", "2", "]"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"sol2", "=", 
     RowBox[{"sol", "[", 
      RowBox[{"kv", "[", 
       RowBox[{"[", "3", "]"}], "]"}], "]"}]}], ";"}]}]}]], "Input",
 CellChangeTimes->{{3.767004525319589*^9, 3.7670045425362573`*^9}, {
  3.7670045982572203`*^9, 3.7670046429352636`*^9}, {3.7670050550468993`*^9, 
  3.7670050624704437`*^9}, {3.7964573317792945`*^9, 
  3.79645734620033*^9}},ExpressionUUID->"cc350f12-0441-4653-ae39-\
35a174fb0f9f"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"The", " ", "plots"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.7670049613952703`*^9, 3.76700496240701*^9}, {
  3.796457353374492*^9, 
  3.79645735596348*^9}},ExpressionUUID->"f7a4f988-3c07-4139-8a55-\
9a1d3b78a268"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LogLinearPlot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       FractionBox[
        RowBox[{"om", " ", 
         SuperscriptBox["a", 
          RowBox[{"-", "3"}]]}], 
        RowBox[{
         RowBox[{"H", "[", "a", "]"}], "^", "2"}]], ",", 
       FractionBox[
        RowBox[{"\[CapitalOmega]\[Gamma]0", " ", 
         SuperscriptBox["a", 
          RowBox[{
           RowBox[{"-", "3"}], 
           RowBox[{"(", 
            RowBox[{"1", "+", "w\[Gamma]"}], ")"}]}]]}], 
        RowBox[{
         RowBox[{"H", "[", "a", "]"}], "^", "2"}]]}], "}"}], "//", "Abs"}], "//",
     "Evaluate"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "ai", ",", "1"}], "}"}], ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameStyle", "\[Rule]", "Black"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<a\>\"", ",", "\"\<\[CapitalOmega](a)\>\""}], "}"}]}], ",", 
   RowBox[{"BaseStyle", "\[Rule]", 
    RowBox[{"FontSize", "\[Rule]", "12"}]}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Black", ",", "Orange"}], "}"}]}], ",", 
   RowBox[{"PlotLegends", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
     "\"\<\!\(\*SubscriptBox[\(\[CapitalOmega]\), \(m\)]\)(a)\>\"", ",", 
      "\"\<\!\(\*SubscriptBox[\(\[CapitalOmega]\), \(\[Gamma]\)]\)(a)\>\""}], 
     "}"}]}], ",", 
   RowBox[{"Epilog", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Black", ",", "Dashed", ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"aeq", ",", 
            SuperscriptBox["10", 
             RowBox[{"-", "6"}]]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"aeq", ",", 
            SuperscriptBox["10", "6"]}], "}"}]}], "}"}], "//", "Log"}], 
       "]"}]}], "}"}]}]}], "]"}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.735625107069229*^9, 3.7356251838925276`*^9}, 
   3.735625464138838*^9, {3.7670003658853207`*^9, 3.7670004421057844`*^9}, 
   3.7670011505763164`*^9, {3.7670150237819614`*^9, 3.767015035667384*^9}, {
   3.7964573592538147`*^9, 
   3.796457368965387*^9}},ExpressionUUID->"00468739-5733-4ef0-a34f-\
d0f2b343e8a8"],

Cell[BoxData[
 TemplateBox[{GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          GrayLevel[0]], 
         LineBox[CompressedData["
1:eJwV1nc8Vt8fAHB7ZGXL3ns8DxkpfcqoCAnJSF8rUWb2aiiZZZUUlSKjgYoU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          "]]}, Annotation[#, "Charting`Private`Tag$2911#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          AbsoluteThickness[1.6], 
          RGBColor[1, 0.5, 0]], 
         LineBox[CompressedData["
1:eJwV1nc4le8bAHB7lL335hgH55AR1d2gIkQUqWSlhIxkFJIUhVCyJSXrG1Jk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          "]]}, Annotation[#, "Charting`Private`Tag$2911#2"]& ]}}, {}, {}}, {
    Ticks -> {
       Charting`ScaledTicks[{Log, Exp}], Automatic}, 
     GridLines -> {None, None}, FrameTicks -> {{Automatic, Automatic}, {
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, DisplayFunction -> 
     Identity, DisplayFunction -> Identity, Ticks -> {Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , Automatic}, 
     AxesOrigin -> NCache[{-Log[100000000], 0}, {-18.420680743952367`, 0}], 
     FrameTicks -> {{Automatic, 
        Charting`ScaledFrameTicks[{Identity, Identity}]}, {Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, GridLines -> {None, None}, 
     DisplayFunction -> Identity, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None, 
       "ClippingRange" -> {{{-18.420680368020108`, -3.759322600806605*^-7}, {
        0.00008469390938749063, 
        0.9999153060906124}}, {{-18.420680368020108`, \
-3.759322600806605*^-7}, {0.00008469390938749063, 0.9999153060906124}}}}, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, 
     BaseStyle -> (FontSize -> 12), 
     CoordinatesToolOptions -> {"DisplayFunction" -> ({
         Exp[
          Part[#, 1]], 
         Part[#, 2]}& ), "CopiedValueFunction" -> ({
         Exp[
          Part[#, 1]], 
         Part[#, 2]}& )}, DisplayFunction :> Identity, Epilog -> {
       GrayLevel[0], 
       Dashing[{Small, Small}], 
       LineBox[
        NCache[{{-Log[
             Rational[979917, 83]], -Log[1000000]}, {-Log[
             Rational[979917, 83]], 
           Log[
           1000000]}}, {{-9.376382545385876, -13.815510557964274`}, \
{-9.376382545385876, 13.815510557964274`}}]]}, 
     Frame -> {{True, True}, {True, True}}, FrameLabel -> {{
        FormBox["\"\[CapitalOmega](a)\"", TraditionalForm], None}, {
        FormBox["\"a\"", TraditionalForm], None}}, FrameStyle -> GrayLevel[0],
      FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, PlotRange -> 
     NCache[{{-Log[100000000], 0}, {0., 
        0.9999153060906124}}, {{-18.420680743952367`, 0}, {0., 
        0.9999153060906124}}], PlotRangeClipping -> True, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}],FormBox[
    FormBox[
     TemplateBox[{
      "\"\\!\\(\\*SubscriptBox[\\(\[CapitalOmega]\\), \\(m\\)]\\)(a)\"", 
       "\"\\!\\(\\*SubscriptBox[\\(\[CapitalOmega]\\), \
\\(\[Gamma]\\)]\\)(a)\""}, "LineLegend", DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    GrayLevel[0]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    GrayLevel[0]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    RGBColor[1, 0.5, 0]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    AbsoluteThickness[1.6], 
                    RGBColor[1, 0.5, 0]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], "GrayLevel[0]"], 
                    Appearance -> None, BaseStyle -> {}, BaselinePosition -> 
                    Baseline, DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False]}], 
                 "]"}], ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[1, 0.5, 0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[0.6666666666666666, 0.33333333333333337`, 0.], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "RGBColor[1, 0.5, 0]"], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[1, 0.5, 0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[1, 0.5, 0], Editable -> False, Selectable -> 
                    False]}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{{3.767000368010023*^9, 3.7670004013788047`*^9}, {
   3.7670004335415735`*^9, 3.76700044311601*^9}, 3.7670004836153307`*^9, {
   3.7670011468620987`*^9, 3.767001151688015*^9}, 3.7670017428801546`*^9, {
   3.7670032252232046`*^9, 3.7670032461188173`*^9}, 3.7670034163149495`*^9, 
   3.7670034481304073`*^9, 3.7670034854837584`*^9, 3.7670035267233562`*^9, 
   3.7670035721226935`*^9, 3.7670036885828443`*^9, 3.767004131058307*^9, 
   3.7670043923737946`*^9, 3.7670046497914095`*^9, 3.7670049662562513`*^9, 
   3.76700511447254*^9, 3.7670150383596225`*^9, 3.7670702038494215`*^9, 
   3.7670702343473463`*^9, 3.767347323195079*^9, 
   3.7964573766553583`*^9},ExpressionUUID->"d23c91c2-717f-42f6-a26e-\
beabce0f2d5b"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LogLogPlot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"kv", "[", 
           RowBox[{"[", "3", "]"}], "]"}], 
          RowBox[{"3", "/", "2"}]], 
         RowBox[{"\[Delta]m", "[", "a", "]"}]}], "/.", "sol2"}], ",", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"kv", "[", 
           RowBox[{"[", "2", "]"}], "]"}], 
          RowBox[{"3", "/", "2"}]], 
         RowBox[{"\[Delta]m", "[", "a", "]"}]}], "/.", "sol01"}], ",", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"kv", "[", 
           RowBox[{"[", "1", "]"}], "]"}], 
          RowBox[{"3", "/", "2"}]], 
         RowBox[{"\[Delta]m", "[", "a", "]"}]}], "/.", "sol003"}]}], "}"}], "//",
      "Abs"}], "//", "Evaluate"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", 
     RowBox[{"10", "ap"}], ",", "1"}], "}"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", "All"}], ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameStyle", "\[Rule]", "Black"}], ",", 
   RowBox[{"Epilog", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Black", ",", "Dashed", ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"aeq", ",", 
            SuperscriptBox["10", 
             RowBox[{"-", "6"}]]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"aeq", ",", 
            SuperscriptBox["10", "6"]}], "}"}]}], "}"}], "//", "Log"}], 
       "]"}]}], "}"}]}], ",", 
   RowBox[{"BaseStyle", "\[Rule]", 
    RowBox[{"FontSize", "\[Rule]", "12"}]}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
     "\"\<a\>\"", ",", 
      "\"\<\!\(\*SuperscriptBox[\(k\), \(3/2\)]\)\[Delta](a)\>\""}], "}"}]}], 
   ",", 
   RowBox[{"AspectRatio", "\[Rule]", 
    RowBox[{"1", "/", "GoldenRatio"}]}], ",", 
   RowBox[{"GridLines", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"PlotLegends", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
     "\"\<k=2.00 h/Mpc\>\"", ",", "\"\<k=0.10 h/Mpc\>\"", ",", 
      "\"\<k=0.03 h/Mpc\>\""}], "}"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7670007843028517`*^9, 3.767000785958127*^9}, {
   3.7670037041181507`*^9, 3.767003704298748*^9}, {3.7670038667263975`*^9, 
   3.7670038949586353`*^9}, {3.767004137375663*^9, 3.767004139600129*^9}, {
   3.7670046545273056`*^9, 3.7670047223864164`*^9}, 3.7670049734094257`*^9, {
   3.767005145416571*^9, 3.7670052057626343`*^9}, {3.7673473315655203`*^9, 
   3.767347391915212*^9}, {3.7673474245954247`*^9, 3.7673474918882785`*^9}, 
   3.7964573793168592`*^9},ExpressionUUID->"944ed33a-9f40-44c5-a365-\
187f001b6345"],

Cell[BoxData[
 TemplateBox[{GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.368417, 0.506779, 0.709798], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVkHk4lPsfhmNmiBZCyjKyzZj3HdKJFlHfb6ZsyQkd2iNLKlFJWkRSIkuR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          "]]}, Annotation[#, "Charting`Private`Tag$3545#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.880722, 0.611041, 0.142051], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwV1Xk0lN8bAHAkIjvJHuad3VIkVLpvSSRRkqWQrKmhRaUSSt9sISVRSKIS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          "]]}, Annotation[#, "Charting`Private`Tag$3545#2"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.560181, 0.691569, 0.194885], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwV1nk0VW0XAPBCylxCRXTceUjKUKl4TilTEhJRSqGIqFQaDL16CxWJhDIk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          "]]}, Annotation[#, "Charting`Private`Tag$3545#3"]& ]}}, {}, {}}, {
    Ticks -> {
       Charting`ScaledTicks[{Log, Exp}], 
       Charting`ScaledTicks[{Log, Exp}]}, GridLines -> {
       Charting`ScaledTickValues[{Log, Exp}], 
       Charting`ScaledTickValues[{Log, Exp}]}, FrameTicks -> {{
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}, {
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, DisplayFunction -> 
     Identity, DisplayFunction -> Identity, Ticks -> {Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& }, 
     AxesOrigin -> {-13.815510276015079`, 0.4069180265176433}, 
     FrameTicks -> {{Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}, {Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, 
     GridLines -> {{-13.815510557964274`, -11.512925464970229`, \
-9.210340371976182, -4.605170185988091, -2.3025850929940455`, 0.}, {
       2.302585092994046, 4.605170185988092, 6.907755278982137, 
       9.210340371976184, 11.512925464970229`}}, DisplayFunction -> Identity, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None, 
       "ClippingRange" -> {{{-13.815510276015079`, -2.8194919506049536`*^-7}, \
{0.4069180265176433, 
        12.63051527197537}}, {{-13.815510276015079`, \
-2.8194919506049536`*^-7}, {0.4069180265176433, 12.63051527197537}}}}, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 0.4069180265176433}, 
     BaseStyle -> (FontSize -> 12), 
     CoordinatesToolOptions -> {"DisplayFunction" -> ({
         Exp[
          Part[#, 1]], 
         Exp[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Exp[
          Part[#, 1]], 
         Exp[
          Part[#, 2]]}& )}, DisplayFunction :> Identity, Epilog -> {
       GrayLevel[0], 
       Dashing[{Small, Small}], 
       LineBox[
        NCache[{{-Log[
             Rational[979917, 83]], -Log[1000000]}, {-Log[
             Rational[979917, 83]], 
           Log[
           1000000]}}, {{-9.376382545385876, -13.815510557964274`}, \
{-9.376382545385876, 13.815510557964274`}}]]}, 
     Frame -> {{True, True}, {True, True}}, FrameLabel -> {{
        FormBox[
        "\"\\!\\(\\*SuperscriptBox[\\(k\\), \\(3/2\\)]\\)\[Delta](a)\"", 
         TraditionalForm], None}, {
        FormBox["\"a\"", TraditionalForm], None}}, FrameStyle -> GrayLevel[0],
      FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {Automatic, Automatic}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, 
     PlotRange -> {All, All}, PlotRangeClipping -> True, 
     PlotRangePadding -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     Ticks -> {Automatic, Automatic}}],FormBox[
    FormBox[
     TemplateBox[{"\"k=2.00 h/Mpc\"", "\"k=0.10 h/Mpc\"", "\"k=0.03 h/Mpc\""},
       "LineLegend", DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.24561133333333335`, 0.3378526666666667, 
                    0.4731986666666667], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "RGBColor[0.368417, 0.506779, 0.709798]"], Appearance -> 
                    None, BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.368417, 0.506779, 0.709798]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.368417, 0.506779, 0.709798], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.587148, 0.40736066666666665`, 0.09470066666666668], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "RGBColor[0.880722, 0.611041, 0.142051]"], Appearance -> 
                    None, BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.880722, 0.611041, 0.142051]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.880722, 0.611041, 0.142051], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.37345400000000006`, 0.461046, 0.12992333333333334`], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "RGBColor[0.560181, 0.691569, 0.194885]"], Appearance -> 
                    None, BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.560181, 0.691569, 0.194885]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.560181, 0.691569, 0.194885], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}]}],
              "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2, ",", #3}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.7670039472590375`*^9, {3.767004133510419*^9, 3.76700414049401*^9}, 
   3.767004394025344*^9, {3.7670046765721254`*^9, 3.7670047239881277`*^9}, {
   3.76700496937527*^9, 3.7670049744089127`*^9}, {3.7670051204966173`*^9, 
   3.767005146198245*^9}, {3.7670051982943273`*^9, 3.7670052269702873`*^9}, 
   3.7670702379066343`*^9, 3.7673473269486847`*^9, {3.7673473687454495`*^9, 
   3.767347376041238*^9}, {3.7673474407354*^9, 3.767347493060093*^9}, 
   3.796457380172393*^9},ExpressionUUID->"17a758cb-c05d-497a-a230-\
3dd632d3ecd2"]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"f01", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]", "[", "a", "]"}], "/.", "sol003"}], "/.", 
    RowBox[{"a", "\[Rule]", "ap"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"f02", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]", "[", "a", "]"}], "/.", "sol01"}], "/.", 
    RowBox[{"a", "\[Rule]", "ap"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"f03", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]", "[", "a", "]"}], "/.", "sol2"}], "/.", 
    RowBox[{"a", "\[Rule]", "ap"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.767003802842705*^9, 3.767003814155143*^9}, {
  3.7670041497285476`*^9, 3.767004150175553*^9}, {3.7670048169745626`*^9, 
  3.7670048561265907`*^9}, {3.7670049929643607`*^9, 3.7670049983189063`*^9}, {
  3.767005253973484*^9, 
  3.7670052573479514`*^9}},ExpressionUUID->"32f1c521-22c3-4f5c-a99c-\
8d08e948855d"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LogLinearPlot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       FractionBox[
        RowBox[{"\[Phi]", "[", "a", "]"}], "f03"], "/.", "sol2"}], ",", 
      RowBox[{
       FractionBox[
        RowBox[{"\[Phi]", "[", "a", "]"}], "f02"], "/.", "sol01"}], ",", 
      RowBox[{
       FractionBox[
        RowBox[{"\[Phi]", "[", "a", "]"}], "f01"], "/.", "sol003"}]}], "}"}], 
    "//", "Evaluate"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "ap", ",", "1"}], "}"}], ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameStyle", "\[Rule]", "Black"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
     "\"\<a\>\"", ",", 
      "\"\<\[Phi](a)/\!\(\*SubscriptBox[\(\[Phi]\), \(i\)]\)\>\""}], "}"}]}], 
   ",", 
   RowBox[{"AspectRatio", "\[Rule]", 
    RowBox[{"1", "/", "GoldenRatio"}]}], ",", 
   RowBox[{"GridLines", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"BaseStyle", "\[Rule]", 
    RowBox[{"FontSize", "\[Rule]", "12"}]}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "0.1"}], ",", "1.1"}], "}"}]}], ",", 
   RowBox[{"Epilog", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Black", ",", "Dashed", ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"aeq", ",", 
            SuperscriptBox["10", 
             RowBox[{"-", "6"}]]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"aeq", ",", 
            SuperscriptBox["10", "6"]}], "}"}]}], "}"}], "//", "Log"}], 
       "]"}]}], "}"}]}], ",", 
   RowBox[{"PlotLegends", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
     "\"\<k=2.00 h/Mpc\>\"", ",", "\"\<k=0.10 h/Mpc\>\"", ",", 
      "\"\<k=0.03 h/Mpc\>\""}], "}"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7625168700408125`*^9, 3.7625169021553583`*^9}, {
   3.76251712800572*^9, 3.762517148948697*^9}, {3.7670005014731812`*^9, 
   3.767000615088724*^9}, {3.767000691624955*^9, 3.7670006948145638`*^9}, {
   3.767003428915862*^9, 3.7670034299534373`*^9}, {3.7670036997674494`*^9, 
   3.767003699961059*^9}, {3.767003819508198*^9, 3.7670038414614596`*^9}, {
   3.767003967197608*^9, 3.7670039726001277`*^9}, {3.7670041594627047`*^9, 
   3.767004196915502*^9}, {3.7670047478141136`*^9, 3.7670048820931873`*^9}, 
   3.767004985508012*^9, {3.767005241202551*^9, 3.7670052659092054`*^9}, {
   3.767347501035697*^9, 3.7673475012757664`*^9}, 
   3.79645738557469*^9},ExpressionUUID->"ea4d4744-627b-4c51-bfb1-\
7f00a2ed0ab3"],

Cell[BoxData[
 TemplateBox[{GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.368417, 0.506779, 0.709798], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwU13k81M8bAHBFJZUiUVns7dwl6ZBjQiIhR5QjlCKVQiIpIpHU2l33faQS
5UiOqPZJUUmlcvQVct9XrnLEb35/7ev92s9nZp5nnpnPDOnEBYtTy/n4+GAl
H9//fw+2Pw/QkFYBjlD30tLSGBokRoX+IagAOu3OncW+dfwcq0hSBXpWF0pP
Yld3SqcqblEB+bvG23qx9/bceCkppgL7mXGptdg7B00W5lerQPKL6tAU7HoF
WYEyQRVwyw94HYd98SzfWu9VKnCk+NsvDnbB8FPJYQEV2OzDqLuJbd2m7FO2
pAxLk92vz2Hr9L+ZvDykDF/Zp7TUsQPcfPh/9yvDjyFajyr2i0G5jW69yvBZ
xiBF6f/9j9xVtelQBk3ZV6JEbKWJI57qTcrA3XdddQW2+MLQ6GylMpi+YRu/
XxxDh6+lLnqCMohN39oA2JxFM+HBl8rwqOZmUSn22mUljOYyZTiZ+SXhAfay
ldfPlecpg3nNvMcN7OH1YoNXEnF7mgwpDeyJ7AC/F3HKEN7yoUcF+4/OgOBi
tDKUnjYNp2Mv934lGxSpDHwHS81EsDf/OH0qLEQZHocqyvT+G0PSnt+n3gcp
g1+b79+f2BQh7RChQGU435FM/obN1Nx4766fMoQNhbnwsPXSX7ZHnVeGsmus
mBRsQ3U5j4azynD44b22GGzTb9wlcTdlMLOx8mRhHxU4LZ3orAz+yfXnr2M7
pHx78tNJGXg3CU1+2M47tbSkHJQh5/vUjYvY50+L2qcfVQbye9Z9V2zvZdeG
OqxwvnyRxAlsv8S+KxRLZWgqPvbOHjuk9kXCAxNlODU+/tMC+/YpWfl+I2X4
aVihaYoducgpkzdUhtzfyfUHsBNVXJse6yqDZ+3ZDB3s9A9fXUaRMowQV3zT
wr5/QnNGWUsZjOH2qDp2zvyDm557lCHjdteRHdgF0SKbinYpg/7aCSEV7BLG
1awpNWU45NiyRgG7orp3+05VZZBPnXWgYIOj+RtfZTx/Et8FCdjVfyssnisp
w7EwmrAYdi2H3jknrwx7bDovrsH+qsDx1JRVhslhFbQcu8Xehc0jKcOo307C
6MIY6pyuk1kmowxVDVbqXdh9LI18XYIyrLzU/K0JewI2fK4SVwb7cJ9oHvZf
G/9jq8SUod615cNT7H8TPcOGInh+LAxi72ML0irW1K5RhtnyJJHb2NLhpw58
41OGxlMrww9hHzAvf9s2yATmwwufG+bx+Ll17ab9THihmF1Xjb2vvnfhVQ8T
7p4VjijF1j6ycUdaOxMUdC3447FVj52779jIBMu95mOW2FvdpEPbXzOhYLR2
W/ncGIrLUcs04zFh9Kzw64fYYsNGr+AFE4L5I2JjsIUv+M6klzLhlQp33hOb
/1Kdy/EnTEh8EDithH29pDf4aw4T1GyzliSxF/8spOpkM6HtzKYta7D/XpFv
It1jAttewHZgFq+f68EGnfFMcNoZix5gu1XGO1vEMuHvzFBeLHYvf35gZRQT
5KRLm8OwnfWrklQ5TNAgmL7zw24P/VmayWLCXNFWv3PYx97//i56hwmaRolT
jtjNqwXHg8OZ8HOng/Zh7KMHpddOhjLhq/hauwPYDXfU5JxDmFAo43AIYVt8
Ntr3PYgJx7wpxJ3YdeuPO+kFMoHsv/cbA9vE3Pdq0VUmbN2Rf4aOXcO9G0+5
woTp2MBRGWyD+nvPonyZINPVdGIr9ttN5XX8l5hwdMV07SZsnSN1wxe9mPBk
TEFVFPtVfK9g9wUmfPmnEr4eW6N5gXrYnQnrJQvH12KXSW7UeXuGCcanmuSE
sJ+mIr+sU0xw6DA5KoCt3G4VI+bMhMjDpb7LsB+TzhWGODEha3CpdPHvGJJ3
Dv40dYwJNV7AXMB+kBU/cNKOCdt275yYxab05q1oOMoELc/9W/5ip8tWkfSt
mRD/KqhiBlvK7adWsSUTrpTL/p7GTsz5bUMzx/UTr/D9/5YYXuUTY8oEnQZV
//8/H82Q5q4wZsKl2Uz+/7cnekEt79IBJgyeSQqbw44sMKrp2c8E5ebVHf+w
U3kb3p7Yx4QNq65qLWHnfW58+UuHCdYNzxr///+r1uRSe8SE0+u+cP8/3s/D
Jwr/08T1+tz8yAR227xcrvUeJuSdZazuxx4RGsv6vosJppee32nGXthSnGq2
A49PvLDqA/Zaef/4T6pMoKSoxBVjE3brcI1UmGB+9WlXCraiwao77xhMkLJY
ezcYe4/1p5v7FJlg1X3W2xnb6FRU4Gs5JjwTPnxlL7att42fNp0J+7MSArZg
u92QuVhBYYKJnpPT6B/8/czIdSmWZsKMlYRVJHZ8gaeTKoEJmd2pro7YD3m7
bPO3MEFYplVAGbu69Y3JIzFc3+ucN36cGUONw+EGdFEmLNTSY2Oxe+YP6dxb
z4T7zy5VO2ILbG1RSxFiQvjtIcWBabxe5TOZWwWZQL3LZ52NTdl9Wi5uBRMk
n1hLOGPrWk9Jcvhw/uIzZD5M4Xo/Vb5JeJEBJGYt1Rv7uPf19RHzDKiLdKyX
xfa4sX+14CwDfqgp3e+fxOubu5b/5gwDipTmlpdgszO+LSybYsCC3b7Dkdhp
BfEzAb8ZECYmOXcRO4/nMD4/yoBo6kozZ+xXn6mDl4dxe7a9ufbYn1sHu6YH
GCBxrfGYI3brcEGrVx/+P29NwRns4XmfprFuBnTqF/dew14Q0vp6rpMBa4Va
UDz22q38Hwd+MSBw1fvj/x+PpPyHty6tDFj3d/dsFbbC7shXXc0M+PipSvD/
3mNgVeb0gwHisiPOxdhG1pJPWxsYUGksO/L/9mxPdeTafmfArc6dsZ7YZ7wf
3m+qY8BxIX8nHWy/G+5phz8zYJZP75gQ9i3u9oSvHxkQoE7J+zyB5zdjlmv6
gQFblw7eZGNnF/DufKxmwJFZz5sW2GW8m6GGbxnw91wBZSv2+88Hr1e9ZkAs
s9C/7/cYamoVuaLLw/ncYC3/HHtmPsVds5wBG7VkXjljr1xz0vV5Kc7H0on7
6tibtioc31nMgC0yop3rsNV2lxxWKWCArdVwc844/h4YXDV98oQBFm3vYy9i
W1rrGirkMsAw+lLxXuwTpwR1H2bjfJy9cUwU28v7swb1AQOeDV016B/D+Uky
Lj5zjwEPNpVs5WGTKmuYhel4fgzqdiVi/+03zP6TwgCD75nqfmP/3y/fkbST
GJA76lZqh529Uz8pJJ4Bdn6F+3Swrx97I/YxhgE2dVb1stjKubxVR9kMuBAp
fXZqFMf3TSso9S4DrPivCLdgt/2tmOu+zQDHnfuKq7BLZPZ4K97Cz99N1H6K
zdpfNuJ5kwGnQxLZmdgu7jtdy4IZsOQUHh+HrRX9rH0xENeDm9rHO9hiFaq2
+tcYsO9E2V8/7OGOgu8RVxiw5uVUkSP2W0Flk2++DJCn9ZdoYycrP6nefIkB
/6oolRLY3taKex29GFDa/Tt5aGQMHbz26Pn9CwxI0Li2qQKbnCW7ffgcA8o/
dE2HYs/V3H+seoYBHXn/PTPC/vabQvdzxfm7JnacHztnc2Ya7yQDDhsrVRUO
j6FgRNyy8gQD6jMIRdbYNi6pXGNHBpSoewhMDo0hlbuEtVH2DPiU7p4Yhi34
LPHmfzYMcNfqlxLHbm/evCRzhAHcHyubigZxvS2Lu+xymAGNpOPXXbDZcpsm
HpszoKuVvkoe2/VQ1NlJUwYsuy3OmBnA5wUfkR51YwZMSASHvccWT4l0uH6A
AfQa04BE7OrBCPN1+xjgd9zjsTJ2qojQR0sdBjDfJsBU/xi6tPvWvkRtBmyv
FxcpxaaGhuySVWdA5KXC3zuwFx4vL3TfyYD1LRX1k334fvD9usKz7QzQjlBN
LMS+QbompcNkQNr+6gJl7M4Xl/jtKAxwTSzcbNA7hsq7Jq9mEBlQcyGqXwib
I+Q10yfFACctCqumB99Hjp4f8N7MgJtGAnYIO23S5QtrHQOkIxX/reseQ/YK
R5Mq/yjBTTcfHU7HGMqdDcrYNa0EWc9kg3Wx597nPnw8oQRbUMnjifYxlOC6
VBQ7ogSH+Dz+O4TdmPWg9kyXErz9ckBkqW0MmctM/xP9ogQajHod1IK/n6PS
ArdqlWDtozzVrp9jaPylodC/D0rQKCR7NxSbZZ8s3vdWCV77ON7/2IzPmwl6
yhXlSjDcktBt8t8Y2i/GdXJ+oAT8H+++kmscQzFdFS4/7inBry1zS7yGMdT9
tOecSYYS/Mh9YmuNfcNc3W9XshI8jAoxvl6Pz7esds4arhKcbTNWrfqG97vV
ym+KrinBXV9DR7EvYyj8x9EPcv5K0Pz1P774z2Pox8PgLymXlWDjeZboVmzf
/Y0/wy7i/ows3bZ8GkPPblybsnNTAhXhSyICH/F6XfxIE7BSgjMXgmRDq/F4
e57k6loogVufg1ZXFb5P1EZuCzqkBOO6nKfa2HcSLbUWDyiB19CQy/gbfL/c
+fPwX20lOLL6Z7jma3weuTB4Y1hOCRbVxJg+Ffj7YF27WpGOx+t2M760HJ/P
tPIi3ShKQCnfUf3nOf7erfVK6pVSgpktY9u8y/D3Knv2abuoElxSKc6wK8bf
8w7BzvoFRQi/+KZgOA/vh+8HXTfOKcLbMGvPrdgH82tHzP8oQl7qFWeDJ2Po
yjX27OffisDsmxBLy8X775bNIh96FUHvYKoXPXsMcS1k976sUwQF1/ofWul4
vaqvrp7/pAgF0vsGVdPG0Gvi0ME9HxXB66DmED0Vz/do3pHSKkVIHD3cvDoZ
378idl4oLFeEbDWSy5O4MbT6rX7q/SxF+NHRPn2Chc9zl5uW/c5QhK4ayZU7
7uL6ZLid0kxTBMXZey8E7uD7Tewdpe8JilD29gM5MRyfX92+l/NFKkLbdoHm
hBCcP+HjTXZ++HmCceWE3xgyezOx56GPIrzPEWCFX8b17BuSOnFRETLKR55I
+uLxdDw8FX5eEfbMqNJVvcfQdNHoZImzIrwuyuRXOo+/FzZXN4iaKkJPJkPV
5PgYGl0n7H3soCLIGdzKuO+Iv2+VaU3ZhoqwfEP46YVjeP9UfJOK9BSB4G69
777tGIpaXM1w360ID98IHfluidfvvbgD78k4fldLygF9XA9H5Z9sJCrCNsuf
b3br4fv6uooNjlKK4LfMgiCug++LPm1N0xKKUNnialKkOYa2HaC5UNbheGX3
8ZzV8Pdj9GlwwB8FuHq8m7GBMoYyD457/Z1SACXvvOsjxDH0Jpvh7DWhAFmH
48cqpfH3xDlbz2VEAc4nMVeabcX9N6UImHYpgGQbU//fBhwP3AqV+qIAZ18O
XW38N4rKpap94moV4OJvvS0e86Po5xV+V5EaBfjPY9Zw3ewokt4RYLCiSgFa
evfnU6dGUdaji4Ij5QpwQ6V+3duBUVTAdbj94oECXLv4TvTTt1H0bSzpys4s
BagxEymY/jKKJo3/O1OQoQCc9tV3iJ9G0c5Vhw9mJSuA8GyPi/67UfTC/8Da
O1wFaBjP+DxTPoren1Rj2QUoANs00H8sfRS17xTizlkpgM1EmlMHvinX5C4t
07ZUgPiuWy+TToyiIuK0Z5AZbj9nuumgwygKFWo3FzyoABGUc6MRVqOI0VYs
Ir5XAWL8bOeD9EbRlZvHOdsUFCBn4lagltQoOjlnveySLO7vU991rc2jyOSC
sWcZVQEqMl7Gq2wcRUSbXeZIRgHW1d7ePi44it4prRMxEVOAsQzp/2onR5BY
fRn79KI8iElo932qHkGLhnl8j+flIZxpkjwII6jv5T2Psb/yELR6X8Pf8hFU
/pBl5jOJ/xe+6TWUN4KO+5/cENIvDxHHmRvkY0dQHnkDO+2bPMxveGpVfXwE
GXi6RjY+kIdVm+TI64eGUTZL8I5hljxkWg1YRnYNI6HHj26VZ8jDZqOKRb6W
YVTbOxSUmiwPPRfuaObVDiNzew9vF648WN1W2yr5ZBjZGfjZzFyTh4yzi4W3
zgyjFye3Wp/2lweO/dmx7uPDSDq4wqL5sjwcf5qho2IzjNpf/Dv46qI8rJSt
iGMbDKNTqsHaoW7yoJJmNapPGUYXpCIo4lbycMdmI8GvcQjV7VEi3rKQB2ON
wzO3aoeQ6tFPhLlD8mB+L984tHIITXHXi7cZyYP8KrlBjbwh5CcYLfhgL47n
vY7m+ZAhdGMyeWSHkjyQ3Q+oOygNobgPeaVW/PKAXIbyjrkNotvPljj1fPLQ
EjeYzLIfRAFpZucsF+Ugir5H/OGhQeRyaYJoPisH2km2p9g7BpEaaedt4zE5
qJobPiTON4i+XX5pr/dTDjRXPD0uHzmAqp2Fd1X+kINCguSd0OsDqNzUUUSn
UQ522Lh/6fccQBnUZdXaX+VAR5NJeHx4AHnU7VPe804OjCkXv+dKDCBhudpl
24rkoEZmbsIuvh8t3yjVkl8gB7uyO5vbw/rRzD/3EmaeHLi7PeDq+/aj1u/r
zyo9koPVgx3TeVb96HGgZb1smhzM5+zauEakHx1oan4gHSEH/Km+FS5BfSgk
dODgWmc5aD0adv60YS/aGz5x/7CTHAjf7V/Ws60XLUTMLyUfk4Phiadb1CV7
0UXOumeMozie9ZaC50d60InkbYRDJnJw/82X9z8je5DOU79hzi45UBY8F7G+
thsttq6+K7FWDvxj+OSzqF2ovF20z3G1HDBF09L9BbuQT5ekTvZKOcgMDzSm
DHeikX7G9O5lckC+t/aATlEnapk0P2Y7IwtTZzSZR/Z2oorViYyUdll46rEQ
bWDegS7vUPhMLpYFb40zt+Rz29Dj9rZnCU9lwcVohV66exv6FRGVtKFAFoqe
l11vUW5D+zsXTi/myEIg1TQkrqgViUV+4W9OlwX/9Y2vbMtaUH6/tzrnjiyQ
RiWuFJU2o94kXtaSsyxIRotWlMQ0oq0G3hE+x2VhfUee5EWLRmQyIec14iAL
OmusnhHWN6Jnhlz000YWTsl/0tW+1YCCp082lxySBWHFw8RvfvVI+pDQhvMa
snDwy9OqVttvyGq5lX+LqCxE/LovYG5Yi8yqXncQNsiC0YzaK4+yj8j4FtPw
2DpZEHr7RsBG7iPaJyy4qW2VLMi9/ODburoGqW6tyPu1QIdAs2UzMd/fIWFV
UmdnLx1UyTcsFUPeIKHpu4aUbjq8XvnhQui/SrSibC7PuYMOu63j0gp8K9GC
Vr1/9086CAXbXMo7/xoNHgjb1FtHh9403Undulfo3fERw4FyOsSdtRNnEUpR
JdU2X76MDgKve++ZbytBr/qqN50ppsO5+blNVYbFqMQ9vXMwnw5uvYh9O7AI
ZflZXh3OooPjpfbfFmsKULomdCpl0oF2V1Jc+2weSlpSOuCeRoc1y/Rliqof
I+7NleKjCXQ4e1t/3+u4R+g653n+GIsOx982PiDfuoeuHqaLq9yhQ2nx+j1b
VmciX4moqx7hdLBXvG7dHZaOzqecO/D7Bh7vRYnrVXeTkP0jma6Jy3TQD2Jo
/FHloKPn7hzY7kMHakaJimnIXWSpPJt/8SIdHGoIO8ye3kJGxd+uTrnT4fBW
XQFPZiBSf31TfOYEHU7TJzXdH+jz3ptn2uo50aHV/Xfz4bKTPOvOV6nsYzh/
G2+naj735Hnx/6UrHqWDq7/WPK8ykMcXLXb2shUdFgKU5WJUb/BY1G35VRZ0
OHG1RcfXMZSXo39ml5MJHSaXC+nnW93h7W4M9X9iRIfvxyxWDRAiedUu93hz
BnQ4X6zY+KODzTv8h8dvqE+HqpftWS+pXF5nWItBjC4dKiSUnl6ziuJ5bJ6N
6ER0YKmGWmd5R/MWszfVKWvR4fmZoIt9wTG8rTWmR2t20eGiRHHP3rNxvGzb
s8kSO+igQ7siIrA/nrdzKKz9pCodrhEutC9bn8B7659FfapMh3em12XVqxN4
Fmtfn15SosP8f/8OOZ5L5LUntz42VqADcfE0Y9eyJN55xtx4giyezzPZDz+H
JPEWXorv6KPSoUPdzv76nyTebdPtfmpkOnzV/yrw7mgyb/OvQy+DZOjwha9R
5El2Mu/BhXPLvhDooFHmytDpT+ap8YXrE7biei4b//RHPIVXyb4f7iZBh8sB
4q3t21N4ZqTKTyVidJDdrzv7WDuF11bYJiIgSgdhE9Nmxp4U3jndeSvz9XTY
s83/33Z6Cm/um0Ri6lo6bJ0QvvJnWQrvlrNa29BqnP/838c3fErmiU+ZkdVX
0aHY3T33U1gyLyvE3SVUALenoLcuQjWZp7rpds73Zbg+bym8yvmUxIP7D0aJ
SzR4t1NqyxWbJJ7pzjeq5xdo0OAoHkxrTOS1VP/yqZilQUnM9oEfeom8M0cW
ygX/0EBwX+mfiswE3t++zUtWUzSwP58Tt2kynhd6eYfevd80+DEpkbpjRzwv
M+H8R61hGiytqe3JiIzlqShErI8YoIEMXeO3fXYM71X5Q8sfvTRo3ieXOF4U
zWv+2f7Tq4MG7QYilhfuc3mnz/2TgTYadHUIrB6P4PBmFracXNdCg30j4cE5
LmyeqLTl8MNGGqiZe2Wi6QiekVPVQstHGvxeRdNebRTE+zHesVfhAw1EA3SC
/rVf5bkELYb4VtMgu+dS9dgaX15w5q51oq9poC4t0XZO0IVX3v1IyqCEBmf/
3ev4nX0OgZTXaGURDZ6UttaHeXmjKus9PO1CGvz63DM9nn4F1b2vcdr5mAYi
791muqJuoL7HQ/domTRgbNG+f0OejcQuMRQEWPh97vrre5enoK1503OBETTw
DFh/5HNsKiL2vfw4f4sGtg5pAepi6UjRxtR98gYNQrfv7vqVkYF0tS4UdPrR
YM/2n/5vUrKQoc+uIAdfGjCNnq/zt7qPTPKXzJu9aeAfXetQu/oBOkpiT369
QAOB36VGO88+RBdWFO58fYoGVy+38BTmHyFvbb9VWs40OJeQ7HuWm4Ou+Or8
KHOiwfGtoS9aKLkodOCrX4EdDab+CJyeQY/RHXKikZIN/n9PvNT7d48Rx+6E
ZLY1DayXbm6RMHyCkj9NvEgzp8EODf3PB3bkocyVFXclD9FA5cgKVbnUPJSN
bjjEGdNgvz/r71O+fJR3+aDyRiM8/xnTRULO+aiocCNfpAGOV0jX5/TbfFQ2
+LNOSJ8GiScUyfnCBegVJSsjVJcGIaolfoG7C9Bb+3Ney/fiel23Q+ONQwGq
iVHTC9CigfbUZLDNtQJU93lh49weGnBDcg27owpQw6qq7ku7acCvyT1vmVuA
mvfeLf69gwbV2YI3Hr0pQO1+VqHu22mgFRWzfOFHAep9KnVkQIUGVBfhQPZo
ARoe6pE9xaTBdLZ4ryR/IZqg5v1tV6TBl0K1Z7dWFaI/x3w+2MvToLdMcE39
UgFaiNVO/EHH+c27djl/ZSFaXrfy7GEqbs/yslO4cCESXP1Fo45Eg7ZPbE97
gUIkrBu31liGBsu6JVObZguQmL9j6zsCDSrcXYJCxgvQlmeyeXpbaZDrlRvQ
3lGApEfGAngSOL/r3R/8/FKAqPSyQxqbaEAW9gp/WF6AJnx2p18XxetnVyD7
TVoBgndl41XrafBPg3vmYlABYm1W11mzjgbDO5ZvC3EsQHZuzzlmQjQwaRkn
zKgXILly9c6YVTR4fUxtcmFDAZoRKlf9KUADvjOrQkw/5CPu4/Lvp5aowGgY
v/BZIB85/dtDzV2gArV2hdnxhDzENK3wHp+lgt0c7/d2uTz0Yaxik/8UFR7Y
pShk7HiC+La/PMIeoMJo5q3MBolc9PmGVnZDLxWk1Rqung/NQcn1L/9u7aYC
UlsbWfb7Edrl8yrhfhsVtD7/qfnIy0bnn/N+ln+nwu+jXfZ6+x6gFlTp2POS
Cn5n9n2r3peBcti6BQoVVGhg1yk8yk1Hlzsq+TzKqJCpvqfEelM62njjTcbc
UypUJDa9EO9JQe3f9SZQARUy9AZMuZ+SUR71re7NJ1To6DYiBLxKQkbVb7vW
Z1PBkKY2qv8iAW2W0Fezuk8F9q43r47XxKNe16qQxEwqlPUUqZ3rikPPyvQb
fqVRgRx1x2piVRwKWl1No6VQgfbvQuPl3bHI1Ha/z5lEKsydmMvwvxSDCLnV
1flxVLjRw1Z050Wjwfn9EtPRVFBTrvxcLB6Nyozfue7hUkElr/BRkX8UCk0x
KAuMpELQ2ga18lEuOjz6TrDqDhV8mharRTy4iIwMbYRuU8H3xasVf5Y4aCzy
/aNDYVSwym+oTkzloBfthnPRIVTwcjevIxlz0O1tH4yag3B+y7l3swTx+SL4
QJJMIBWcNQXKmPVsRP/+YejkVSpsKH9P5uWx0RTFSDPHjwovsrz5DOPZqNK7
5s6YDxX6TcOu57HZiF1l1KrmjedDPEXtfRwbOYh/ZFzxpIJSk7qx8xM2UnI9
GMA7T4XzG7y5e76y0Vzpx88C56igd+1i29JyDnovaCxj5EaFUp8uxhFdDoqx
qb0Q6UKFSxElgw0sDmq2+GvMf5IKP/sCpp91cBDRmKrge5wKJ6/M39yDuOiU
vtmqIQcqnH5GCJp+wEU52le7HeypcCDO7urZzVFofFf26282VGAGnCdviYpC
O7bVp+4/QoWk5e+2eON8A0XJhmlBhR27T4S0q8WglVJHd2YeooJ6YLVq6ecY
dFA8ZKO4Ca6/WqUV78/HokbBlk//DKjwnFnZ0v4qDg2PROh+0qJCipZa4ejt
RKTaVyqjo0EFzWWnUip1k5Bve9fCs91USK2/pbo0l4SWf9coTd5OhTAbvfwM
txRkUOsavWEbFTp3XTgUJJ2K7lRFeYYwqTARq0Z6WZeKJMqGFM/J43pM5ZnK
43q2L5RY3U6nAsX3nzHhdDrKzNHrtaRSocnyiMPBsnSklJKUriFDBW5Jo/6K
QxnIK/bdtXwCFaoXD9cNR2eg0shJW8pWKnxXOfn8Q2MGWrglsztOAtePHeX2
YbFMpBt8cNOaTVRwaeHPPGCSiW75+04EiFKhW/ZR6pmgTPTJ+96XifVUcGrZ
5xBQkIlEz3957LKOCkuqrXLmzZnoqOt8eLMQnt9vCWHFi5koxUnW1VSQCreW
2wXdkLqHOm0s91WuwOtz2GNzws57SM4ykLSTnwqyXjP8Pw/cQ+7GuYuP+Kig
UeG6eeeRe+ipftNPqUUK1AqEWiQ73EN/tfmfc+Yp8Pi1tPMqp3tIa7dy7IpZ
CmRvJC/3tbuHgrfZXfSboUCz/wrRCbN76L1CmNnIJAUEfIV3hu29h9ZRixjH
f1NgRELO0UzhHrKQ+iXUMEqBhBM+j9yE76F48TX9hsMUeClwpGpyOBO1rd9V
9WKAAvKd1k/FqzMRZbVzpkofBabSPdGKxEx0enlkYFY3BVq/f9/K55aJ8ubL
7Td3UqB/SkTJZHsmmprqVb/ziwLcUX4mZTYDqY+KSvC1UoC3QryzrDwDvW0/
87WviQKs9oz3eaoZSKg5Ns+ugQKk9pq5sYF0dOh7ZcSXbxT42bpFUyI1HTVX
bd1f+okCp4I7aWpLaWgsp6Y89A0FHsYV+jKdU9BCRZ7EW6AAZeuWGsenyUjo
E9d72SsKxLzZubhNIBnRx2yZV8so+P57OPdlfCJS40MRz4spoOJqtDL7dwLS
EaH0zzylgJ7MGrbOwQRkv30ow/MJBQ7wB/edFY9HZ/Z9WczLoYDV48rgnrA4
5GtVZDf8kAL5B5Of3Vweh0Jc4srk71PAu+rNFomwWMT19d/kmkmBEuKNjQbi
sSj9lqNXVhoFYpf/xywpiEFPEvS+dCRT4F6/VeuIdQwqz5FVkknE+ZUwLDi+
Oga9r1gTbh+H8+/RHXH+fTRqrB3rSYimQEDl/c020dGoq/W7bhOHAvPTmcdN
3KPR+GhpmlgkBYo83RkOltFocSlpwfwOBaKOP00N2R+N1opct4kMp8BG0sny
dOwt5JMltaE4P4edNH3w8/TthhuFQiiwfYByt+hcNFLbp+RhEESBXZKu2wc4
0UjXasOnkAAKFOt293Ero9Ehlyn5Sn8KfMxabBNdjEb2vj9Cly5T4BiPP6hY
LwadufWiS9OHAhEupIFCbgy6nJC+98pFCnxLXxWjOBiDbuaEpJR6UKDqAzl5
+wGcn1rjI6pnKZA4nufevSUO5bWqPLtwmgJ8E1f5XMPjUMWomMiTUxQ4NJQ4
qLwQhxo3tNbIOuH50VII7O+NR92k17KnjlHgwrQZxc8+Af1WvR+SaUsBR/NJ
blxdAlpr5a4tZfX/+Y3o+56fiLa6mCfZWuD4WubN8zcnIVnfHX/jDlHgx3wh
I8E/Cand2mLVYEyBO9QhkcimJKSb8K9Q1IgC8XyUx2mKycgsp0PYzIACWnsT
VUd8ktGxiqqzd/dRQD2iICe/LBmdqX30vkaHAmXer//sHE9Gl1vv0gQRBbw2
7/00TEhBoaOewfqaFPgSW7bfWjsFRS1Z/QpWp8D7A3ke4odTUPqGPZqwkwIu
jt26I8dSUB5JOuHfdgoY/cuwem6Xgl6oLp/Zsw23b3txwN8kBX3Q67W4zKSA
9ODYLsvtKajxcE1+sSIFnKY0qNfWpaDuU3lrJ+UocO5I7gbrlmR8fuK6qdAp
sK4u9+WJ9GS0FOZT7U6hAGGqUnqXbTJam2BLySXi/cSxLixUKBltzdG+3i9F
gV+Tqh0qT5OQXAW5lSZJgYEueriYRRLaUbtyj/NmCjR6/FunMJSIzEY/T7aK
UiBQw6P+/LpENF57gim3gQIzXTruK5MSEDt35rTXOgqkHCa9vEtPQHWnpdtW
ClJg8FFD11rNeORh8HSz+QoK+L/gt3R/H4c20PdbJi2n4O/VCNvrSBwy63J/
r/yPDKIWm7qGg2LR+Otl/FfmyOB9Z6AuTCYWsdNjtN7+IQMqzfIXeBuDVALl
LwtPk2HZ/h05Fh4xqO7Yy6dHJ8hwK8gTKdJikIem+UjmGBnaFpv+aXRFow2S
PbIjw2SgJTeuO5gbjQpmL5/YNUiG7aqRVO61aGT2Y21KUB8ZziyK0yxt8for
SW/62E0GAX/agJ5uNGLHqImKd+LxyZ9ZfWBHNFLxfm/s9IsM5hzjqk2q0ajO
wj4sp4UMe1ttC5fviUYe28ZfT/1HBqG7W6+omeL+N4QsaDeRwZpaqCTshvsf
ldgVXk+Gg2b21ndYuP9PuZ7fv5KB56LnuPMF7j8XPZb6Qga70sa/ReO4/9vf
e11ryZBY7Hk3jIHjd3MlPf2A2xvJOiBzAcdvMG+3UE2GO8cDjC/k4fjpkbH7
35LhwgpnR2OBWLRhBeUr+zUZNMhLk69MYtGhSqP9tBdkoFOy+vJ7YtFoetv1
C8/JsJl0t7VdJQ6xAr0qnpeQoWBovNrWLw4xHVbO8D8jgwPfN6uRl3Hos2ai
imkhGUpEFzr8/8Wh85LMs/F5ZKg7fUp3YWc8Ep57fb8zlwzNDZL1/m7xKO+H
VbvSIzJof+D+I0fHI9PSga2+D8hwTj8QdpfEo9GYa1av75FB1v8gl/YlHrG8
RdhrMsjwPkxjlU9bPGJa3q+xSiVD73O33JauePR5m/qK9CQypFWtTt/+Kx6d
3/AJDcaTIWu5sfS1Otz/mNMVtVgylI/v9+0uxf1/mnoWEIXzW69u/CIG9//4
1th7NhmKsr3+BJzB/d8mKGxkkeGBxTN6LR4/y63g5LEIMlQf35vtMo/jN9yX
9vAWGV5f/h6r8hzHT//x3++bOD+/7qyW98TxrzgnpnmDDFHPrFxcqXFIuHvJ
NPQ6GeI9HA0k6vH5OUP27VZ/MtxP06x8xcD5DqxYPHmZDAEjqzfoNccglsMh
9fxLZGDXKPTuvxWDmFpdF2e9yDCyJ+m0kHoM+izpm6fngeup+VwdjESj83NC
A3fdyfCY/MYw4mE0Ev4vlfLjDBlcTz1wuegajfJKVR3Ip8ngOeix34UZjUxj
q+PPnSIDpaAiUuVfFHKRWunUcwLHX+8uN9oYha5l6cs6OJGBYUT4z68sCkUr
3hxtPIbj6TqrQsiMQo+fvi0+ZEeGr9NJn+fx+fOtusC190fJQN47zZBmRaEW
0NunY02G8SgPh1PsKDRlcGNNuSUZLin3b2IlRqE1Xyq/qZqTodT1NEXzcRQi
Wy9PzDUlQ5jV5ud9b6PQnlad41RjHN/bFXZ/uqKQxckguZQDZPByIs3mCEaj
M0MwtsmADCLFKxK24fUW7MVXytpHhr/LO3MzHaNRwiwKWKWL94f5FSYr2NHo
/Sre2llNMtxdHz14fToatbMWv3vuIYON6+gdf7kY9GeTdtLgLjKE6H2KV7ON
QcIp104478Dre2SV3b3wGESnvpRvUcXz/7LpZk1xDNLOXRg/rIKft7td86I1
BlmrapZ9YpDhScd9jwy+WHy/8w/cr0gGY8dQozKpWHRzb8V+nhwZro3mZxxT
i0XJ7+bW7aaToW9iSW3dvlj0zHRPQwGFDKYPioinjGNRbYNfsjyJDASGXq4s
Xp/d9s+dM6Xx/mdh+lNzfyxa6PqrIEkgw8LixJOnu2LRxrO7J6K24PW/dFf1
FSkWKUz4Pl8rQQa//RlZZXi96/qVXr8pRoZ3yT+XEztikC3fH4NFETKoaTQf
iCqNQZ5hO9f7rieD8kNd3hdcb7eEfRrH1pJhy+KI/xOrGJQeU5xyWogMjXt/
bZKUiUFlhOmTHavI8GXkddXmXrzf3VNTsl1Bhhz5cEoR3k+XCovKD/KRIdPp
wsl1u6KRuPpk0Nt/JKBJ1hEnFqMQE1QPaM2TYIIzk1n4IQrtN/DaUPKXBJ0R
17x14qOQw+fCJuYMCbbpXqgJOhuFfKx+pz6cJMENIY7p7n1R6G6LigvxNwn+
qd2eWUOJQlnOHoyEURIMBuTyBwlGoReD+VMiwyQ4eK9+YXqKi+o9xypuD5Bg
0dAvIKOfi4b+Mm/w95GA3mebwu7iouXXzxtd7SaBqHTUqwc9XLR1VZ7IdAcJ
ds7fv5+C76eqrJEf7r9I0NWA9q9a5KIDmxjpvS0keDT5ofnQxijklHzO1bEZ
t7e0z/Y1Iwpdpjxm/mgiwcPF/a5hxlGInTM0bdZAgkMX+I8/Ox+FHm5TfPnh
GwmMdYrL9uP104hyDlZ8IkHWYsEf619RaLR6QFTtIwkWkv1WPBSIRitN5Zsf
vyeBeVKAurBcNJJuOJ1BqyZBH3uz3y3DaLTDPvt06hsSnNb/uZp2KhqJzafl
twMej8bkr28B0WgqIW6G/IoEL8eDGtZzo1H97kitUxUkSPDza6emRqNnTaEh
D8tI0Os5eYCbGY2ifQI+DhSTwO31d3mxtGh0cZOPqFIRCYReHXfIiopGFs/c
bc4XkCAmpkDN/3o0UrU8lV7wBI8v+9bQOpdoJDph3zeRQ4KNY7ESa/Sj0QT7
MHNHNgk2R5ryW0tHo2/Kxpd875NAKnji4+JEFCr8rPfieSYJiu8Qtmrg9c92
1+BfSCOBI6nrqh3Oj8fa7UbaKSRQcT7E//p4FFIxIv+ojMP9mxpVji6LQhsG
tsisiCHBkx9Hnp9s5KLxMBEXAy4JfGZ1fE7mc1EdffWT8EgSfJH4GjNzl4vy
q/imPt4hwW9oHJ3y4iLWyb97hG+ToDRH2G/RgYvO848HmYWRgD85KcLXgotM
M/vec0NIIDuX0H3dhIuYOr/WNwSR4CPt2u0/h7hIuL3RWiKQBEe3r1x+zYaL
RgM+p9hcJUHK9EufM25c9FmqujvJjwRvskr/9gVw0ZMXLxXbfEhAON3o0hvP
RXfsir2I3iTwuKFzYHsJF52be/z8hCcJeFbJX/0buEhpd7JB31kSPHAIPOIh
FoXWNkWx5N1IMPZxWciRbVFo+FJEw1kXEtSvaub/aBSFasVuEPKcSaCzFFdy
3zEK5RZdcR53IoElX6vsOK6/2xZeOaoOJIjwads66BuFzvx2++1th+f7Bp+4
6JUoZMQ+vrv0KM5/96MkhUtRSEHZJnDWigRpR3hvNN2ikNBns2oNSxLc63BR
ibeOQoPnDNcFmJHgx/ubM1+0olDNmr2HwYQEdeP/nItlotCjnF1Jyw+SgF2U
cunQPy66dUC5c58hCfapXL9f18RFp/vp8mH6JDiQ11Z0u4CL5OjipWv2kiCM
/i5b/QQXCVatWzTRwvUp3XdtTIuL+p1X6LP3kMCfRHmnK8VF75f/i/i2C89v
9BM542Vc9DBj6pvYDhJcFv8SNz7IQaF7h7ccUSVBS0K+Y1MzB7n86nJKUMb9
tYX1nfjKQfsDfj78qUSC17965R9/5iC61PdRKQUSRB08YhX0jYNWvqjZ4SRL
AlILf6drCwf12FZezaSSQGTgd2rGEAe9nX3+pptEAjh5ZkZtiYOy4guFZGXw
fpVDHPiziYtCdj0ydyOQ4IpliH4Kk4tONqbH527B+2Gf3oM3Bly071L8rxFx
EoQj64AVTlxEEWPTVcRIENjtHbjah4uu2G2fSxchwdsUCt0vjIu+ZTZ+EllP
gkLCUb2laC6SH/TLCF6Lx5ts5diTwkXXt0ldmlyN8/MtM/NBBhf9uAyGJ1fh
/WbE5vZ0GhcpgzOhQYAEe25oq/Jw/YWuWjWuv5wEzEBhfaU7XNRmmvOmZIkI
CuE1D9KucNGOWJM42X9EOCHx63fhSVyvreNn4ueIwOzKzlA8yEXd1Gjt1X+J
IE7q2bKA49M4t0v0yjQRgnpnYpRFuIhb1NwzOEEE6Q7PyK4JDhqcu/bcbpwI
vCebGHsaOEhHl3S3doQICRr2V02ec1B8+FsnrSEiHD5xcoCczkHjda5qef1E
SL0U7pF2m4MMNq8RlOklwqk3tobX/Dgo1THvZ2QXEW5VFBzVc+egmQfm+Xwd
RHAJ0GvTduEgk9GpYM82ImwaHvkReBLPz454686fRHjFnynzw5WDFq5qKFj+
R4SivXCw9QIHWb5t+/emkQgGXfZPRvw5KGdN8Fe1eiLk7O1/uyuCg5Zb0u7f
/0oEr6PLFt8kc5BN4ntcakSIX9prdz2fgwo6zhqH1RJBsd1pxqSSgwTl1xP/
fiCCVOHnPcL1HOTo8XTy9DsixHpIEAq7OKik1Ordf2+JoCGyIU53nIOEl/4m
GlUSwe2Z/peZv7he9yefr+ARYTd119Ht/zjo5V2kq/SSCPWb55nxCxwk1tC5
KaWcCCqvbQMF/nDQOULowLoyIhj/fdwSN8xBb5zlXwYUE2HAe01zfisHbc2t
ZY89JcJg6Mek3hoO8pq4cNKpgAimfzS0BYs56IP6xt1fnxDhm0Oo5OoUDiIG
lazRzSVCl9Cc3swNDrr83ubX02wiTPbwFsbOcNCX9f+eUh4Q4dfxDj9RS7x+
jqSHRt/D8R6x2uKqxUEBqXq2KzJwPMzEl00KHNTQ08vwSSWCYVTyKSqBg5QY
t5f1JREhy0k0skOEg0K8GQ1HEoiQuNa40WItB7VU1GW/jyXC9r76635rOGg7
v/dV9WgiWKS9VKSt56DbRhJmORwiVJ9NkvstzkGdnHKKZCTOp47g0AYyB6n/
d+xPxB3c34PKqUfKHMQmLvu4EI7n+88523JtDup3zUp1DyPCPZuLUkxTDkL5
Bl5tIUSoE6s37rPnoNiZQf1DwUTYEO1w4LsbB41qsbZAIBEKP+xYXO7NQfo3
t42oXCMCNfdKx5MrHJRcWw8ZV4iQ8saGp3eNg6Y2Xo4WvUyEmi8N2q64ng7a
SZ6+cYkI1lkDh1Zc4qDMzFcaU15EmFv7Xd8L53Nu4Pj6Ux5EcEpx9l2G++9e
fAad7kQoLlj766MRB33eKOjldJYIixZz3u67OKhMzo7SdhqPT+Lu1hYcb4ZW
Xr2dC66XQTCTEMb5sVgW+p8zjte31FVujo28XQ/vOnKcCOMlKyeF+9nI4erD
/noHIsR9rt+Y9YONDDlzCRb2RHA4ZvImoZaNVB+YHKyzwflc9/fD6bdsRKhI
XzA5QoT9L3skbICNVtZNPvl4mAh3cv7z8sYe797veMAC95/huMMZP988m7Dh
3SEibO7OcMz4yEZvhUde7zMhwl4kahXewEZ5lL0XK42IwBfyYbl2BxvF7Y6i
7jUkwpL+0S9NI2x03aS34aU+Ec7xRCq08PjPnFAP09DD/0vmLyit4KDDvnd2
P99LhC/Trdxf6zhI+86vgZ3aRLgpdm3z9EYOkstQTXqmgd8XeaJShetDtOSm
sao6rucex7bYTXj91/z4l7+TCPxh9peMNnBQ7y/FfIYaEVpub3ZZWMlBdVMB
TrnbiPB08O147ywbla/+JiKvjOvPo8C+bpCNsqRpbx4oEeHJviNKl/5jo7vb
L3tTFXA9nBO4FlHNRj6GH2mZskRgxzzsqShlI6dj0k0yNCJQtMK+aD1kIyMv
z1spZCJEPtlY3BrPRmphb9UliXg//c16anqXjaSTJYbipfB+27ZtOfkmGwkW
nkkWlyQC4cM8f+51NpqoemkSvZkI6ctj0WwgG/1s3rAkIk6Eh2G/SU3BON9j
zgWRG/H6dBpv/nQL51ug9Pg6ESIwBOImGBw2it8itDFCmAhkot5fySQ2CmYe
eyu4lggftmokp99no3N6BZdCVxNh63DqYHk+G1kf5ZcVWEWEdo0rJy6WsdFe
d+sfQQJEOBv23XP0FRspBD8KX1qG6zft5uLVSjaSY85emluUgcM2NYWPsOeP
pP2YnpWBkrXGrTo8NvocpK/xe0oG1nyIfmiD28vMHUoZHpOBlTkNJKM8nL8G
Dl//oAyIfDF5npPBRgeWdjl39ciAxnKhVv0oXI/ybVVt7TLQa0b+JRiC688i
RK75pwxcuBe3a8wbx39VIaKhUQYq76stm3bB8T6oG6n7KgPlCvvnJ2xxfHU+
ZrW1MkDzcjLINWcjNEcoevdOBqTvrAmIP8hGotQ3m95UysDjkIgXM4Zs1Gvi
dvnVSxlI7jcVLT+A68F3/c/nZTIwr/DLN9KEjVgZxVrFRTKgWXPya5clG534
aJdekCcDzo8sZ8bs2GjH9DL+x49kYNuWp4c/nMTzKZN96mGWDPCdNE+45M5G
LYam7zPTZMDz47PvS3j8ezNPKmxLlAFlvrxr/Vdwvc1fucOLloHiFScup1/D
71txRk0iZaDFYF4z4yobnc17aNYSLgOtN72o1b44v6tePT0Tgt+vEtljdgGv
5+P1YrMBMsDe0X31lzMbxZYP+oT5ycAGO4WLJ6zZaG7jsv82ecvA5ZUPTf4Z
4P3BXUIj67wMxH2r2z+kzkaV1YwUVTecr+TiE8cYbEQn7lsCZxnoM54XtKWw
0W0/2+OHHGRguV5kRzqBjUa/ebxpPSoDtmIahrFb2MhCKYx2zlIGRK9terZ7
KxuV3EwJmzPB8etuIxGl2WjLr6KBW4YyUHE9kBtOZaOru2sOSujJgKupXoK7
Ehu1c9qf3NeSgeDVHN9UNTbaNzSzXm23DHxScu7r1mCj7H3rvCpV8fyMxkrT
dNhobSql3owhA4pNS6p39NjI44/6zl+yuJ6ehJGyddmo3sws3p0sA17iR0Jc
tNlod47L3DxBBqRqGVYzO9gomf+a/W0JGZiyin4fqshGfMeiXm0WlQGG6m/t
UzJsdLLkEfHhWhmwklfe9VqUjd6th+Adq2RAZfN/xz+vxOvBrbH7zTIZKNru
0Ws5G4lYlcP7LRakgf3Fj+45GIkmJPkftc9Iw9PcF1WbOiOR9aUtay78loaD
oRH88e2RqPyzsvu/IWmoOn0r5HJPJJKW2/8lolcaKLnmk55jkSg4yH7b1g5p
uK9VuTljIRL1NntFZf+Uht7sTDOnFXh/UQuf3tkoDWLuQTVla/D6v5t2pKpO
GjocXpTrbcD13Vf83PIjfn/PmV2yePyX9tZKdlZJwxerg4c78P8/EjqveYA0
1AullR1dy0aak39/LZZLw9SOqvZfAmyUbrxe926xNHD5P3YfnolEAg9oWZIF
0lC6OzTnLB7/6SWNlTk50kAW/I9gVhOJao9anN59Xxqc2BPCFiWRSOXp6Zrq
NGl4+dunwudBJIpeE6hklSgNko06TduTI9HXjG4TDT1p4H/gfhecIhHV2OID
XVsaNG201DRsI9HlGd4+UXVp2LZEMmg8Gok+pjPg33Zp2OK5SWebPc7XwSSN
AaY0JJopnzdxjkSe04Kl9fLSMPvHKSb/XCR6m+ajClScz3RZXa5PJJIw6n6S
KyMN4inLtpEDI9GZKXP5uK3SUHs5e+xFSCR6mcrLCt4kDY1K4TvlQyPRhgMM
4vkN0vB15Urz18GRyHkyMclmjTQc2CuaGH4lEpWkCEror8T9H26pVz4fiVYb
+nBVluH+/rtPmnSIRPYTXesIC1JQUTJc4msaifKTzcNX/ZGCvhLuoTvakWi5
AU9g8rcUxMWtMhhQiURqwwkVOelScDmYsyiwxEJhsat2xSZJwanKGILDNAs1
7730NChWCkJ1HXr/G2MhxlAnw50jBQcnJ3++HWWh6zFmj47ekYJAw5Pny36z
0Hf0irovTArGa0V+Ff1hIfqgYrpysBQ0DYok78btX4lOkJS8JgXO7t8bA1dG
ok/aq+JWXpaCosBAbu+aSEQc8Bad8JKCG/qvJMnCkcg7qvNuq7sULHc/elpn
bSR6p2W2+sNpKfAVq7m7Db+/tf9lyDNnKXDsuDJisMBC7lxFvnQHKZB3X7Nl
BI8XNBP8I2yk4GLp7DdOJwtt7Fv5x+ewFAg2XOSON7CQC8fb68QhKVBga1DS
PrLQc43OERMjKfj6osDv81sWWtt7yE1dXwr2pNZ9EK9kIUf2y27qXikQeWVW
d+wNCz3do+i0QUMK8j8yvH6+Y6EVPfE/53dIgev2VCPNzyx0NHLlkT4VKWCK
B1m11LPQY3Xvb98UpWD+zFz/+mYWWurqMHlFlwK/NbzD51tYyIJ16MMjEs73
L1MJkZ8s9GD3y30xBCkgByieeoHHO9upANclpMCzsipuoJaFjO/Ga5wTlYJl
nUZ2ha9ZKG3XytIj66TgY7jR9oRiFprouKiqJygFhRLlsiY5LKR/p+MJk18K
Hs9ny1pksFD8zkPyWxcJ4GERpsZLYiH/DYys+L8EeAxohWgiCzkMCslITBIg
K2XenpPMQrpv+xNiRgjwruDG3Xn8Pi21WkysnwDauXIiF7JZSPByViS3kwA/
qz551D9hoWHzYCGRVgIsLPxmHS5goS+KTjcjmwiQu/yVyus8nL8V2nzC37Ct
bxv+wO/H/pL0v1NLAJ/+rVct0ljI7/nstNA7AkxPXhF9HsVC9lFNHuGvCRAT
YW2w9hYLnxeKh1a9IECzotJy0wAWohhEuYSWEOAjWTh7zWUWWkny7BAoJMBt
7XvWy31Y+L5oan8jlwA2mvXKk74s9KleqWnZAwKs9K49/fQqCxXkCVlcTyeA
uMd9pfPBLBR9q792MZEAqzNOSEuFsZDviWqDazEEkJSMZ83h/m01syrnIwng
cMjp/UQoC2mLB2tduU0A899paj3XWYg07lj2N4QAkdsGe3NxfwI1Wtt9Awnw
qiRErPccC/Xfk8yb9iMAfcfUKu4JFvp4bVbO25sADzsHt0faslDekaZ7E+cJ
cOpunJOsNQtxtxVLe7oR4GJF2pMXVix0aU1UwpgzARSKak0Kj+J66/EQO+9A
AIr0Y4g8xkKaPNPI4aMEONh/7qcbbl8mQUnorCUB9JOuUY6cYqHlF4VuDpgQ
YK5XbcXESRbqMe5fcjUkQNj6J3zPnFjoPb36Sq8uAd68uGG42QbXL1/W9Ekt
AtxPP/rwyiEWimwO8ujaRYCrH0L6X+izkNczx6HjqgRYYVEtWq3FQtYsLZd2
JQIILXtjsEydhdRPS3Y4yBIgeeeFwqVdLETQnbVrJRHAWpq//eAeFuIjNDXa
EQiwRvHtd5Y2C3VNPzNvFv9fxXUeD9UXBQDcTnaDGTMTqWyFdonUE0WkQiUi
SyJJEUJItpBmiLIkSSkhZUmyxDslS5ZSRKmkHyJLJPvW7/TffD/nLnPveffc
95bCVW/j/N96MUTN2/hGS4mlsM13/Ww8zpeT7WHYLrQUEn+77OFBs8P2vTzE
txTWSZjK/NWNITyOqm1r5VgKTzPGK7Jx/oOagiXmc0wYofTala2LITQl+je8
m2BCalyw9bhyDMEYrH60f5QJB/fUccovjyEWX2WovBlgwt3R4arTsjHE97SQ
DJNeJuTOmD86gX7lZyfX8I0Jo6LpCUz5GCLLfNsNow4m/NjZ/XqtQgxxRY0p
VdeK4z3R2F2B45/hm4kxeMuEyjBO004VPO9dbUuqXzPh5RX9GxwY1ygrCtd/
xQS39RvS23F+mevxf19UMuG3IGvfNXoMMX/aw1+3lAnTKe0ueRIxxDfDfROV
T5jg2v9T4qpQDPFyuZrHtsdM6HIszPzMjfVjbslgeRYTolLrNI9xxRCXP/Q5
aWcw8Xvx0PI3IjGEW151V8ktXN+WkyPC0jGEtwI3zSyZCfZiUgk0jAfe0N37
M54Jf5tleutEY4hw0aCwEDYT+l0/KjwQ+bff5aX0KCZsyrNbOLMEz+/09EhB
KBNqvh7/9RbnSzu9Wcn4Au5HypevGnNsIvM/L5v/fJlg6OJtaznOxue7IN7f
kwnLf1pyF46wieLGX3WU00yIl2w2EPzFJip3qP3NOcEEZZ7hbZoYryk+qaF/
jAlqaQnmqmNsrCcPTn22YcJEN8VqzwSb+Jjec8frMBMSw3dTxibZRJf0io9C
5kxoEaIen8J4f7Sd6D0TJng4512nYP/Rv6k7dQyZQM6saxweZBPT3h3+rTuY
cJpryLCrh01wDNAK3HSYcC+CPnOnk00I2B3q49HE/BCXL3N2sAmJ1njZW+uZ
0FFsZ+T4jU0wjJoPaKgx4c7n85K5GF9RKRLdpMSERzbRqY+/sonVG/eA03Im
aIoHL+ztYhMbsqImF5j4f7yuGa/7zia0ZWvUEqlMyImYO+OC4+nFczuukWDC
0TUdQxM4njH/jhs1Qkx4svp02KtWNnEgMOitLR8TTBp2D1u9YRM2v8t5pziY
4F76+Mne12ziuPPM1tg5BsjINJuV1bAJt8+bPZUnGZAZ/X7sE9rb1DuLHGVA
b1iNxVgdmwisLug8PMiAfNOSkqQGNhGuPSI12suAA7d9gtc2sQlWntqeqC4G
6GnqbA1vZBMJCq4h8p8ZsO+c9SkNnC/txoNnJR8Y8LcUVDuqMN+ivcOmzQxw
1ok0el2B+Q5bofCzngEKO3QyUkow39N2R0KqGbBThUPi6lPM9+lbV+nAAI+A
nW2y6Jr/OmoKyhjg/uNZGlnMJt4cllkwesoAxe/HPq8uZRPtjYc2/pfHgF87
hYnGMsz3jmsn/XMY8F5aZpcDur+4+TblPgMM1X+aeD3DfKuKtuXcZsB++e2F
4YWY7/Q9wvopDKj8+7L5ay7mm3pZ7/N1BlRnag0LZ2G+r9T4ecUy4LvScnfj
+2xCnIMnTyiaAQ3Hn9ou3mMT9HM7ejPCGRC8ReZ9UibmeyCIqXMR19/ySboW
+6+2e27Wep4Bxpypqjw5mO/WmUg3bwZUbbq3Uy0b822kWcnjzoA4noGOSRxf
r9J7PPUkA5IWvGMpdzDfGwtXaxxngM2X/Xu+3mQT5lkj9k22DOj21DVqTWIT
R2TVk5ysGBB9WrR+MoFNHIt3bVo4wIBU8ut1m0Q24cqfxZ24jwGqYVzFasls
wjOwV2uNEQPuVFT2Td9gEwG/V3jU6DPAMqbxsGUKxp89I8u3M0A3zaW8ENu7
XjARK9BiwI8iqQea/8bX/26buYkBfT6Hdz+5ivMv8Xl8cy0DDm7exHWAhf/v
reDi1dUMOJ/9uFPjMpswSri9N0KRAW0+bR6LkWxC13rTrQB5BqyIzlBPiGIT
W5a/HvJgMuB39uYf66PZxLq+ozrOVAbMDuVEZF9hE8qPxq5YSzAgTLs77D+0
nFfkZ1Nh/L8NeiZ62J6qtVTVgB/3TzyhrjeCTYj+zfffysUAyuoTJexQNsFb
vat+3QId+NaeHbAPYhPz0R10pWk6xHgUBoQHsIlxU/eTzD90yLP4FH/Rn00M
UXlKxX/RwSFx/ZQ0uvtLsgDfTzqwJBKiBS6wiY676pZz3XTY+cNVVymYTbx3
eflgtJMOwd2aZ3acZRP1aw5P9X6iwwrP9Y5bAtnEi/FBg8+tOP91E1qhH5so
KQtObH5LB2pFwo973mwiP1j6R3U9HQZdOc9pYf8sgxyN8mo6LMnfbHPQnU3c
FiYu5QMd6mLNEo3Rie9bWu+X00FLIO2FmQebiEl2UbhZTIcm0xLrfOwfYbvg
dbWADluddv1R82QTFxTiqy7l0iHr/YnDwhj3HlCSDHhAB5EF1mHDM1gP8suP
edylw8T+W8fZrlgvfEwLnW7R4fjQj6YpZzZhrdPLaZ1Mh30L16JjjmN+ufzN
TK/RwW6gz6LVEZ/POtE7u2Lo8JphYcHGuF5Mxqj2ZTo8TaUqdjixCa2DW3TX
hdPhXaXCx5s43jpGU6ziRdzvL3154xhX6XL4xvCng01UBPkQx5PPnFwjfg73
Vx22G9izCZrblSBeDzrwy5qfrbfB/G6QfzPrSoeplKN+hkfYBN90keyoEx32
Ziypt7RiE4sVRqd77XF87tqqTvREWOfzDms6bCRL20SsMb9GXsLNFnQoM6Z1
NqF7xARsqs3oMNq99vI29JcPqQ/LTOhQmPxedyn2b7m5fi7PkA5b5Bz1+w9h
fh1qjO/r0UEsTZ9+15xNvFS2TknZRoe5KalHZqZsomx45GfsFjp8WWFwPXI/
myh4Eq51aSMdEobG7tIwnnWeftl/DT5/3pFf15ixiXTi8Uf3VXRgjiiVlqGT
efVVnBToUH0ponAE28c2tPseWUaHfGMn7Uf7ML9xbrX7GXSw8LA0Z+9hE0GH
OWm7pOnQP1x5IWk3m/CRTXTWFqdDJ1v123MDNnG6e3XxWiE60Jyoxnbo49kk
ryIfHTj2OR2WN8T7wv3gIQYnHbTtl7+WwP49x63u3Z2TgQ8mc9wr0G5HbP+s
npSBnX6vIRL7j+931HsyKgNfT2xkxutj/djlErd1UAbijSeGHHTZBNfW011V
vTJgXjQ7rbWdTUSv81xr0iUDMz5K9LJtbIKi5BvU2oF2Odp5AeMpzMAmmw8y
IN3qt9+awPu7NmPX7WwZcMgT6lmpySYOyv/RUDCXgRdaRHu3EuZHejo810QG
ZstbPa4p4HqEFlo2GcrAVUmXiIXlmF8OrpUVO2TgnJzD9B55NuE1yee5S0cG
uHTur+FBzw4KvWjaLAO6zgqrs9Ch38XFLdbLwPbAe5sp2F+wXdquU1UGVmjn
yxtjPL6R8dhZSQZYM0HJWnJ4v79ctvBLXgZEd3hFlzPZxN1nCia+TBl4KHuT
OUvHev9o1c2/0jIwrBCkGo8uuLtmIFJcBiKn339fxsB6l7xRS1xIBi5bfNx6
BQ3sLVHJvOiA9Su+YPvdYdva5TlkQKbN5MkojU00++kpZc/SQHe3i/AVaTZh
ecbw3PoJGrzhWvH2tiTed44mr0pHaIAfSQ9PUtiEi5WZpN4ADXY5zGZdwfjI
Potj9T00UDe/f6of7bPTusD8Gw2aflKfs9GLWvYcnz/R4LniYEWiBD5fa532
O7bSYCCshXFUDM+bomva4BsaSN7U/jUlgvWH4T7s9ZoG7Q82CrWiZcW9dear
aPA7VuTrUVE2cY/3/JXwShp8vDBf74dWm7vQIVxKg6XySYpO2L5oNHRVwhMa
pP3xX2ItxCZ0fkT6yT6mgdbSyfvWS9jEq8+s2vtZNEisdBo+KsAm9ryLo67J
oIG55KOQMPRL2q8gyTQarDQSH51Ga9sa900n08Dupa36B3TBvcz9nddo0BIt
8uIBP9aXQa6SqhgazJkHi9bwYj1dbyeffZkGBfq22kI8eH/4lUfFhNOAmF7Y
uYmbTbArab+9LtIgdelatjmah9fbysqfBrJaaxxIdMCe5hfbz9HgYd2TVyno
sTi11QoeNMg98eD7OS42cfJjVPySUzTI9EvSecCB+ZHrnf3lRIP1yZVx/H9Z
xGGnHY6t9jRwqZ/YZrzIIt48vNVQak2DMLunZz3Ru8ZmNt62oMH2EyXH3qGf
b7FIDTejwZGJ5JxY9MaLhTyuJjQQ+yGTZ7HAInKqRU/vN6SBgsjxkOtzLGK5
8KkPm/RokJ5QUrdylkUkm9duY2zD/x94ROjWDIsQu7Ey868mDYasiuu+oCO+
XRTt3UADz+jGenlsv6D4xadeHfdbk953H+Peblu+5anQoLPkzVGDaRYxWHjd
MGEl5qsmZChpkkUcmxnN85ejgc9IeKHHBIv4ROyVsafT4AYrRWkz2jQiO3iX
FA04yP6Uy+jaRt6fq8VowDkadyMJvV3ymJm4IA0mW1Nb6sZZxFOrytIJHtwv
qbUTO/6wCLV0xorPf6kgW3jOofo3i8j44RMNs1QQE7uqPTfKIhjqLWP3J6gQ
yj+z6Qs6zmut9ZVRKpidd7rjju0Fyq5UeQxSoY6zKOIS+iJHv6rFD+xv5UbG
jLCISYOd17d+p8LgadH71r9YxGl2+rz8Fyr0jyQc2DbMInpa5o/ztVPh4DFB
n7khFmHNsGoafEcFg8i2kB0Yf2//VONdIxVC+KzW70QbPZBIK66lQltY1XUf
bA/Dp/lSX1KhduWdJP5BFqG5qf5MSAUVdpwOXfHsJ4t47K/U7lxCBerryhsq
aMUXoYTJEyqUPJeW2Y9O5f/2YP1jKpQuvuw7j5bct1Wclk2Fgl9nMyf7WUT0
9SS/+QwqvM1R5nraxyI4P//p+p5GhWKGoOShHyzCb7mpUe0NKmgl9u6M72UR
IydyC3KvU+HmZZdX19HOjwUY8bFU2E0pi21Efx0/HuobTYXlD4djzqMPbn0x
YHOJCn+aumUcelhEQ4jsAb1gKvB+ubnA080i9OrOlysHUOHaoAKv8n8solS0
baWID+6n8AN1MfS6QxtYYx5UyAxUnDVAP7gZM95+igrO2rMgjJb7b8CmwpkK
Pyq+j4t9ZxEJKobVdx2oYC3to05+YxHC7hnqUTZU8LtnONnRySKyTO3MIg9T
YY+W3YZn6J0bmOcizHF9gXpBA+guyfbkS3upYFMRlEeiAyfin4fvxvYrOU91
fmURMu37usL0qWDvZqqc/oVFFJUI8oRtp8KyPxphJZ/xeU2pUQ7VokKZIXfm
JfRQQOiekE1U6HsYe6QIHXV0u3vwWly/wHaIRysQs/EXV1Ph5NuAm9CB+ZYv
Lg5SpMKYsOPVmE8swobLs+OCPBXoQzXfcj+yiOlu9cVAJhXEV8ns9Udfr/65
PJBKhcmrU2L30ese3N8VIEEF27VrDgehG6IcTvoLU0E1eJt5bjvm++6y+27P
peGNZyysa2YRSuGf6049kwarJvOo7Lcs4qVT0pBroTTk2Cd4Z79hEbaGB8Rd
H0lD5c3WzY1NLGJWRWzTySxpiCxZPn8JnSjYcNglQxpazsHoHfSGoYiAE2nS
YHKMf08Q+k2T3m3nG9JAux7KXdXIIlzzFl86XZcGuz5OoeoGFsEXV/bjeKw0
+EumjCig73r6CB6PlgahLnt9Gnr7wQ1rHC9Jg0qPRZcKukPjl9mxYGkY+JG2
/1U9i/Ch5ZxzCJCGvkMKTZ2vWQRlxumGvY80TOV9p/GgH3csr7A7Kw2nzmhl
Ftbh+Xr+tcvWDedfuSysFi3QdnOp2VJpuD6waLHhJYvY1FdLdFOlIY9y7psg
2n76z7FzEtLwrE2ryeQFi2AtkY/gF5YGTu4/ep7AIkoYJtk3+HB/Zh+2yKF7
VP0aVTml4bX9MWV1tPi2eyMVc1Jgcjm9qY9kETr7mimmk1KwckFBlY4+aTev
8d+oFAzHCUhYV+J+eqhYeQ9KQbqtaoQ6+mXIwUC+H1Igyq9E00D/ig++ndwl
BYnuo461FVi/7uW+XP1ZCqpS1edGnrMIg6cfe59/kIKGLVuOH0V71vAs2d8s
BSxZ4Z/m6Pp+m/1e1VKQJNptllmO65mTDU04IwXPQgoMzj3F9Qgb31c+KQVm
K+6Nu6J7ZH3qSh2lYHals+BAEa5n7d3BPbZS0N7g9GkjWkf3jWinpRQ4Coyf
7HnCIobDg24q6UvB6VGD/WcKWcQ+0W9Fz2hSkLMq6OerPBYhsTy97xNIAql9
aKYrB+tz2EqF1+WSsL1vmOqK3t2baV9SLAlF+9armqIdDVffelAgCTolVX+2
ooOyH31KzJWE2KoQ46FsPF/ryxsOJEuC6tF+WnMWi6jTa8trcpeEg6317OJM
bH9c2O+FnCQ0Lsq5pdxlESk1sUX5dEnoP6JDTGTgfaEi+fu2lCRocP5c+eMO
i2iOTlSPFZOErG75iafooSG6a5CgJHwIlR59iBbYfyvzNK8kmF1u/uOAXlkg
323DIQmcDWtq/6T/u4/uLTOZowBfyBWNGvSRc8o2WycpsJsjW6Qc7dOek7z6
NwX6Y2Jt3NFxWms+0IcoEONyTZsXnXuzQGJJHwXSZu5Rv97G9Sxs2jf9nQJR
Xl1X3qK77Uqi+75QQFIlLPYievHF1tq2dgqUJFpeZaAZCiR3zXsKOEdcD3qR
xiI0IvR0nzZRoGK4dQfkYn3qrw68V0eB4y+4vOIx7mZsVHqtigJ2KkbLN6Mj
cxsnQispsPCXaSmHvitqusGzlAKpvHzFPOhKj5YzDkUUsNryRzvrFt7X7y0e
muZR4Eq7UfZh9Pimjj4ihwJFccdVd6PFk44qrL1PgcfFYtOpaLWZLnu5dApI
v+KxVlVhEYbWx2+J3KRAds3Al+BUfB+o6Ps0n0CB70MBbefQQctOUYeuUoBT
cH7fDnRKyC/zz1coEKCx61LXTcxf99nY+ggKtDQfDCxEN++aaCgNocC3W9K1
99FDD/wEsgMpUPZkp6Q9OlnLbaeqLwV6X509wYXWb7ALfnSWApd5Myd+puD5
sjnwfK0bBfTCU413YDzll8F0gTMF3C+0jxzCuEGw9qZNDhTwV+/cthY9JrHG
o9iaAgPSPUw5dFrG8twtFhS44L2wruEG1hsN6f4yUwo8231MPQg9XiOgsG0P
7if5ars7Ot1y3o7cRYG2olMnNqNNBkZu7tClgLjd+sCOZKz3Ad3tVdoUkJsI
kaxEZ74sqG1UoEDHhT+9i0lYn74ORIw2SkBEfKHa1PV/913nS89aCbhWsPjY
F/2M4/3i+AsJUBsa/6DPwv2Nr9b2fS4BK9yEd+3AuKhCqc9MsQTcPvjtDhe6
7GluYUCBBFgWRSfANbzfDdN/LTyUALnD4uP5aMqna6uDMyVgS52x+Xl0hWuk
M9cdCejZlKq5Ce0y7383/KYEJLt7q69ES8Wc6eRLlIDAgx1iv+Pxflt2jHH5
qgQwm9SKctBuBYcshK5IAJdJ3tZkNGN6pYHUOQkwq+Pf3BPHIlopR3m3yEpA
Eq2cWyWWRbhPPDhCk5EATxedHjpa8ONY3iRFAmxq+Sj9MSziftk23jYRCejO
ypMrRuveijryVEACaF2NYw/Qny+25F3nloDdvplp59E+x+R4vRfFYUXV4lkD
tMSuk0cOzIjD6jGnKU10rnJR3oZxceh1V1oujA7TETqi3iAO35716o7jfm5w
Ln284rw46OcHeNVGs4im3Tw8XN7i8KgDDPLQLqr7rb6fEYeIPMkNSWhu0ZTH
cFIctAQac53QaaM93OnHxSEhV2TGHL0kQNxK1lQcXqvd9n9/Gd9HYk9wyyiL
w1MVIqo7Ct93vFTzGTlikJBye44Wif3PBL8TTheDP8vOvB+JwLjrh7HFBDFw
f+F3qAud5rRa6vcVMdAMOxD4Ei3rcFGjO0QMTvXQre+gU21aLT74ioGRcid5
A73UcpVf7WkxeHTquLD6v/iBoBuljmJg49RePIbzLd3fUvbQSgzGT7eY23th
3Fjly639YvApaVJJFtszDS4sxO4SAy1tBwkxdMqO93KhW8XAah1//cglzO82
ZV3v9WLAWbV/4j06ZUugg7OyGJTGvXld9S++6V2opawYxPToHkj7F1+rdM9Y
UgzCLuSvZKPpqgHVOkvEoC94cdELLdVd32HrIgp1Wl+eDYfh+1DnsnlTW1FI
W3nWb3kgxju8ZfUPioLUzuWHqq9g/MPr7RrGovC0skIqF9tLvpOzV9YVBUke
0WPJaArp2TtCEwXXNSU829BiNxmjwbUisGQ6/frVUHzeDrjyZiiJgFwQb6hR
CH4/PbqtWM4UAbbH/qvKaGOBD7taxEVAeT4/dm8Qixg4Jug8yCsCfsrzxzkx
Hl1BRHDPCcM+7kbbsWAWIaQizX/8rTAQyY86b6GFF0gBRT9hcLC7oz53Ec/n
A2nhB6+FQPj+ld43ON5jzj1qZKUQLFuxyrUMvc8m2KT9iRDwhZV4PkTHSAyy
+NOEYG93yOEotFggiLh4CkFVUsdbPbS42SmxVUwhOJ32I6LkAj7vc0B56CYI
dkJJX8pxv65O3XO6cEwQmI/3KuSgRcejSvZbCkJjhY76bbTwsKntuL4g3Fvn
fi0UzfetK3sbUxAGAva3GaHnX3LsaH69BDz7+IQ6A1jEj2hd9wmlJWB1wbdC
Eu0UqfCydukSeLNTQE4Q3RMmIJ1CWQK+mmtd5/1ZxPfA5rLtiwIQdOxZZS/6
yxl7vqgPAnBSc0dbMfqdefAtRrgANBDJQtbo5wz8+Z0fWmymqI/O4/eVT7js
ljZ+iNwukZSBnn23231dAz/42TVoJ6O3XH5HWf6UHyrCffaEooum/rPiuswP
6s+UZy3Qj1p5+1+t4wfDXdRLnGiTdfVaFYr8MJftHTXlh/fJlZgrTxn8AGr8
Ur/Qq/VpazN5+OHDFe3GT+jMAhWfyI98IL9y1qUQnRazh8c4mA8Cy06/sUdv
GxCz0DvHB4meBkZH0F92tT7QduUD3aVjVWbogDvJMxvs+OBXH0vYCM1YsNmj
epAPNqX+bd2OLrVcfmulER+88N3QWuHLIiyLen8xt/PBX6scEXWMT4vl6Ept
RP/JUFVAJ506Ey+swgdVR2yNhdGbazf08MjygWqcx4w4um3FlMaCBB94OrpP
CaDPBZVHTvDxQUeMjfyJmH/n5+Kn4TleKLLnMp3E+Z5o7FT9McoLU/zbTw6j
zeMELnT28oJF19mdaejfQ41v2jp44XH7K+cCPJ9Xd8fJv33LC/U83FbSJ1nE
2nuHPGtf8UJDvXREPbZ/85f+iizlBa9aVvlL9GnrTumSx7wQ0DN+MhLzK/zs
7on8DF54ovLOoADjDyknSrOSeYGnq8EuC218RlXoDpsXHFICTv2b/+frEZsb
obzgMSr1KhkdpVj0OM6XF5aRfkuuopVD/Dii3XihrLLSOd2TRdR80TEPdeAF
t/VtPBcx7rSF856/BS+soNhe9UXzXK+e8NzDCwcvP3Z7coJFZIxcNjylywsJ
QZz1jhjX27PvhqMGL6xaMSjjjPfBf5mUQevVvHBF5uafU8YsIoSrXefgMl5I
DTAqPYvrcb1+hDdYihdy37nGe/vg/il1Nj1cwgt6/gGfo/H8by1xSGxf5IH/
jo9I3D+I34fGvbbc4zzwiEPssjbOJ/LFRXntTx4ovab0WgU9eXpo5EgnD7xR
MaiTR3/7614S0cIDibP+LTLourg/wYV1PPAzx8pbAl2w0teos4IHViioxwui
U57OSgg+wfmkSiy50W6fODMcbvHAKK2x9A/+P/Hl4tXinjzAiI4xbEfPFF5j
65zgAcGR99lv0f/tpFm42PBAucKTS3XohrYUuetmPCC5ru/rC3SRi1wfacAD
PKuT15Wjb83eyRvcygMc0kXSRegIlqIfbT0PvF/TlZeLdpfL1tVX4oGK+i0B
99CW+WpL3Jk8sGpfgMQ19A69/Hcp4jyg2OvbkYZe3boxpYaXBwQ0fcvi0JLO
z46NzXLDsElkgDN6fkpbVW6UG2b92yvD0L2XK/8Y9XLDF/WE/ED0G6be83Md
3GBdfdjLHV38qDr8zltuENUImM5C3yaM9ja94oZfekcvnkRHvWuUninlhvDr
ld+PoT0cTTsV8rjhsj5d2AatF3nYPfAGN1Ss+vJkH/rHG6c41TPccGelRJMG
+q39T6vDjv/mX9Bbiy4Zc1sRZskNXL/+JKugo6neTzr0uEEt6sT4UrRX1lQA
3xZuiLsmMiuFttEO2LlBnRti3Ql1EfSuxkVh2xXcsMOb84EQeo1t6IfLNG4Y
8zxiyIOmjfKmPRXmhnceLeX6Z9kER+hl5++c3LDsmvD7qXN4XiRF1opMcUGn
rPyT3+j3969ObRnigr9jerfkRVhEuaYUHP/OBcMJCv1bbl8h7r1OirraxgWK
FmN+Oh9YBNuaafa8gQseLeEVGVbH96nhNHo/cMHrkH2trTie3cUV/0kWc8HU
/mv8b9G7JTJziIdckPiplvoavT5jldepdC7YULP7qBeaofFoa1ICF9gHVC59
hvWIu3YdT1U0tnfKkDHH+JBlUeOvi1ywRWp7/yZcX+uAZgLjHBeMqZs55mK8
IrD8qIErrqdOc1QZv9/D84ct1ey5gGez3sgdjJv0LDtIseCCc6O7jjxGS8mY
75/ewwX0/hnNBPTnPeHGnTu4IPVCLTseffdi8a5XmlzQ7DSzk412fdKvm6PO
BTd+9tOi0DOMvZo+dC7Yrei2LQgN+y5usBHjguQ3y86eR0eFFqjr8XKB48of
yd5o2oC0guhvTnCUXXrbFa1R2imeWccJlbtdpi3Q80PiwqxKTri07orpv/W+
ktfn9yzihOPuz/L3og9EPVjYls4J/jo2h3eizx72GPjgywkTPnOm69GPJjhf
8Spzgu2x94EiaO9Vm8jBpZzw4P1GgSVonaPOZe8onPAyzyueB93wqj7/1iIH
UM225c954/Nw7dotjTYOiHj41XwArbhB0dfpEge8TUpUeo3W+nr1YHTtX5Iz
2kYlGi3AXbz7bsFfMmrBVzQC/VHls07Zzb/kRjfnXyFoP29FxQH3v2Rz8pO8
8+gSobIJI5m/5O6rq36cRG/W7k5ccnKR/J42y9iD3pik8SlKcIH8OiilJoHm
qjjSlD4+T0p9sAQR9Pv/Lr4o6ZwnGwb/nhFEe66pz+4vnCeTaMJJ3OjCalv/
3dbzJNtBQn8S30/XjUcyBXLnSKNMS6MvaHWzjqORe2dJ+aWf2Q/RZWU2zyS1
ZskjP1RfZKENFL6JpyvMkr/SXolmou2nuqtK5mbIeXp2czr6+q3hVQPZM+RH
HjHdRPT8T45JE74ZUp4/93kIuiFUOYZCTpFzOtJ/j6Ath7L70nKmyEKHh2CJ
7jmktkM1cYpcLI70s0Avqqwf1z89RW70mv1qht7wVvvIOeYU2TEl07cbncLc
p/TRd5L80yJ/ShN9ssi78ta6CfL3trIAKjo6+hxv3NIJ0sq+7YIUOsfexyRc
YIJUaIvPoKCHhP06XLvGyVfXMutE0WecAiY1r46TCWeUt/Kjvaiha96P/CG9
8pZmT+N9eMEvNo0vb4xU3S8t+BWdtu9q70zKGPkymPvsZ3SlQpzacMQY+far
QOUnNMe7+LIW2zFyEd/G2tChqxLb0kXHyM+/TUSa0ZEdqaJbz/wmB1vWJ1eh
43RygtzVR8l3Qb6zD9FjzaeqHtNGSUcTMe8c9EEndYFfnKNkaqTjeBZ6zVKH
lZ9yR0i3y7V999Ha5GmjlsVfZI1J6vRttBlvZELt3WGSwyzd5To6OK5UPX9g
gHyRbvEyCK1fpvNsDTlACsyv8b2A5usmdR9dGyA1vlxcE4hmbao5kKMzQJrp
emecRye1vz9/L+YnKb1pPN0b/Vh2qDp5Qz+52YLsdUV/zZazDfHvJZcGR6VZ
ohlHbAr7zXpJ+Y+KNw6jLQVT+ExX9ZJVG9USLNDvXaXz5D72kM88z8cd/Pe+
oSrM8VyjhzQ575piis5/NHNncvQ/cqA35OdudFhBa++pE13k4sIjV200OFC0
W7Z3kekpOmwt9KKEaYw2tYss1prN24Ie7Uk8TYn+RiobyU5uRn9yDhfbu6uT
NKs8lbgRnetqd+Bl+WeyrMeKW/3ffnlKf87NaiOPbfyaIf9v/h/q6oL72sgg
D6XmZWhRa4OLJ/58IDuq7Ofl0PX6vitXbP9Axv/gPyKL3in96VRSSwu54YG/
EgOtWZI6H7LYTCoecxyVRGvNG+o8zqohT0TI7xNAT/b4dbn8qSZZZP9NfnRh
U3b4yu3VZC4n7xAfWu22UFNySxXZLKoXy4tepvf2aNgikAeSYr5yoQXFHy3m
ZxWTg8KLWQtnsZ5/FzVU2v6UfFfcx/fPC4UesTdbnpDi4eVO8+jRQ5vkIxbz
yUv+nxXn0LmavmHFWZkko+Nd3jT6doe21J2WDPIL6Sr1z/EXFu9dWUwnmzm6
zk/96x8hxH6RlUhOHrQxnES3NcY/Tly8SmYvsc2bQPsqbkpvz4ogx4LXy/xz
xLFLNdzZPmTD1+aQcbSEUWSaimRipYro5qE/6P8BqA7heg==
          "]]}, Annotation[#, "Charting`Private`Tag$6132#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.880722, 0.611041, 0.142051], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwV2Hc8lV8YAHAhe+917e0aaRdeUXZWi5KUkUgKIRqUXyqSUVKySkJ2KiPu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          "]]}, Annotation[#, "Charting`Private`Tag$6132#2"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.560181, 0.691569, 0.194885], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwV13k8VU8UAHD7Xva97MvjvedJiIpbSNZCmwpRKaSU0KJEKWmxliSRSJI9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          "]]}, Annotation[#, "Charting`Private`Tag$6132#3"]& ]}}, {}, {}}, {
    Ticks -> {
       Charting`ScaledTicks[{Log, Exp}], Automatic}, GridLines -> {
       Charting`ScaledTickValues[{Log, Exp}], Automatic}, 
     FrameTicks -> {{Automatic, Automatic}, {
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, DisplayFunction -> 
     Identity, DisplayFunction -> Identity, Ticks -> {Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , Automatic}, 
     AxesOrigin -> {-16.11809532201759, 0}, FrameTicks -> {{Automatic, 
        Charting`ScaledFrameTicks[{Identity, Identity}]}, {Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, 
     GridLines -> {{-16.11809565095832, -11.512925464970229`, \
-2.3025850929940455`}, Automatic}, DisplayFunction -> Identity, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {0, 0}}, PlotRangeClipping -> True, ImagePadding -> 
     All, DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None, 
       "ClippingRange" -> {{{-16.11809532201759, -3.2894072757057793`*^-7}, \
{-0.02855940123897063, 
        0.9999999999863621}}, {{-16.11809532201759, \
-3.2894072757057793`*^-7}, {-0.02855940123897063, 0.9999999999863621}}}}, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, 
     BaseStyle -> (FontSize -> 12), 
     CoordinatesToolOptions -> {"DisplayFunction" -> ({
         Exp[
          Part[#, 1]], 
         Part[#, 2]}& ), "CopiedValueFunction" -> ({
         Exp[
          Part[#, 1]], 
         Part[#, 2]}& )}, DisplayFunction :> Identity, Epilog -> {
       GrayLevel[0], 
       Dashing[{Small, Small}], 
       LineBox[
        NCache[{{-Log[
             Rational[979917, 83]], -Log[1000000]}, {-Log[
             Rational[979917, 83]], 
           Log[
           1000000]}}, {{-9.376382545385876, -13.815510557964274`}, \
{-9.376382545385876, 13.815510557964274`}}]]}, 
     Frame -> {{True, True}, {True, True}}, FrameLabel -> {{
        FormBox[
        "\"\[Phi](a)/\\!\\(\\*SubscriptBox[\\(\[Phi]\\), \\(i\\)]\\)\"", 
         TraditionalForm], None}, {
        FormBox["\"a\"", TraditionalForm], None}}, FrameStyle -> GrayLevel[0],
      FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {Automatic, Automatic}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None}, 
     PlotRange -> {{-16.11809532201759, 0.}, {-0.1, 1.1}}, PlotRangeClipping -> 
     True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {Automatic, Automatic}}, 
     Ticks -> {Automatic, Automatic}}],FormBox[
    FormBox[
     TemplateBox[{"\"k=2.00 h/Mpc\"", "\"k=0.10 h/Mpc\"", "\"k=0.03 h/Mpc\""},
       "LineLegend", DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontSize -> 12, 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.24561133333333335`, 0.3378526666666667, 
                    0.4731986666666667], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "RGBColor[0.368417, 0.506779, 0.709798]"], Appearance -> 
                    None, BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.368417, 0.506779, 0.709798]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.368417, 0.506779, 0.709798], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.587148, 0.40736066666666665`, 0.09470066666666668], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "RGBColor[0.880722, 0.611041, 0.142051]"], Appearance -> 
                    None, BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.880722, 0.611041, 0.142051]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.880722, 0.611041, 0.142051], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontSize", "\[Rule]", "12"}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.37345400000000006`, 0.461046, 0.12992333333333334`], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    "RGBColor[0.560181, 0.691569, 0.194885]"], Appearance -> 
                    None, BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.560181, 0.691569, 0.194885]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.560181, 0.691569, 0.194885], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}]}],
              "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2, ",", #3}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{{3.762516888390074*^9, 3.762516903070718*^9}, 
   3.762516960795412*^9, {3.7625171180054917`*^9, 3.7625171498777895`*^9}, {
   3.7670005204798765`*^9, 3.767000615576269*^9}, {3.7670006532748256`*^9, 
   3.7670006954236865`*^9}, 3.767001170557514*^9, 3.7670012317129374`*^9, 
   3.7670017480307236`*^9, {3.7670034230535975`*^9, 3.7670034530829163`*^9}, 
   3.767003499376281*^9, 3.7670035413111963`*^9, 3.7670035792949905`*^9, {
   3.7670036953937397`*^9, 3.767003700561793*^9}, 3.7670037641230774`*^9, {
   3.76700382285056*^9, 3.7670038419693527`*^9}, {3.7670039565293765`*^9, 
   3.767003973557585*^9}, {3.7670041524422994`*^9, 3.7670041979513736`*^9}, 
   3.7670043983575234`*^9, {3.767004772681619*^9, 3.7670048095278296`*^9}, {
   3.767004870628583*^9, 3.7670048833715267`*^9}, {3.767004980671635*^9, 
   3.7670050009992943`*^9}, {3.7670052425353823`*^9, 3.7670052672290325`*^9}, 
   3.7670702444643064`*^9, 3.767347503540399*^9, {3.796457387202384*^9, 
   3.796457391347041*^9}},ExpressionUUID->"333a80bf-2b33-4106-94a0-\
0fac108c40a6"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"(*", " ", "end", " ", "*)"}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.735625441878965*^9, 
  3.735625464138838*^9}},ExpressionUUID->"80e1a4f5-996a-440c-8808-\
e0b2376a8e9f"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"Case", " ", "4"}], ":", " ", 
    RowBox[{"Matter", " ", "+", " ", "Radiation"}]}], " ", "//", " ", 
   RowBox[{"T", 
    RowBox[{"(", "k", ")"}], " ", "and", " ", "P", 
    RowBox[{"(", "k", ")"}], " ", "evolution"}]}], " ", "*)"}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356254487086363`*^9, 3.735625464138838*^9}, {
   3.735625508308816*^9, 3.7356255123786583`*^9}, {3.7368546415020256`*^9, 
   3.736854646924579*^9}, 3.736855204917408*^9, {3.766999036665384*^9, 
   3.7669990400885253`*^9}, {3.767004044047413*^9, 3.767004045491911*^9}, 
   3.7670049286730413`*^9, 3.7670150490767164`*^9, 3.767365288328416*^9, {
   3.767366381545531*^9, 3.7673663838388343`*^9}, {3.7964640169051657`*^9, 
   3.7964640258145323`*^9}, 
   3.796479297891097*^9},ExpressionUUID->"316a5955-1fd9-4466-98a6-\
6f065a4aa32f"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"Cosmological", " ", "parameters"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalOmega]\[Gamma]0", "=", 
     RowBox[{
      FractionBox["415", "100"], "*", 
      RowBox[{
       SuperscriptBox["10", 
        RowBox[{"-", "5"}]], "/", 
       RowBox[{"h", "^", "2"}]}]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"h", "=", 
     RowBox[{"7", "/", "10"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"om", "=", 
     RowBox[{"1", "-", "\[CapitalOmega]\[Gamma]0"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"ai", "=", 
     SuperscriptBox["10", 
      RowBox[{"-", "7"}]]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"ap", "=", 
     RowBox[{"10", " ", "ai"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"H0", "=", "1"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"w\[Gamma]", "=", 
     RowBox[{"1", "/", "3"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"cs2\[Gamma]", "=", 
     RowBox[{"1", "/", "3"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"aeq", "=", 
     RowBox[{"\[CapitalOmega]\[Gamma]0", "/", "om"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{"Friedman", " ", "equation"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"H", "[", "a_", "]"}], ":=", 
    SqrtBox[
     RowBox[{
      RowBox[{"om", " ", 
       SuperscriptBox["a", 
        RowBox[{"-", "3"}]]}], "+", 
      RowBox[{"\[CapitalOmega]\[Gamma]0", " ", 
       SuperscriptBox["a", 
        RowBox[{
         RowBox[{"-", "3"}], 
         RowBox[{"(", 
          RowBox[{"1", "+", "w\[Gamma]"}], ")"}]}]]}]}]]}]}]}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356253342187953`*^9, 3.735625344922727*^9}, 
   3.735625464138838*^9, {3.7670002343494787`*^9, 3.767000240007764*^9}, 
   3.7670008362373605`*^9, {3.7670019254231896`*^9, 3.7670019257496815`*^9}, 
   3.767002010116867*^9, 3.767003150769675*^9, {3.7670044141212025`*^9, 
   3.767004446241784*^9}, 3.767004943746189*^9, {3.7670050080960464`*^9, 
   3.7670050243241205`*^9}, {3.7670052183047867`*^9, 
   3.7670052221352024`*^9}, {3.7670701651352663`*^9, 3.767070208317212*^9}, {
   3.7674219237742405`*^9, 
   3.7674219340138683`*^9}},ExpressionUUID->"1aecf578-9dca-42d8-b7ce-\
8b6fbf7c426a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"The", " ", "initial", " ", "conditions"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]0", "[", 
      RowBox[{"a_", ",", "k_"}], "]"}], ":=", 
     RowBox[{
      RowBox[{"-", "1"}], "/", 
      SuperscriptBox["k", 
       RowBox[{"3", "/", "2"}]]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"\[Delta]\[Gamma]0", "[", 
      RowBox[{"a_", ",", "k_"}], "]"}], ":=", 
     RowBox[{
      RowBox[{"-", "2"}], 
      RowBox[{"\[Phi]0", "[", 
       RowBox[{"a", ",", "k"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"\[Delta]c0", "[", 
      RowBox[{"a_", ",", "k_"}], "]"}], ":=", 
     RowBox[{
      FractionBox["3", "4"], 
      RowBox[{"\[Delta]\[Gamma]0", "[", 
       RowBox[{"a", ",", "k"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"Vm0", "[", 
      RowBox[{"a_", ",", "k_"}], "]"}], ":=", 
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"k", "^", "2"}], " ", 
      FractionBox["ai", 
       RowBox[{"H0", 
        SqrtBox["\[CapitalOmega]\[Gamma]0"], " "}]], " ", 
      RowBox[{"\[Phi]0", "[", 
       RowBox[{"a", ",", "k"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"V\[Gamma]0", "[", 
      RowBox[{"a_", ",", "k_"}], "]"}], ":=", 
     RowBox[{"Vm0", "[", 
      RowBox[{"a", ",", "k"}], "]"}]}], ";"}]}]}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.7356246272481565`*^9, 3.735624689719591*^9}, {
   3.735624872839239*^9, 3.7356248730495434`*^9}, 3.735625464138838*^9, {
   3.735625551077653*^9, 3.7356256094488688`*^9}, {3.736854660690071*^9, 
   3.7368546715821238`*^9}, {3.73685504106439*^9, 3.7368550813497915`*^9}, 
   3.76699984139952*^9, {3.7669999090055523`*^9, 3.767000052173439*^9}, {
   3.7670002989900303`*^9, 3.7670003119008837`*^9}, {3.767000804122465*^9, 
   3.7670008110914736`*^9}, 3.76700086483064*^9, 3.767001208940386*^9, {
   3.7670017011887536`*^9, 3.7670017105864887`*^9}, 3.76700176727139*^9, 
   3.7670018401406918`*^9, {3.7670032129104495`*^9, 3.7670032340102596`*^9}, {
   3.767003616150666*^9, 3.767003616994241*^9}, {3.7670036522042994`*^9, 
   3.7670036793012247`*^9}, {3.7670044530416827`*^9, 
   3.7670044836567087`*^9}, {3.7964574037588863`*^9, 
   3.7964574094474487`*^9}},ExpressionUUID->"6ce379b4-b15b-41c6-816b-\
35ecc349a2c4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "This", " ", "solves", " ", "the", " ", "ODEs", " ", "as", " ", "a", " ", 
    "function", " ", "of", " ", "k"}], "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"sol", "[", "k_", "]"}], ":=", 
    RowBox[{"NDSolve", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[Delta]m", "'"}], "[", "a", "]"}], "\[Equal]", 
         RowBox[{
          FractionBox[
           RowBox[{"-", 
            RowBox[{"Vm", "[", "a", "]"}]}], 
           RowBox[{
            RowBox[{"H", "[", "a", "]"}], " ", 
            RowBox[{"a", "^", "2"}]}]], "+", 
          RowBox[{"3", 
           RowBox[{
            RowBox[{"\[Phi]", "'"}], "[", "a", "]"}]}]}]}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"Vm", "'"}], "[", "a", "]"}], "\[Equal]", 
         RowBox[{
          FractionBox[
           RowBox[{"-", 
            RowBox[{"Vm", "[", "a", "]"}]}], "a"], "+", 
          RowBox[{
           FractionBox[
            RowBox[{"k", "^", "2"}], 
            RowBox[{
             RowBox[{"H", "[", "a", "]"}], " ", 
             RowBox[{"a", "^", "2"}]}]], 
           RowBox[{"\[Phi]", "[", "a", "]"}]}]}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Delta]\[Gamma]", "'"}], "[", "a", "]"}], "\[Equal]", 
         RowBox[{
          RowBox[{"-", 
           FractionBox[
            RowBox[{"V\[Gamma]", "[", "a", "]"}], 
            RowBox[{
             RowBox[{"H", "[", "a", "]"}], 
             RowBox[{"a", "^", "2"}]}]]}], "-", 
          RowBox[{
           FractionBox["3", "a"], 
           RowBox[{"(", 
            RowBox[{"cs2\[Gamma]", "-", "w\[Gamma]"}], ")"}], 
           RowBox[{"\[Delta]\[Gamma]", "[", "a", "]"}]}], "+", 
          RowBox[{"3", 
           RowBox[{"(", 
            RowBox[{"1", "+", "w\[Gamma]"}], ")"}], 
           RowBox[{
            RowBox[{"\[Phi]", "'"}], "[", "a", "]"}]}]}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"V\[Gamma]", "'"}], "[", "a", "]"}], "\[Equal]", 
         RowBox[{
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{"1", "-", 
              RowBox[{"3", "w\[Gamma]"}]}], ")"}]}], 
           FractionBox[
            RowBox[{"V\[Gamma]", "[", "a", "]"}], "a"]}], "+", 
          RowBox[{
           FractionBox[
            RowBox[{
             RowBox[{"k", "^", "2"}], " ", "cs2\[Gamma]"}], 
            RowBox[{
             RowBox[{"H", "[", "a", "]"}], " ", 
             RowBox[{"a", "^", "2"}]}]], 
           RowBox[{"\[Delta]\[Gamma]", "[", "a", "]"}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"1", "+", "w\[Gamma]"}], ")"}], 
           FractionBox[
            RowBox[{"k", "^", "2"}], 
            RowBox[{
             RowBox[{"H", "[", "a", "]"}], " ", 
             RowBox[{"a", "^", "2"}]}]], 
           RowBox[{"\[Phi]", "[", "a", "]"}]}]}]}], ",", 
        RowBox[{
         RowBox[{"\[Phi]", "[", "a", "]"}], "==", 
         RowBox[{
          RowBox[{"-", 
           FractionBox["3", "2"]}], 
          FractionBox[
           RowBox[{"a", "^", "2"}], 
           RowBox[{"k", "^", "2"}]], 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"om", " ", 
             SuperscriptBox["a", 
              RowBox[{"-", "3"}]], 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"\[Delta]m", "[", "a", "]"}], "+", 
               RowBox[{"3", "a", " ", 
                FractionBox[
                 RowBox[{"H", "[", "a", "]"}], 
                 RowBox[{"k", "^", "2"}]], " ", 
                RowBox[{"Vm", "[", "a", "]"}]}]}], ")"}]}], "+", 
            RowBox[{"\[CapitalOmega]\[Gamma]0", " ", 
             SuperscriptBox["a", 
              RowBox[{
               RowBox[{"-", "3"}], 
               RowBox[{"(", 
                RowBox[{"1", "+", "w\[Gamma]"}], ")"}]}]], 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"\[Delta]\[Gamma]", "[", "a", "]"}], "+", 
               RowBox[{"3", "a", " ", 
                FractionBox[
                 RowBox[{"H", "[", "a", "]"}], 
                 RowBox[{"k", "^", "2"}]], " ", 
                RowBox[{"V\[Gamma]", "[", "a", "]"}]}]}], ")"}]}]}], 
           ")"}]}]}], ",", 
        RowBox[{
         RowBox[{"\[Delta]m", "[", "ai", "]"}], "==", 
         RowBox[{"\[Delta]c0", "[", 
          RowBox[{"ai", ",", "k"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"Vm", "[", "ai", "]"}], "==", 
         RowBox[{"Vm0", "[", 
          RowBox[{"ai", ",", "k"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"\[Delta]\[Gamma]", "[", "ai", "]"}], "==", 
         RowBox[{"\[Delta]\[Gamma]0", "[", 
          RowBox[{"ai", ",", "k"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"V\[Gamma]", "[", "ai", "]"}], "==", 
         RowBox[{"V\[Gamma]0", "[", 
          RowBox[{"ai", ",", "k"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"\[Phi]", "[", "ai", "]"}], "==", 
         RowBox[{"\[Phi]0", "[", 
          RowBox[{"ai", ",", "k"}], "]"}]}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
       "\[Delta]m", ",", "Vm", ",", "\[Delta]\[Gamma]", ",", "V\[Gamma]", ",",
         "\[Phi]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"a", ",", "ai", ",", "1"}], "}"}], ",", 
      RowBox[{"MaxStepSize", "\[Rule]", 
       SuperscriptBox["10", 
        RowBox[{"-", "5"}]]}], ",", 
      RowBox[{"MaxStepFraction", "\[Rule]", 
       SuperscriptBox["10", 
        RowBox[{"-", "6"}]]}], ",", 
      RowBox[{"StartingStepSize", "\[Rule]", 
       SuperscriptBox["10", 
        RowBox[{"-", "5"}]]}], ",", 
      RowBox[{"MaxSteps", "\[Rule]", 
       RowBox[{"10", "^", "8"}]}], ",", 
      RowBox[{"Method", "\[Rule]", "Automatic"}]}], "]"}]}], ";"}]}]], "Input",
 CellChangeTimes->{{3.767000062606375*^9, 3.76700007587354*^9}, {
   3.767000117818261*^9, 3.7670002200253425`*^9}, {3.767000334646226*^9, 
   3.7670003497741113`*^9}, {3.7670006449247065`*^9, 
   3.7670006680052977`*^9}, {3.7670007068011217`*^9, 3.767000728449688*^9}, {
   3.7670010196211257`*^9, 3.7670010490387807`*^9}, {3.7670014088407974`*^9, 
   3.7670014276572733`*^9}, {3.7670018032029686`*^9, 3.767001808313428*^9}, {
   3.7670033800456734`*^9, 3.767003386616849*^9}, 3.7670034819158373`*^9, {
   3.7670037198004446`*^9, 3.767003750469631*^9}, {3.767004056704776*^9, 
   3.7670041193844557`*^9}, {3.767004219094764*^9, 3.7670043887432117`*^9}, {
   3.767004491866829*^9, 3.7670045185115004`*^9}, {3.7674219165444036`*^9, 
   3.767421918746636*^9}, 3.767422672815238*^9, {3.7964574144863625`*^9, 
   3.7964574245538287`*^9}},ExpressionUUID->"0fada880-40ca-4085-ba7e-\
62eb5f4a19ec"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"We", " ", "need", " ", "many", " ", "values"}], ",", " ", 
    RowBox[{"so", " ", "parallelize"}]}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"LaunchKernels", "[", "7", "]"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Off", "[", 
     RowBox[{"NDSolve", "::", "ivres"}], "]"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"ParallelEvaluate", "[", 
     RowBox[{
      RowBox[{"Off", "[", 
       RowBox[{"NDSolve", "::", "ivres"}], "]"}], ";"}], "]"}], 
    ";"}]}]}]], "Input",
 CellChangeTimes->{{3.767422039861061*^9, 3.7674220444959574`*^9}, {
  3.7964574495847883`*^9, 
  3.796457461176392*^9}},ExpressionUUID->"bc62daac-1f46-4a36-b26d-\
1a63f4985057"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"The", " ", "wavenumber", " ", 
    RowBox[{"k", "/", "H0"}], " ", "and", " ", 
    RowBox[{"in", " ", "[", 
     RowBox[{"h", "/", "Mpc"}], "]"}], " ", 
    RowBox[{"(", 
     RowBox[{"second", " ", "line"}], ")"}]}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"kv", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"3000", " ", 
        RowBox[{"10", "^", "log10k"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"log10k", ",", 
         RowBox[{"-", "3"}], ",", "1", ",", 
         RowBox[{"1", "/", "5"}]}], "}"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"Round", "[", 
    RowBox[{
     RowBox[{"kv", "/", "3000."}], ",", "0.001"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.796460539967727*^9, 3.796460570495513*^9}, {
  3.796476845502134*^9, 3.7964768765058336`*^9}, {3.7964785937003613`*^9, 
  3.7964786013800745`*^9}, {3.796478728281082*^9, 
  3.7964787638361077`*^9}},ExpressionUUID->"ebf7bcbb-1234-4fed-a007-\
29b93cd39c78"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.001`", ",", "0.002`", ",", "0.003`", ",", "0.004`", ",", "0.006`", ",", 
   "0.01`", ",", "0.016`", ",", "0.025`", ",", "0.04`", ",", "0.063`", ",", 
   "0.1`", ",", "0.158`", ",", "0.251`", ",", "0.398`", ",", "0.631`", ",", 
   "1.`", ",", "1.585`", ",", "2.512`", ",", "3.981`", ",", 
   "6.3100000000000005`", ",", "10.`"}], "}"}]], "Output",
 CellChangeTimes->{{3.796460540588432*^9, 3.7964605709116545`*^9}, 
   3.7964617083835154`*^9, 3.796474664949297*^9, 3.7964786031249547`*^9, {
   3.796478729997715*^9, 
   3.796478764323908*^9}},ExpressionUUID->"65e68414-e0ee-4cad-973e-\
deb3ab599ed9"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "The", " ", "solutions", " ", "of", " ", "the", " ", "ODEs", " ", "for", 
    " ", "the", " ", "k", " ", "values"}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"allsols", "=", 
     RowBox[{"ParallelTable", "[", 
      RowBox[{
       RowBox[{"sol", "[", 
        RowBox[{"kv", "[", 
         RowBox[{"[", "i", "]"}], "]"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", 
         RowBox[{"Length", "[", "kv", "]"}]}], "}"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"CloseKernels", "[", "]"}], ";"}]}]}]], "Input",
 CellChangeTimes->{{3.76736539254949*^9, 3.7673654432596045`*^9}, {
   3.767366134887046*^9, 3.767366141795056*^9}, 3.76742195657323*^9, {
   3.7674220646207314`*^9, 3.767422065740223*^9}, {3.7674223573629923`*^9, 
   3.76742238363859*^9}, {3.7964574713269663`*^9, 3.796457480792892*^9}, {
   3.7964605282745295`*^9, 3.7964605377319956`*^9}, {3.7964605829991856`*^9, 
   3.796460592590663*^9}, {3.796461805257654*^9, 
   3.796461805531621*^9}},ExpressionUUID->"c85d671c-c7d5-48a7-8254-\
6e789fce4441"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Rasterize", "[", 
  RowBox[{
   RowBox[{"LogLogPlot", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"Flatten", "[", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            SuperscriptBox[
             RowBox[{"kv", "[", 
              RowBox[{"[", "i", "]"}], "]"}], 
             RowBox[{"3", "/", "2"}]], " ", 
            RowBox[{"\[Delta]m", "[", "a", "]"}]}], "/.", 
           RowBox[{"allsols", "[", 
            RowBox[{"[", "i", "]"}], "]"}]}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "1", ",", 
            RowBox[{"Length", "[", "kv", "]"}]}], "}"}]}], "]"}], "]"}], "//",
        "Abs"}], "//", "Evaluate"}], ",", 
     RowBox[{"{", 
      RowBox[{"a", ",", "ap", ",", "1"}], "}"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"ColorData", "[", 
         RowBox[{"\"\<Rainbow\>\"", ",", 
          RowBox[{"i", "/", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Length", "[", "kv", "]"}], "-", "1"}], ")"}]}]}], "]"}],
         ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "0", ",", 
          RowBox[{
           RowBox[{"Length", "[", "kv", "]"}], "-", "1"}]}], "}"}]}], "]"}]}],
      ",", 
     RowBox[{"PlotRange", "\[Rule]", "All"}], ",", 
     RowBox[{"Frame", "\[Rule]", "True"}], ",", 
     RowBox[{"FrameStyle", "\[Rule]", "Black"}], ",", 
     RowBox[{"BaseStyle", "\[Rule]", 
      RowBox[{"FontSize", "\[Rule]", "12"}]}], ",", 
     RowBox[{"Epilog", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Black", ",", "Dashed", ",", 
        RowBox[{"Line", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"aeq", ",", 
              SuperscriptBox["10", 
               RowBox[{"-", "6"}]]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"aeq", ",", 
              SuperscriptBox["10", "6"]}], "}"}]}], "}"}], "//", "Log"}], 
         "]"}]}], "}"}]}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<a\>\"", ",", 
        "\"\<\!\(\*SuperscriptBox[\(k\), \(3/2\)]\)\[Delta](a)\>\""}], 
       "}"}]}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", 
      RowBox[{"1", "/", "GoldenRatio"}]}], ",", 
     RowBox[{"GridLines", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"PlotLegends", "\[Rule]", 
      RowBox[{"BarLegend", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"\"\<Rainbow\>\"", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "10"}], "}"}]}], "}"}], ",", 
        RowBox[{"LegendLabel", "\[Rule]", "\"\<k [h/Mpc]\>\""}]}], "]"}]}]}], 
    "]"}], ",", 
   RowBox[{"RasterSize", "\[Rule]", "1500"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "Large"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7670007843028517`*^9, 3.767000785958127*^9}, {
   3.7670037041181507`*^9, 3.767003704298748*^9}, {3.7670038667263975`*^9, 
   3.7670038949586353`*^9}, {3.767004137375663*^9, 3.767004139600129*^9}, {
   3.7670046545273056`*^9, 3.7670047223864164`*^9}, 3.7670049734094257`*^9, {
   3.767005145416571*^9, 3.7670052057626343`*^9}, {3.7673473315655203`*^9, 
   3.767347391915212*^9}, {3.7673474245954247`*^9, 3.7673474918882785`*^9}, {
   3.767365556154033*^9, 3.767365606114623*^9}, {3.767365764958556*^9, 
   3.7673657695383625`*^9}, {3.7673661737639427`*^9, 3.767366201000299*^9}, {
   3.7674219661651573`*^9, 3.7674219814965267`*^9}, {3.767422206957065*^9, 
   3.767422297834478*^9}, {3.7674224087119026`*^9, 3.7674224233236575`*^9}, 
   3.796457433472886*^9, {3.796457773754018*^9, 3.7964577868996615`*^9}, {
   3.7964578192752132`*^9, 3.7964578235021014`*^9}, 3.7964578602679358`*^9, {
   3.796460678562361*^9, 3.7964606911893187`*^9}, 3.796460810404089*^9, {
   3.796460884215969*^9, 3.7964608844727926`*^9}, {3.7964615394458227`*^9, 
   3.7964615527010274`*^9}, 
   3.7964786354609957`*^9},ExpressionUUID->"eed1b495-a3de-4448-98e7-\
8eabe5ee2adf"],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzs3Wt0HOWd73uvc9ZZ67w872bP3nvmTGYmO8xMBvbMhEyiZLJzxwwbJ9yG
IRyCYbDwNttAMPhChGXHQOIQB4HBwYAwjsHBOMIXsHy3JFuyLV8k32W7Zd0s
qaVWy5Jal9bVOi03brdaXU89VU93V3U938+qNYtB3dXV9fwt3L9U/foLjz51
d+7/MW3atHn/d+T/3P3Igu8888wji+75fyL/z31Pzvtfjz8567Hbn3x21uOz
nvnao/9n5F8+Efk/5/6vadMm/nkcAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAADAAp+v2IgvcwfhwGu6lSsWBAAAAAAAPfgKcqZdl1OQzk/exbnT
DEx63bQeUOwYJu052ZHlFlvfe/yhK+wmUyQXBAAAAAAAqNMqgbmx70nJSNIj
s56dJAlgSGAAAAAAAMAEpxKYnDi5GUpgDAIYgyjCaniSLIBxeQITvwjxB08C
AwAAAABAijmTwAiCifQdkFEAY3QxiLX0JGkA4+oEZrLMDQIAAAAAABrSKIFJ
XgKTcGTxF4NYiU9uHPaky0lIYAAAAAAAwLhOCYxhADP5yAoK7EQwcQFMQYHc
G3UZEhgAAAAAANJImwTGOIBJODKfjQgmPoDxSb5RlyGBAQAAAAAgjXRJYAxL
YKYemfUIZlIAI/tGXYYEBgAAAACANMqWBMZXXJAbX7By7SuUiqWPVxTATDky
qxHM5ADGVgLj8xUne3sW18M3cZYm7Ud+TyQwAAAAAACkkdQH74QvC8qxdWWH
7QRmIpyYZmTyV1mb7zTZK085MmsRTEIAY/ZG434c/anJ+5OKma4FVIY7kTlT
JDAAAAAAAKSR+QfvhHzAXvwybjeBSf4lzwnZgmlgICiBSXpkViKYxADGYgJj
8FXYVt6fzDmKvaLRrkhgAAAAAABII5MP3gkf7m3HL+P2EpgcyWjBJDIQBzDJ
jkw+gpkSwFhJYApksxPjN2ghfhG+HRIYAAAAAADSSPTBO4Xxy7itBGbSa8dV
mSS550Z0bCYBTNIjizsMYR4xNYCxkMDEvb9JbzCxFcb4KKbsLKH0xZfkVCXf
EwkMAAAAAABpZPjBO7Xxy7hCAmNw50zCA413KS6BMToyuQgmSQBjPYEx2L8v
4ZFT95W4RkbVxOZ7IoEBAAAAACCtkn/wTnn8Mm43gRGFAZNyBcMHmgYwBkcm
FcEku7zGWgIjeYVNsp2Jf2rxsSQwAAAAAACkUZIP3glXTKTq47itBEYcK8Tv
0ugwzQMYoyOTiGBiz4zft5UExvTkTjodk/cm+JHBjia+mNrom6lJYAAAAAAA
SKMpH7zTE7+M20tgTHIFmdTArARGcGSmEUzSAMZKAiNxbZHh+bAWwFh6HRIY
AAAAAABSbPIH74TW1pR+Erf3bdQWDl7csivYl+GRmUQwyQMYCwmM1Pk1utBH
5gIgC0hgtHOtpTnuD/y1OminDwoAAAAAPEv8Zcap/CjuSAIjE8AIjkwYwRgE
MFa+jVrmyhWjN2nxWhq7LwNPMv5zz+IDAAAAQHqIE5hUfh5zIoGRKIERH5kg
gjEKYOQTGMlza1oUnJJFIoHRh9kf+lQUbwMAAAAAEiT5MJZbXGCpLFaSAwmM
XAAjPDLDCMYwgCGBgatNXunYfUe+4rh/TwYDAAAAACmXmMB83gRh7Qt7pGQ+
gZEMYMRHZhDBGAcwqU5guAsJqXRjbqZOjeyfGAAAAACAdZMSGKMP+Cn5SJ75
BEaqBMb0yJJGMIIAJtU9MEa7o4kXNogmN+7HjAAAAAAApJrgg3eKM5iMJzCy
AYzZkSWJYIQfY1P7XUiG79Hyt1H7CnInvu6moKDYJ6zM4eO3h5GwAAAAAIBT
RB+8J9+hpPqZLdMJjHQAY3pkcS8T/bH4OgL5BEYiORHkLBYjGMOrnab+mI/n
XnV9lVlhAAAAAMg48QfvVGYwGU5gLFRamB5ZQgQjeSOHTAJj8hYnn39BS7D5
25z8ssKbrfh87lnXp+Dz9fcVF+TGLXtObtLLowAAAAAAqSD/dUKqH80zm8BY
6RQ1P7JJEYxJAGMtgRG8y4SSZGHVsMnp8pnHPiQwGogtcmQujb6SOocSXgAA
AABIC/MP3qnKYDKawFj6UheJI0v2cdVoz1YTmM8/9sZdfeDzFecmvp759Taf
76hg8p4mdpUjsSsSGCclXJsiyXftGpZJfzwT1j/xCddvQsqdMl9Sow0AAAAA
sE/mg7elu10MZTSBkS+BkTyyqbmJ4Xuwk8CYELwLo0sZDElkTCQwmSX6gmgj
4nU3WMHEi6bic79JdyQxAQAAAACQcnIfvFORwWQygbEUwMgdWWJwYvwWLCQw
kU/A5gGK6W0hFkIYuSXm83cG+ST/YMSTSvHEX+uVfJEtXTsGAAAAALBC9oO3
egaTwQTGWgAjeWRmVbayu5vy4yS3HN14V7nFUmHI5EpVO7sigXFCQkGP5LeK
ywZuU9bxxlMNl5ivqwYAAACANJH/4K2awWQugbH6P+RLHpnstQpWE5hrhzy5
0CMn2uchc/DxPu8FmXyniUk1SOy5JDCZliRKsfil4p/3/sR+NiXMS1xJsxbp
yWW9AAAAAICsZONmC3tcfidFxs6DNSQwGWVw4ZOlb06Xqs6evEfzfCVW1ssM
AAAAAEC2cmnykHEuPQ8kMJkiuuvMbCDih0ewSsYzZnoRDHchAQAAAEDWc2ny
kHEuPQ8kMBkwNXyZaOeJmwiTgZCenfjVNIhgku7AYoMSAAAAAMCNXJo8ZJxL
zwMJTAZMvkHoejeydAJjnKtMEbfPhNWM28nkEpli+b1PMg0AAAAAMk7+M4uW
XJo8ZJxLzwMJTAbETvKkL6aSTmDkL5YRRTDmX6ZkbSzt/bYEAAAAABVWPrVo
aHLy4Ivn9KFllIsSmPg1KCaBST9fQbIvuZINVuKvXjFdIfGDfcZfXm558VPw
qxMAAAAALLL2uUU7k77FZRK9PvC7JoFhQVxCNoERXNZiZ6/Xvrt60leg2/kG
dBIYAAAAAE6w/tlFK3zgjyKBwWQ2EhiJwbH2aAVTfwEa/TK0+u8dl9YDU9y5
1afLP970keIHCH7KAKRwz54ZANeu/riLB8DG0yWfIvMwewOQdX/8xz00AK79
4y/zik5x7erb2AMDYINrB4Df/7HnTv0AgmT4wB9FAoPJJLMSCz28CXtN73pO
/QXomb+Bp/XA+Bu4/Cs6xbV/AbOxh8wPQOw3Q35+vuCJrl39cRcPAH8DzwzP
DAC//21w7erb2AMDYINrB4Df/7HnTv0AAgCyvJLACBy7JvYPCYz+vePSemCK
O7f6dPnHmz5S/ADBTxmAFO7Z/QNw49dVbq7gia5d/WMuHgAbT5d8iszD7P0G
yLo//sc8NAD8/rfBtatvYw8MgA2uHQCtfv/LfMSYRgIDwAavJDAqe0jh8Wgi
fSct83t27QC49sBsSOF7Ee8q9tPYb4bYNTDpPrCUy8Y/Ze55RUmuPTA3y8bJ
zLrf/0hK8ve//I9cOwDZ+KcsA6+Y2oNX/OxAAgPAPhIYfnnqjQHwJA8kMMgA
BkBzDIDmGACdkcAAcIznmnht7CGFx4OswwB4EgkMZDAAmmMANMcA6IwEBoBj
0vJt1BavmFFAAgNFDIAnkcBABgOgOQZAcwyAzkhgADhGNoGJC1UsJTBp/mqr
aclY3UOajg1ZgQHwJBIYyGAANMcAaI4B0JmNzwvqHzoAYIL0/UJWbizKWA0M
18BAFQPgSSQwkMEAaI4B0BwDoDOugQHgGOlgxcKdRZkLYFKQwBw7diyFx4Os
wwB4kmQCw+prjgHQHAOgOQZAZyqrTwIDQIn8pS1xj5SujElzCwwJDJQxAJ5E
AgMZDIDmGADNMQA6I4EB4BgLNxfFJyvG17bEXyuT9gCGBAaqGABPKrmuvr5e
8DBWX3MMgOYYAM0xADojgQHgGJv1LtOm5SR9eHFuXP6S/gAmBU28/PdXcwyA
zlh9zTEAmmMANMcA6MzG6qt/6ACACVYSmMnXt0RTGN/1a2F8vsnpS/o7YCZw
DQwUMQA6Y/U1xwBojgHQHAOgM66BAeAYSwnM1AzGWCbyFxIYKGMAdMbqa44B
0BwDoDkGQGckMAAcYzGBGU+4F8lI+u8/iiKBgSIGQGesvuYYAM0xAJpjAHRG
AgPAMdYTmPFxX7HwSpjMXP0SRQIDRQyAzlh9zTEAmmMANMcA6IwEBoBj7CQw
1/h8xQW5OfFJTE5ObsGNXpjMIIGBIgZAZ6y+5hgAzTEAmmMAdEYCAwD2kMBA
EQOgM1ZfcwyA5hgAzTEAOiOBAQB7kt4GZWkP/PdXcwyAzlh9zTEAmmMANMcA
6IxvowYAe7gGBooYAJ2x+ppjADTHAGiOAdAZ18AAgD0kMFDEAOiM1dccA6A5
BkBzDIDOSGAAwB4SGChiAHTG6muOAdAcA6A5BkBnJDAAYA8JDBQxADpj9TXH
AGiOAdAcA6AzEhgAsIcEBooYAJ2x+ppjADTHAGiOAdAZCQwA2EMCA0UMgCfF
fjPk5+cLHsbqa44B0BwDoDkGQGckMMgYf/HS3IilxX6Jx/qriwuXXnv8NUuX
FhZX+yWeCGSO+hfD8d9fzTEAnkQCAxkMgOYYAM0xADrj26iRIZ/nLxIJjL+6
8Eb0MtnSwmpSGLgG18BAEQPgSSQwkMEAaI4B0BwDoLOqPXsCeXlXBwdtPJcE
BtKqC2+kKOIEJu6RyRVWZ+qgATESGChiADyJBAYyGADNMQCaYwC01f3BBxe/
/GXfF77QsWyZjaeTwECKf3KoIkxg4pOaifuOYrsoXkoGA7chgYEiBsCTSGAg
gwHQHAOgOQZAQ0O1tZfvvtv3hS/EtsGzZ63uhAQGppLcUSRIYPw3cpYkKcuN
dEaqSgZINxIYKGIAPIkEBjIYAM0xAJpjALRydXDwyqpV8dlLdGu64w6ruyKB
gci1Lt24Kl3z+ORGAGNwlUssgyGCgRuQwEARA+BJJDCQwQBojgHQHAOgj4FD
hxq///2p8Ut06yostLQ3EhgY8sffNXQtMDFPT0wDGCIYuAsJDBQxAJ5EAgMZ
DIDmGADNMQA6GOvpaX/uOaPsJbJFfhp5jKV9ksDA0I1vPrr+7UXm4YlMvEIE
AxchgYEiBsCTSGAggwHQHAOgOQbA83o2brx0yy1G2Uvj978/cOiQjd2SwMDQ
RAIz+aujTbMTiUtgJB8EZMa0ZCztgf/+ao4B8CQSGMhgADTHAGiOAfCwodra
5gceMMpeam+66dx/PNLy2IzRzg75fap/6ICOTBMYqctbSGDgIlwDA0UMgCeR
wEAGA6A5BkBzDIAnXR0cDP7617U33WQUvzTNmNG++On6734psrW/8ISNlyCB
gQXyCYwwW+E2JLgHCQwUMQCeRAIDGQyA5hgAzTEA3iNu3L10yy0dLy9tuvcb
0fgluvWX77b6KiQwsMAsOpG8uoUEBu5BAgNFDIAnkcBABgOgOQZAcwyAl5g2
7vpnP+5/6qfx2Ut0a7r3G1eHBi29FgkMLCCBgeeQwEARA+BJJDCQwQBojgHQ
HAPgGaGtW+u++lVB425gyYKG6TdPjV+iW/f61ZZejgQGFpDAwHNIYKCIAdAZ
q685BkBzDIDmGAAPGL58uWXmTEHjbuDni1rn3GuUvTTOuNVq/DJOAqPs7T/5
E5ds8+fPPynN5rulBwaeQwIDRQyAzlh9zTEAmmMANMcAZLsrq1YJGncv/+hH
Hb/KM8peIlvbgscsfRdSDAmMorf/5E8E94tlclu8eHHa3y3fhQTPIYGBIgZA
Z6y+5hgAzTEAmmMAspdp42774oUJjbsJ3S+n3/pN90fLrw5ba4CJIoFRRAIz
SSxckUpguAQGziOBgSIGQGesvuYYAM0xAJpjALKRaeNu6yOPJG3cjW2db77c
V1rUNPebzf/x9z1FBTaOgQRG0dt/+p98f/2XbthckcBI3YbETUhwERIYKGIA
dMbqa44B0BwDoDkGIOuIG3cjPxI37jbPvH3g0O6OXz/a/B9/H91aZn9luKnG
6mGQwCgigZlM4g4jAhi4ybRkLO2B//5qjgHQGauvOQZAcwyA5hiALCJu3I1s
7fOfbXn0TqPspWH6zV3r3gxte7dl9ldi8Ut0C7z0oOQxqH/oQBQJTALTCCa2
D1pg4AZcAwNFDIDOWH3NMQCaYwA0xwBkC9PG3fafPy1u3O2vKG7Lm5GQvcS2
vpINlo6HBEYRCUyiGxFMbmH1lEfdyF+4AgauQAIDRQyAzlh9zTEAmmMANMcA
uF+4qkqxcbf/wM6eogKj7CWyBV+dPdpt7RuRSGAUvf1f/1PtTX/phs0tCUx8
yhJ54I0Uxl99I5vhAhi4BQkMFDEAOmP1NccAaI4B0BwD4GZjPT2BvDxR4+7D
D/ufmSlu3B04ssv/zHeMspe2BbeFT5fbODYSGEUkMMLHGSF/gVuQwEARA6Az
Vl9zDIDmGADNMQCupdi425L744GKPcFXZwsufbnw5kJ7X0U9TgKjjAQmOX91
4VKD9GVpkpuTAKeQwEARA6AzVl9zDIDmGADNMQAuZN64u2hBy2MzjLKXxhm3
dq9fnbRxN756d6S17viRw70H1lwdGbJxkCQwit7+8z+t/bu/csPmrgRmgt9f
XVy49EYQs3TpRPhC+gJXIYGBIgZAZ6y+5hgAzTEAmmMA3MakcfeuHwdeXChu
3A1XlbUvuc8oe2mZ/ZXeXesiLzToq7j8ym2tS/8pVLraxnGSwCjSK4EBPIcE
BooYAJ2x+ppjADTHAGiOAXAP08bdQP7PxY27fXu3dn+03KRxt6NlLBzq3rKk
dek/RTf/S18f6ai3erQkMIpIYICsRgIDRQyAJ5VcV19fL3gYq685BkBzDIDm
GAA3MG/cfeSRtmcfE1z60rF8YX/Fp4LG3ciP+iuLI6/VX721bfm3Y/FLdOtc
m2v1mElgFJHAAFmNBAaKGABPiv1myM/PFzyM1dccA6A5BkBzDIDjZBp3G2fc
qtK421NUMNYfGumo71ybm5C9xLb+6q2WDpsERtE7f/Gnl27+azdsJDCADSQw
UMQAeBIJDGQwAJpjADTHADjIvHF3/rOtc+41yl4apt9s2rjbvuS+4aaaqyND
odLVRtlL9BoYqzcikcAoIoEBshoJDBQxAJ5EAgMZDIDmGADNMQBOMWnc/dGP
TBt3+0o3t+XNEDTuhra9G3mhofpjgTfvMcpe2pZ/2+rVL1EkMIpIYICsRgID
RQyAJ5HAQAYDoDkGQHMMQOaZNu625y0QN+72bt/Y9V6eSeNud8dYbzC+cXfq
dum9uWPhkL13QQKj6J2/+tNL//jXbthIYAAbSGCgiAHwJBIYyGAANMcAaI4B
yCSZxl3/Uz8VXPoSfDW/r7RI3LgbPl0+btC4G9vaX7tz0FehsvokMIpIYICs
Ni0ZS3vgv7+aYwA8iQQGMhgAzTEAmmMAMkamcbdh+s1G2UvzzNvlG3eDhTON
shf/S18Pla6+OjI0fm31R0Ot8m8h6ScOEhh7SGCArMY1MFDEAHgSCQxkMACa
YwA0xwBkgEzjbstjM1QadwMvPTh06eRE4+7elf6Xvm4UvwQLZ8Yad8eGei9+
8mzz6q8N1JfZeFMkMIpIYICsRgIDRQyAJ5HAQAYDoDkGQHMMQLqZNu62LZgj
btwNV5UJGndb5+b0lWyIvNCgr0LcuNt39I+xoxqoL/Ovu6N59dciW+Qfro4O
WX1fJDCK3v3if6679Ytu2EhgABtIYKCIAfAkEhjIYAA0xwBojgFIn3BVVdMd
dwgad9uefVLcuNu3d2vXumWC244633hqonE3HBI37kZ+OtYbjB7VaKg1WPx0
NHuJbd0HC6y+OxIYRSQwQFYjgYEiBsCTSGAggwHQHAOgOQYgHcZ6ejqWLRPc
dtT8wL+LG3c733w5JY27gTfvGfRVxA4sVL22dc0PEuKX6DYcvGjpPZLAKCKB
AbIaCQwUMQCeRAIDGQyA5hgAzTEAKde7c6dK427r//43mcbdq8ODIx31nR/O
FTXu7l0ZbdyNGGo73f7HnybNXiJb65of9J//zNLbJIFR9O6X/nPd177oho0E
BrCBBAaKGABPIoGBDAZAcwyA5hiAFBoNBEwbd5v/vx8aZS+NM26VadwdbqqZ
aNwtXS1o3O1cmxvfuNtTucooe4lsweKnR/uDVt8sCYwiEhggq5HAQBED4Ekk
MJDBAGiOAdAcA5AqGWjc7d21LvJCQ5dPiRt3+6u3xo4qvnF36tb03g/DTYfs
vV8SGEUkMEBWI4GBIgbAk/KvKykpETyM1dccA6A5BkBzDIA69cbd/gM7U9a4
Gw5Fj2q0Pzi1cTehffd45QHb75oERhEJDJDVSGCgiAHQGauvOQZAcwyA5hgA
FeaNuw/+RNy4e+WdFRYad1f80PDSlxU/TGjcbXn3W0bZS/sffzrUdnpcbfVJ
YBS9+7f/pe6b/80NGwkMYAMJDBQxADpj9TXHAGiOAdAcA2CbSxp3I1t84+5w
8GJg82NG2UvLu98KVa+NvQUSGAeRwABZjQQGihgAnbH6mmMANMcAaI4BsMG0
cbftZ0+2PHpnBhp3g4UzY427V0eHzBt3Q63xbySy+iN9fnsngQRGEQkMkNVI
YKCIAdAZq685BkBzDIDmGACrxI27TbdPb//506lp3K0/Jm7c7Tv0Yeyowk2H
2tbfZZS9+NfdMVBflvBGxoZ7z+1ccGnjN/ua99s4DyQwigq//F/q/8eX3LCR
wAA2kMBAEQOgM1ZfcwyA5hgAzTEA8sSNu7U33dS+4JnMNO52FS0a6/38+6NH
+4NX9rwguPSlp3LV2FBvwnsJ1Rc3fDrj0sZvRrbIP1wdHbJ6NkhgFJHAAFmN
BAaKGACdsfqaYwA0xwBojgGQkenG3eXfNspe2l+7M75xt+/sJ61rfmCUvbRt
uD/auBtvONTQWjo3mr3EtuCJ162eExIYRSQwQFYjgYEiBkBnrL7mGADNMQCa
YwBMmTbutufNS1XjbrBwZjoad6Oujg5dOVtY98l3E+KX6DbYddHSaSGBUVR4
838RRHaZ3EhgABtIYKCIAdAZq685BkBzDIDmGACBzDTuDl06OdG4u3dlCht3
R7oaEt5LOHimefcjSbOXyFa/eXqovtjSySGBUeSeBGb+/PknDTh9kgD3IoGB
IgZAZ6y+5hgAzTEAmmMAjKSkcbd9yX2Cxt3QtncjLzToqxA37vZXb40dlbhx
t3XND/rPf5bwRsaGeztPv2WUvUQ2/4F5o+FOq+eHBEaRexIYroEBbCCBgSIG
QGesvuYYAM0xAJpjAKYyb9x9fr64cbdv79buj5YLbjsKvjp7onG3N9hVtEhw
21H3liXyjbtdpcumNu72Ne+PNe5O3S5uvqfff9jeWSKBUUQCA2S1aclY2gP/
/dUcA6AzVl9zDIDmGADNMQDxxnp6Anl5Ko27nW++3F/xqUzjbt/RPwoadwNv
3qPYuDvS5/cfmCe49KXz9FvHj1qOX9Q/dCCq8B/+a8MPb3LDRgID2MA1MFDE
AOiM1dccA6A5BkBzDEBMaOtWceNuYMmCxhm3GmUvLbk/HqgsNW3cHesPiRt3
/S99PVS6Ota4O9LVIG7c7T310dT30lWzzqhxN7I1734kHDwzrrb6JDCKSGCA
rEYCA0UMgM5Yfc0xAJpjADTHAEQMX74sbtxtn/9s6+P3GGUvDdNvTlXjbufa
3LQ27tZ98t2umnWxB5PAOIgEBshqJDBQxAB4Uv51JSUlgoex+ppjADTHAGiO
ARA37l6+68eBJc8JbjtypHHXv+6OpI27wROvixt3R/r88U8hgXFQ4T/914bb
b3LDRgID2EACA0UMgCfFfjPk5+cLHsbqa44B0BwDoDmdByBcVdX4/e8bZS+X
brklZY274VD3liUmjbvhUPSo7DXu9jbtETTuRn7U17x/6hkggXEQCQyQ1Uhg
oIgB8CQSGMhgADTHAGhOzwGQadxte/axlDTu9ldvFTfuDl0+FTuwNDXujg0n
JjZRkdUfHrT8PdRRJDCKSGCArEYCA0UMgCeRwEAGA6A5BkBzGg6AaeNux4t5
gsbd5pm3mzbudn+0PNq427k2N1WNu6HqtVdHh+LfSOT/lWzcNXK87NcndtzW
13XWxmkkgVFEAgNkNRIYKGIAPIkEBjIYAM0xAJrTagDMG3cXLVBs3G1fcl+0
cbf3wBpB426wcKZ6427TjgeNspf6zdPjG3en6u2sPrf/4ari70S2mvJZY2ND
ggcnRQKj6L1b/6zxf/6NGzYSGOjCV5AzTSSnwCe/MxIYKGIAPIkEBjIYAM0x
AJrTZwDUG3cHKvcIGndbZn8l2rg7dPlUuht3A0dfFjfujoYN7y0aHe5tPLU8
mr3EtrbaD62eTxIYRSQwQKYV5woDGBIYZBQD4EkkMJDBAGiOAdCcDgNg2rgb
yP/55Z98V9C427t9o2Tjbs+2l00ad3uD0aNKU+Nuv/+w4FQEL28/vfeehPgl
sp3YcdvQgF/wxKlIYBSRwAAZZnYJDAkMMooB8CQSGMhgADTHAGjO2wNg2rjb
+sgj7c/PEVz60rF8oWnj7kDV3shrDZze2bbih0bZS/trdw76KmIHluHG3Yhw
b6Ov8mdTs5fYFry83dK5JYFR9N4//1njj/7WDRsJDDRx/RKY3OJU7I0EBooY
AE8igYEMBkBzDIDmPDwApo27weVLU9K4O9Yb7PxwruDSl9DelSqNuxHixt2W
fY8Pdl00Og9jY0OtF98/seM2o+zlwsEnbJTxksAoIoEBMuv6JTCWrnQxRgID
RQyAJ5HAQAYDoDkGQHOeHADTxt1A3vOtc+4VN+72bHnLtHE38lqON+721G4S
nIq+rrOxxt0k245/7WjcYu8kk8AoIoEBMirFAQwJDFQxAJ5EAgMZDIDmGADN
eW8AzBt3X1wobtztLy+WbNztWP2AoHG379CNelt7jbsdVb8R3HbUdnCRuHG3
6cyrgtuO6quXHTtSYvs8k8AoIoEBMiq19yCRwEAZA+BJJDCQwQBojgHQnJcG
QL1xN7R1fde6ZVKNuztXCG47urJhXnzjblfpsgw37na1HUjauBvdzu1/uCdQ
Oa62+iQwitbk/FnTPX/rho0EBjqIvwTGV5ybEyvlzcnJLSi2cVkMCQwUMQCe
RAIDGQyA5hgAzXljAFLSuNtXWiRu3A2fLo+8VrimRNC4G/lRfONu//nPxI27
gy3HE96LTOPu1KKYmOHBztqjCwSXvrRceGds7POnk8A4iAQGyKTrl8Dk5CT/
QiTLNyeRwEARA+BJJDCQwQBojgHQnAcGIAWNuxV7xI27PUUFko27Y+FQ9KhM
G3d7jr2T2sbdiLbaDy017pLAOIgEBrB5Z5DPV1yQOylIiV7HIopQrr+SiLUQ
hgQGihgATyq5rr6+XvAwVl9zDIDmGADNZfUAZKBxN/DSgzKNux2rHxi6fCp6
VNHG3ZZ3v2UUv3R8OsdG4273xQ2CU9HXdbamfJZR9nJy951ttR9OfRYJjIPW
fPPPm/7t79ywkcDAITdSEfkE5vrNRBYzlBtPS8hqfBN3JNnKYEhgoIgB0Bmr
rzkGQHMMgOaydwBS0rjbljfDKHtpnZsj07jrf+nrvQfWxI5qonF3w/1G2Uvr
mh9kvnG37vgLw4PJn04C4yASGOjNF39VimQCI3Mhi52m3RvHYiGCIYGBIgZA
Z6y+5hgAzTEAmsvGAXBP427nh3Odbdy90rpX0Lgb+VFX2wHB00lgHEQCA435
ErIUmdREfPVLPBvfNx3bufxzSWCgiAHQGauvOQZAcwyA5rJrAFLSuNtf8WlK
GncjD4gdWOYbd8O9jb7KnwkufWk+9+bocGLgk4AExkEkMNBVkijFPIFJeNKk
7y+afCeRvQwmtn/pS2hIYKCIAdAZq685BkBzDIDmsmgAUtC4W1maksbdnp0r
sqtx10hk9QPtB+0tBwmMojXf+vPLD/ydGzYSGGTOlKxEMvaYdM1M8nxlckZj
9V4kEhhkHAOgM1ZfcwyA5hgAzWXFANC4G9PXdfbc/oetNu4mFR5o21/65N7d
01tbdllfExIYVSQw0IxB+CIVe8QHMILrW2xUyyQ+lbuQkDEMgM5Yfc0xAJpj
ADTn/gFISeNu+5L7ZBp3A2/eY3jb0fJvu7xxt/boAqPG3cQjGRuqu7SuZO+M
vbunR7YDZQ8MDV2xui4kMIpIYKCPqeFLTm5xfBeMSVwinazEXwcT/0DTK1xi
ryAf3JDAQBEDoDNWX3MMgOYYAM25eQDCVVVNd9whbtxtuvcb4sbdnqICqcbd
bS+nsHE38piE95LWxt0zJQ/0BColz2p3d82hg7Oi2UtsO3d2hdXVIYFRtOY7
/+/lh77sho0EBuk2+QahifRl4t9KJzBGuUoScfucdDWL+MuO7HwtNgkMVDEA
OmP1NccAaI4B0Jw7B2Csp6dj2TLBbUfND/6k7dnHxI27A0d2yTTuDpzeKW7c
HfRVxA7MtHE33HQo4b2MhjtVGneHBvy1RxcILn1pufDO2Jjh0+ONjPSdO7si
IXuJbVeunLS0RiQwikhgoI8b3zSUe6M/Vz6Bkb9YxjiCiUtxJg7iRotvccGN
63MsFfiSwEARA6AzVl9zDIDmGADNuXAAenfuFDfudryYl5LG3dGuFnHjbmjv
SguNu5WrbDTuhoNnBKfCtHF3oEf280IweLSs5F6j+OVA2QMkMBlGAgN9+Apy
Jn15UZRssBIfnphGJMYPTvgC7KksfoESCQwUMQA6Y/U1xwBojgHQnKsGYDQQ
EDfuti9a0Pr4PaaNu61zc4yyl/Yl98k07gYLZ8o37gY2P5a0cbd59yOCxt2u
mnWCU2HauNtev1HyrA4NXTlRlWeUvUS2Wt97IyN9VheLBEbR+9/7i+aZf++G
jQQGzpBNYIwua7G8V1EXsPXvryaBgSIGQGesvuYYAM0xAJpzzwCYN+4ueU7c
uDtQuTvw0oNG2UvL7K9krHE3eOJ1wW1H/gPzVBp3646/INm4G9FQv0Fw6UtZ
yaz+vibLS3UNCYwiEhjozkYCI1HTYvboa7cdxQUxya7OkUICA0UMgM5Yfc0x
AJpjADTnhgEwbdxtf36+SePuto9d0rjb17w/fY27kR8pNu7GZS/3Nl/+TGX1
SWAUkcBAd5LJioUe3oS9Wr+uxQoSGChiAHTG6muOAdAcA6A5ZwcgM427/ZUT
f2l3Q+Pu2HCv0alIbePuxQurBbcdnTr5i+jXT5PAOIgEBrojgeEvYHpjADwp
9pshPz9f8DBWX3MMgOYYAM05OACmjbuBJQtMG3c733hKcOlL17plko27V0c+
DzeyvXE30H7wQNkDgsbdNn9p7MEkMA56/wd/IRjdTG7z588/acDpkwRP80oC
I3BMqKioSPwAeBsD4Ek3fl3l5goexuprjgHQHAOgOUcGoGrPnpq77xZc+nLu
4Yd8D003yl7qfvjls68sPlf48uU5/2z0obLh2R+e3LYh8lrnP3qxZdk/G2Uv
ja/fd3Lf5uhRHT96+MKWxYLbjurWP3SifFvCe6ku/+PFLfcbZS+1RT84s+dX
onNxqKhq1/2Gl77s+Ndj+38reVaPHi3dX/qk4NKXA/sXHz1aHv8U09WX+YhB
AmOPexIYroGBM7ySwNjewzH+JzC9MQCeFPvNwDUwEGAANMcAaC7zAyBu3G26
fbpp4264qiwljbt9hz6MHdVE4+76u7K6cbdk7wyj7OXYkae7u2umPktl9Ulg
FJHAQHfONPGmDAkMFDEAnkQCAxkMgOYYAM1lcgDEjbu1N91k2rjbu3uTaePu
SGudaePulQ3z4ht3r+x5Qdy4OzaU2N+SLY27JXtnNDVuMnouCYyD1k7/i5bH
b3bDRgIDZ6Tl26gtXjGjgAQGihgATyKBgQwGQHMMgOYyMwAyjbv+p34quPQl
+Gq+eeNu+eZxi427fWc/ETfuDrYcT3gvpo27wROvZ6xx93zNSsFtRyeq8qKN
u0ZIYBxEAgPdySYwcaGKpQQmvTchkcBAFQPgSSQwkMEAaI4B0FwGBiC0dau4
cbc9b55q4+57ealv3D32jtXG3ebdj2Sscbe1ZVdZyb2Cxt1A+0HTnZDAOIgE
BrqTvl/Iyo1FGauBSd7Ea2kP/AVMcwyAJ5HAQAYDoDkGQHNpHYDhy5dbZs4U
XPrS9rMnWx690yh7aZh+c/f61b271rXOzTHKXtryZgxdmvjGlt4Da/wvfd0o
ewkWzhz2X4ge1dXRoZ7KVYLbjjo+nTPS1ZDwXsLBM827HzHKXuo++W5XzTrB
qejrOntu/8NG2cvJ3Xe212+UPKvhgbYTVXmCS19qfe+NjPTJ7MrG6qt/6EAU
CQx0Jx2sWLizKHMBDNfAQBUD4EkkMJDBAGiOAdBc+gZA3Lh7+Uc/Mm3cHajc
Y9q4e3V4MNsbd2uPLkh3464RroFx0Np//ULLnFvcsJHAwBnyl7bEPVK6MibN
LTAkMFDGAHgSCQxkMACaYwA0l44BCFdVNX7/+0bZy6VbbjFt3O3bu9W0cXe0
o2UsHOressTZxt3IAwSnoqvtQGYad8tK7m2o32B1pUhgHEQCA91ZuLkoPlkx
vrYl/lqZtAcwJDBQxQB4EgkMZDAAmmMANJfaARjr6Qnk5ak07na++bJ5427l
xN+sXd64OzzYadq4O2r89HimjbtnTv9S3Lhr5MjR8hO+90bHhm08lwRGEQkM
dGez3mXatJykDy/Ojctf0h/AkMBAFQPgSSQwkMEAaI4B0FwKB6Bn40Zx425g
yQJB425L7o8HKkuDr84WNe6uW5byxt1Q9drMN+72dZ2VPKtt/tIDZQ8IGneD
waM2Fiuise1AUclDH++7+9SlD2w8nQRG0do7v9A69xY3bCQwcIaVBGby9S3R
FMZ3/VoYn29y+pL+DpgJ05KxtAf+AqY5BsCTSGAggwHQHAOguZQMwFBtrbhx
t33+s62P3yNu3A1tezfDjbvB4qfT0bhbUz5L0LjbVvuh4OmTjkSicVfye6sT
9zzUtf/kso/33R3dikrv7+qtk3li0k8cJDD2kMBAd5YSmKkZjLFM5C9cAwNl
DIAnkcBABgOgOQZAc4oDcHVw0Lxx98WF4sbdcFWZaeNu5LVS2LjrX3dH0sbd
jqrfqDTuNp97M2ONu6HQJevLNeFC06eb9j8Ui1+i295jC63uhwRGEQkMdGcx
gRlPuBfJSPrvP4oigYEiBsCTSGAggwHQHAOgOZUBMG3cDeT/3LRxt/uj5TKN
uz3bXk5r425v0x5x426//7DgVGSycbepcZONxYoI9lzYdWReQvYS22qbd1ja
GwmMIhIY6M56AjM+7isWXgmTmatfokhgoIgB0BmrrzkGQHMMgObsDcBYT0/7
c88JbjtqfeQRdzbuDrWdTngvI31+ceNu5+m3XNK4e+rkL+w17g6P9J269IFR
9hLZ9p9cFh7qsrRPEhhFa3/0l61P/3c3bCQwcIadBOYan6+4IDcnPonJyckt
uNELkxkkMFDEAOiM1dccA6A5BkBzNgbAtHG348U8QeNu88zbTRt3uz9aPtYf
GusNOtu427LvcQ807vqDVVvLHzXKXiI/ijzAxm5JYBSRwABZjQQGihgAnbH6
mmMANMcAaM7SAJg37i5a0DrnXpXG3fYl90k27g5dPhU9KtuNu007HjTKXuo3
T1dp3D2x4zY3NO72hdvjG3enbjvKXxwe6bOx53ESGGUkMEBWS3oblKU98Bcw
zTEAOmP1NccAaI4B0JzkAJg37t71Y9PG3YHKPZKNux2rHxA07vYeWBM7MBp3
jdQ0FBWV3m+Uvew6Mi/Yc8HGH3/1Dx2I+v1df+l/9h/csJHAADZwDQwUMQA6
Y/U1xwBojgHQnMwAqDfu9m7f2FNUYNK4290x0bi7c4XgtqPOD+cqNu72Ne9P
a+Nu5AGSZz6tjbt7jy00yl6KSu+vaSiKPlLljz8JjCISGCCrkcBAEQOgM1Zf
cwyA5hgAzYkHQKZxt+3ZxwSXvgRfzTdt3B2o2ht5rXBNSVobd0fDnWlt3G0+
96bjjbujY8Omjbt94fbY40lgHEQCA2Q1EhgoYgB0xuprjgHQHAOgOcEAuKpx
dywcih5Vmhp3B7suCs6SuHG3pnxWVjTuNk65PocExkEkMEBWI4GBIgZAZ6y+
5hgAzTEAmks6AFKNu4/fky2Nuy37Hvd24254qOvw2d8KLn05demDpI27JDAO
+v09f+mf/w9u2EhgABtIYKCIAdAZq685BkBzDIDmEgZAqnF3yXPixt1wVZlL
Gnc7T7/l+cbd2uYdm/Y/ZJS9bD88N9hzwei5JDAOIoEBshoJDBQxADpj9TXH
AGiOAdBc/ACIG3drb7qp/fn5Jo27uzeZN+52tGSgcbfffzhbGncb6jdYWbG4
g+ytEzfunrr0wejYsGAPJDAOIoEBspr6F8PxFzDNMQA6Y/U1xwBojgHQXHQA
TBt3mx/8iXrjbn9l8bhE427kAbHD6z//Wcobd4MnXhc07g4N+FPYuHvxwmrB
bUcnqvLS1LhbWr041N9suh++jdpBv7/vr/yL/tENGwkMYAPXwEARA6AzVl9z
DIDmGADNRQbAtHG3fclC08bdzjeeElz60rVumUzjbs/OFYqNu90XN9Rvnm6U
vTTvfiQcPCM4G+LG3QsHn5Bv3A20H0xf4+62g7ONspdN+x+qb90nuSuugXEQ
CQyQ1UhgoIgB0BmrrzkGQHMMgM6Gamtr7r5bcOlL27yfmTbu9u5aJ2jcbcub
IdO427H6gYTG3ZZ3v2UUv3R8Omdq4+5g10VB427dJ981bdw9t/9ho+zl5O47
5Rt3h4aumDbujiTrxTVl2rh7pGZl0sZdIyQwDiKBAbIaCQwUMQCeVHJdfX29
4GGsvuYYAM0xANoSN+42fu97KWzcDbx5j1H24n/p64mNuxvuN8peWtf8YGrj
7tXRIcXG3aYzr2ascbe7u8byUl1j2rgb6BJd3pMUCYyDfn//X/nz/tENGwkM
YAMJDBQxAJ4U+82Qn58veBirrzkGQHMMgIYy2ri77WX5xt2u0mUpb9ztbdoj
OBVXWveKG3d7ApWSZ7W7u+bI4SfS0bgb6m8WN+7WNBSJG3eNkMA4iAQGyGok
MFDEAHgSCQxkMACaYwC0MtbTE8jLEzfu+n/2aEoadwdO7xQ37g76KmIHZtq4
O9hyPOG9jIY72w4uElz60lH1G5XG3ZYL77i/cXf/yWUyjbtGSGAcRAIDZDUS
GChiADyJBAYyGADNMQD66N25U9y427bg6ZQ07o52tYgbd0N7V8o37vYce8dq
427Tjgc937i7tfxR+cZdIyQwDiKBAbIaCQwUMQCeRAIDGQyA5hgAHYwGAi0z
ZwouffE/npuSxt2rI0Pixt1g4Uwad02lvHHXSOSPv6/tsL3nksAoWvvvf926
+J/csJHAADaQwEARA+BJJDCQwQBojgHwPHHj7oVvfCMzjbtty7+dgcbd4VBi
YhOTycbdI4efSF/jbrDngr09J7jS3/r7soWr9s4802znWhoSGEUkMEBWm5aM
pT3wFzDNMQCeRAIDGQyA5hgADwtXVV02/rLpaOPuJePbjiw17nZvWZLCxt3I
YxLei2njbqi+WHAqMta4W7J3hgsbdxNEdnK4duOqvTOjW2HZnP6hbpknJv3E
QQJjDwkMkNW4BgaKGABPIoGBDAZAcwyAJ4319AR//Wtx427rkw+npHG3v3pr
Cht3w02HEt6LaeNu4OjLio27Y2OJPTNJZW/jbry27tp1Fc/G4pfotuvM76zu
hwRG0doH/1oQWmZyI4EBbCCBgSIGwJNIYCCDAdAcA+A9/aWlmWncHemoN23c
vTryebhh3rhbuWpq425Xzbq0Nu4O9Pgkz6pp427kATYWazwjjbtRgyP9+869
m5C9xLbG4ClLeyOBUeSeBGb+/PknjTl9ngCXIoGBIgbAk0hgIIMB0BwD4CWj
gYD/8cdd0rg77P+8riTauCu47Siw+bGpjbvh4BnTxt2piU2MaeNue/1GybNq
2rh78cJqlzfuRtS0Higsm2MUv/zh0KK27lpLOySBUeSeBIZrYAAbSGCgiAHw
JBIYyGAANMcAeEZXYeGlW24xyl4av/c908bdgcrdKWnc7Tt04+uEJhp3199l
qXF3bLjXtHF3pM9vdB5MG3frjr+gW+PulqpfGWUvq0tmHanbZKNehgRGEQkM
kNVIYKCIAfAkEhjIYAA0xwB4wFBtrWnjbtPdOYLG3dC2j1PSuHtlw7z4xt0r
e15IeeNub9MewamgcTdeZCdH6jatLpllFL/8vmzhlf5WezsngVG09qEvtr74
FTdsJDCADSQwUMQAeBIJDGQwAJpjALLa1cFB08Zd/7xZgktfzs+flZLG3fbX
7oxv3O07+4m4cXew5XjCexkNd/oPzBNc+tJR9Rsad+U1Bk/94dAio+ylsGxO
TesBlT/+JDCKSGCArEYCA0UMgCeRwEAGA6A5BiB79ZeWNvzLvwgad9t//qxp
4+6lFx/OdOPusXfc3Ljb5i/1duNuZIv8NPr10yQwDiKBAbIaCQwUMQCelH9d
SUmJ4GGsvuYYAM0xANloNBBoe/JJUePunNmmjbuh4jUpadwd6aiPHpVp427H
p3PsNe4KTkUKG3fDA23ixt1a33vZ3ri7ruLZ+O88IoFx0NqffrH15a+4YSOB
AWwggYEiBkBnrL7mGADNMQBZx0WNu0f/GDuqcNMh/7o7vNq4e+zI07Ybd+tb
92WmcTcU7vjsxApB4+7h2o0J9TIkMA4igQGy2rRkLO2Bv4BpjgHQGauvOQZA
cwxAFhk8e9a0cffyv38nA427XUWL4ht3O3cuEDfujg0l9reYNu72Ne8XnIrU
Nu4eOjjLKHspK7k3fY27Z+o+SknjbsTxhs8Ejbtbqn6VtHHXxh9/9Q8diCKB
AbIa18BAEQOgM1ZfcwyA5hiArKDeuNuxfKF542755nGLjbu9pz4SN+4OtZ1O
eC+mjbvBE69nrHH3fM3K9DXuFpXebxS/lFYvTlXjblt3rWnjrtFzuQbGQSQw
QFYjgYEiBkBnrL7mGADNMQDu119aWvfVryo27na+8ZSocfe9PKuNu8PBi+LG
3VD1WquNu827H3FP424weNTGYo1fa9zdfniuUfayaf9DKWzcLTu/VtC4u+vM
76KNu0ZIYBy0duYXW5ff6oaNBAawgQQGihgAnbH6mmMANMcAuNloIOB//HHF
xt3e3R8KGncbnv1hyht3g8VPu7lxd2joSvoad4/UrHRD4+77B56Kb9w1QgLj
IBIYIKuRwEARA6AzVl9zDIDmGADXUm/cDR8rM23cPV55KIWNu5Ef2WvcHQ0b
VuZ6pnE30CW6vEeeuHE3sk1t3DVCAuMgEhggq5HAQBEDoDNWX3MMgOYYABfK
TOPuSGvdWDh06T3RbUeON+52tR1IYePukcNPeLtxt+jYsrbuWvm9kcA4aO2j
X2x95VY3bCQwgA0kMFDEAOiM1dccA6A5BsBVxnp6TBt3W5982LRxt23Bbdne
uDs82JnCxt2LF1Z7vnH3eEPi1UemSGAcRAIDZDUSGChiAHTG6muOAdAcA+Ae
5o27eeaNu1dWz9ehcbev66zkWQ0Gj9K4a4QExkEkMEBWI4GBIgZAZ6y+5hgA
zTEAbjAaCLTMnCm49KVt7hOKjbtteTMy07g72HVRsXG3pnyWoHG3rfZDybPq
jcZdX9vh9w88JWjcbblis7hmnATGUSQwQFYjgYEiBkBnrL7mGADNMQCOu7Jq
VQYad68OD6awcbd1zQ+mNu5eHR0ybdwd6fMbnYfR4d7mc28KbjuqPbrAUuNu
Wcm9jjTu+oNV9vacIIWNu0ZIYBy09rH/1vrbr7phI4EBbCCBgSIGQGesvuYY
AM0xAA4KV1WZNu423fuNlDTudm9ZIrjt6MqGeelu3O1t2iM4Falt3D125Gmj
7KVk74z0Ne6euvRBxhp3O0KN6q9CAuMgEhggq5HAQBEDoDNWX3MMgOYYAEfI
NO76f/ZoBhp3L//qO5Yadwdbjie8l9FwZ9vBRYJLXzqqfqPYuDtq/PRJZ3Vs
qNb3niONu3uPLcxY4+7Jpp0peaFxEhhHkcAAWY0EBooYAE/Kv66kpETwMFZf
cwyA5hiAzMtY4+5oV4tp4+7xI4ejR2XauNtTuWpq4273xQ2Cxt2mHQ+6p3E3
0H7QxmKNu6lxd/up1+017hohgXHQ2tz/1vraV92wkcAANpDAQBED4Emx3wz5
+fmCh7H6mmMANMcAZJJ54+7PnjRt3A0VrzFt3I28lmnj7rD/wvi1ATBt3A1s
fiwdjbsXDj5hlL2c2HGbpcbdUyd/Ibj05XzNyvQ17kYeY2PPU5k27jYGT6Xk
heKRwDhIgwTGX11cuHRpbszSpUsLi6sNq6CArDItGUt74C9gmmMAPIkEBjIY
AM0xABkjbtxtun26eeNuVYoadw/dCDdO7f195ht3Wy68k6rG3abGTYLG3UMH
Z9G4K2Djj7/6hw5EeTyBqS6Mi14SLC0mhUH24xoYKGIAPIkEBjIYAM0xABmg
3rjbu3uTZONuz7aXU9i4G3lMwntRbNztCVSKG3e72g5IntVQ6BKNu4r2V5Zu
PbO+dzBk47kkMIrWPv6l1pX/7IYt9QlMdaFh+kIIA68ggYEiBsCTSGAggwHQ
HAOQVmM9PR3Llpk07j71U8GlL8FX8weO7PI/8x3Txt2B0zsFjbuRH8U37vad
/UTUuLv+rnDToYT3ku7G3eZzb6awcTc80GZjvUwbd0urF2dj4+5UI2Mj5XV7
Xto1b9mup7eeWW9jDyQwirybwMTnL0sLq/3Xs5ZrNyWRwcArSGCgiAHwJBIY
yGAANMcApE/vzp1mjbvzTBt3O994KiWNu1dHPi/RHelqEDTuRrY0Ne6e3H2n
UfZSUz4rhY27rS27bCzW+LXG3W0HZ3uycTdBwxXf7yp+uWzX07Et8m+s7oQE
RpFnE5gbAUyykCXu7qTC6lS+LJBhJDBQxAB4EgkMZDAAmmMA0sG8cXfez0wb
d3t3rUth4+74tf4Wbzfunju7wnbj7uGzvxU37g7b2vNUjjTuxoRHBraeWR+f
vUS331X8cmRsxNKuSGAUeTWB8RcvFScssYSGq2CQ1UhgoIgB8CQSGMhgADTH
AKTclVWram+6ybBx91//1ZHG3XDTobYN9wsad3tPfZTwRjRp3K1t3iFu3A10
iS7vkedg427UiZbKV/Ytmhq/RLfyOlGBz1QkMIrW/q8vta76ZzdsqU1gTAOW
WERDAoOsRgIDRQyAJ5HAQAYDoDkGIIXCVVVNd9yh0rjbt3drOhp3u0qXCS59
6dy5INsbd8fGEm+bktqzWePumbqPvNG42zXQub5qtVH2Etn2+T4LjwxY2icJ
jCKvJjBcAwNNkMBAEQPgSSQwkMEAaI4BSAmZxt22Zx9TbdytLI68VrimRL5x
t//8Z4LGXf+6O07t/X3Ce1Fv3K07/oLg0pemM69KNu6OjPSZNu729zXZWK9o
467gtqNMNu4eb0j8vu/UKq/b88s9zxllL+9VFjR3J956JoMERpFXE5i4CCZZ
xCL+KZA9SGCgiAHwJBIYyGAANMcAqDNt3A28+IJp427w1dmixt11y8b6Q2O9
QUuNux2fzjFt3E0YAMXG3fb6jYLG3XP7H05h426bv9TeemWycbf84nrBbUef
nViR1sbd5u6Gtw+9YpS9vLJv0eYKm1/YPU4Co2ztnC+1/O5rbthS/m3UcV+G
NOm7kPxxNbz08CLbkcBAEQPgSSQwkMEAaI4BUGHauNs+/1nTxt3QtndT0rg7
dPnzBteJxt1j77S8+y1B4+5w8GL0wbEBUGzcHejxpbBx90RVnuDSl/M1K93f
uFsXOO5s427xuY2C244+OfX73sGQyh9/EhhFHk5gJr54Oi5rmWoil0n1SwIZ
RgIDRQyAJ5HAQAYDoDkGwDb1xt2Byj2mjbuRFxq6fKpj9QOCxt3eA2tiRzXU
dtpS425kABQbd8fGhkwbd4cGZD9vNdRvyPbG3f6hbtPG3cGR/pS8VlLn20+9
WrbYKHuJ/Ki24/NzSALjIE8nMOMTIUxx0hRmaSF3H8ETSGCgiAHwJBIYyGAA
NMcA2CDTuHv5J98VNO6Gtn1s2rg72tEy0bi7c4XgtqPOD+fGGnfHhnptNO6e
2L8urY27V1r3Sp7V7u4a08ZdG4s1LtG4W9NQlLHG3bbu2pS8UFK9gyFx427x
uY3xjbskMA7ycgJzo+yFi2DgWSQwUMQAeFLJdfX19YKHsfqaYwA0xwBYItO4
2/78HMGlLx3LF6apcde/7g5B42646VDCe8muxt3wQJuN9TJt3N1/clkKG3c/
PrLYtY27bx96ZWrjLgmMg95/4kuCvDST2+LFi0/KkXpj8fHL0sLiG1HLtR4Y
imDgFSQwUMQA6IzV1xwDoDkGQJ5p425wxcumjbudbzwluPSl+6Plko27Y+FQ
9KhGQ63B4qdNG3cT3ot7GncD7QcdadzdWv6olxp3f1fxS0Hjbnld8guZSGAc
5KoEJnVvKy5/Sf5lR3EPIINBNiOBgSIGQGesvuYYAM0xADJMG3cDec+bN+4W
r5Fs3G1b/m2Zxt2IUPVaycbdGBp3tWrcXV+1uncwZPR0EhgHeTOBkfmyab6Q
Gp5AAgNFDIDOWH3NMQCaYwBMiRt3L9/148CS5xqm32wUv/ifeSh8rEyycTdY
ONMoe/G/9HWVxt3xa9+RlC2Nu0cOP5G+xt1gzwV7e07geOPuiZZKcePu+XaT
8IcExkFr5n7p8jtfc8OWygQm9kXUonDlRgTDVTDIXiQwUMQA6IzV1xwDoDkG
QEDcuHvpllvan5/fPPN2o+ylccat8o27ob0rLTTuHlgu+F+0kzbu9vsPixt3
Q/XFglNB424CcePux0cWp7Vxt6OvXdy4u8/3WXzjrhESGAd5MoGRzFakchrA
3UhgoIgB0BmrrzkGQHMMQFKpatxtW3CbaePuoK9C3LgbrimJHVg6GncDR192
T+Pu0NAVG+vlnsbd1SWznG3cfa+yYGrjrhESGAd5MoHhGhjoY1oylvbAX8A0
xwDojNXXHAOgOQZgKvPG3eVLxY27/RW7rqyeL7j0peu9PJnG3Z6dK2KNuyNd
DTYad3tqN4kbd6vL/yg4FSls3A0Gj4obdwPtB20s1jiNuxKNu0Zs/PFX/9CB
KG8mMPTAQBtcAwNFDIDOWH3NMQCaYwDiqTfudr2/snf3h+qNux2rH4g17l4d
HbLRuDscajBt3I3s2WgAMtm4e/HCag807kYekJIXSsq0cffjE4WCxl0jXAPj
oDVP3nS58Otu2Bz7LiQCGGQzEhgoYgB0xuprjgHQHAMQI9O4K7jtSLJx9+rw
oBsad4dDn9+oMnUAaNxNYNq4W35xfVobd8/4qwSNuysP/KK2w+Y5JIFxkEcT
mLgbkSYylsLian/sl4W/urhw6Y0fcgsSshoJDBQxADpj9TXHAGiOARiXa9xt
uvcbpo27LbO/ImjcHWmts9y4W7rMRuNuY/F98o27CQNA424CZxt3uwY6TRt3
R8ZGbO+fBMZBnk1gJjKYuJzFwFLyF2Q5EhgoYgB0xuprjgHQnOYDINO42/bs
Y+aNu3kzjLKX1rk5/eWbxzPSuNt+ON9q425sAFLYuDs2NkTjrjrTxt22kOo7
JYFx0Jqnb2pa83U3bClPYMYn7jUSpDBLC7n9CNmPBAaKGACdsfqaYwA0p/MA
mDbuBpYsEDfuDlSWdr2Xl/LG3Y5P56S8cTccPJP0JEQHIJONu23+UnvrReNu
rHH3cIPNc5iABMZB3k5gJviv3XUUH8QsnXxXEpDNSGCgiAHQGauvOQZAc3oO
gGnjbvv8Z8WNu93rV5s27g6ePzJ+rXHX/9LXJRt3e469k6bGXaNTcfzw1hQ2
7p46+QvBpS/na1a6v3HX13bYe427RkhgHOT9BAbwNBIYKGIAdMbqa44B0JyG
A2DSuPujH4kbd9sWPGbauNuzZVW0cbdj9QOSjbuDLcfFjbt9Zz9JeCOWGnen
Sm3jblPjJkHj7qGDs9zfuBsKd4gbdw/XbnSwcTfyI9uNu0ZIYBxEAgO4gK8g
59pvsdxi88dORgIDRQyAJ8V+M+Tn5wsexuprjgHQnFYDYN64u+AZQeNu5Ee9
uzf1FBUIbjvq+PWj0cbdnp0rUtW4e2XPC+qNuwlS2LgbCl3yfONu0bFlWd24
a4QExkHvPf03je/nuGEjgYG2inOv/xYjgUHGMQCeRAIDGQyA5jQZAJnGXf9T
PxVc+hJ8NX+icXfBbaaNu+GaEkuNu61rfmCUvbStvytVjbsxGW7cDQ+02Viv
aONuUen9RvFLafXiFDbu/uHQIqPspbBsjgcad42QwDiIBAZw2I38hQQGDmAA
PIkEBjIYAM3pMAAyjbsN028WN+52vvGUGxp3uy9usNe4G5UtjbvbD881yl42
7X8ohY27ZefX6tC4a4QExkEkMICj4vMXEhg4gAHwJBIYyGAANOftAZBp3G15
9E6Txt1d6zLfuDvSldjfMth1UaVxd6DHZ9q4OzZm+PR4aW3cPVKz0iWNu43B
Uyl5oaQy3LhrhATGQYXP/E3D73PcsJHAQEOT8xcSGDiAAfAkEhjIYAA05+EB
SEHjbpVJ425o27veaNwN9zZKntX0Ne7Wt+4TN+4GukSX98jTsHHXCAmMg0hg
AKdcr9/NKSj+/J9IYJB5DIAnkcBABgOgOU8OgHnj7vPzxY27fXu3iht3g6/O
TnnjbuSnSRt3Gz6dodK4e6bkAUHj7rEDv5M8qw427p6p+4jG3XQggXEQCQzg
jBv5iy/2zyQwyDwGwJNIYCCDAdCcxwZAvXH3yjsrBo7s8j/zHaPsJfKj1Dfu
brg/aeNu28FFKo279dXLBJe+NJ5aPjrcKzMANO6mhIONu4aHdKTij6e328t8
SGAUFc77m4YPctywkcBAJ5O/fpoEBs5hADyJBAYyGADNeWkAQlu3iht3O17M
a5xxq1H20pL744HK0uCrs0WNu+uW2WjcDWx+zCh7aXn3W+lo3O1o3CLZuGs6
AOlr3A10nXFJ4+72U697u3E3qYrG4wu3//qZbS9tPrfbxtNJYBSRwACZd73+
5XrgQgID5zAAnkQCAxkMgOa8MQDDly+bNO4uWtD6+D3ixt3QtndbZn9F0Lg7
dOnkuNXG3cpVVht3w8EzKo274d5GS427ggFIX+Pu8EifaeNueKjLxp6ncrxx
d9eFTYLbjtZXrc5A426Chq7m1yref2bbS7GtucfyJUwkMIpIYIBMS8xfSGDg
JAbAk0hgIIMB0JwHBsCkcfeuH8s07rYvuU/cuBt5IXHjbtvyb8c37oabDokb
d/vPJ97zMjbcm/nGXaMBcLBx1x+ssrfnBDKNu6mql0nqfPsplzTuxgwMh7ed
L4nPXqLbigPvWt0VCYwiEhggSSQiw+crLsjNyYn7HZSTk1tQ7PNZfzESGDiH
AfAkEhjIYAA0l9UDEK6qavz+91Uad3u3b+z+aLm4cXe0o8VS4+5ofzAdjbu9
TXsEp8K0cTd4eXvSJ04dgLQ27pZWLxY07p669EHGGnc7QrJf/2RD72DItHE3
PDKQvgNIqiZQm7/ntanxS3Qrq6u0tDcSGEWFz/1N/fpvuGEjgYFDbnwjtHz+
EYtMkpoo1zV5sckvRQID5zAAnkQCAxkMgOaydADGenoCeXnixt325+cILn0J
vppv2rg7ULV3XKJxd9BXETswe427/gPzBJe+dFT9Rr1x1+jp8QOQ1sbdM3Uf
CRp39x5bmLHG3ZNNO1PyQkZMG3ebuw0vZEqTgeHwH05+apS9RLbITyOPsbRP
EhhFJDDQm+9G/iKdf8Q/xVCyfcV//VHSH5DAIPMYAE8igYEMBkBz2TgApo27
weVLBY27zTNvN23c7f5ouUzjbmjvSsXG3a6adZlp3DUSGwAad9U1dze8fegV
QeNueZ3oQqY0OXL55M93rTDKXn5V9lZt0E4iRAKjiAQGGvMlZCky+Yf46pd4
iTmLYf5CAgMnMQCeRAIDGQyA5rJrAEwbdwN5z8s07rbOzTHKXgIvPSjTuBss
nKnYuDvYddG0cVdwKqw27hqJDACNu+rc2bjb3htcdfgDo+xl/vbla8s+tr1z
EhhF787/m7qPvuGGjQQGmZUkSjHPPxKedK33JfYzX3Hu5F1Oylrko5vkKY0B
EhgoYgB0xuprjgHQXBYNgHrj7kDl7sBLDzreuHt1dMi0cXekz290Huw17hqp
KH+dxl1FLmzcHRkb2Xa+ZP725Ubxy2sV77f3BlX++JPAKCKBgY6mZCWSCcyk
a2aSpySTg5a4PZLAwJ0YAJ2x+ppjADSXFQMg07h7+SffFTTuhrZ93FNUYNK4
292RFY27p/feY6Nxd6q0Nu7uP7lMcOkLjbtpVROo/VXZW0bZy893rahq+fze
NBIYB5HAQDMG4YtUAhMfwEi07U7eJQkM3IkB0BmrrzkGQHMuHwD1xt2O5Qvd
07jbdnCRSuNu3fEXbDfuTjqr6WzcrWkoonHXqcbd0GCvuHF3XfWmyGNijyeB
cRAJDPQxNXzJyS2O74IxSWCkS3vjwxapZhd6YOAcBkBnrL7mGADNuXkAUtK4
2/nGU4JLX7rWLctM4273xQ0qjbvt9RsVG3dj0te4G+y5IG7crW3eYW/PCVze
uPvLPc+5s3G3JlCb8BQSGAe9s+BvL234phs2Ehik2+SrUCbSl4l/K53AWMhV
4vYpdTULCQycwwDojNXXHAOgOXcOgHrjbtf7K3t3fyho3G3Lm5Hyxt2OT+dM
bdwdDjWoNO4O9PhS0rgbYdq4e+7sijQ17h4++1sad9OnvTf4WsX7gsbdHRf3
j4yNTH0iCYyDSGCgj1jOcT18uUY6gZG/WMZyBEMCA+cwADpj9TXHAGjOhQOg
2Ljrf+ah8LEyGncTNDVuSl/j7tbyR42yl20HZ+vTuBt5QPpePalo467gtqPX
Kt7vHDC8HIgExkEkMNCHryBn0pcXRckGK3GXwJiHKpYeTAIDJzEAOmP1NccA
aM5VA6DYuNs441apxt2OlpQ37o4NJRaw9PsPNxbfJ2jcDdWL/r6Xwsbd/r4m
QePu3t3/s6F+g+RVNAlo3I1tuy5scqRx98V9bwgad49cPineAwmMg95Z+LeC
eDaTGwkMnCGbwFi7rMXKBTMkMHASA6AzVl9zDIDmXDIAqWrcbVtwm6Bxt798
83iqG3cHW44nvJfRcGf74XzB553A0Zfd07h7pFI2yZn0Hmncvb69fegVRxp3
3zu2UXDpyx9OfjowHDbdDwmMg9yTwMyfP/+kMafPE7zLRgIjkZNYe7QCEhgo
YgB0xuprjgHQnBsGQL1xt79i15XV80WNu+/lpb5x99g7Uxt3Q/XFKo27bbUf
prBxt6L8YUHjbmvLrnFbA5DJxt3yi+tp3E1Q0Xhc3LhbG5RNhEhgHOSeBIZr
YOAMyazE4vcbWe3itY0EBooYAJ2x+ppjADTn7AAM1dZq1bg7NbGJ6es6a9q4
K3tWh66cOf1LycZdSwOQycbdusBxGncT2G7cNUIC4yASGOiOBIa/geuNAdAZ
q685BkBzTg3A1cFB9cbdgcrdpo27V4cHM9O4W/fJd40+4PgPzBsOGV6WMDrc
a9q4OzRgWNiboPnyZ5Yad+UHIGONu/1D3aaNu4Mj/Sl5raSysXH37SMftV9v
LpJHAuOgtxf9ne+P/+KGjQQGzvBKAiNwTKioqEj8AHgbA6AzVl9zDIDmHBmA
E2vXXvjmN42yl4tf/vK53Ecv3ftNo+yl7o5/PLPqlQtvLhTcdlT3wn0ndn96
/NB+37oFgtuO6n/3cHXFvuhRVR3aV1v0tKBxN/LT45UHEt/L/nUXN99jlL3U
Fv3gVMkq0bmo+KBqxwzD+GXHjOMV70qe1SOV28tKZgkadyvKf3vs2OGEZ8kM
wKEjOz878Jzg0pddFb85eqxS8jjFtlas/t3e/zDKXn5ftrD08GcpeaGk9leW
vlVqeNtRZPvgwOpDRw+m7wCS2lqxffGO3xplL89v/01Ruc1zYrr6Mh8xSGDs
IYGB7rySwNjewzH+N1C9MQA6Y/U1xwBoLsMDMNbT0/7cc4LbjlofeUSmcdf/
zHeMspfWuTnZ0rhbe3SB4NKXpjOvprBxt7+vKelzxQOQ4cbdj48sFjTuHm9I
vPootcSNu+9VFjjSuLuuepN6464RlT/+JDCKSGCgO5p4+Ru43hgATyq5rr6+
XvAwVl9zDIDmMjkAPRs3XrrlFsXG3c43nnJD425P7SZx4+5AQHRLTuYbd40I
BsA9jbufnViR7sbd31X80ih7eWXfInc27jZ0qWZfJDAOWr3o7y4W/YsbNhIY
OCMt30Zt8YoZBSQwUMQAeFLsN0N+fr7gYay+5hgAzWVmAIZqay/ffbdJ4+6c
e8WNuz1b3sqKxt0rZwtd2LhrJOkAaNW4W3xuY9Y17u6prbDUuGuEBMZBJDDQ
nWwCExeqWEpg0nsTUvIeGEt74G/gmmMAPIkEBjIYAM2lewCuDg4Gf/1rk8bd
FxeKG3f7y4vd07gruO2otXSuYuPu8GCn5Im12rhrZOoA6NO4e8ZfJW7cre2Q
OocplKbGXSM2/virf+hAFAkMdCd9v5CVG4syVgPDNTBQxQB4EgkMZDAAmkvr
APSXljb8y78YZS+XbrklkP/zpnu/YZS9NM64NbR1fU9RgeC2o+Crs0da68bC
oZ5tLwtuO+r8cO7Y9c+to/3BrtJlgsbdyE/HhhILWPr9hxs+nWGUvdRvnh6q
F/3FsCdQeXrvPUbZS+RHV1r3Sp7VUOjSsSNPG2UvJXtnNNRvGBszvAgnQfwA
hPqb959cJrj05dSlD0bHhiX3LHa84bPVJbOMspePjyxu665NyQsl1TXQub5q
teDSl32+z8IjA+k7gKRqArUv7nvDKHvJ3/PakcsnU/uKXAPjIBIY6E46WLFw
Z1HmAhgSGKhiADyJBAYyGADNpWkARgOBtiefFDfutj37mLhxt7/iU0HjbuRH
/ZUTfxUbOL0z3Y27bQcXZXvjrpHoALincXd1ySwad1PeuGuEBMZBbz3/5fOf
fMsNGwkMnCF/aUvcI6UrY9LcAkMCA2UMgCeRwEAGA6C5dAxAV2GhuHE3sGSB
aeNu8NXZosbddcvG+kOjXS3pbtztvrhBsXH3xI7bUtW4e6DsAduNu0YiA0Dj
rucbd42QwDiIBAa6s3BzUXyyYnxtS/y1MmkPYEhgoIoB8CQSGMhgADSX2gEY
PHtW3LjbPv9ZceNu9/rVPVveapn9Fccbd8PBM+LG3a6adeLG3ZryWalq3D11
8heKjbtJDY/0bS9f4pLG3cgDUvJCSZk27n58otDDjbtGSGAcRAID3dmsd5k2
LSfpw4tz4/KX9AcwJDBQxQB4EgkMZDAAmkvVAEQbdwXZy+Uf/ag9f57gtqO2
BY/1lxe35c0QN+5GXmvo8qnAm/ekr3F3bLhX3LjrPzBP3LjbfO5NceNuuLdR
8sQ2NW5KSePuVBlr3A2FO8SNu+UX19O4m9bGXSMkMA763c+/XLPpW27YSGDg
DCsJzOTrW6IpjO/6tTA+3+T0Jf0dMBOmJWNpD/wNXHMMgCeRwEAGA6C5lAxA
f2lp3Ve/Kmjcbc9bIGjcjfyod/vGrnXLxI27o90dGWjc7WveL2jcjfxI3Ljb
1XZA3LgbvLxd8qymtnF30p5p3NWscdeI1T/+ST9xkMDYQwID3VlKYKZmMMYy
kb9wDQyUMQCeRAIDGQyA5hQHYDQQ8D/+uKhx9+GH/U/9VHDpS/DV/L7SInHj
7kDVxPcEZaBx139gnuDSl46q36g07jaeWu5g4+7n7zGzjbt/OLTIKHspLJtD
427GGneNcA2Mg0hgoDuLCcx4wr1IRtJ//1EUCQwUMQCeRAIDGQyA5lQG4Mqq
VaaNuw3TbzbKXlpyfzxQsUfcuNtTVJCZxt2umnV1n3xX0LgbDp4RnAr3N+5O
7DmDjbtl59e6tnH3l3ue061x1wgJjINW5X357Ob/4YaNBAbOsJ7AjI/7ioVX
wmTm6pcoEhgoYgA8iQQGMhgAzdkbgHBVlWnjbvNPp4sbd0Pb3hU07gZeejBj
jbvNux8RN+4KToVp427rxfcl7xUaGrpyoipPcOnL+ZqVtht3j9SsFNx2VHH6
V6lq3PW1HRY37jYGT6XkhZIybdxdX7Vaw8ZdIyQwDiKBge7sJDDX+HzFBbk5
8UlMTk5uwY1emMwggYEiBsCTSGAggwHQnNUBGOvpMW/cXfy0uHE3XFUmaNxt
nZtjs3F3/V0pb9wdDXcanQrTxl1f5c/kG3cb6jc40rhbVPJQqhp3r/S3iht3
D9dudLZx93x7GsOfpKKNu/O3LzeKX1Yd/iADjbtGSGAcRAIDZDUSGChiADyJ
BAYyGADNWRoA88bd558VN+727d0qbtztfOMpG427V/a8YLVxt99/WNy429e8
X3AqxI27J3ffKd+4291dY9q4K79G8frC7aaNu0ePVdrbebzRsWFx427RsWXO
Nu7uurDJkcbdX5W9ZZS9/HzXiow17hqJ/PEfHh2191wSGEUkMEBWS3oblKU9
8DdwzTEAnpR/XUlJieBhrL7mGADNSQ7AaCDQMnOm4NIX/5zZ4sbdK++sMG3c
DZ8uH7fYuNt39pOUN+4GT7yemcbdkZE+08bd8ECbzK6mMm3c7eqtG0/FbwDT
xt2TTTsVX0JgZGxE3Lj7u4pfOtK4+4eTn7qncdfI2pIdc7e839QlexGOQfUC
CYwdb+Z9+cyW/+GGjQQGsIFrYKCIAdAZq685BkBzMgNwZdWq2ptuMspeGr/3
PXHjbuv//rf+smLTxt2rw4OZadyt3zzdKHtp3v2IYuNub2e15Jk3bdxt85dK
7ipxz2aNuxeaPo09WOU3wOBI/75z7wpuO9p+6nUNG3ePXD4pbtytDWY6EZqq
NdT10r7N//HHtyPbkj1FNvZAAqOIBAbIaiQwUMQA6IzV1xwDoDnxAISrqpru
uMMoe6m96ab2Rc8JGncbZ9zaveFd08bd4aaJhhOrjbuC246SNu4Odl1s2fe4
Sxp3T538RZoad49fWG2pcdf2b4Ca1gMub9ztGjDs8EkT08bdHRf3O9K4G294
dHTL2WOzNxVG45fotuui5cUigVFEAgNkNRIYKGIAdMbqa44B0JzRAIz19HQs
WyZq3P23+9oWzRHcdtT+whOmjbu9uyZCj3Q37l4dHXJP425T4yZB4+6Rw0/Y
btxtbDsgaNyN/Chp466N3wAyjbujY8P23oUM08bdyAPS9+pJubxxN+Z8oDVv
58fx2Ut0m72psDtsrSSZBEbRGy/8/emt33bDRgID2EACA0UMgM5Yfc0xAJpL
OgC9O3cKGncjPwrkL2yccatR9nL5J9/tP7Cz6708mcbd7i1LsrdxN/Ij+cbd
UOiSg427RpGIpd8AMo27HSHZMMoG08bdrWfW07ibVP/w0HvHSqdmL7YvgyGB
UUQCA2Q1EhgoYgB0xuprjgHQXMIAmDbutj39pH/uA2aNuxtlGnf7q7emu3G3
7eAiwaUvHVW/yUzj7tjYkOONu0bkfwM427gbQeOubeUNF+Zued8oe8nb+fH5
QKvVfZLAKCKBAbIaCQwUMQA6Y/U1xwBoLn4ATBt3O158Xty4Gz5WJtO4O9JR
37k2N62Nuz21mwSNu007HlRs3O3rOit5hoPBoxXlDzveuGtE5jdAVjTuZr5f
JVsad39d9plR9pJb9M6Wsza/kJoERhEJDJDV1L8Yjr+Ba44B0BmrrzkGQHPR
AQhXVTV+//uixt3n5zfPvF3QuNuzaZ1M4+7VkaFQ6Wpx4+6w/0L02Ow17g6H
GtLauNtW+6F84+6Z0790SeOuEdPfAO5v3O3oa0/fASRl2ri77XyJGxp3i04f
SWjcjd9e2rd596Fyq7tV/9CBqJWL//7kp992w0YCA9jANTBQxADojNXXHAOg
ueNlZYG8PMFtR80P/iSQ/7TgtqPAsmckG3cHfRXixt2+Qx/GDixNjbvDIcPL
Ekwbd2uPLhga8Eue2ObLnwkadw8dnJXhxl0jgt8Ajjfunm8/JW7cPdFSmb5X
T8q0cfe1ivfd3Lgb3eZueb/k0rlxtd//JDCKSGCArEYCA0UMgM5Yfc0xADoL
bd164R/+QdC42/GrpYLG3eaZt8s27vYGxY27XUWL1Bt3G4vvEzTuhuqLBafC
tHH3SuteybPa39dk2rgreRVN4nr1N9tu3DWS9DeATONuW3etjbcgqXcwJG7c
LT63kcbdpEwbd1dX7o197REJjINIYICsRgIDRQyAzlh9zTEAehqqrRU37gby
nm99/B6j7KVh+s3Rxt3WuTlSjbvLv22UvbS/dqd642774XzBpS+Boy+rNO42
nXk1hY27/X1NNtZrdGxYsXHXyNTfAKaNu8cbEq8+Sq0sbdyNPCbDRzVVecOF
Zz77wCh7WbD9D6f9k8aPBMZBr+fffGLbd9ywkcAANpDAQBEDoDNWX3MMgG6u
Dg6KG3cv3/XjwJLnTBt3O379qEzjbrBwprhx9+rI51eDuLBxt6Z8lqXG3UMH
Zwkad1tbdtlYr/EUNe4aif8NYNq4+9mJFelu3H370Cvixt30vboR08bdmkAa
LweS1NEferV8u1H2MntTYdHpI1Mbd0lgHEQCA2Q1EhgoYgB0xuprjgHQirhx
99Itt5g27oa2fdyzZZW4cXektW6icXfvSkH2EiycOdJRHz0qe427g10XxY27
V84WTk1sYmQadyXP6tDQlXNnVwgufYn81Hbj7pGalSlp3DUS+w1g2rhbF0i8
+iiFwiMDuy5sEjfudg10pu8AksqKxt2IbeerxY27raHkQ0IC4yASGCCrkcBA
EQOgM1ZfcwyAJsZ6etqfe05w21HrI4+0Pz9H0LjbsXzhwJHd4sbdvpIN49ca
d9tfu1PQuNtfvTV2YOGmQ/51d6S2cbe1dK5i4+7woOyH/daWXeLG3StXbHaD
1LfuS2HjrpHIbwDTxt3yi+sHR/rVX8uIaePuGX8K3qkl2dK4e6mzXdy4W95w
QfB0EhgHkcAAWY0EBooYAE/Kv66kpETwMFZfcwyADno2bqz76lcFjbvB5SaN
u30ln4kbd6+snh9t3L2yYZ7g0pfuLUviG3c7dy6w0bjb8OkMo+ylfvN0xcbd
noDs1+uYNu7WXVrnnsbdpCI72VqxWtC4+/GRxTTueqBx1wgJjINeW3KzIAfO
5EYCA9hAAgNFDIAnxX4z5OfnCx7G6muOAfC2odra5gceEFz6cnZ2buucewWN
u13vrxQ37rYtuC3auNt36ENB427gzXuG6m8MW++pj2w07voPzEtf427zuTdT
1bhbdWy+7cbdM3UfpaNxdypx4+7qklkZaNx9Zd8io+zl7UOvONK4u656U1Y0
7s7d8r5R9vLMZx8kNO4aIYFxkHsSmPnz55804PRJAtxrWjKW9sDfwDXHAHgS
CQxkMABedXVwsLOgwKRx98WFgtuO/M88FD5WFnjpQaPspWX2V6KNu8P+C4LG
Xf9LXw+Vro417g61nRY37oaq107tb+mqWeeext2K8oeNspeyknttN+4Gus6I
G3drm3fY23OCwZH+svNrxY27oXBHSl4rqebuhvcqC9zWuFvReFzcuFsbzHQi
NJW4cTeyFZ0+0j8se+WVjd//6h86/n/23vy/qXrt995/Dj88LznPOerz2H3r
Sz2IuDn2VkC5udlq3RtKt1hF2IJsy2QpUtzdVIFqN5TBQqVAaRkaOs/zPE9J
mzRthiYr87gy1JNSSNMk3ysrWUlWhuv9yssfWCsra7hcbT69rvdCVomdBAZ7
YBAkBLAHBmEJFkBCggkMwgQsgITE3NkJG3eVOSfFO/4YwLhblh/QuOu06GHj
rupWptu466QNsHGX4h3wNe5aqDHYuKuZLI6acXds9HtOjLtd4/9iadx1w5d3
wcZd1wph+SC/WOzmBv5j2LhrsOojtwN+CWjcrZppiQvj7qxqKagNYg8Mh+Sf
fKH/yZux8MIEBkFCABMYhCVYAAkJJjAIE7AAEgyHQgEbd2UZ++R/3xPIuFsD
G3dNbRW/PzXuyvM2E427eZs9jbtmYTNg3HUt8jXuOm0GaugnYOxI1nrIbpQR
TwUD4y5tJr7dC/F8eeSMu+UtH5Gyl8qOjLAYd12YaC1s3G2YuBJR465AOQkb
d6eWRiL36X5ZNe4CY0cFXTfjwrhbMxPKqcMEhkMwgUGQuAYTGIQlWAAJCSYw
CBOwABIJ3d27sxs3krIX0auvKk9nwcZdU3sNbNx1LV017qpuZQKtL7rKM07L
s04Gh4mieAeCNe4aF1sA465rkUEMDaoENO6qpfUMz6pePxs5427T4PEoGHdd
9IseA8bd3zqPcmvcrZku58S4e7rhYuwbd4sHWlkad0lgAsMhmMAgSFyDCQzC
EiyAhAQTGIQJWACJAS0QLGzbBrS+LB09Aht3tSWFhtpbkHE3K3XVuGtovRbA
uLuw9ud4/eANyZXXAOMuLR/1OpaAxl3lwD/ZGHfFY+cZGnftdiNs3B0ayIqc
cVdvWgxhy77ItYI7PcdJ2cvP9X/tFz0OV87jlzg17pptlijvlS/dYsHBxzfZ
G3dJYALDIZjAIEhcgwkMwhIsgIQEExiECVgA8c6y1UqdOwdkLwv/+Z/Ks1nA
2JH8yB6Gxl16YQQ27hpar7l3jJaPykt3hmDcnbv/Jil7Waz9lI1xd6Llk6CM
u63Nu0jZi2tRXBh3uwR3YeNua09tWD7LL2jcDRkmxl2bw8HyUzCB4ZDzp17o
q3ozFl6YwCBICGACg7AECyAhwQQGYQIWQFxjamoSvfoqZNw9lwMYd12Lxgp+
0JXlA2NHynN/YWjcdWgkq3vlpA2a1twQjLviqt2wcRc4FUbN+ETLJ7Bxl+Gs
EE2rhway4t24O0+NwMbdSWnr7xG7A6Bxlw0BjbtiTXjUNJjAcAgmMAgS12AC
g7AECyAhwQQGYQIWQJziUChk+/YBrS/yLzJlX34MtL5Q50+Ye2rEmf8R0Lhr
mWyEjbvm0Wr3jpmmHodg3FUO/BM27josKuKpsBnEY+dh467FMM/wxIqEpbBx
V6udDP5yrQAbd590ZXJi3I3EHQCNuyETIeMuCUxgOAQTGASA74Lnzco/cr1j
iBtMYBCWYAEkJJjAIEzAAohHNEVFgHF3ftMm5elvYeOuubuJOp8BG3edJn1Q
xl27RhSCcdcgroONu8bFFuBUqKX1sHGXWnjC8KxqtZM9XfsB465IWBrCxfo9
kHG3rGlnGI27w+LqoubPmBt3w3sHMFj1d4aKgNYX3sRdNO76JaBx92JHTcjG
XRKYwHDIv05t7Kn637HwwgQmRuDz8vPTU1JS/hCQlJT0/HwepjHcggkMwhIs
gIQEExiECVgA8YVlYAAw7go2bFB8e1SyJxU27uorr8DGXetUz+9PjbuynFdI
2YuycJfbuLvsoEMw7tqNMti4qxr9BTDu0mYZbNydH8llbtydmS6EjbsWszyE
6+Vw2kZmb0bHuKvUz5f1ZZOyl8LGvX6Nu2G8A3SJmgDj7s/t36Nxl0Skjbsk
MIHhEExgkFX4fF46k9iFlMXwsDGGG/xekKC2gL+BJzlYAAkJJjAIE7AA4gWn
Tgcbdxc/+rMi+zBs3DV31y2d/AAy7j4oWDXuKgt3MTTuWiX9YTfuShr2sTTu
GlSDDE+sYqkDNu7KZU1BXqtnyKgB2LgrlDaEtmUvHE5bQOOu2iT1+96w3AHk
+kU07oZGdIy7JIK9+qRvgRHavcQGE5ik52n0EmLy4pPEYBATddjfAPE38CQH
CyAhwQQGYQIWQFxgqK6ee/llUvbiWqT8/hRs3DU8uaspzgbGjqjzGavGXV11
HmzcdT7XZawYd5uyw2vcFVZs0c5Awz4BjbvSmevhMu7OTBeGZty10BrYuOta
GkbjbnH73wMad0mwvAPYnfaAxl2NmejwiRABjbuVU41JZdwlgT0wHPLP7zZ2
Vf/vWHhhAhN9wOwlJSUl3UW+jweGl7/y78Cc0koOw/WhJQ+YwCAswQJISBqf
IxQKgdXw6ic5WAAxjkOhkKSlAa0vS98ckR1MA1pfVJfOGJvKZAffIGUvrkWr
xl3zaDVs3LVMNrp3zDT1WHrtrfAad+UdR2PEuNvTtT8ujLs1Yz8zNO6SYHMH
CGjcHZOF50iZE9C4+2P79SQ07pLABIZDMIFJUvyHLytmF15wbSx8/koi47st
jGGiBCYwCEuwAJIZvPpJDhZALKMuKBBs2EDKXsR/+pMy97hoywuk7EWS/p6p
mQcbdwU//M1p0js0kgDG3eo8T+Ou8tFnkHG3NdfXuKsX8mDjrknWBZ2KsBp3
+3oOkLKX5sYdbIy79X3fRMe4O7bYEJRxl0RodwA07oYMJ8ZdEpjAcAgmMEkI
P98rMXkavLDOS1aiGK/GmBQU9UYcTGAQlmABJDN49ZMcLIDYxDIwML9pEyl7
md24UXHy2GLaO6TsZT71JU3xJX3lFUnGi4Bxl54ddhUAbNylitI8jbu6vsvB
GndtelFA466vKMZNGI27Tict4F+Fjbs0rQ7hesW+cZdECHeAODXuutaJ8l75
wpVxlwQmMByCCUySwc/3zEhSItCp4u2VcX1EmD8B8QQTGIQlWADJDF79JAcL
INZw6nRLX38NjB1J//pXxYkDAYy7PbXyrFTAuKuvvOL6LHphRJy3lTh2lPt6
sMZdr2NZdtABjbtWzQxwNgIad42acYYnlqJ6YeOuYqkjuEv1nLgw7pII6g4Q
j8bd0w0XJxWM2oEiCrfGXRKYwHDID9/9j47qTbHwwgQmGvDS3blIZNtTPHMY
jGAiCSYwCEuwAJIZvPpJDhZATKG7excw7opefZXKOzOf+hJg3DU2PtZczYKN
uw6lZMW4W3mGoXHXYaICGndd63gdS6SNu3LBLYZnlabVI8PfJYBx93rrlyEb
d0kwvAPEo3HX9ULjLgwmMByCCUxysZLARNHQsvqEa0xgIgkmMAhLsACSGbz6
SQ4WQIxACwSLu3YBrS+KrH/IvtwNtL6oL+cZm+5KM1Mg4273yu9jpsGHsHHX
ym9371hA465F3Ol1LE6bQdF7JqLGXdosY3hixfPlgHG3s2NvXBh3n4z8xNK4
S4LJHQCNuyEzq1o6WVcGGHcrp5g+Nj0SYALDIZjAJBd8Dh4Wjc+njiiYwCAs
wQJIZvDqJzlYAJyzbLVS584Bxt2F999Tns0CjLvSzz+09DUrcnYDrS+a4myn
SW9XCmHjrr7+gqdxV1GxB2h90XUXRN+461qB4Yk1GcWAcbexPjUujLvD4uqw
GHdJwHeAODXuts/3R3mXfDHZ6NvDncDY0fm2J1Ez7pLABIZDzmX/j7aat2Lh
hQkMgoQAJjAIS7AAkhm8+kkOFgC3mJqaRK++Chh3ledyYOOurrxYV5YPGHeX
Tn5Azw4v2+mAxl2bbHp1r1aMu90FgHFXUbHHRnn7WyJt3F2cuBRG467FLA/h
esWvcZcEcAeIR+Pu1b67sWDcHZAIY8q4SwITGA7BBAZB4hpMYBCWYAEkM3j1
kxwsAK5wKBTyL74Axo7kX2QuHcsExo6Uud+Y2nmMjLvCPsWl7YBx19i55lSx
iDsB46702luGkdtexxJp4+5k296gjLvtbZ8Axl25rCnIa/WMuDbukvB7B4hH
4+6Juh9jwbirtZgCGndNNmIOGWUwgeEQTGCQcMDn4XOnOQITGIQlWADJDF79
JAcLgBM0RUWzGzeSspf5TZuovBzAuLuY9o6pvSawcVerdFr02gcngbGjucK/
ehp31XXHgLEj11Jf465ZMRA54+5w7btBGXfHRr8HWl8mxvMiZ9y1hbRlXyJk
3CXhdQdA4y4bAhp3Z1VLXO/jOrp7e8tHpkN7BhMmMCzBBAbxhs/j8fLz0xmR
4gKfeMQpmMAgLMECSGbw6ic5WABRxjIwsLBtGyl7EWzYoDieJf1sByl7EW15
QVtSaKi9BRt3LaNtv68ad3Nfh4277gIwjt8HjLvykvc5Me7arEy/7C8uPObK
uKvQjIW2ZS9MtPbxUB7Q+tI8dSNk4y4JzztAnBp3F3WhDJSFlxg37vplcFG+
t+Thh9fv3xmcCOHtmMCwJDf7/22t2RwLL0xgOIfPy3c/QTp4MIHhBr8XI6gt
4G/gSQ4WQDKDVz/JwQKIGk6dTpmdDYwdLXz4gfJsFjB2JD+yJ6BxV1eW/8y4
eyMdNu4u21dGIVwFABt3JVde03UX+Ppb9EKesGILKXuZ533A0rirU3QzPLFM
jLtOZyhzHwGNu2Nzt8Nl3O0XPY6ocZfE6h0gZo27Z5t/AYy7zXNMiyRyxIVx
1wuN2fJDQ9eH1++vvnYXP1jU6hm+l/2XDmQVTGCQZ6w8qJoNmMBwA/sbIP4G
nuRgASQzePWTHCyA6KB/+HDu5ZdJ2YtrkfLMKfGOP5KyF9ciQ225riwfyF4U
Obtt4sllO61vKoSNu3alcHWvlh309IPjwNiRomKPXeNtW7XpRdKmzMgZdyXT
l8No3DUZQ1GeBjTuNg0eD5dxV64VBDTuhuWD/OK6A8Smcfe34UdA60vxYDka
d0Ojcpz/ackjd/yy+jpVFfRoGyYwLMEEBnkKi/wlJSUlPZ+Hz5zmCExgEJZg
ASQzePWTHCyASGNbWJCkpcHGXdnBNKD1RXXpjLmnRn7kbVL2Is1MMTauuFas
/PYAxt3ee+4dWzHulrwPGHdNU95f/JcdtHq8CDbu2vTQt3XYuDvdsd+sY/rL
JEX1tjbvImUvzY07pJKaIK/VM6Jm3LXaTQGNu3qLMiyf5Re5fvFCQ06sGXd7
FoYB4+7photo3A2NGYXq8MN6r+zF/WrkB5ezYQLDku+z/2dz7dux8MIEhkvW
BTCrgcoqvOdzSSn5fPe/rZtWci3geu+TGkxgEJZgASQk7jvDiRMngNXw6ic5
WACRY9lqVRcUCDZsAIy7ytzjoi0vkLIX6ecfmrub1IWHgdYX19IV466Bgo27
mrKjnsZdVfURoPVF05TtpL0bDAIad3WCcuBsBDTuLgnvMjyxNK0eGf4u4Y27
fDk0xsUSNO6yATbunqwrizXjrs3hKOkfI2Uvqz0wzAeRVsEEhiWYwCC/rwtg
vAMV9yLvBfz8FMISJJpgAoOwBAsgIcEEBmECFkCEMHd2zm/aBBl3Tx5b/HgL
KXuZT31JW1JobCwFjLvyrNRV466x9x5g3FVc2m7lt7t3zDByGzLulu60Svq9
jsVhUcHG3aWuE7Bxd34kFxg7mus/xty4K54vh427avVw8JdrhcQ27noSs8bd
w09yAePuksH7OVzRBzbuZpQXxaZx96vyWlL2kl7KC7b7ZRVMYFiCCQwSIExx
RzC+ohdgERItMIFBWIIFkJBgAoMwAQsg7DgUiqWvvwbGjhY/+vNSViYwdrR0
bL91oh8w7koyXtQ9KFi2WW2yaaoojZS9yHJecRt3XdioGdi4qx+8EXbjLrXw
JFzGXa12MuGNu3d6jkfIuLtKnBp32+e9U8HoE6fG3YK2fqD15czDGiMdYmFj
AsMSTGAQoM9lBXc842chtAyJDpjAICzBAkhIMIFBmIAFEF50d+/ObtwIGHcV
p4/Np74EGHdNHfWwcZc6n2GXzq0Yd+svAGNHnsZdJ23QdRcAY0cU74Bf466k
YR8pe5m7/yZs3LUY5gMadxkGJna7EY277IlB467ZZkkM465rBa730ZuaqTlf
4677dfhh/YxCxeb+jwkMS85k/8/G2rdj4YUJDHfAvSzg0ucLMYLhCkxgEJZg
ASQkmMAgTMACCBe0QLCwbRtk3D3whXTfdqD1RX05zzzQKDv4Bil7cS0ytVX8
/tS4K8/bDBh3TYMP3TtmEXfKireSshfx1c1+jbuq0V9CNu46nXTUjLuuRYlh
3DXR2rB8ll/k+sWr3fmwcTf6fpXEMO7eHu6MNePuolb/bWUTKXvZXfzgzuDE
6pqYwHAIJjBIoAQGbHTBLhiuwQQGYQkWQEKCCQzCBCwA9ixbrdS5c0D2Iv7T
nxQnv4aNu5axPup8BtD6oinOdpr0TgOlupUJtL5oH5x0Wp4ZNR0miuIdgI27
/d3eT6E1ybpg465eCA2eh9e4OzSQBbS+TE1eiH3j7pyiHzbuulYIywf5JaBx
95emH5TGaGtjlwxUQddNUvZy+EkuGndDY9W4u7v4ASl++bayydO4iwkMh2AC
gzBOYMAmGHTBcAQmMAhLsAASEkxgECZgAbDE1NQ09/LLgHF36R+HF/77TcC4
a6i6r6+8Isl4kZS9LJ38gJ5dscsaWq/Bxl16YcS9Y/rBG7Bxl5aP/r6+ABwW
1VLXCcC4q+g9EzXjrkhY2lifChh3tdrJ0C5Z7Bh322ZKIm3cvdD6HWDcHZJ0
R/kOYHfaq2Za0LgbCWDj7qclj2qm5rzeggkMh+Rk/y8gXo7mCxMY7gjQyAJq
YjCB4RhMYBCWYAEkJJjAIEzAAggZh0Ih27cPMu7u/m/53/cAY0fK3G8sA81L
Jz8gZS/SzBR95RXXZ9ELI8rCXYBx19B6zb1jtHw0oHHXvbK7AHSCcsC4K67a
HU3jbk/Xfti4G+zFWiXKxt3Cxr0cGnfvj/wKtL48HCtZNe5G8w4goESwcbdn
IcTnWIWRODXu5jf3AMbdgrZ+1zq+b8QEhkNiJ4E5fPjwMAGuT1Lis/Y0an85
CtQEAzfIIJEHExiEJVgACQkmMAgTsABCQ11QEMC4e/IIYNxdTHvH3N2oKc6G
jbsOrdJp0euq84CxI9WtTIdGsrpXTtqg7ciHjbsOvdTzQFwFYNXMwMZd9XhR
1Iy7U5MXgG8KQwNZFrM8hOsVZePunZ7jHBp3+xbamRt3o3MHCGjcdS2NBeNu
t1iQYMbdr8prZxTExjNMYDgkdhIY7IHhkrUI5g/pPPIDqX1jFrA/BokCmMAg
LMECSEgwgUGYgAUQLJaBAfHWrUDry9Lhv0v2pJKyF9GWFzTXLxib7sLGXcto
m+uzzKPVkHE3b7NlstG9Y2ZhM2DcdS0yCbyNtcsOerz2JDB2JG3KhI270pnr
4TLuymVNsHFXsdQR9NV6SjSNu20zJRwad5XGpWCNu1G4A8DG3bPNv8SCcVdp
0sPG3bLRnlgz7gpVWti4W9I/ZnM4gC1gAsMhmMAgT/GIYFbilPT8dTnM2tL1
Qcu64Cbau4w8BRMYhCVYAAkJJjAIE7AAmOPU6RRZWUD2svCf/6nIPgyMHcmP
7LH0NSvP/QVofdGV5S/brA6NBDbu6usvrBl39VLYuKvtyHfS3g0GJlmX6FFq
yMZdg2owXMZdi1kOG3dnpgvRuAsT0LhbMlDo17gb0TtAYhh3cxoqYtO4C4wd
napq9TTuksAEhkOyT/+v2rp3YuGFCQy3rI0TreUwa2mLZ0CTspLP8PLz0z3f
gAEMV2ACg7AECyAhwQQGYQIWAEP0Dx8Cxt3ZjRsV32WJd/yRlL24Fhlqy3Vl
+YBxV5Gz2yaeXLbThtZrspxXSNkLVZTmZdyVXHmNlL0s3ft41bjricOikrUe
go27ThtxJCSgcVc4mB0u425fzwE07gaEiXGX9N4I3QHiyLibVX2HlL1kPrge
m8bd/feqAONuI5/Yt+YFJjAcggkM4oaXvj6E8UxV/AQ0pFWRqIIJDMISLICE
BBMYhAlYAAGhBQJJWhrQ+iI/8IXsYBrQ+kKdP2HuqZFnpQLGXWPjil2WFvYp
Lm0njh3lvm7svLW2Y/JReelOhsZdN5rJ4rn7bwLGXbNiADgb1MKT4dp3SdnL
WOOuoIy7nR17SdlLc+MONO4GhLlxl0Qk7gAizWJcGHeLB1oTz7hrpIMobExg
OAQTGMQTvkdni3eqsn5QyQNUwHAJJjAIS7AAEhJMYBAmYAEALFutqvx8wYYN
pOxF/Kc/KXOPi7a8QMpeJOnvmbub1IWHgbEjzdWsVeOu9sFJYOxIU3bU+bxn
wEkbNE3ZAYy7Ju8GAws1tlj7KWDc1UwWw8ZdfvdXUTPu0rQ6hEuGxl2ScZe4
kbDeAQIad4sHy+PCuDsqE3O9j94ENO6Oy5TBbhMTGA757vSL1XV/ioUXJjAx
A5/Py09P8WPkXRfQPEtf/K2GRBFMYBCWYAEkM3j1kxwsABLmzs75TZugsaNj
RxfT3iFlL/OpL2lLCg01xdLMFFL2Is9KXTXumgYfynNfJ2Uvikvbrfx2946Z
ph7Dxl2LuNPrWJw2g3Lgn8DYkaz1EGzclUxfho27FsM8wxMb0LhLUb3BX64V
omncbZ66EV/GXRJhvAP0LAyfqPuRlL2cbriIxt3QWNTqYeNu+cg0bNwlgQkM
h2ACgwQPn8/n8Vz/4Xo/kN8xgUFYgwWQzODVT3KwAHxxKBRLX38NjB1JP/1U
ceIAMHa0dGy/ZaBZkbOblL1IMl7UPShYtlntSiFVlEbKXmQ5r+jrLyzbn30f
tGtEykefAa0vuu4C3yYWg7gOMO7y778DG3d1im7YuKucf8DwxAY07gr4Vxl2
0XhvmdZ0jf8rOsZdvrwrHo27JMJyB1gyUP/uuQ20vqBxNzQCGnfP1LYzMe6S
wASGQzCBQZC4xu9gWFBbwN/AkxwsgGQGr36SgwXghfbmzdmNG0nZi+jVV6m8
M/OpLwHGXVNrtfZ2LjB2pDz3F7t0btlO6+svAGNHqhvpdqVwda+WHbSuuwAw
7ioq9tioGa9jselFAY27/T2tpFNhs6pg465rKUPjrtNJBzTu6vWzoV0ywWJV
dIy7apMUNu52Ce5G2rh7vvl4aMZdEizvAHanvU7QngDG3ZqZkcBbiS6wcTe9
lMfcuEsihKvP/ksHssqpnJeq6rfGwgsTGAQJAeyBQViCBZDM4NVPcrAA3FjH
xxe2bSNlL4INGxRZ/5B+tgNofVFfzjP1VMkOvkHKXlyLTG0VK5/Fb5fnbSYa
d/M2mwYfunfMIu4EjLvSa28Zx+97Hcuyg1aPF8HGXQu1kkuQCgA27k60fBJG
4654vjyUCxZF465rI7Bxt6wvO9LG3TtDRWyMuyTY3AECGnfb5yPYDsSQgMbd
ix01CW/cJYE9MByCCUxywcXoEE4rRRRMYBCWYAEkM3j1kxwsABdOnY46dw4Y
O1rcuVN5NgvIXqSff2gZ6qTOZ0DG3eJsp0nvNFCqW5lA64v2wUmn5dlYgcNE
qeuOAWNHmqZsX+OuWTEgrtoNG3fdK/sWgMUwP92xn5S9DFW9HZRxd2I8Dxg7
Ghn+jo1xFxg7Cq9x97fOo6Tspaj5s0gbd7tETeyNuyRCuwOgcTdyBDTuziiY
Puo9IJjAcAgmMMnFyvOMoijPfar0xUdVRxRMYBCWYAEkM3j1kxwsAEN19dzL
L5OyF9ci6vw58Y4/AsZdfeUdfeUVScaLgHGXnl15/K6h9Rps3KUX1kYhjOP3
pdfeImUv8tKdVol3g4HDolL0noGNu3ajzPMtngUQXuOuVFITOeNuZUdGdIy7
DRNXODTuyvWL4TLukgjhDpAwxt3Q7LWRI3LGXRKYwHAIJjDJhfuJ0inp+RHN
Yfh8nvuxSZjARBJMYBCWYAEkM3j1k5xkLgDbwoIkLQ1ofVF8+w/5kXSg9UWZ
+415oFGelQoYd/WVV1yfRS+MKAt3AcZdQ+u1tR2jZhQVe0jZi+TKa7q+y77G
Xb2QJ6zYQspeRI9SDeI635PgLgCdonuscRdg3KUWnjA8sSajOAGMu5PS1qLm
zwDj7jwVQXNI2I27JIK6AySMcVes4V5N40mkjbskMIHhkJM5L/Hqt8bCCxOY
qMDP93ycdCRymNW+F8/PwAAmkmACg7AECyCZwauf5CRnASxbreqCAsGGDaTs
ZWHbNmXuccC4u5j2jrm7SXM1Cxg7os5nOLRKp0WvqzwDGXdvZTo0kmc79tS4
C4wdUbwDdo33yIlNL5I07ANaX6ihn5w2/yMhrgKwWVXCwWw07rpRm6QPBs4m
mHGXBMM7ABp3I0cUjLskMIHhEExgkhD++ogkTEEM3zt6+UNUB56SFkxgEJZg
ASQzePWTnCQsAHNn5/ymTaTsZXbjRmVOtiT9PVL2Itrygrak0NhYKs1MAYy7
ltE212eZBh/Cxl3LZKN7xyziTlnxVlL24lpkEtR4Hcuyg1aN/gIYdxdrP101
7pLoa/kXbNw1qAYZntjEMO72zJVza9wtGSiMhHGXBJM7ABp3I0TUjLskMIHh
EExgkhQ/cckf/pDyNIoJKjNZyV38bOlpqoPhSzTABAZhCRZAMoNXP8lJqgJw
KBRLX38NjB3JPvubMucwMHYkP7LHMtqpyNkNtL7oyvKXbVa7Uqi6kQ60vujr
L3gadyneAci425rrpL2bWEyyLubGXV+MmnHYuCuduZ5Uxt15aoRb427bXF3k
jLsk4DsAGncjRzSNuyQwgeGQ4zkvPW74P7HwwgQm+viNYdxpTEq6i3yeD/n5
rn93LSa8MfKOGcQTTGAQlmABJDN49ZOc5CkA7c2bsxs3krKX+U2bqLwzgHHX
tcjUWq0ryweyF0XObpt4ctlO65sKZTmvkLIXqijN07irH7whufIaKXtZuvcx
LR/1OhYmxl3XOqRT4bAZJNOXgbEjfvdXSWXcNdFabo27i1pRQONuhD4auAPE
hXFXqtegcTdkMIHhEExgkBVnLjlOCQrMXqIPJjAIS7AAkhm8+klOMhSAdXx8
Yds2UvYi2LBBeeqE7MvdQOuL+nKeqf2R7OAbpOxFmplibCxd+Sx+u+LSduLY
Ue7rxs5b7h2j5aPy0p2k7EV67S394A3fw9EJymHjrknWBZwNjbx1tH47KXtx
LQrKuDvQdxiNu2yw2M0BjbsacwR7IfzeAeLCuGtzONC4yxJMYDgEExhkFX8a
F4xe4gBMYBCWYAEkM3j1k5zELgCnTkedOweMHUk+/lh5Nku05QVS9iL9/ENL
bzN1PgNofdFczVox7hooTdlRYOxIXXrI+VxV6qQNmqZs2LjrMHl/eQxo3FWN
/uL7jCQ3tFkm6D0CG3cdBGGv94l10nOzxWjcZUlA4+6YbCByn76K1x0AjbuR
g0PjLglMYDgEExjECz5vJYph0hWzMqcUrDcGCTeYwCAswQJISBqfIxQKgdXw
6ic5CVwAhurquZdfJmUvrkXU+XML//0mKXuZT31JV16sr7wiyXiRlL3Is1Kt
Uz2uzzL23pPnvk7KXpZ+fNfKb3fvmGnqMWzctYg7vY5l1bgLZC+Shn1WzQxw
NuSCW0NVbwPGXaNmnOGJVauHYePu4kKIvhQ07rpfvIm74TXukvC8A8SLcfdq
XxMad8MCJjAccizn5YcN78bCCxOYGITvwlcEw8fQJYbABAZhCRZAQuK+M5w4
cQJYDa9+kpOQBUALBJK0NKD1RfHtP+RH0oGxI2XuN+aeWnlWKil7kWS8qK+8
smyz2mTTVFEaKXuR5byir7+wbH/Wl2LXiJSPPgNaX3TdBX6Nu/O8D0jZi7Bi
i04APWPIqBmfaPmE3PryllxwC3i7JwGNu2Oj38e+cVeinkxC4y6J1TtAvBh3
20TTcWfcbeSLODfuksAEhkOSIoGRDfKKTp1Kd3PqVBFvUBapT0OQaIIJDMIS
LICEBBMYhAkJVgDLVqsqP1+wYQMpe1nYtk2Ze3w+9SVS9rKY9o6pvUZzNQsY
O6LOZziUEqdFr6+/AIwdUUVpdqXw2Y45aF13AWDcVVTssVHeTSwOi2qp6wTQ
+qLoPQMbd+dHcoGxI0Hvkb6eRobnViqpaW7cQcpe2ts+iX3jrtVuSlrjLgnX
HQCNuxEioHH3zuAEt/uMCQyHJHoCI+MVeUQv6znFwxQGiXswgUFYggWQkGAC
gzAhkQrA1NQkevVVUvYyu3GjMidbkv4eKXsRbXlBW1JoqL0lzUwhZS+yg2+Y
B+pdn2WZbJTnbQaMu6bBh+4ds4g75SXvA8Zdw8ht38OBjbviqt1mBSQJoRae
wMZdjbz1d2YFYDKK+3oORMi42z56NmrG3eutX8aycddgjZ6CdZUlA/XPun+j
cTfsrBp3dxc/IMUvp6pao2ncJYEJDIdk5fx/FQ3/GQuvCCQwMh4xfXlG0WC4
PzOm4eWn5+fjEFFigQkMwhIsgIQEExiECYlRAA6FQrZvHzB2JPvsb8qcw8DY
kfzIHstAsyJnN9D6oivLd5r0TgOlupUJtL5oH5x0Wp59t3KYKFX1EWDsyLXU
17hr1cwAxt25+2+qx4sA467FMM/v/gpofZFMX3Ybd+ECcDppAf8qbNw1GUOc
+5gWP4qacffxUB6Hxt2ppRHYuOtaIXKf7pd4Me5OKaTxaNz9qryWlL18WvIo
+sZdEpjAcEgCJzCDRR7tLh5jR7JBz2AmqTIYXvqz/1fSeVzvCiueH0dwh7H6
tCdPw3BKStAm4djZyHMwgUFYggWQkGACgzAhAQpAXVAAjB3Nb9pE5Z0R7/gj
KXtxLTLWP9SV5QPZiyJnNz077PosQ+s1Wc4rpOxFcWk7vbD2fdAwclt67a3w
GndlrYdseuLXN6eTlkxfBoy70x37zbp1v2oABUBRvbBxVyqpCe2SUbrpqBl3
+0WPY9m4WzNdHh3jricCSoTG3UigMVsK2vpj0LhLAhMYDknYBMaj/8XftNFa
PJNMs0jBJTArD0XKf3qO8lf9uxHfP2a4jyKIBIafDz3bKSWf0bHFzkY8wQQG
YQkWQEKCCQzChLguAHNn5/ymTaTsRbBhg/LUCdmXu4HWF/XlPFP7I9nBN0jZ
izQzxVBT7PosemFEcWk7YNw1tF5z7xgtH1VU7IGNu75NLCZZl+hRKil7cS0y
iCFJiE7RDRh3h2vfXRLe9X2X3wKgaTVs3HUtDc24a7MbYeNuy3B2uIy7cq2A
c+Pu93Vfk7KXf3f+EE3j7ipxZNzNfHCdlL0cefJbPBp3x2VKrvfRG0xgOCRR
E5iACctaQpNEEQzDBIbPSyelBCkp6dxGMfy1/IVxAuP5FiKBthU7G/HC/b6g
3uVJXP8GjrAHCyAhcd8ZMIFBAOK0ABwKxdLXXwNjR5KPP1aezRJteYGUvUg/
/9DS20ydzwBaX9SFhx1apdOi1z44CYwdqW5lOp/Pazhpg7YjH8heFBV77Brv
790Oi0rWeghofVEO/NNpI34vtllVwsFsYOzItdS1jt/3+hYAbNzt7NgbsnF3
UdH9sO0vpOzFtSiMxt3mqRvcGnf/3flDTBl3XfQsDH9bkxf7xt1zzY/RuBsd
XP/7q02hGJx+xwSGNf/I+f/vN7wXC6+wJjBrAQxxymgtgkmeQSQGCQw/n5i+
eLCSw0Rzx5/vnFeCwSSrgHtO1h0Uuf8kdjbii/tdzN/iRZz+Bo6ECyyAhMR9
Z8AEBgGIxwLQFBXNbtxIyl7mXn5Zden8wn+/Scpe5lNf0pUX6yuvSDJeJGUv
8qxUy2ib67NMgw8h427eZiu/3b1jJkGNrHgrYNw1jt/3cziTxXP33yRlL4u1
n1ooSIeinH8wXPsuKXsZa9ylU3QDb/csANi421ifysa42zKcHR3jLl/exa1x
t2a6PAaNuwVdN2PfuPtgvA827kr1Gm530ot4Me6SyK/o2HaxZUqmC+G9nl9b
wr5jyUBiJjDudCWJ+luYECiBYZ4RBJ0TsMfPzgVOYLze9NS24l7m0+tDOKLY
2Yg/3O9h+gYf4vE3cCSMYAEkJO47AyYwCEB8FYBlYGBh2zag9UX53SnZwTRg
7EiZ+425p1aelUrKXiQZL+oeFCzbrHalUHUjHWh90ddfWLY/iyMceinFOwC0
vqjrjvkady3UmLhqNyl7EVZs0UwWA2fDrONPd+wnZS9DVW9Lpi8HDExWC2DV
uAuMHbEx7k6KysqadgLGXUo3HdqWvdBblGjc9cLutFdONQLG3TM1l+LCuNs4
O8H1PnoTR8ZdX6Zkuoxfe/5PfpPr9cWtPtrhDHYLnl9aIrGHCU9CJjDJ2N7C
CDCB8Y4IUp7qX/guViww+enr1bHPc4ToKH0Jc1EBP31dz4z/VGP9YfvbYuxs
xC+YwCAswQJISDCBQZgQLwXg1OkUWVlA9rLw/ntU3nfA2NFi2jum9hrN1Sxg
7Ig6n2GXzi3baX39BcC4SxWl2ZXC1R1bdtD6wRuSK6+Rshd56U5f467TZlD0
noGNuw6L/7mh358bd4Gxo+mO/RbDPJMT6yoAiuptb/skQsbdJ12ZgHF3UlQW
FuOuCzTu+jKpEAQ07nJ+Bwho3C3srkfjbhihHc7r7XOr2Yv7VTGwEOx2MIFh
ydGcV+41vB8Lr0gkMM9bYGSyQV7RqTU1r+ejkZIJIIFZ9/UfGqXheY0pRboT
BpDSBKVdAfYTVsvEzkb8gwkMwhIsgIQEExiECXFRALq7d+defpmUvcxu3Ej9
K3cx7R1S9iLa8oK2pNBQe0uamULKXmQH3zB1r/zUtfLbl358lzh2lPu6sfee
e8eskn556U5S9iK58ppf465eyIONuyZZF3Q2FN2j9dsB4y618IThiaVpdUvz
Qdi4aw9pOMhmN/ZPF6Jxlyvjrt5qYGjc5fYOkJDG3cFFOdf7SKRToPzocodX
/OJ6bbvYEqwQBhMYliRkAuO2wKwkMLLBIo+HT3tyqijZYhhiArOuQ4PBbA+P
YV7DDt/wZcU/47GvAfaUcZ7hGT9Bp4bbjRDABAZhCRZAQoIJDMKEGC8AWiCA
x47kX2QuHcsExo7kR/ZYBpoVObuB1hft7VynSe80UOrSQ8DYkfbBSU/jrqYp
Gxg7Uj76zNe4a9OLpE2ZQOuLavQX38Rm7e1WlaD3CND6Mj+S6yALe70QCUth
465aPRzaVZuXt6JxN2aNu2ebf/E07nJ1B4CNuxnlRfFo3C3pH4u1fXajNtEn
KkZ8s5eQ22AwgWFJYicwRYNrSl7/JNeYkjsIWBearJ9+YZyneMQjEcpgvHbs
mf2XcQITRJrhsU2vY4mdjZDABAZhCRZAQoIJDMKEmC0Ap05HnTsHZC/zmzZR
eafnU18iZS/iHX80tVbryvKB7EWRs5ueXckZDK3X5Lmvk7IXxaXttNDDWDv1
WHrtLVL2Iive6lrB63CWHbRq9BfAuCtp2GfTQ50ScsEtwLg70fKJQcX0F1qt
dhI27s7NFodm3NWbFqNm3J2UtqJx14slA/Vj+3VS9nL4Sa6vcTf6d4CENO5+
W9kUy8bdu73zO39uI2UvGb/2jCwEfcIxgWFJIiYwaxoYz24Xd7vL04mk5Mxg
1pIA97d778cLBfUs5LX3RsQH497ddY9eYpzAMG+WAYKP2NkICUxgEJZgASQk
J57T2NgIrIZXP8mJzQIwVFfDY0eKk8ck6e8BrS/qy3mmnirZwTdI2Ys0M8VQ
s+K5pRdGlIW7SNmLLOcVQ+s1t3HXrhEpKvYArS+a1lwn7d2FYpJ1zfM+AIy7
OkE5cDaMmvHJtr2AcVcuuMUwMLHbjTPThcDY0dBAVmjGXYfTFjXjrtokReOu
F6vGXWDs6Mf2636Nu1G+AwQ07raJwlMkYSTejbuHSgdI2ct7PzXd7Z0PQcP7
OyYwrDmSkwKE1dF8RSyB8Ttr5LFOMj0vybMZIyXdr1w3qCxlbXuRiGBcW1/3
yKBVmMYZHscaOMogrRw7GyGCCQzCEiyAZAavfpITawVACwSStDSg9UX6178q
cw4D2Yvs4EeWvmbqfAbQ+qIuPOzQKp0Wva7yDDB2pLqV6dBIVnds2UHruguA
7EVRsYeWj3odjsOiknccBcaOFL1nAOOuw2ZYnLgEjB0Jeo8wNO66kMuaWpt3
kbIX16LYN+66NsKtcVdjVsWmcfd0w0XAuNuzQBwoi9odIKBx17U0Ho27rnW4
3k3/GK32Sw0zwNjRiYqRuraekLePCQxLYieBOXz48DBj4IPyHD0itrisZTDJ
FMEQHjidkp/vd0ApMIEecB1+mCYwwTWTELYaOxshggkMwhIsgGQGr36SEzsF
sGy1UufOCTZsIGUvoldfpfLOwGNH+so7ugcFkowXSdmLPCvVMtrm+jjT4EN5
3maicTdvs2Wy0b1vFnGnvOR9UvYivfaWYeS27xFpZ0qFFVtI2Yu4ajds3FVL
6wHjrmsRc+OuxSwfGsgCWl+mJi+EbNztmbwA/G4fRuOuRD0Zy8bdq935nBh3
iwfLgdaX34YfrRp3SUTnDgAbd7Oq76BxN7z0iVR+jburL9eiToHyd3ZXHxMY
lsROAhMJDww4Y5Ssz6z2yWCePlI61G6WtUmhCD8TyU0ICQyDQwocnnC8ESKY
wCAswQJIZvDqJzkxUgCmpibRq6+SshfBhg2K41nSz3YArS/U+RPmnhp5Viop
e5FkvKh7ULBss9qVQtWNdKD1RV9/wWl5pnRwmChV9RGg9UVdd8y1jtfhWKgx
ScM+UvYyd/9N9XgRYNy1GOZh46547DxD467TSYuEpY31qaTspanhU612MrSr
JpQ2lLd8RPqtPrzG3YaJK9wad39u/56UvfzQcJQT4277fD9s3BVQgROhSN8B
0LgbZQIady81zBitz1xAmMBwyOGclNsN22PhFYmnUcPtLesemZRUPH2k9Cpu
w0qoTpeoN8EwTCiCfKqQ/z6V2NkIGUxgEJZgASQzePWTHM4LwLawAI8dLX70
Z+XZLCB7kX7+obm7SXXxS2DsiDqfYZfOLdtpff0FWc4rpOyFKkqzydY0FPrB
G4BxV1660yLu9Docp81ADf0EjB3JWg8Bxl2nk5YLbg1VvQ0Yd42acYbnVqud
7OzYCxh3RcLS0ApAb1psGjwO/FG1f7owjMbdoubPODTu8ibuxp1xt2qmxcu4
SyJyd4CENO6eqmqNZeNuxcACbNydkuk818cEhkMSMoFhmK0kbwLjl2B7L7zf
hgkMJjBIXIIFkMzg1U9yOCyAZatVXVAAjB3Nvfyy8vtT4h1/JGUv86kv6e5d
11dekWamkLIX2cE3TN0rP3mt/PalH98ljh3lvm7svefeN1o+unTvY1L2Irny
mq7vsm8Ti0FcJ3qUSspeXIv0Quh3AINqcKLlE9i4y/Dc2u3GifE82LhL0+rf
gy8Ah9M2Nnc7asbdsr5sDo27Y7IB2LgrUIbYPhQyAY27BV03/Rp3SUToDjCr
WopH4+7+e1VxatxdUJsA4+7On9vu9voRRmECwyGJmcAwGjBK1ikkIus0vUxn
iiLr4vUHJjDrwQQGYQkWQDKDVz/J4aoATE1N85s2Aa0viqx/yA6mAa0viuyD
loHmpZMfAK0vurJ8p0nvNFDq0kPA2JH2wUnn8y+tTtqgac0Fxo4o3gG7xvuL
mE0vkrUeAlpflAP/dJLnhmxW1fxILmzcda3D8NxKJTXNjTsA4y5F9bpXDqoA
FJqxgMZd5lsDcDhtXYK7sWzcbeA/5sS4e6LuR1L24loEGHdJhP0OENC4W9hd
H4PG3fzmHti4a6TDoJKOBLTDeb19Djbuqk3+Zx4xgeGQr3P+o6Thg1h4hTOB
YTKHlKQmXpB1iph1j4AmsRYXxJoHJnbCk2glMAB9IGVlZfAKSGKDBZDM4NVP
cqJfAAN1dRN//jOQvUxt3jxxKGNu8/9Dyl4E//XGyPUCwQ9/A7IX4T/eG66+
7/q4qdunJWdeJWUv4ryto9Ul7n0br7oovrqZlL24Fo3VFnodTn9v13jdacG9
10nZy8yDnYNt94AT0t96YaDqT8T4pSq1r/06w3Pb0/2kuZE4duR6tbWe6evr
8nwLwwLo7m1+0nYSGDt63Pq1ax2G+wlT11l2rfErUvby74Z9D9u9r0J4KW29
kVNziJS9XGjIqeuqiugO+NLW0/6vustA68vFhusdPZ0hbDm8d4CbTdWZ5ddI
2ctXD27cb6kP48eFhSs1TR8XV5Cyl7/99qi8uZ3rfSTyW23XR7+0kLKX/ypo
vlndBbw94NVn8hXjD5jAhESCJjABnzbtsRw7YDzw1vT6eRo0YeVYfhYSmnhB
+vCP4MkNFkAyg1c/yYlyAagLCmY3boTGjs7lAGNHoi0vaEsKjU13gbEj1yJj
Y6nrs+iFEWXhLlL2Ist5xdB6bdn+7E/Ddo1I+egzoPVF113gpL2bWEyyLnHV
bsC4q5ksBs6GxTA/3bEfaH2RTF9mbtwV8K8Cxt2+ngN6/azvG5kUQEDj7qKi
m8lOBsREa2vGfkbjrhfNc92wcVekCf1pU+G6A0j1mvNtT4DWl3g07t4ZnIi1
fXajNtGnH40BrS/5NVNu4y4JNlcfExiWJGoCs/6R1KdOFQ3KnucwskFe0al0
MJ9Javw9rzolJT0/n8fjP4P31OS7boVodcBE6GnUpD6V2NkIEUxgEJZgASQz
ePWTnKgVgGVgAB47WvrmCDx2JD+yxzrcqcjZDbS+aK5mrYwdWfS6yjPA2JHq
VqZDI1ndsWUHresukFx5jZS9KCr22KgZr8NxWFRLXSdg465rHdLZcDppyfRl
IHuZ7tjP3LhLUb3tbZ+Qspfmxh2LC8THNMMFoDct1vd9A7S+jMzeDJdxd2yx
AY27Xizq5LBxt07QztC4S4L9HcDmcFRODcajcRcYO4p94+62i8TWF1/jLglM
YDgkcROYdX0uJDB/8Qufl+6bwkBErQHmd+YJjEeUEVTusW7l2NkIEUxgEJZg
ASQzePWTnCgUgEOhWPr6ayB7WXj/PWXucdGWF0jZi3jHH431D3Vl+UD2Is9K
pWdXJBimwYfy3NeJxt28zZbJRve+WcSd8pL3SdmL9NpbxvH7vkeknSkVVmwB
jLsmWRdwQnSK7tH67aTsZbj23SXhXYbnlqbVY6PfA2NHrqWrxl0SpAJwOG0j
szcB42593zdo3I0cAY27/+65HZRxlwTLO0BA427j7AT7nQwvg4vy9FJenBp3
ZxUGwLi77WLLrU4h7XAy3BomMBxyKOc/bjZ8GAuv8Ccwv6/0uxQRU5hTRRi/
ADx9YjWj+CWK/S8rMJ7SCWach9imEjsbIYEJDMISLIBkBq9+khPpAtDevAmM
HbkWKXNOLqa9A7S+qC/nmXqqZAffAMaO9JVXXJ9lk01TRWlA64u+/oLT8uzv
2g4Tpao+AowdaZqyXet4HY6FGpM07ANaX1Sjv/g+I8mNzaoS9B4BWl/m+o8x
N+6K58sB425nx15P4y4JvwUgowYA4255y0fT4kcMdxJm1bgLZC9o3A2XcZdE
yHeAODXu/tDQlajG3aP3hhbUwZ1wTGA4JMETmKc8HTvyCGJOnSrirQ0lIRAB
cpgURrre8MI4zghinocce8TORkj4vTBM3ugGv4IlOVgAyQxe/SQncgVgGRhY
2LYNaH2Rf5Ep//seIHuRHfzIMtRJnc8AWl9UF790aJVOi15ffwHIXqiiNJts
rWFDP3hDeu0tUvYiL91plfR7HY7TZqCGfgKyF0nDPpse+tO5XHBrqOptUvYy
1rhLx9imotfP9vUcIGUvjfWpAv5Vp5MYBHniVQAWWtMzeQEYO2ofPetah+F+
wsxTI8XtfydlL0XNn/WLiMNTYaFtru77uq9J2cvV7vxFbbR7IfRWw797bgOt
L78NPzLbLGH8xNDuAG2i6cwH10nZy5Env43KxGHcybBQOc7/tOQRKXv5qrx2
XKbkeh+J9IlUH13uIGUvrkV146F8rQ3h6rP/0oGskgwJDBIGnspf8tdirBUr
DDd7wryhxGNNxsoYnxVjZyME2N8A8StYkoMFkMzg1U9yIlEATp1OkZUFZC/i
rVuVucfnU18Cxo4MteW6BwWSjBeJY0dH3raMtrk+zjxaLc/bTBw7yn3d2HvP
vW+0fHTp3sek7EVy5TX94A3fJhaDuE70KJWUvQgrtuiF0A9so2Z8ouUTUvYy
VPW2ZPoyw8DEbjcK+FeBsaOBvsMmYxBffj0LQLBYBRh3KzsyZNQA8y0DoHHX
L3WC9sgZd0kEeweQ6jXnmh/Hl3F3RqGCjbvlI9Oxts9u1Cb6RMUI0PpyuVkQ
0LhLAntgOORgzqu/NuyMhRcmMAgjghjp8cwzyB0lAR7pFDsb8Q8mMAhLsAAS
khPPaWxsBFbDq5/khL0AAo4dKY4dhceOVJfOmHtq5FmppOxFkvGirix/2Wa1
K4WqW5lA64v2wUnnc1eGkzZoWnOBsSOKd8Chl3odjk0vkrUeAlpfFL1nnOTH
FTlsBvHYedi4azHMMzy3FNXb2rwLMO5KJTXBXq/VAoCNu2VNO0dmbzqc4ZnO
GEPjrg8izWJA426EPpr5HcDmcDwY74sv466RtsHG3TO17bFs3L3bO7/z5zb2
xl0SmMBwCCYwSJwRolRl5cna/lZfLx32t0rsbMQvmMAgLMECSEjcd4YTJ04A
q+HVT3LCWAABx45kGfsUJw4A2Yv08w8tgx2qi18CY0fU+Qy7dG7ZTuvrL8hy
XiFlL4pL2+mFtS/ypqnHsuKtpOzFtcgsbPY6nGUHrR4vmrv/Jil7EVftNiug
nhBq4Qls3HWtwPDc0rR6aCALaH2ZGM+zh/RMot6+7pHZm8DYUX3fN3pTeFov
lPr5gMbdcOU8folB467ZZglo3FWZI9gOxPAOMCoTx51xt2deAhh3XYti2bg7
JdPBxt27vfPMjbskMIHhEExgkDgjmATG59HaK96a5x0ofJ9HPhG6U2JnI/7A
BAZhCRZAQoIJDMKEsBRAwLGj+U2blKezAowdVd3XV16RZqaQshfZwTdM3Ss/
9a389qUf3yVlL7KcVwyt19z7ZteIlI8+A1pfdN0FTtq7icUk6xJX7SZlL3P3
39RMFgPGXYthnt/9FdD6Mj+S6yB3znghEpY21qcCxl2tNsTcQEYNlDf9BTDu
CqUNoW3ZCybGXaWeaS9QCMSpcXdAwvSJ5CET8A6gtZji0bh7prYdaH253DkY
s8Zdo9V+uVkAjB2dqBhZ0oXHBYQJDIdgAoPEGUElML7JBxkg9YidjfjC/gaI
X8GSHCyAhMR9Z8AEBgFgXwCaoiJg7EiwYYPi5DHpZzvgsSPLaAcwduR6aW/n
Ok16p4GCx47UpYfcY0fLDlrXXSC58hope1FU7LFRM16H47ColrpOAGNHstZD
diNReul00pLpy4Bxd6LlE4NqkOG51Wone7r2A8ZdkbCUoUDGCwutaR89C7S+
9ExesIXUVOPLPDVyvfVLwLg7thienIdEPBp3740+Ca9xlwR8B4hH4275yPTu
4geAcXdGwfRxY9GnU6CEjbvN00th/DhMYDjkQM5r1xv+KxZemMAgjAgygfnd
awKIRDBOGW434oXfbQS1BfwKluRgASQk7rsBJjAIAJsCsAwMiLduBVpfpH/9
qzLnMDx2ZO5u0lzNArIXRc5um3hy2U4bWq8BY0dLP75r5a+5MiziTnnJ+6Ts
RXrtLeP4fd8j0s6UCiu2kLIX0aNU42ILcEJ0im7YuCsX3GJu3J2avACMHQ0N
ZAVl3PVkWvwIMO4+6cpUaMZC27IXJlr7ZOQnoPWlZuxnNO5Gx7hLgnQHkOo1
OQ0VpOwlo7woNo27hx/Ww8ZdrveRSEDj7qWGmZCNuyTwWUgcggkMEmcEn8A8
fag21H/CrOckdjayDvY3QPwKluRgASQkmMAgTAitABwKxdLXX8NjR1TuKWDs
yLVI//A3Q00xMHbkWmRsLHV9HC3sU1zaDowd6esvLNufJRsOE6WqPgKMHWma
sl3reB2RhRqTNOwDWl9Uo78Axl2bVSUczAbGjgS9R2gz08fFSiU1gHHXtUgu
awrhqrnQGOZg4+7Y3O1wmViGxdWAcfe3zqNo3I2mcZeE7x3A5nCUjfYAY0ex
ady93DkIG3c15mj0FIXG3d75bRdbSNnLF7f6WBp3SWAPDIdgApMM8FdhsOLz
500HHwREjVASmKfw+bz89BTP/CMlJT2fx+S8xOJGnoMJDMISLICEBBMYhAkh
FAD7sSPq/AnLaIciZzfQ+qIpzl4dO9KUHQXGjlS3Mu1KoXvf9IM3gLEjeelO
q6Tf63CcNgM19BOQvUga9lk13sNKniwJ7w7XvkvKXkbrt6ul9QzPrckoho27
U5MXQjPuOpw22LjbMpwdHeNuYeNeNO5G37hLwusOMCoTH3nyG2DcbRPFXBtJ
+9wCbNztmZdwvY9EpmS6jF97YONu5D4dExgO+TLntasNu2LhhQlM+Hn6RX99
j8XKF33yG9zpRpCTMAh3YAKDsAQLICHBBAZhQlAFEJ6xo/4OTXE2kL0snfyA
nh12fZyx85Y893VS9iLP22werXbvm1XSLy/dScpeJFde0w/e8HXnGsR1okep
pOxFWLFFJygHTohZx5/u2A+0vixOXGJo3HU66bnZYsC429O1n41x92Eb0bjr
WhQ14+6DgbNqk/cjv8NIbBp3R+XTnBt3SbjvAFqLqbC7Hmh9udrXZLKFIh2K
HItaPWzcvd4zEsvG3UsNM7BxV22K7AnHBIZDMIFJWADxK3HWBROY+AMTGIQl
WAAJCSYwCBMYFgCjsaO8HHjsSHfvuqmtAh47MtQUuz6OXhhRFu4CWl901XlO
y7MREoeJ0jRlA2NHFO+AQ+/9rd+mF8laDwGtL4reMw4L0djpsBkk05eB7GW6
Y79Rw/RrNUX1dnbshY27DDflhYXWtAxnA60vT9pORs24OyltDcsHkUDjbgis
3gFqZkYA425W9Z0pRQRzsxCwORywcffbyqZYNu42Ty/Bxt1OgTIKu4EJDIdg
ApOgrHO+pjybLUrx/Cd/GQsmMPEHJjAIS7AAEhJMYBAmMCkAeOzItWhl7Ojz
D4HWF2XuN5bRzgBjR1ezHFql06LXVZ4BsheqKI1eWPOHGEZuS6+9RcpeZMVb
zcJmr8NZdtDq8aK5+2+Sshdx1W6zYgA6IfLW0frtpOxluPbdJeFdhuefptVj
o9/Dxl3XOgy35sWkqAw27lK66bDcAUy09vFQHtD60jBxxWqP4AOLY9a4e/hJ
bowYd0lUd7TCxt0H430xaNz9qryWlL18WvIolo27SzoLbNy93j4XduMuCUxg
OAQTmITEo/9lfb/LOhGsb8yCCUz84bfLKagt4FewJAcLICHBBAZhAlwAAceO
ZJ/9DR47kqS/Z+7v0JXlMxk7Mg0+hMaOcl839t5z7xstH1VU7AFaX3TdBU7a
ewLIJOsSV+0mZS9z99/UTBb7DiutfahZJug9ArS+CAezbVamf3lfXHjc3LgD
MO5SVC/DTXlB6aZh4+6kqGx1TfZ3gH7RY9i4K1FHULoSs8bdvNYrsHHX7ozS
t2wSAY2759uexKBxt6CtHxg7+qGhK36Nu4dKByJk3CUR7P/+fr9xYAITGl/k
vH6lYXcsvDCBCR9rAYy/eSP+Wn+M92JMYOIP7IFBWIIFkJBgAoMwgVQATp0u
PGNH3TzZwTeAsSN95RXXx9lk01RRGtD6on1w0ml49gAjJ23QtOYC2YuiYo9d
4z1v4rColrpOAGNHstZDdiP0uCK54NZQ1duk7GWi5ROdopvhadfrZ/t6DgCt
LwL+1dCMuza7cYh/FTbuGi1L7vXZ3AHkWsGdnuOAcbdf9DgJjbsVE7WwcXfp
eSVzyKhMfPDxTVL24loUg8bdRr4INu4OLsq53kcisHF3589tETXuksAeGA7B
BCYRcQcpJOGLRwizLmvBBCb+wAQGYQkWQEKCCQzCBL8FEK6xI+W5vwCtL+rC
w8/GjqrzgOxFcWk7LVzbSdPUY1nxVlL2Ir32lmsF3yPSzpQKK7aQshfRo1Tj
Ygtwloya8YmWT0jZy1DV29KZ604nI2em3W4U8K8C2UtfzwG9fpbJpnxZVHTD
xt15ubeJJbQ7gNVuapspAcaOHg/lRdS4qzQuxaNxt2dhOMq75IvWYrrYUZNg
xt2S/rFYG5VyEwvGXRKYwHAIJjCJCJMgxW+fDCYw8QcmMAhLsAASEkxgECZ4
FUC4xo7gpx3Js1KtUz2ujzOPVsvzNpOyF1nOK4bWa+59s2tE8NiRpjXXd+zI
Qo1JGvYBrS+q0V+c5McVOWyG+ZFcYOxI0HuENkOdM54oljpam3eRspfmxh3i
eejRSwBGyxJs3O2fLvRr3A3hDsCXdwHGXdci1wqhHQUT7E57PBp3fxt+xK1x
d5WAxt1Z1VLgrUQRJsZdoYqDp3gzpFOghI27fSIudcGYwHDI5zlv/Lvhz7Hw
wgQmfDAMUnxbZTCBiT8wgUFYggWQkDQ+RygUAqvh1U9y3AXgUCjkX3wRYOwo
92TAsSNjYyn8tKPVsSO7Uqi6kQ60vqhLD3mOHem6C4DsZenex7R81OvQnDYD
NfQTkL1IGvZZNTPAyaEWngzXvkvKXkbrt2t8WkpIWMzyoYEsoPVlbPR7Nsbd
sqadpOylpucQpSNOlAR1B9BblLBxt3nqRkSNuyI1H427oSHWULBxt3JqMNba
SMZlSti4WzlOeLJrDKA20bFj3CWBCQyHYAKTiLj7W4iPnX6G9zASJjDxByYw
CEuwAJIZvPpJzmoBqAsKAo4dSdLfCzB2NNAMP+1IdfFLh1a5bKf19ReA7GXp
x3et/Hb3HpqFzfDYkX7whu9xGcR1okeppOxFWLFFJ4C6TSyGeX73V0Dri2T6
soPcOeOFSFjaWJ9Kyl46O/ayMe4+6cpkYtwlwfRx5E5bv+hxYeNewLgr1wpC
OwomWOzmh2MlwNjRnaGiGDTuVs20oHE3BAIad8/UtqNxlz2YwHAIJjAJydqI
UaAoZf0wEiYw8QcmMAhLsACSGbz6Sc7Q5cvzmzYBrS/yLzIVJw4A2cti2jtM
xo4so22uj7NMNsJjR/qmwmX7MyWCXSOieAeA1hdV9RGHyVtqatOLZK2HgNYX
Re8Zh4XY+e900pLpy4Bxd7pjv1nH9C/vWu1kZ8deUvbSWJ8q4F9lKJDxPky7
sWfyAmzctdCBv1YzuQPItYLfOo/Cxt0QDoE5Q5LuHxqOxpdxt6DrZowYd488
+Q0w7t5squZ6H71p5Is+LXkUv8bdL271xZpxlwQmMByCCUxisu5x1Dz4dwWP
EMb9Jkxg4oY/+COoLeBXsCQHCyCZwauftDgUCtm+fUD2It66lco7DY8daUsK
4bEjScaL+soryzarQyNR3coEWl9cS13rrO7bsoPW9V2WXHmNlL3IS3daxJ1e
R+R6l2r0l7n7b5KyF3HVbrNiADgnOkX3WOMuUvYyXPuucv4Bw9NrtxsnxvOA
saOBvsMmoziEC+dCKG2AjbsyCjpMT+A7gNVuapi4Aht39RZlaEfBBKVx6de+
i7BxN/pNJrBx99uavBgx7hZ21wc07sbUj4BFrf5UVSts3DXSEXy6FhuMVvvl
ZkFsGndJ4NOoOWR/zpu/1H8cCy9MYMKLRwYTOFFZvzImMPEE9sAgLMECSGbw
6ichy1arKj9fsGEDMHZE/XAWHjtSZB8MYuyoqVCW8wope5HnbbZMNrp3zyLu
lJe8T8peJFde0/VdXnZ4f5ExybrmeR+Qspe5+29qJot93+XGZlXN9R8Dxo7m
R3Jd6zA8w1JJTXPjDsC461oh+Ou2gt602DR4HGh9GZm96de4SwK4A0xKW4ua
PwOMu3OK/tCOggl2p72B/xg27iqN0dbGqszauDDuNs5OMDTuxsiPAJvDcWdw
Ajbuzii4lNbCxLhxlwT2wHAIJjAJDD8/nekw0tMHVKdgAhOHYAKDsAQLIJnB
q59smJqaRK++CrS+LB05rDx7NMDYUXej9nYuk7EjK7996cd3obGj+gvusSOH
iVJVHwHGjijeAbvG+zE3DotK3nEUGDuStR6yG6HHFckFtwDj7kTLJwbVINPT
axT39RwAWl8mxvPswSQka4fptI3N3QaMu/V932gMc8Fu1u8dQG2SPhg4C7S+
tM2URNS4K1BOwsbdvoX2wFsJNwGNuwIq2s9g8kWq1wRl3I2FHwEBjbvlI0SV
NOfEhXGXBCYwHIIJTKLD5/Py09MDGHndK/OehTaYwMQNmMAgLMECSGbw6icP
tEAgSUsDspeFbdtUP50NOHZkaquQHXwj4NiR00CpSw9BY0c30u1K4eq+LTto
/eANYOxIVrzVLGz2PSjNZDEwdiR6lGpcbAHOiVEzPtm2l5S9DFW9LRfcYnh6
nU5awL8KG3e12hB1JTJqADDulrd8JFisCm3LXncAh9PWJbgLGHfv9ByPqHHX
YNXfH/kVGDtyLeXEuHu2+ZeENO5y+yNAY7agcZdDXFd/QhxKIPw7JjCs+VvO
/wZS7mi+MIFBkBDABAZhCRZAMoNXPxlYtlqpc+eAsaO5l19W/fivwGNHo50M
x44MrdfgsSPz6Jr/0yrpl5fuBFpfdN0FTtr7wUMWakxctRtofVGN/uIkP67I
YTOIx84DY0eC3iO0Geqc8YSietvbPgGMuyJhaQgXbuUwaQ1s3O0a/xcT4y4J
zzvAPDVS3P53Do27fQvtgHH35/bvOTHu3ht9kgDG3TaR/zYSDn8EJLBxd9vF
lpgy7vpFZbAd+rl/U1Z/w0goE1KYwLAEE5iE5WnrC4/P1NiPxCmYwCAswQJI
ZvDqJzz6hw/nXn4ZaH1Rfndq6VhmGMeOFJe2A60vuuo8p+VZD4PDRKnrjgHZ
i6Jij42a8Toip82g6D0DZC+Shn02PTQPQi08Ga3fTspeXIs08laGp5em1SPD
3wFjR0MDWRZziF8kBYtV5S0fkbKXJ12ZzI27JFbvACZaWzP2M/Ar+uOhPNc6
LD8LQGlcutqdT8pevq/7mhPj7oBkHDDuuhbFkXGX9HZOfgQsavXfVjYBrS/X
e0bQuBtRqgeo904PbcpaSWA+PDtisDgCv2c9mMCwJHYSmMOHDw8T4PokxSfu
J0u7SEnPxzAmQcEEBmEJFkAyg1c/gbGOjy/u2gVkL66lE//4MmpjR1RRmk22
9od4/eAN6bW3SNmLa5Fpyk/HhU5QLqzYQspeXIv0QmiK2mKY53d/BbS+SKYv
O8idM16I58sB425r8y7FUkdQl8yN3rRY3/cNKXspa9o5Nnfb4QzDV1TXHWBs
sYFz4y4wdvRr30U07pJgbtwlEeUfATaHo6R/DDDuHn5YH8vG3ebpJdi42ymI
4KPBwoJYYTlUNLOavbhf+Q+Dfi4bJjAsycjZdLH+01h4YQ9MuHlqgEnxeshR
CoYxCQYmMAhLsACSGbz6CYlTp1NmZwPZy8rY0aXzi2nvAK0vytxvAo4dUecz
HEpJ4LGj3NdNgw/du0fLR5fufQy0vmhac33HjqyaGUnDPqD1RTnwT2DsyOmk
JdOXh6reJmUv0x37zTqmvxxptZOwcVfAvxqycXdk9iYwdtQynK03LYawZV/U
Jumvzd8AfxvtEtyNtHH3fPNxwLg7JOmO3KeTCGjcFWnCc/7ZINZQQRl3SUTz
R8Dgohww7u4ufhDLxt0lnQU27l5uFsSscXcV2u68US/908lBr/hl9RWsEAYT
GJZgApMU8HnkMIbrfUPY8Qd/BLUF/AqW5GABJDN49RMPTVHR7MaNpOxFsGHD
ytjRP/4GZC+S9PfM/R0Bxo6OvM3kaUcrY0eVZ9xjR07aoGnKhseOaPmo1xE5
bQbV6C9A9rJY+6mFGgPOiU7RPda4i5S9DNe+uyS8y/D02u3GmelCIHvp6zmg
188Gf91WkFEDD9v+QspeXIuE0obQtuzFqnEXyF7K+rIjbdy9M1QEtL48HCux
2M2R2wG/BDTu1gna49S4SyI6PwI0Zkt+c0+cGndphxM27n5xqy/Gjbsuhub0
f84b9Zu9uF7vnR5qm2BUM+y/dCCrYAKTbPB5+f7DGB6GMXEJ9sAgLMECSGbw
6icS5s5O8datQOuL5OOPqfwc0ZYXgLEj/cPfTN08eOxI96Bg2WZ1aCSqW5nw
2BG9MOLePeP4fXjsyDBy2/egDOI60aNUYOxIM1kMnBObVTXXfwwYOxIOZrvW
YXiG5bKm1uZdpOyluXHH4kKIrloLrWkZzgZaX/qnC20hNdX4Mk+NXG/9kpS9
FDV/FmnjbttcHWzcFamj/QtpQOPuv3tuq8wRNOEwZFQmPvj4ZgjGXRJR+BFQ
MzUHG3d75iWR3oeQmZLpMn7tiWvjrsHi+OG+iJS9uF6upeiBiT6YwCQvfHIY
g1NK8QMmMAhLsACSGbz6iYFtYUG2bx+QvYhefVXz74sL//0m0PpCnT9hnepT
nvsLk7EjfVMhPHZk7L3n3j1aPqqo2AONHTVlO0zeD5Sx6UWy1kNA64u846jD
AoUncsGt4dp3SdnLRMsnOgXTIReLWT40kAW0vkyM59G0OtgLt8qkqKysaScp
e6npOUTpwjOdYaK1T0Z+AlpfXEsjatxd1Ipg427bXF30m0x6FoZh4+6AZDzK
u+SL1mI63/YkZOMuiYj+CIh3425+zVQiGXd9X7ty+4fmQny2OyYwLEnPeeun
+r/GwgsTGA55FsZ4/u+UDrnskNgBExiEJVgACYn7znDixAlgNbz68c6y1arK
zweeNL0ydnTmtPzvfwGyF+nnH1pGeyM0dqTtyAeyF3npTt+xo2UHrR4vmrv/
Jil7EVftNsm6gNNi1IxPtu0lZS9DVW9LZ647nYy+PblWEwlLG+tTSdlLZ8de
tTrEZ0ZQuumankOk7KW85aNJUVloW/ZlWFwNGHeL2/8+T40E3kqoWOzmmuly
YOyoZKAw+sbdJQMFG3crJmpjwbhbOTWYUV7ExrhLIkI/AlaNu0D28m1lUywb
d+vGZTt/bgOMu32i2N35VQRSU2bhFCl7+dPJwd+aZV09oV99TGBYggkMsp6n
/t70FExg4gVMYBCWYAEkJJjAJAMBnzQt//xz1flsYOxo9j9fNlTdNzaWRmLs
yDT1WFa8FRg70g/e8D0ok6xrnvcBKXuZu/+merxo2UEMTxw2w/xILjB2xO/+
ymJgOjigVg93duwlZS+N9akiYWmwV20Vm93YP10IG3ctNFObB4xcKyjry4aN
u2F5shKJqaURwLjrWjQmY/tY7WCxO+2wcTev9UosGHdnVUsn68rYG3dJROJH
wOCifP+9KlL28mnJo1g27i6oTUfvDQGtL9fb52LfuHulRkIy7rpeh4pmxIqV
XJHN1ccEhiWYwCBIXIMJDMISLICEBBOYxCbgk6bFW7eqL18Q7/gj0Pqivpw3
cv86w6cdBRg7ytvs+bQjGzUDjx2pqo/4jh3ZjTJ47Mi11KYXAaeFWngCjB2N
1m93rcDwDNvtxonxPGDsaGggK+SxI6G0ATbuyqjwJBJWuwk27v7a/I3aJA3L
Z/lFY1aVDBQCrS+8ibuxadyN8i75YrLRxQOtsHFXaQpxhMRNeH8ExLtx93r7
HGDcPVQ6MKtg+qh6ruiZ0X364zgpe/nw7Ej1wNqNFxMYDtl7+u38ur2x8MIE
Jro8e1b1M9D9ErdgAoOwBAsgIcEEJlFxKBSKrCz4SdPU+X9KP/8QyF5kBz+y
jPVpirOhsaOsVKZjR9V5QY0dWcSdXge17KA1k8XA2JHoUapxsQU4LWYdf7pj
P9D6Ih477yA/q9oLqaSmuXEHKXtpbd5FUb3BX7oV9KbFpsHjQOvLyOzNcLWj
BDTuTkpbI3oHaJur+77ua1L28u/OHxa1UJ4WCcw2y2/Dj2Djrt7K/bfsbrEA
Nu66VgjLB4WxACrH+bBxd3BRHq7PCjt9IhVg3N35c1vFwALX+xgAlcEWrHEX
ExgOwQQmsVmdKXKx7rHT/Hwv/e4zUtLxeUhxByYwCEuwABISTGASEvhJ066X
4tt/KM8eBbIX8Y4/mtpqDTXF0swUYOxIX3mF4diRTbY2UACPHUmuvKYfvOE7
QGSSdYmrdgOtL6rRX5zk8MRhM0imLwPZy2TbXqOGqUnVZBT39RwAWl8E/KsM
BTLe++m0jczeBIy79X3faAxzIWzZz1HQ2sdDeUDrS8PElVXjboTuAIta0c/t
38PG3Uh8LkzPwvC3NXmAcXdSEcEHcDNEqtfAxt3bw50hGHdJhKUAhCotbNwt
6R+LWeOu2kSfezIBjB25lsa+cfdRtwIw7n764/iE2M+T1DCB4RBMYBIXn5wl
Jf9pvsJL9xu/rF8JiRcwgUFYggWQkGACk2CYmprmN22CnjSdlqYq+Od86kuk
7EW05QX15TzLaPvSyQ+A1hd14WGHVhn2sSOKd8Ch9552cVhU8o6jQPYiadhn
1cwAp0Ujbx2t307KXoZr35ULbjE8w04nLeBfBbKXvp4DJqM42Au3iowaeNKV
CRh3BYtVoW3Zl37RY8C4+1vnUYl60r1y2O8AFruZN3EXNu4arGxnZ4JlyUAV
dN0EWl8qpxo5N+7aHA7YuHuyrixk4y4JlgXAxLgrVHH/FG8SvBEJYNz969Wu
2DfuihWWgMZd2u70+15MYDgEE5gEhdDmks5z5y+eDS98nsfqmMHEFZjAICzB
AkhIMIFJGGwLC5K0tIBPml5MewdofZEf2WOdGVUXHobHjqxTPb8/HTtSXNoe
trGjkvdDGzvSC6EnAlgM84LeI0Dry1z/MZuV6bcniuptbd5Fyl6aG3dIJTXB
X7qn+0lrusb/BYwd9UxeCKNx907PcVL2Uti4t1/02GvEKbx3gCFJN2zcnVqK
4OOW/GJ32qtmWgDj7o/t1xd13E/HTCmkWdV3SNlL5oPrlVODkfhcNgUwuChP
L+UBxt3K8dj9PrGgNh0qHSBlL9sutlxvn6Md/oOLGGHVuAuMHR29wV817pLA
BIZD9pzecr4uPRZemMCEk7U+l5WgZYX1nS/+Upa1NfA5SHEEJjAIS7AAEhJM
YBIAp05HnTsHPGl6duNGKvcs/KTpxbR3TB31+sorwNiRa5Ghptj1iWEfO9L1
XfYdOzIrBuCxI2roJ2DsyOmkpTPXh6reJmUvEy2f6BTdDE8yTauHBrKA1pep
yQt2u58GfiZMix+Vt3xEyl6edGWG0bjbNlMCjB09Hsrza9wN1x1AaVyCjbsN
/MfRN+5OKgSAcffbmrz2+f4o75IvJht9ta8JGDu62FGjtZgi9OmhFYDGbPmh
oQtofXEtjXHjLjB2dKh0YEEdqRMeLtomNB+eHWFo3CWBCQyHYAKTkKw1ungk
LR4ZDCFica+BEUz84K/VKbibIX4FS3KwABISTGDiHe3Nm/CTpmWf/Q1+0vR8
6kvakkLzQKM8KxVofdFczYrm2NFS1wl47MhCjQGnRafonmj5hJS9DFW9LZ25
ztzTIhKWNtankrKXnq79Wu1k4K34g9JN1/d9Q8peypp2js3dDpdxly/vAoy7
rkWuFUjvZX8HsDvtzYIngHH3and+9I27eqsBNu4WD5bHgnG3TTSd+eA6YNwd
lYU4+MaQEAog3o27H13uAIy7deMyrvcxACqD7cSt2aCMuySCvfp+v3FgAhMa
mMAkIu4RpPWtLu5/JgUsa6NLGMHEDdgDg7AECyAhwQQmfjF3doq3bgWyl4Vt
29Q/n4efNK3M/cY60a+6+CWQvShydg9Xlro+0TLZKM/bHNGxIxeayWJhxRZS
9uJapBOUA6fFZlUJB7OBsSNB7xHazPTbk1Y72dmxFxg7EglLg7xuz/fTbhzi
XwXGjpoGj+tNi6Ft3Au9RQkbd9tmSqx26K/5LO8AIjUfMO7+0HC0S9TEZvuh
ARt3zzb/EiPG3XPNj4HWl7LRHpuD0fdoNgRVAEyMu1HY59BQm+jTj8aA1pf8
mimj1c71bgagrH0JNu4OzQUhWcIeGA756+l38uoyYuGFCUz4IPWyPP93Yr6C
TTDxByYwCEuwABISTGDiEdvCgmzfvgBPmv7hLPykaddSc3+H9nYukL1IM1OM
jSsJw1DTY3jsSHUj3XPsyDh+PxJjR8qBfwJjRy7kglvDte+SspfR+u0aeSvD
k2y3GyfG84Cxo5Hh72haHeSle8aiovth219I2YtrkVDaENqWvXA4bf2ix4WN
e0nZy52e43Jt4Jwh5DuAwap/OFYCjB3dGSrixLj7Y/t1UvZy+Elu5VSj3cnx
t2ybw/FgvA8w7uY0VEj14VEDBYRhATAx7i5qo325mXO3dx4w7mb82vN/2XvT
rjbOdO+3P8Kz1vkEe62z1smrznnRnb3avdNPJyfpZMdPvB3HibfjdsdOxzYZ
sGO7scO2E5zgCTvEEDwpBoKNDcYYM4MZhUBiEEKAhEAgITTP81wanFMOCZZB
dVVprALu39KLXq2idKO6XUi/XNf/Emmy9IYnzazaCyfu3u7TEyXuEoEMDI0g
A7MRITMwhFm7yMCsP5CBQaQI2gAbEmRg1hekkS/4w/LtN+Zvj8GTpj2PH3nZ
dUDkC/5w3i+K+tzRgNvdVwa4F2Px3/zirpUVYkZxcm1H5vHzqbQdeR2SOe4B
oPRFN3+LetuRXtfNYe8kci887odW63ii1+7XdQZMg9OFQOnLxDwrlGyezCqM
TnntSD6Re6ngfDqhbKN4quTuAAIN73J/PpF7KRv6Vm5JsnsracLRcLuUDSTu
Xh+9a/KQJ2NkGtLEXfbibDbXQ2UDkCbudkvTM0U9E0gNLjhxt35cxfDEXU8g
cr1Dk0riLhHIwNAIMjAbkWS7ieLGxyCYDTIwiBRBG2BDggzMOoI08kX3j39Y
S8+TTpr2C9nwpGnzuT0h9dOvxn5xF9B2ZDj3J3dfWWzbkWOoKIm2I+dCHdx2
hB8AvC2RkEc9cwVwL7KxowGPiuKb7POqBfwviNwLu2+bXFZJ3eQ8t85oaE7Z
0DCwC0jctbrmyU9EgWDY1z9bDrQddYp+8GEJDP9N9A5g8Zoqx0rgxN3sF5nM
meVn+68SuZeC3lK+ZjrLS1oLaeIu/mzmEneJgDcAaeJuCYfP2MRdbzAMJ+4W
NIk2QOIufkDSJ0cGhkaQgdmQxMyipu5gnv0QEjDrCGRgECmCNsCGBBmYdQFp
5IvqtdccP17VfPBqipOmDcde8Y09/TgQMsxbK/ZBbUf3csOWpZUVeiWP9D+9
nmjbUcA6o+3ZD5S+mMfPRwLQqGirphNuO8IPoPgmR6OYXFYJtB0JBSd83iTz
Ts2Omc7RXCBxd07ZkNyZ1zKnH6rgfAok7qqsCc96pn4HCEfD/bI2wL1UjpVY
vKZEF5Ai7qCnerIRSNytnW71h+hXBHDi7umuB5lO3CUC2ABw4u7Rxh4mJ+6O
yC1A4i7+FGc+23s1UYwOLP+2DCh9Ke/WUUzcJQIZGBr5qPCtyz2fMuGBDExa
iXEwv9vydCI1fHRHycFnx6MWpPUEMjCIFEEbYDODrj5dkEa+PJ00/d0l48mP
qUya1uX8gci94E+5GkqW245cXcWAezGVvh2YY6+sMENtR9qe/Sm2HWlnr0XA
0JhYrNbxIc5uIveCP6XXdSd46X4lgDn4c2VA29HgdCF+THInX4Xdp28QFAKl
L6Py+uQmK1G8A8gtc1c4XwOJuwINL4lXTxGOYgxO3FU60pN4nAp6t+NcfxOR
e8lprMhO4i4RcTcAnLi7p7qZ4Ym7BU0ioPTlWv8CwxN3sXC0lmN468wkkXvJ
ZUln1WloaUQGhkaQgdm4dFCWKjFzqlH9y3oDGRhEiqANsJlBVz/7UIl8MX/9
lfX7M4B7+W3SdJ/x5BtA6Yv1Sk7EosNf1Dv+0Fj0V6jtaID1JPxrKQv5tKO6
XZloOwoFbSpREeBe5oc/8zokFN9nDLNPCU8DpS/SubJwssEscu3jxsG9QOKu
wSpM7syriERDo/J6IHG3QVBo9632YNQhvQN4gu4HUxVA6csj0Z3sJ+5qXUY4
cbdXToMRWkUoEmkQ8xmSuEvEqg2wARJ3d1wdBBJ3pQYX3WskYVbt3V8qIXIv
289O1XLSNi8bGRgaQQZmQ7NS20LNwJDWyiCYBzIwiBRBG2Azg65+liGNfDEc
OGC7dgmIfMEf1isFwdkJ65UcwL0YT28LiLn4K2IaEdx2ZK87HnHoVlbok7bB
047ck7eTaDsyjRbAbUcWVTPQdoQ/hR9A8U2ORjHlUh27bxuRe+GPfuZ0JhkV
6/Ao+gRfAqUvosW7yZWjrEVlFVXz/gUk7s7pqU6AIgK+A3AVvUDi7g3eBboS
d4G2ox/5991BqiVSmUNsUMOJu1xleqKBUiR2A5Am7rJlShqXCiM1uHLu8OHE
XbrXSIInELn8SAkn7to86bm3LIMMDI3sK9xa1PMZEx7IwNCHrKOkA7mXdQoy
MIgUQRtgM4OuftYgjXzBn3XcIol8oThp2tNdjb9i1GN1Np8B3Iv52rvijqqV
FZK2Hdm6TkZ8q6fJkLYdqR/v8ZuhghCvQzI//BlQ+qISFYWCkL2JxW6fHhk+
QOReOOydyiWoDgcgFPaKFu8C7qVP8KXbl562Fx/m7J65AbQd9c+WB8NpCBEl
ugNonUogcfdCbx5dibsFvaVA4q7YSL/WcAZ8DEzcJWJ5A5Am7l7nTnixdH73
TyPeYPha/wKcuGv3JROynU26hNbtZ6eAxF3+Qvqrd5CBoRFkYBCIdQ0yMIgU
QRtgM4OufhYgjXxRvPSS9cp3hmN74UnT3v42L7vOcOwVQL84qgsjTsuTMOYZ
+gloO8Kfwg/AD1veAOTTjpJtO3LMVQPvDOm0o9nBDxNqO5oRXwDajkTT3+LH
JHEFcbTmsRbuR0TupXFw75K+P7kzr2Va3QUk7taO5Ovsaas8WXsHCIT93fON
DEzc/ZF/Hyh9aZrtQYm7SYBvANLEXYnBQvcyCemVGODE3RE5cxe/jNocyGVJ
M5q4SwQyMDSCDAwCsa5BBgaRImgDbGbQ1c8oEbMZjnzBn7J8+43lYj7gXpRv
/t5RVRaUjpvP7QHci+XSR8uTpoMynvnau0Dpi6MhP+r5tZQF3wAe0X142lGG
2o6smk5x37tA25FRfo/6W61WNXLYO4ncC4/7odU6nuDV+xVvwDQ4XQiUvkzM
s0LJ5smswuiUA4m7LPaBCWVbulqclll1B5CaRHDi7pRuLI2vTpFeOQ9I3C0e
KkeJu8mxYLZ98aAdSNx9MDnLtDWvoLH74MTdKp6C+Ym75d26LCTuEoEMDI0g
A4P4hY6DKIJ3fYIMDCJF0AbYzKCrnyGeBIP269cXX3wxxcgXc+ExbGmedNK0
X9iHv2jYsmS7lwu4FwtrN7b07IpjRvHSnZ2Jth1FQx6L8LtU2o4CHhXcdrQ0
WUi97cjpnOOPfkbkXth92+Syymg0mR6ESDQ0p2xoGNhF5F66+cetrvS0vQTD
Pu5CDdB21DZV7A6k/7/mr9wBHH5bjZAFlL60zNRkP3FX6dCixN1M4MVCcOLu
+R4eYxN3sUj03sgSkLh7vE7I/MRd/oLr78Xi7CTuEoEMDI18WPj2hZ5cJjyQ
gaGVX8J40Rjqdcjv4pHQGdBXsE0O2gCbGXT1M4G7pQWO26UY+RKYHne3l+tz
t0CTppuvPwkFowG3u68McC/Gor96R56VlER8VnvvV3DbUVA3sfZXc8kbU2w7
0s3fgtuOXGaqVRbhsFc6Vwa0HQkFJ3zeJJs+zI6ZztFcoO1oTtmQ3JnXIjOO
Vg0dIXIv+FP4Ael6rVXgd4BwNMxV9F7ozWNU4q4/FGia7UGJu5mAr9IBibv4
U0xO3BVpHEDi7q4bXOYn7to8oXN1S9lM3CUiiQ8AqX/pQCyDDAziF34dh4QK
YdYdqAYGkSJoA2xm0NVPL6Rxu9QjX0gnTdtZJyLOp0URTydNF/8N0C+u9vMr
bUc47snbQNsR/hR+wNpfLWCd0fUfSqXtyK7vA9qOph6/YZTfo16sotd1A21H
Q5zd+AGJXLqY3xRz8OfKgLYjnvgifkxyJ1+F3advmyoGSl+4CzVpSdwlonf0
8Q3eBSBxlyPvzH7irtg4vy4Sd1ljfeslcXcZhz9wvoe3ThN37T6spFsKtB2d
bZ1hfuJu65g5+4m7RKAaGBpBBgbxC78NpE5WwsjQMCWaQAYGkSJoA2xm0NVP
F5hcDsftUol8UW3743Lki+XSR4B7MZ15Lyjl/0xh0jT+bMjw7LtqQD1irNsF
lL44Bgoz1HYkHz8JlL7gz1JvO3K7FwX8L4DSl4V5VjjZYBa59nHj4F4i99I+
nGOwQr8pdSLREF/RyGIfIHIvDYJCo1OelteKSyDsb5mpAdqO7giuZj9x1+Z3
wom77VI2ExJ32Yuz6ytxF6dRNL+nuhlI3F0wU/03mH16JYZdN7hA4q5AydzF
LyPX++hK3CUCGRga2Vu47VzPESY8kIGhlWcGJrFuJJms4+CWLQn+ECKNIAOD
SBG0ATYz6OqnTsRsNp8+DbiX5cgXa8k5OPLFeqWASuSLl/10mHLUY3U05ENt
R8V/84u7ni3SZ7V1nQTci+nhPzCjeO1vB7cdKR69CrcdRaOYbv7W1OM3iNzL
DHt3Qm1Hclkl4F4E/C+cziT7ZRweRZ/gSyL30jCwS7R4N10puCqrqHYkn8i9
VHA+nVC2peWFiJjSjV3uz2dg4u6JziIi91LKq2JC4q7aYV2PibtHG3uAxN1G
Ef01RURo7L7jdUI4cReLROleJsRy4i7gXjKduEsEMjA0ggwM4ldkJVue/WMi
L4SRdZQc3BL77w8ZGHpABgaRImgDbGbQ1U+FJ8GgraQEjtvV7Nhhv1Wm3vln
wL0Yju0Nzk25mq/rcv4A6BdXQ0nU516eNG049yci94I/5e4rww/7dZERzCW4
pSv/C9B25JU8WvvbkbYdGYfz4bYjl3lshr0baDvSzd+i3nZkNAwMcXYTuRcO
e6da1ZjoFVwmFPaKFu8CbUcDk1+7fen57u/DnN0zN4C2o07RD/gxaXmtuFi8
pjuCq3DibiDsz9wC4qJ0aC9ybhK5l1PdxcxJ3AXaji5x2hiYuHtrZBJoO/qy
odPsYVar1ApYJFrFUwDu5XidcNFMfxYQDH/B9f5FEZC428DLdqXZCsjA0Agy
MIhnUJIwa9QLMjC0ggwMIkXQBtjMoKufNK76ejhuV/nyy/Zylv7z9wH3ovng
VR+3xzfWYTj2CuBerFdywnoF/qKBObap9G2g9MVedzzi0K0s0r/EMda8A5S+
LDz6VxRb/RUmGvJYp36A2458BigbFvMbSNuOAh6qgZk+r3pKeBoofZmVFGOY
PcEL+Cta81gL9yMi94I/pTIOJXfmtUyruyo4nxK5l2rev1RWUbpeay3haLhf
1gYn7mqd2Y5g9YcCD8Wd6yJx91jb3fWVuMuWKeHEXZ5Cw9g/AQKl7Z+Vo0Di
bpNQQ/caSbB5QgX3FoHSF/zZ7CTuEoEMDI38vfC/ABOezQcyMIzgOQnznFR5
1m8Ujy0HUYYvPSADg0gRtAE2JOzfWFpaAg5DVz8JSON2F1980Vbyvel/PoEj
X5w1rKB03HxuD+BejKe3BcTcnylMmjZfezcoe1YnEHYorR1fAO7F3PRxyLqw
dgO45I3K1m1w29GTCGHhSjSK6ReqgLYjcd+7DspOAz+bXFbJ7ttG5F5Ghg/Y
7dOJX8OneAOmwelCoPRlSlYZSjZPZhVGp7xBUAiUvozK69PV4hQXuWWubOhb
IHGXq+jNfuIuXzMNJ+7OmTOYhEMRZ8B3hdsJJ+76QswKgNU63d88HgJKX26N
TC4n7jLwT4Ddh51tnQFKXy51zjI/cbeBZwISd/9eLM5m4i4RyMDQCDIwiNXE
SpinhTBERS/LB2w52CFD7oVGkIFBpAjaABuSlTtDQUEBcBi6+gmByeW6ffvg
uF3z119Zvz+jfPP3KUa+6HO3eLqfRqxEA25XVzH1SdNRzOMcLgHci6F6q0/6
a8xI7AbIdNsR/tDN34qEqNYzWK3jPO6HRO6F3bdNuVRHvYkplkg0NKdsaBjY
ReRe+gRfWl3pKWkIhn3chRrAvTQICu0+fVpeKy6eoPuR6A7w8btGyHL4s51i
avJY10Xibrt0Ek7clZozeO2SIBSJPJicBRJ3T7T0xSbuMu1PQJNQAyTu/rNy
lPmJu7NqL5C4+9aZyfJuHRZmRHANMjA0whwDc+LEiWkC6H6TNh8dxMrlmXkp
QdOPmAAyMIgUQRtgQ4IMTHqhGLdr/7EEjts1nvwYW5wjjXxx3i+K+tw/L0+a
LvoryaTpgHtlnV7JI0P1VkC/uMaux7YdLW+ASMBmHj+f0baj+eHPqLcdYZgd
bjvCnw34jclcyJ9/NjtmOkdzidxL4+DeeXVrcmdei8w4WjV0BEjcndOnrcUp
LqPKASBx9wrn6/bhJMNzkiYcDT9eGGR+4u6izXSmtwFI3G2XTjItcXdSa0w0
cZc5fwKkBtd6T9z1BCJw4u7xigW1mX6vuAIyMDTCHAODamCYBZGEQeaFYSAD
g0gRtAE2JMjApAuKcbuOKpbmg1cB96Ld959+/iBp5Ivt6pGI5WmQC7YksLB2
A+7Fdvtg7KTpoG7C9PAfgHuxtH4adqxO+cA3gGOumnTaUYptR1ZNJ8V3Gz+b
cqkOaDsa4uy2WscTv4xPCWAO/lwZ0HY0KvkePya5k6/C7tO3TRUDpS/9s+UZ
TdzVOpWVYyXAp+7u+cZA2J/lO4DcqoQTdzkKGmYwrcIXwu5PjwBtR1e4nUxL
3HX4A9e5E0Db0fkeHn7M2h9kwp8AbzBMmrirsTM0LngF0sTdLqGV7jWuBhkY
GtlTuP2b7mNMeCADwziem1CN2o0YCjIwiBRBG2BDggxMWkhL3K5q2x9dD6uC
Uj4c+WI68x5+DP6iEYfOXnc8oUnT9t6v4LYj/xJn7W/nM4wuNL2bStuRwzgk
7nsXKH3Rzl5LqO1oZPgAUPoil1Um13aEI9c+bhzcS+ReOkdzDVZhcmdeRSQa
4isaWewDRO6ldiRfZ09yZDYVAmF/v6wNcC+VYyUribtZuwO4g57a6Vag7ahS
UM+ExN0xtRxI3MWfYmDibrdUsb+mFUjc5at0RD9L+5+AEbll761hIHG3Q0S4
eIZg84Tyb8uA0pfLj5SeALPKpZZBBoZGkIHZZDy1Klu2bDl4sKSjQ0ZqVGQx
EgbNOmImyMAgUgRtgA0JMjAp4unqUr32Gnnc7lefAe5F+ebv7beKQ8p529Uj
gHsxHHvFy677+ZfIF3dfGcmk6QEW9UnT+FP4AWsrWCIBm3E4H2478pshIxHw
qEjbjvwuqv/ZBsPsM+ILgHsRCk74vOrEL+NTHB5Fn+BLIvfSMLBrRnE/XSm4
KquodiSfyL2w2AcmlG0ZTdyVmkRXOF8TuZfL/fmjyoHY47NzB+Brpk91FxO5
l7P9V5mQuGvxueHE3WrhEAMTd0+1DwClLzUTM8uJu0TQ+CfA7sMKmkTrPXG3
lmMAEnf3l0pm1elJ884EyMDQCDIwm4zn6lp+q26BhEyMhCGaT42gE2RgECmC
NsCGBBmYpPGPjGh27IDjdi1nC0njds2FxzDFvKuhBIh8wZ/CD3gW+VL8tzRO
mrZ1nYz4Vte9P4lgdkmF4tGrRO5lqelNx1w18P5Eo5hu/la62o5wlEt1HPZO
oO1Ir+tO5AI+IxT2ihbvAm1Hg9OFbl96Ikd8mLN75gbQdtQ2VZzpxN0aIQso
fXkwVYEfs+qnMn0HMHmspbwqOHE3+zOYVhGKRNqlkzmNFUDi7qLNRO8iV4Gv
uWZiBnAvp9oHYhN3iaDrT0D9uGrH1UEi95Jzh78BEndrOQa610gCMjA08sG3
75zp+hcTHsjAZIU4BoZUyMQMR0KFMIwDGRhEiqANsCFBBiYJghIJPOoIfxg/
/9x27RIct2s4tjcomfJ0V6cr8gV/Fj9mZZ0h64Kl9VPAvZge/iOom1j7C3q1
g6qO94DSF/P4+Wy2HTmdc3DbkXSuLJzsPGiteayF+xGRe8GfUlEeik3KtLqr
gvMpkXupGjoiM0I5xqnDVfRe6M0jci9lQ9/KLfH7njJ3BwhHw+1SNuBeSnlV
Jg/9yRiLNtPprgdw4i7da1zNpNZ4sK6DyL3sr2ldm7hLRPb/BEgNrpw7fCL3
suPq4LpI3L3eoQHajvJvy4wOplfv4HT0TOR9r7A5k1GgyMCkCDIwmwxZyZYt
ZHOO4giZZ7G8W5CEYRbIwCBSBG2ADQkyMAkR0miMhw/D7kW7e7f9Vhkct4s/
6+P2+IV9xtPb0hX54h1/uLJO0knT+p9e94jux/kF3UrD0HHAvWh79gesM8Bb
RNp2JBs7mlDb0aykGHAvAv4XTmeSYSlun3ZwuhAofZmSVYaSFTurMDrlDYJC
oPSFu1ATDGcwRFTrVN7gXQBKX/plbUCRSYbuAHNm+dn+q0DiLl9D/5xTXwir
FAzAibvOALMCYB3+wOX+0SQSd4nI5p8AbzB8rX9hvSfu9otsQOIu/hR3llkp
zXHx+CKsh4b//dEU/vjmJtUpdbEgA5MiyMBsVmQyWUdHycGDCQqZZQlDKUQG
kRWQgUGkCNoAGxJkYCgSMZsthYWwe1Fv3eqopBS3iy1OWy59lLbIl76y2EnT
HtF9/U+vA/rFOVwSO2l6mWjIYxPfBNzLUtObLjk0kphK25Fd30f9Pddq2oC2
I/wp/IBEruEzItHQjOJ+w8AuIvfSJ/jS6kpPkmow7OMu1ADu5QH/a6Mzg/Em
gbC/Y7YeTty1eEl6Z9J+B3AHPZWCeqD0pXa6lQmJu1zlfG5zFZC4K9Qt0b3G
1bRLZHDi7qQ24RHtWfsT0CsxwIm7+AHZWUnSqM0BOHG3vFvHzMTdVYyJ3btO
zC7rl+UH/v8kehJkYFIEGRjELyQrZH6pkPlFyND9G2xWkIFBpAjaABsSZGBI
ibpctpIS+QsvwKOOHHeqSON2bdfOh3QKO+sE4F5SiXwJ6iaMdbvgSdMh68La
39G91KFs3Qbol9muk1GwaYi07Ug3fyuhtiMB/4sMtR0ZrML24Rwi99I4uHde
3ZrcmdciM45WDR0hci8VnE8nlElKJIpM6cbgxF38ACrnSe8dgKMYAxJ3L3Ju
yq2r56FnH73bca6/CSh9uT89wrTE3QWzjTRxNxRJ5rt/Fv4EaOw+OHG3pFvK
8MRdLByt5RjeOjNJ5F5yWVK5nunVOzg2Z/hU2VKse1l+7Doxi4WeJHQqZGBS
5L+/fffrrhOMeCADwyCeCpmSX4QMZReDupPoARkYRIqgDbAhQQYG4Ekw6Kio
WHzxRcC9KF56yVb2g+3aeThu1/TVZ6Rxu6lEvkTcelvXScC9GGveiTtpOmCd
0fUfAtwL/mzQsQBsACptR/gxFN/zcNgrnSsD3At/9LOk244CmIMnvgi0HY1K
vsePSe7kq7D79G1TxUDpS/fMDR/mTMtrxcXiNcGJuy0zNYGwn+LZ0nUHUDq0
QOLuic6ixwuDTEjcbRDzgcTdc/1NTEvc9WIh0sRdrTPhAoYVMvonAItE740s
wYm7Ig3Te3amFO79pRIi97L97FQDj1l7hoh77abXD4nW6pflB/5sQmdDBiZF
kIFBUOCXAhkSIYMMDD0gA4NIEbQBNiTIwBDhqq9XvPQSPGbaeqnIUX4FjtvV
f/5+YGqMNG7XfG5P0pEvrrHryU2atgi/A9yLsnWbe+nXv9hxN0A0iukXqtLY
dqTXdQ9xdgNtR2oV1AYFM6dsANqOOkdzDVZorDZ1ItEQX9HIYh8gci+1I/kq
qygtrxWXcDTMkXcCibs3eBe0zsSKTFK/A/hDgabZHqDt6ProXSYk7ooN6pOd
tUTuJbe5aj0m7nZLFSm+ROb+BAiUNjhx997IEsMTd22e0OVHSqDt6FzdEn4M
3cskRyL3fXJORuReXj8k+qnJgGpgsgwyMIjEiStkkIGhh7g6LKEzbMKvYIhY
0AbYkCADsxZPV5fqtddIxkyfOeOsvqne+Wc4btfb30Yat2s8+YZv7OkfxiQi
X7ySR4bqrUDpi733q7WTpnEcc9VLTW8C+sUmvhnbdrR2A6S37cjnVcNtR7OS
YgyzJ3wtf8HsmOkczSVyLw0Du2YU9yPR9Hw/UllFtSP5RO6FxT7AVzSm67Xi
IrfMAYm7F3rzuIreJE6b4h1AbJwv6C0lci/4U0xI3HUGfKyxPqDt6OpwNwMT
d8/38IDSl+vciYQSd4nIxJ8Auw8r6ZYCbUcFTSLmJ+62jpm3n50ici9/Lxbz
F1x0r5Gc2MTduI+87xUqfZDi2Yj+G3xGf4WNyq5vdp5+/CUTHsjArFuWhUwJ
SoKhBVQDg0gRtAE2JAW/wWazgcM2ydX3j4xoduyA43bNp0+7HtyGRx2ptv3R
WcMKSsfhuF197hZ3e/mTUPBJGEsi8sX08B9JTJr2GUbVj/cA7kU/kBtyry6Q
iN0AaW87kssqM9d2xJ8rA9qOBqcL3T5tcidfhQ9zds/cANqO2qaK7T59Wl4r
Lp6gu2WmBmg7qhGy8GOSO3nSdwCb3/kj/z6cuOsPpUERpEj3gghI3D3ZWSs2
qOle42oaRfN7qpuJ3MvRxh6JwZKu10r7n4AOkW7XDS6Re9l7a5gzz/SeHbne
l8uSwom7WJjR1TvL9I053jkqIXIv+FPl95OvD0QGJkWQgUEg1jXIwCBSBG2A
zcyGv/pBiUS3bx/sXgyHDrka7+sObieP21XOw3G7+MN5v2g5btcv7jKVvp3G
yBf9T697JY/W/o5hrwGeNK1s3ebVDsZ9f5Y3QCTk0c3fAtxLom1HRsMA3Hak
XKpL7ELGMK9ubRzcS+ReWrgfac2UQmipMK3uquB8SuReqoaOyIyj6XqtuAg0
vMv9+UTu5Qrna7klSYv16/kTvwOEo+FeOe9EZxGQuKt0pEd/pcKizQQn7jaI
+cml12aOBbPtREsfkXvZU93cKJpP75rT+CdAY/cdrxMCpS/X+he8QZqzgGCw
cLS8Wwe4l1yWVG2m3yuSYnOG875XAKUvrIcGjy+SytVHBiZFkIFBINY1yMAg
UgRtgM3MBr76mFxuOHQIdi/a3bvdjXWGY3sB94I/LEVfUozbDeufxjJgSwJr
xb40Rr4QTZp+EsFs4puKR68SuRf8KbukYm1WzAr4BrDr+4C2o6nHbyTadiQU
nABKX2bEF5JuO7K65vsEXwKlL6LFu6FkRymtwuiUNwgKgdIX7kJNMJzBTgqL
11Q5VgKUvvTL2lLPtk30DiC3Ki9ybgKJu71yXopLSh1fCGsQ8wH3cq6/Se9m
VgCsFwtV8UVA29H5Hp7Zk/79lpY/AVgkWsVTAO7l8D2B1MD0nh3+guv9iyIg
cbdLSH+cERXgxN1Pzskk8l83EjIwNIIMDAKxrkEGBpEiaANsZjbk1Q9pNKa8
PNi9qLdudT+8D4+Z/nXUkWzW012tz90CuBfTmfcoxu0mEfli7fgi7IgTr+pR
98KTpk2jBZGADXij/C6ZsGcfUPoiHz+ZxrajkeEDdnuSqSChsHdingW4lz7B
lw5PqqmkywTDPu5CDeBeGgSFRqc8La8Vl3A03C9rA9xL5ViJxZueVg7qdwB3
0FM73Qq0Hf3Iv2/zZ3AIFEWEuqVjbXeBxF2ucp7uNa6Gp9AAibv4U3yVjvws
SZH6nwCB0rb31jCQuFs/TvUeQhc2Tyj/tgwofbn8SLleEnf/eWaeyL1s/Vxc
322OPR4ZGBp5/8z7/9N5igkPZGAQiCRABgaRImgDbGY22NWn4l6UL7/srKux
FH0JuxfDsb2BqTHSuF3DsVd+jdv1WF1dxYB7SSLyxVi3K6AeWftrBh0L8KRp
bc9+vxlq8I+EPNrZa4B7mWHvdhiHqL/zGW07WtL3t3A/InIvjYN78QOSPvkq
ZMbRqqEjRO6lgvPpjDZtrxUXuWXuCudrIvdyuT9/Spe2HqufKd8BeKqJU93F
QOKu2Ei/1rD43Fe4nUDpS6VggGmJu2aPD07creKLvFgGv/un8ifA7sPOts7A
ibv4MWlcbSao5RjeOjNJ5F72l0rWS+JuWa0OaDs6VbZkc64umUMGhkaQgUEg
1jXIwCBSBG2AzcyGufpU3IvipZfsrJu2a+eVb/4ecC+6g9t93J6glG8+tweO
23U1X1+O2/UM/WQs+ivgXqwV+xKNfPGI7q/9NalMmnbJSSY7W1TN0z1vA21H
+oWqaJTq9ya3ezFzbUdunxZuO+LPlaWr7cju0zcLLwKlL/2z5T4sgzUenqC7
RsgCSl9aZmoCYX96X5T0DmDyWEt5VUDpS7uUzYTE3XbpZE5jBZF7Od31QGrO
YFpyEoQiEThx91T7wIIZqmFLC0n/CagfV8GJuwJlxhefIrNq7/5SCZF7eevM
5HpJ3O3k2uDE3TFx/KRuZGBoBBkYBGJdgwwMIkXQBtjMbICrHzGbLYWF8hde
IHEv1685yq+otv0RHjPtefwopJ6zXsmB43Yd1YUR59NxJH5xFzzqyFT6Nn7M
ymqTjnzBcS7UJTRpei1eh2R++DOg9EUx8RXmN1B858Nh78I8C247slrHE7mY
zwiFvaLFu4B76RzNtbrSU3cRiYZG5fUs9gEi91I7kq+zpxR4SwpX0Qsk7t7g
XdA643SipQ5wBwhHw+1SNuBeSnlVDEncPd31gMi95DRWtEsnmZa4KzFYjjb2
ELmX/TWt7ZIszRdN4k+A1OA6fE8AlL5U8RQMT9z1BCKXHymBtqPjFQvrInFX
pQ9+ck4GJ+5ioSdEP44MDI28d+a/8zu/YsIDGRgEIgmQgUGkCNoAm5l1ffWp
uJfFF1+0lf3gvF+u3vlnwL3gz7oeVoX1S6SjjqxXclbidi2s3VDcbtFfPUM/
PQk/KyZJOvLFbxbCk6YNQ8fXTpqOJRS0qURFgHuZHfzQlcgIIb2uG2g7Yvdt
Uy7VUS+kWYXWPAa3Hc0pG5I781pUVlE1719E7oXFPjChbItEM9gGonUqfxy5
TOReLvTmcRW9mXt1ojvAnFle0FtK5F5OdRdzFOlshkoOXwirFAwAbUdXuJ1M
S9x1+APXuRNA21EJh48fk7X1JPQnwBsM3+LIAfdyvE64aKaa2k0XXULrBkjc
xUJPWA8NgHv55JxMpQ/CJ0EGhkaQgUEg1jXIwCBSBG2Azcw6vfpU3UtJifvR
Pc0HrwLuRbXtj46qspBO4bxfBLuXlbjdsGUJjtvFH66u4ti43aQjX8Jeg3E4
H3Av6sd7fAaSgcimpXqg7Qh/yii/l1DbkYD/BVD6Ipr+NpW2o8HpQqD0hSe+
GMDS853ahzk7RT8AbUdtU8XugCUtrxWXQNjfMVsPtB3VCFmeYPz2gXSx9g7g
Dnp+5N8HSl8qBfX4MRldFRW4yvnc5ioi93Ks7e6YOoNpycnRLVXsr2klci9H
G3smtcYsL4n6nwDOvAlI3N11g9shylRccLpQmwPHKxbgxF1PgFnlUnEZE7uB
tqOtn4ub2ZQkEjIwNIIMDAKxrkEGBpEiaANsZtbd1Y+6XLaSEiruxdNWD7sX
5Zu/t107H9IuuZqvw2OmjSffSDpuN+nIl2jIA0+aXmp60zFXDUyaxvHYJmcH
PwRKXyb6joaCVOMaMtp2FImGRIt3GwZ2EbmX9uEcgxWKF06ICWUb0HZUNXRE
YZ5I12vFZUo3BiTu4k/JLZnte1pm1R2gV8470VlE5F7O9l+dM9OvNfRuxyVO
G1D6cn96xBdiVgDsks15qn2AyL3sqW6umZihpVWKyp8Ajd1X0CQCSl8udc4y
PHEXC0fLu3Vw4u6UIrO2My3YnOFTZUtA6cs3N1VrE3eJQAaGRpCBQSDWNcjA
IFIEbYDNzDq6+svuZfHFFwH3In/hBUthoaf9oe7gdnjUkaXoy5BGQTpm2nDs
FfwY/NWTidv1WZ3DJYB70ZX/hSjyxSVvhCdNW4TfwZOmQ0GbYuIrwL3McQ94
bJPUNwCNbUcNA7tmFPfT1QpkdMprR/KB0pdReX1G244sXtMdwVWg9KVf1haO
ZilGY2UDKB3ai5ybcOJu1lZFRCgSaRDzgcTdM70Ni7b0zOlOF14sVDMxA7Qd
ffN4SOuk7bs/fAfAItH6cdWOq4NE7iXnDp/5ibv8BRecuFvLMayLxN177aat
n4uJ3MveU1KixF0ikIGhkZ1ndp/sKGDCAxkYBCIJkIFBpAjaAJuZdXH10+te
TF99hslmfdwmw7FXqIw6+jnxuN0nEcw9eVv/0+uAfrF1nYy44wxn8ZuF2p79
gHvR9R8KOhagtyuK6Reqph6/AbQdmZbqlw+msgEy2nbkDZjgtiP8WbcvPXGv
wbCvf7YccC8NgkK7L4MTc8LRcL+s7UJvHpF7qRwrsXizKhDwDeAPBWqnW+HE
XZOH/mQMsUF9srOWyL3kNle1SyfpXuNq+CrdwboOIveCP8WWZSRgmTrAHUCk
ceTc4RO5lx1XB++NLGERRosLmyd0rm4JaDvKvy1bF4m7Ernvn2fmidzL64dE
cOIuEcjA0AgyMAjEugYZGESKoA2wmWH41afiXvCH+fRpb3crqXsxHNsbmBrz
C/uMp7cB7kWX8wfn/aKo7+l/TMSWBNaKfQnF7frk3XDcrunhP4K6OB0upJEv
ytZtHjVJKKvDOCTuexcofVHPXIltO4I3QKbbjmYU94G2oxbuRyrjUHInX8uM
tr+C8ymRe8GfmtOn7bXiIrfMlQ19S+ReLvfnT+loyLZ9MNR8qrsYSNzla6az
v6pVOAM+1lgf0HZ0dbgbP4buZT6Hwx8438MDSl9ujUx6sQyWWlEk7h3A7sNK
uqVA21H+wymNnVlv+FoaeKbtZ6eI3Mv7F0X9IqZX7+B4fJHiO1qg7ejIJTlp
4i4RyMDQCDIwCMS6BhkYRIqgDbCZYezVX87aJXUvprw8T3eb4dhe2L3oDm73
cXuCUr753B6SMdOVp5fHTIcM87Z7uYB7MZz7U6Jxu4bqrV7Jo7W/LGnkC/6U
XVIBR774XTLZ2FHAvcwPf+Z1SFb9FLABMtp2ZLAK24dzgNIX0eLdUNib3MlX
YXGrGgSFQOkLR3o7GM7g10lP0P1gqgJoO2qZqcl04u5aTB7r9dG7QOlL7XQr
ExJ3uxdEQOLuyc5asUFN9xpX0yia31PdDCTuLpiZ8t1/7R2gV2LYdYNL5F72
3hrGD6BlqdSZVXtzWVKg9OV6h2ZdJO52cm1w4i5+QCrnRwaGRnac2ZPX8Q0T
HsjAIBBJgAwMIkXQBtiQFPwGm80GDmPg1acy5yhR9xJSz1mv5MDuxXb1yPKY
6YhD52jIT2/crq78L66x68lFvphGC+DIl0jIo565ArgXcd+7FlVz3J+NuwHc
7kWh4ARdbUcDk1+nse1oVF4PuJcH/K+NzsxGy3IVvZf784ncyw3eBaVdltEF
rCUcDbdL2UDi7kXOTbmV5u4YnEWb6Vx/E5F7yWmsaBDzaUmvBVgw24429gCJ
u42iebrX+ByxdwCN3Xe8TgiUvlzrX/AGac4CgvEEIuXdOsC95LKks+r0qN2M
otIHj1ySA6UvFyrUHl+qmx8ZGBpBBgaxSek4uHzfONiRyE/JZB0lB7dsibnv
bNlysKRDltAHqLSc5DeQgUGkCNoAG5KVO0NBQQFwGKOufkijMeXlkboXw6FD
7sY6Uvei3fefPm5PxKKzs07A7sVy6SNs8WmrxfKoI8O5P5HE7WpEK2uOYh44
bheIfPEZRtWP98CRLwHrDPymwZOm8Yd29lokRFjJsGoDZLrtaE7ZkLW2I4V5
omroCNB2NKFsS9drxUXrVFaOlRC5lwu9eVxFb/azbefM8rP9V4ncy4nOol45
j/bEXV8IaxDzgbajS5w2vTs9Q8nThRcLXedOAG1H53t4Dj/jIkeW7wBYJFrF
UwDuJecOX2pw0b1YErizjvcviojcy/azU7Ucplfv4GChJ6yHhtcPiYDEXYk8
PTV7yMDQCDIwiE3Jb/4lEQMjK4mVJqvZUkJJoKTlJLEgA4NIEbQBNiTry8As
uxdYvCy7F8/jNtNXn8HuRfPBq57HjyIWnaPyNOxeTGfeC4i5+AKiAbd7gAWP
OjKVvh2YY6+smUrcrrnpY8wojvP7upWGoeMpRr6QTpqWj58MeFTwSWI3gFbT
xmHvzFzbUedoLlD6MiWrTFfbkd2nb5sqBkpfOkU/+DBnWl4rLoGwv3u+EWg7
qhGyHP5s96G4g55KQT3QdvQj/77Nn8G3hSJC3dKxtrtE7gV/iqtkVhkJDlum
3F/TCiTuTmqNdK8xPvgdQKC07b01DCTu1o+T3EZox+jA8m/LgNKXgnuLNg/9
qTukjIndu07MAom799rTmdSNDAyNvFPw93+1FzLhgQwMIlvInvkXygYm9kcI
ITtXWk6yCmRgECmCNsCGZL0YmKBEYjx8OM3uxWlx3i/S5fwBcC/Gk2/4xp7e
bJ+EMe/IPXjUEf4sfkzssv1LHGPNO4B7wZ/Fj1n7+0ZDHuvUD4B7oRL5gvkN
8KTpGfZuB7VikuUN4HTO8Uc/y1DbUQBz8MQXAffSJ/jS4VEkd/JVRKIhvqKR
xT5A5F5qR/JVVhH5iVJgxiC8wvmayL3gT0lNmV1AXHrlPCBxt6C3VGykX2tY
fO4r3E6g9KVaOOQLJekAM4TW6T7VPgCUvtRMzDAhcTcudh/2r2rCyBf8UdAk
wo+he5kQWDhayzG8dWaSyL38vVjMX2B69Q6OzRk+VbYEtB3lfa8wWNJ8LZCB
oRFkYBCbDNkqDUJFeMCFK7EARSxpOclakIFBpAjaABsS5huYoERiOHQoXe5F
vfPP7pZaKu7FcOwVT3f18hr84i5T6dvwqCP3ACt21BFmFJubPgbci/6n1z2i
+3EVinOhbqnpzVQiX6JRTDd/C5g0jT9llN+jXqkyPj4wKymmse1oSd+f3MnX
orKKqnn/InIvLPYBvqIRX1K6Xm4tFq/pjuAqUPrSL2sLhP2ZW0BclA5t8VA5
UPrSNNvjD9HcHROKRNqlkzmNFUTu5Uxvw6Itq3O6ScHXXDMxA7iXU+0DSzb6
a4qIqB9XwYm7AiVT4oKJmFV795dKgNKX8m4dFmb0vOxl7rWbtn4uJnIv7xyV
DAkzspGQgaERZGAQm4k4FoTcwKz6oV8iW1aek3UcfP6UBPokLSeJBzIwiBRB
G2BDwmQD4x8Z0e3bR+peTHl53v4uKu7F9bAqbNaRuhd97hZX8/UnoaeDO4My
noW1m3zUkce6suxU4nbTEvli1/fBk6aXJgtjJ02Tolyq6+99ZwO0HfkwZ6fo
B6DtqG2q2O6LE8WTLsLRcL+s7UJvHpF7qRwrMbrTEy9MHX8o8FDcCbiXUl7V
4xGSZrcsIDXrT3c9IHIvuc1V7dJJute4mkmt8WBdB5F72V/T2i1NT1lXJpAa
XIfvCYDSlyqegvmJu5cfKeHEXbWZcak7a5HIff88Mw+UvrAeGlJP3CUCGRga
QQYGsVlYozkoGpjnambiq5Hn9Uq8M6blJHFBBgaRImgDbEiYaWD8IyPa3bup
uBdPd5vx5MdU3EvEYXE1X9fnbgHciy7nD877RVHf04G/mEZku30QHnXkbD4T
O+poOW5XV/4XQL/Ye7+KG7eblsgXr0MyP/xZopOmAazW8ZHhA0Dpy4z4Qkbb
jqyutPW8TCjbgLajqqEjMuNoul4rLnLLXNnQt0Tu5XJ/vkDDy+gC4sLXTBf0
lhK5l1PdxRzF2M903/+dAV+lYABoO7o63I0fQ+MK1+LwBy73jwKlL9e5EwxM
3F3GGwxf618A3MvxOuGimf4R5DBdQuv2s1NA4i5+AN1rJMfjixTf0QLu5ZNz
MpkqsyVzyMDQyPaCvYAez+YDGRhExiCQL5RUR6w7AUpT4GiZtJwkPsjAIFIE
bYANCdMMjKerS7NjByxe5C+8YCks9LG7SOccqbb90VnDou5eIk7Lz7+MmbbX
HYfdi+1ebsjwTA5EMY9LcAuO27W0fho3bjcSsFmE36UY+RIK2pKeNB2XgN8o
mv4Wbjuy26epn/C5X5ms7ahxcK9c+zi5k69FZ5+rHckHSl9G5fXBcAa/v3uC
7hohC2g7apmpwY/J3ALiYvJYr4/eBT5vV082uoO/fsum8f7PXpzNba4ici8n
O2vFBjVdayOiUTQPJO4ebexhbOIuTq/EACTu7rrB7RDpyM9CK2pzIJclBUpf
Lj9SegLMGlAel74xxztHJUTuZevn4vpucxaWgQwMjSADg9jYrJUvWw52xGbB
kJgOylIktoRl9YFpOQkByMAgUgRtgA0JQwzMk2DQVV+veu012L0svviipbDQ
0/5Q//n7pO7FUVUWNusSci9Px0y3n4fdy9Mx00vP3o0nEcwjum+o3ppE3C7+
s3ZJReqRL0b5PXjStG7+FjBpeu0JFYvV7L5tRO6Fw96pVjUmcnmfg7TtaGKe
lca2o/7ZcsC9NAgKLe4MDnAJR8NcRS/QdnSDd0FpT3i4YeqrapeyT3QWEX3S
vsi5OWeWx/4ILfd/tcN6rr+JyL3kNFY0iPmhCLO+Ry+YbSda+ojcy57q5geT
s0xb8woauy//4RRQ+lLSLWV+4m55tw5wL/tLJbPq9NxeMopKH8z7XgGUvnxz
U2VzZqkFDBkYGmGOgTlx4sQ0AXS/SYh1zPO9PU/ty9P/l7KBSUCJxJxzVZlL
Wk5CBDIwiBRBG2BDQruBibpctpISxUsvkboX/DBvd7Pu4HYq7iViN3u6q2H3
gj8clacjlqf/Pfepe+kqNpz7E+BezNfejR0zjeOTd8OjjpbjduP+4u6lDmXr
thQjXxzGoRn27hQnTcdiNg0PcXYDpS+zkuL10nY0re6q4HxK5F7wp+b0lOZA
JY3SLrvBu0DkXi705nEVveFotmM05szys/1XCT9jdxY9Xhhcu6os3/99IaxB
zAfaji5x2vRuRzaXRIoXC90amQTajs738LTObFc6UQSLRKt4ih1XB4ncy74f
B0UaZr3ha+EvuN6/KCJyL2+dmazlGOheIzlY6MlPTYbXD4mI3MveU9IxcVY3
EjIwNPJfX3/4RdsFJjxQDQwiE6zYj9/kyy9QNjDUi2UAe5KWkxCBDAwiRdAG
2JDQaGBCGo359Gn5Cy/A7kXx0kv269fdj+5pPniVunsxHHuFqnsJuEndy9Mx
0+MPYxcfUI+YHv4jubhdv1mo7dmfYuSL3yWTjR2FJ027zGPUL4fPqxYKTgDu
RcD/wumco37CWLLcdmR0yhsEhUDpC0d6O9NtRy0zNUDbUY2Q5fBne4KMO+ip
FNQD/4nzR/59kyd+MkY27/9C3dKxtrtE7gV/iqukfyL2KtgyJZC4iz/FU2jo
XiMhAqUt5w6fyL3suDpYP64aHWf0BwCbJ5R/WwaUvuDP4sfQvUxyxsTuvaek
RO7l9UOin5oMWOhJlleFDAyNIAOD2NjISrY8N3doGapOJKZ6hdyHEB2clpMQ
ggwMIkXQBtiQ0GJgAkIhlQHTipdeclRUuFsfpNe92FknVtyLe4BlLPorPGba
M/TTqjHTltZPAfeSStzuUtObqUe+TPe8ndCk6XDYuzDPAtwLh71zmHc9kSv8
HNlsOwqGfRzpbcC9POB/bXTKyU+UAqPKgcv9+UTu5Qrna7klSZGVCr1y3qnu
YiL3UtBbKtRBEc3Zuf9bfO4r3E6g9KVaOOQLMasLRut0n+/hAaUvVXyRF2Po
d3+7D7vUOQu0HRU0iUyup3HBTP4AUMsxvHVmksi9vH9RxF9w0b1GcmzO8Dc3
VUDbUd73CpU+SMvakIGhEWRgEJsRqgYmsYoUgrOm5SSEIAODSBG0ATYkWTYw
VIJ28Yfy5ZedtbXO++XqnX+mMufoCRb0sutI3Yvt6pGQ+umXXyruxXDuT+6+
MvzIlcWTjplejtsNWRfW/uKkcbv4Az8g9ciXRCdNazVtcNvRwjwrHPYmtwGy
3HY0px+qGjoCtB1Nq7vS9Vpx0TqVlWMlQNtRv6wt+21HSoe2eKgcKH1pmu3x
h0iG8mT6/h+KRNqlkzmNFUTu5Uxvw6LNlNE1JAq+5geTs3uqm4ncy6n2gQVz
tiudqNMk1Oy6wSVyL3tvDQuUzxbPzA8As2rv/lIJUPpS3q3DwlG6l0lOfbd5
6+diIvfyzlFJ3xidLWDIwNAIMjCIzUgSBoaCDiE3MMmfhBBkYBApgjbAhiQ7
BuZJMOi8e1f58suk7kW9dauzttb1sIrUvWg+eBU/LOp1edl1xpNvUHQvT8KY
d+Qe7F7wh6v9fDSmHSPiszqGimD3Ynr4j6BuIs7vTiFu1zB0POiI421iIY18
SXTStNM5J+B/Abcdud2LywcnugGy3HZk9+nhtqPumRs+zJmul1tLIOzvnm8E
2o7uCK5avNkWCP5Q4KG4E3AvpbwqpUNL5VQZvf9LzfrTXQ+I3Etuc1W7dDJz
r54ck1rj0cYeIveyv6a1XZLtgGXqSA2u43VCoPSliqfAIs+JC6Z9APAEIiUt
asC95LKkajNDh33HIlP5PzknA0pfymp1Hh/N0c3IwNDI21/vO9xaxIQHMjCI
7EFRcyQ4mih+sUtaTkIMMjCIFEEbYEPC/o2lpSXgsKSvfsRstpWULL74Iql7
0e3b5+vvc1SVqbb9kdS9eB4/ehIKUuk5sl7Jec69FP8Ndi/2uuNhy7O3YnnM
tK78L0mMOvqZQtyutme/zzAKv4ekkS/ivnft+j7qFwXD7DPiC4B7GeLsNhoG
Yn8koQ1gsArbh3Oy1nY0Kq8H3EvtSL7Ontmunynd2BXO10DbEX5ARhcQF75m
uqC3lMi9nOou5igSWFWG7v/OgK9SMAC0HV0d7saPycRLJ43DHyjh8IG2I/xZ
/Bi6lxkfbzB8iyMH3MvxOqHGHucNZ9QHgC6hFUjc3X52Cj+A7jWS4/FFWA8N
gHv55JxMImfE5kcGhkaQgUFsRpCBiYFRf38R2QdtgM1MElcfk8tNeXmkQbv4
Afhh/olx+61iUveiO7jd8/hR1Od2t5eTuhfLpY+CUv7yYii6l5DhWUfM8php
/U+vA+7FUL2VaNSRzzBKGrfrkpPMdCaNfJl6/IZ+oYp65At+pHKpjsPeCegX
uawyvEaPUNwA3oBpcLowa21HCvME0HbEYh+YULZFohmM4LB4TXcEV4HSl+75
xkDYn7kFxMXksV4fvQuUvlRPNrqDVKeTL5OJ+z97cTa3uYrIvZzsrBUb1Gl/
0RRpl8j217QSuZejjT2TWiPdaySEM2/ae2sYaDvqlRCOCmLIBwC1OXC8YgEo
fbn8SOkJMHTYdyxDQuc7RyVE7mXr5+L6bjPda3wGMjA0ggwMYjOCDEwMDPn7
i6ALtAE2Mwldff/ICJWg3eUB04HJcUvRl8o3f0/qXnzcnqjP7Wq+TjpjOta9
+CZbTKVvJ+Renv6UtI10zLRLcCtuZG7QsQDH7SoevWqXVERD0FdgKpEvKlFR
QpEvVus4j/sh4F6mhKd93vjfeUk3QCQamlHcz2bbUdtUMVD6gj+b0bajcDTc
L2sD3EvlWInWqczcAohW1S5lA+7lIuem3JrMqtJ7/1c7rOf6m4jcS05jRYOY
H4ow63v0gtl2qn2AyL3sqW5+MDnLtDWvoLH7CppEQOnLtf4FbxBKKKL9AwAW
jpZ364DE3f2lkll1eirrMorBguV9rwBKXy5UqG3ObKdFwSADQyNvf/1Rbutl
JjyQgUFkj41oYAAEIA0NDfABiI0N2gCbGYpXf7q0VPrKK6TuZeHf/110/vx0
zU/zuXtg8YI/Fg5sn75XIRzsm7/2pebTf4fdi+Kr90Qtd5cXI2n8QXP5Ddi9
KK/unu55EPsriPruLN3ZCbgXzY//sdD0PxNjQ2t/feEYe7brJBy3O/f4OH4Y
yfvIqxJ2vQu4F2HvxxOjLZSu3C/wxzoHBw4D7oXdv3tkuBo4A7wBBkfqGgc+
AkpfHnPP8se51BcMMC4Ya+KW3ej7J2HiLvtw70hmb1btw42XegmnHV3sOVE/
BL2ZGaKF1/k/nd8RuZd/tV+4zXnAF/CTO3m67v88/ti1nhag7eirttqekfRs
lXTBHeNfausF2o6+bOjs4o3Qvcz4jI4Lvns0vP0HQvfy8a3BJvYY6Xno/QBw
r13w/gXCupc3v574robG1VFldGzi0o/C/+8AoXt57/hUXcsE3cuMA+nVp/IV
AxmY5EAGBrEZQUm8MQjo/i8gCHpBG2AzA1/9qMtlv36dStCuZscOT1eX5/Ej
3cHtpO7FcGxvYGos4rQ47xfpcv5Ave4lMMc2X3sXdi+22wexped+KSpjph0D
hXHHTEdDHrukQvHoVcC96AdySeN2SSNfZti7HcYh6hcuHPbKZZXsvm3ApGnl
Uh1pHxPRBghgDka1HY3K6zPaduTw22qELKD0pWWmxhN0k58orbiDnh/594HS
F/xZmz+liqC03P/H1PJjbXeJ3Av+FFeZtq2SLngKzcG6DiL3gj+FH0D3GgkR
KG05d/hE7mXXDW79uIrqqWj6AGDzhAruLQJtR/m3ZfgxtKwtISRy395TUiL3
8voh0b12ExZ6Qvcy45PK1UcGJkWQgUFsRjIyjZqo2CUtJyEEGRhEiqANsJkh
uvpBiYRK2Av+MB4+7OMNuR5WaT54ldS9mL76DJPNRiw6R+VpUvdivZIT614s
rN2JupeQdYF0zDR+QNwx008imHOhDo7bVT/eQxq3Sxr5Mt3ztlF+j3rkC45e
1w1Pmp6VFGOYncqp1m4A0mlHLdyPlvT91FcLQ6XtCD8mXS+3lnA0zFX0XujN
I3IvN3gXlHYaxt/0ynknOouI3EtBb6nYmAatkeL9X+92XOF2AqUv1cIhXyiB
vZ0FtE73+R4eUPpSxRd5MYZ+97f7sEuds0Db0dnWGfwY6iek5QNALcew/ewU
kXt5/6KIv+DK/qoSxeOLXKhQA21Hed8rDBZmbf5VIANDI8jAIDYjVA1MjA9J
SJ48d3BaTkIIMjCIFEEbYDOz9uq76us1O3aQihf5Cy+YT58OiEW2a+dJg3bx
h6Xoy5Bqcdm9wOIldsY0DrYkIHUv+AGBOXbsbxFx6x0DhbB7MTd9HHfMNI5H
3avqeA+O23Uu1MHvLZXIF/XMlYQiX6hMmsaPoX7CVRuAdNrRlKwyXdOOItHQ
qLyexT5A5F6qho7IjCSCK0XklrkbvAtE7uVCbx5X0ZvRBcRF6dBe5NwESl/a
pexwND2BEknf/0ORSLNEkNNYQeRezvQ2LNqyPacbBl9zzcTMnupmIvdyqn1g
wZzAv8cs0yTU7LrBJXIvOXf4AmXCi8/yB4BZtffQtTmg9KW8W4eFo+Qnoptm
tnXr52Ii9/LOUcmQMINxVekCv/o2e5KyERmYFNn61T8/aylmwgMZGET2oNzq
k0hPEGGtS1pOQgQyMIgUQRtgM7Ny9SNms/XSJcVLL5G6F/wY+/Xry0G7pOJF
te2P9lvFEZslpJ6zs06Quhf8mBX3Ephj224fzIR7MdbtIhozTTrqSPHoVZv4
Jhy3i2PX94n7oMgX2dhRvyuBygoMs89KihOaNE2FlQ3g9mlJ244cHkWi5ycC
bjvCH6Py+mA4gwNbPUH3I9EdoO2oRsiipe2odroVcC+lvCqTJ50DeZO7/4sN
6tNdD4jcS25zVbt0Mo2LTAuTWuNnDx8TuZf9Na3tEhoqnSgiNbgO3xMQuZcd
VwfvjSxhkWTERdY+AHgCkcuPlIB7yWVJ1WaGDvuORabyf3JOBpS+sB4aGNt2
FIvHE/nm/Pjrb48P8SgVTK4CGZgUQQYGsRmh7EQSaAoididpOQkRv4sHlR9c
AX0B3+SgDbCZwa++b2DAePgwqXjBH6rXXnPV1/t4fcaTH5O6F80Hr7oeVkU9
rqCUb72SQ+peHJWnIxbd8qqo9ByZr7272r34rI6hIti9GKq3+qRtcd+KgHVG
P5ALx+2aRgsiAZL/yuyxTc5xD6Qx8gUHnjTN7tsWd9I0FfANEAp7RYt3mdN2
1Cy8mNG2IxyuovdyP2HibtnQt3JLAnVE6YKnmjjVXUzkXvCn+JrptL9oovd/
Z8B3dbgbaDvCn8WPSfs6U8HhD1zuHwXajko4fPwYupcZH28wfK1/AWg7KmgS
aezJv+HZ+QDQJbQCbUf4U/gBWVhGinh8EdZDA+BePjknU+mDdC+TEkM8+zv/
Lfzfb4zhD/x/YBglfRf3GwcyMMmBDAxiM0K9KiXmSMqRMWsOTMtJCEj9Boi+
gG9y0AbYnERdLufdu/P/8R9U3MvTsJeBAc/jR9p9/0nqXnQHt+NH4i/h4zaZ
zryXkHvxTbaQuhdT6dv4YbG/S8RndQ6X6Mr/Ao+Z9ojuxx0zHfYajMP5sHuh
Ercb8KgUE1+lN/LFaBgYGT4AlL6Ipr8N+I3UT7iKgZE7LVxo2tEGazvSOpU/
jlwG2o76ZW3pavBJYFUuYymvCih9eSju9IcyoggSuv+3Sydzm6uI3MvJzlqx
If7QcxppFM3vr2klci9HG3skBgvdaySkV2LYe2uYyL3gT3HmU+3zyvQHALU5
kMuSAqUvJS1qT4Chw75j6RtzvHNUQuRetn4u7uQyt38tFps9lHdKuuxeVh6s
yoRzp5GBSZG3Tn/8afMVJjyQgUFkjwT6gmKlCHFZSmyZS5xTpuUk8UEGBpEi
aANsNjC53Hz69OKLL1JpOLJeuhScn3PWsNQ7/0zqXownPw5MjT0JBT3d1caT
b8DiRZfzB+f9oojz168/vskWU+nbGXIvLsGtKBanbygSsFmE38HuRduz328W
wm9pKGjTzl6DxkxnIPJlZPiA3Z58UYTDo+gTfAm4l4HJr90+bdLnXwXtbUeB
sL9lpgZuO7J4s51b4g8FmmZ7APdSPFSudKTtKqyF4v1/0WY609tA5F5yGisa
xPxQhFnfoxfMthMtfUTuZU91c6NonmlrXkFj9+U/nAJKX25x5N5gGlRh5j4A
YOFoebcOcC+Hrs3NqtNjdzOKSh/M+14BlL4U39F6fAzdSLFgWPSnau3rb4+v
0i/4A/8/ZfLEbr/IwKQIMjCIzUiSySy/+92WuId3HIxRJ3HPmJaTxAUZGESK
oA2weXC3tOj27aNS9KLZscNVX4/J5qxXCkiDdpVv/v7XoF2nxdV8XZ+7hbp7
eRLGvOMPk3AvUczjEtzS//R6cu6FyphpVcd7Xu0g/JZSidtNIvJlRnwBcC8c
9k61qpH6CVcRCnunZJWAe2nhfqRKsE8KgAltRwIND2g7usL5esZAItkygVAn
KegtBdqOeuW8TK+B9P7vC2GVggGg7egKt1PvdmR6nQnhxUK3RiaBtqPzPTyz
h1mtUitgkWgVT7Hj6iCRezl8TyA1pG1UUIY+APAXXO9fFAFtR7UcQyZeN71g
oSesh4bXD4mI3Ms/z8xLEhQXdDE57dr7sWite1kxMH3sxBrBkIFJEWRgEJuR
RAzM86UpywJF9tuHaZnseXFCWOKSlpPEAxkYRIqgDbDhiZjN9uvXlS+/TGXC
kSkvLyAU+rg9VMJeVoJ2KQ6Y1uducTVff+ZeRu4Zi/+WCfeiK/+Lc7gk4ovz
kXJ5zPRS05spjjr6mULc7uzghy7zGPUrFY1iisVqdt82QL8szLOSi3xZRq59
3Di4l8i9NAzsEi3ejUTTM4qXCW1HRre2cqwEKH3pl7UFwv6MrmEtJo/1R/59
oPSlerLRHSRJe04L8P2fq5wH2o6Otd0dU8uzsMiEYMuUB+s6iNwL/hRfpaN7
jYQIlDag7WjXDW79uCrNr5juDwA2Tyj/tgwofTlXt4Qfk94XzQRjYveuE7NA
29G9dmaN+iLC44lc+E5B5F7wx5G8OZU64XsgMjApggwMYjOSkIFZq0+IAdRJ
Wk6ylrhnoP7jP6Mv4JsetAE2MO6WFsOhQ1SKXpQvv2y/fj20tOisYWk+eJVi
0O4TLIgtTtuuHiENezEce8XdXh71PR0uEw24PUM/kboX87V3V7mXJxHMPXk7
affy9A1Z6oDHTC81vWmXVMSNi4mFNG5X3PeuRdWc0MUyGgaGOLsB9yIUnHC7
FxM6ZyxW13w3/zhQ+jI4XegNpO07BRPajrrnGwH3UjlWYnRnsMEnLuFouF3K
PtFZROReLnJuzpmzpzWI7v9qh/VcfxNQ+nJ/esQXSiDRKAtone5T7QNA6UvN
xIwXY+h3f7sPK2gSAW1HZ1tn8GPS/rpp/ACAhaO1HMNbZyaJ3Mv+Ugl/IW3V
O5nD5gyfKlsC2o7wZ/Fj6F4mJZrbTFvfFRC5l//z7liig5BS/9KBWOat0wc/
aSplwgMZGET2SNDA/LyqjYiIRDJlkj7JKlK/AaIv4JsctAE2HphcvnJnyP1f
/wt2L7p9+9wtLZhs1lL0pfLN31MJ2vX2Px0n5Bf2WS59ROpejKe3+bhNT0JP
J0REA273AMtY9NdEZ0w/iWAe0X1D9VZ41JFjoJDIvVAZM22d+oF0zHTAo5KP
nwTcy9TjN/QLVRGy88RCGvnC435oNg1TP+HqNWOOUcn3gHvpHM01WNPWhsOE
tqMZg/AK52si93K5P39Kl0BtUrqYM8vP9l8lci8nOoseLwxmOQd47f3fF8Ia
xHzAvZzrb1q0Meu//ocikZqJGcC9nGofwL8z071MQurHVUDb0T8rRwXKTKW8
pusDwJTCvb9UQuRe3jozWcsxYOFk5mVnmXvtJqDtaNeJ2TFxtofUJ4dkzvPJ
EQlQ+lJ2UzU6mvzVRwYmRZCBQWxGEjcwP/8s6wCLWKgVrqTlJM+BDAwiRdAG
2DBEXS5Xfb1mxw7Zv/0bqYFZfPFF8+nTEx0d3v42/efvk4oX/GH66rPloF0f
t4k0aBd/WC595Bf2/bo2j9XdV5ZZ9+KO/6Weyphp8/j5sJcklyAUtKlnrsBx
u0uThQnF7VKJfFEu1SU0PimWSDQ0p2wA2o4esj/AD9hIbUcWr+mO4CpQ+tIx
W5/9tiN30FMpqAfajn7k37f5aVAEq+7/Qt3Ssba7RO4lt7mqe0GU/UXC8FU6
oO1of01rt1RB9xoJEWkcOXf4RO5lx9XBKp4Ci2RQXKT+AcDmCV1+pATajvJv
y4wOZpVLxWVS6tl7SkrkXl4/JGI9NGChJ3QvkxyPJ1J2UwW4l0+OSCRzT/8b
QSpXHxmYFNmEBmay4uAvfNOxDlKgEJkhGQPzCzJZR8nBLbESZcuWgyXPIl2y
d5LfQAYGkSJoA2wAAkKhKS9P/sILK4IFMDCq115z3r0bNugcVWWL//USlbAX
65WCsEEb9bnd7eWGY6+Quhc760RIPbe8tqjH6uoqNpz7E+xebLcPYktr/nO8
tM1Y8w7sXmxdJ4ncS8itJB0zjR9AOmY6GsX0C1XpjdulEvkinSvDsMRKxGMx
WIWdo7lA6cuo5PuxcU7S51+FzDhKe9tRv6wNcC8/jlzWOpWZWwARvXIe0HZU
0FsqNs5nf1XLrNz/9W7HFW4nUPrCGutzBpgVOqp1us/38IDSl+vcCSa3HZV0
S4G2o/yHUxp7xt/wFD8AtI6Zt5+dInIv718UcWeZldIcF5szfKFCDbQd5X2v
UOmDdC+TEp3d5nf+W0jkXra+K6h/9OzbLzIwNPLmqZxDjVeZ8MiSgfnNvyAD
g9gYIAODSBG0AdYvQMTuWgMjf+EF4+HD/pGRoGTKXHiMStGLdt9/ultqn2DB
iEXnvF9EdciR5degy7Blydl8BhYvcd3Lct0LFfcSssaXJ2GvwTx+HnYvuv5D
pGOmcayaTtK4XUeCk4MyHfni9mkHpwsB99LNP251Pf3Wn5Z//nafvll4kd62
I6lJBLcdcRW9GV1AXJQO7UXOTaD0pV3K9ocC2V/YCvgGCEUizRJBTmMFkXs5
3fVAbFDTuMi14Gt+MDm7p7qZyL0cbexZMGeqcyd1moSaXTe4RO5l763hXkmW
vqIkfQeYVXtzWVKg9KW8W7cu2o6a2datn4uJ3Ms7RyV9Y+tAIuGo1P4jeXNA
6cuF7xQ2+3NCEhkYGtlkBuaZf0EGBrExQAYGkSJoA6xHSCN2Yw3MctFLxGL2
PH6kO7idinsxFx7z858OYg6IuXbWiYSCdnGwJYG97jipe8GPwTTPNTVQ7Dmy
dnxB5F4iARupe1E/3kM6ZhrHYRyaHfwQcC/TPW+bluoTunCZjnwJhb2ixbsN
A7uI3Evj4F659vHK8Sn+8w+GfaPyesC9ZKftqEbIAkpfHonueILZjm5wBz21
062AeynlVZk8iY1/zQSPBvtOdz0gci85jRXNkqeKhu5lPsek1vjZw8dE7mVP
dXOjiLaaIlKkBtfxOiFQ+nKtf8EbzF4WUBJ3AE8gUtKiBtxLLkuqNtPpFSki
kfs+OScDSl/KanUeH7M2f1wwLMqq1Lz+9jiRe9n7sWi57WgVyMDQyKYyMLH+
BRkYxMYAGRhEiqANsI7A5HJLYaHipZdIZxut3BlOf/opJpu1XilQbfsjqXhR
7/yz/VZx2KB9Egp62XXG09sSCtrF8Yu7rBX7qLiXkOG5b0lRzEPFvVhaPw3q
JuK+OZGAzTr1g+LRq/CYafcSeeupxzYpGzsKx+3q5m8lFLeb6ciXn3+ZNN3C
/QgofZmYZ4Wen2Sdyj9/2tuOwtFwv6ztQm8ekXu5wbsgt8xlbgFE8FQTp7qL
idwL/hRfM539Va3CGfBdHe4G2o6ucDv1bmb913+HP3C5fxRoO8KfxY+he5nx
8QbD1/oXAPdy+J5Aasj2qKBE7wBdQuv7F0VA2xF+QIaWmkY8vkhZrQ5wL5+c
k0nkzOq5I2KIZwfajl5/e/zefR2GxS9GQgaGRt78n08OPrrOhEfGDczz/gUZ
GMTG4HfxSOgM6Av4JgdtAOYTG7FL5aHdvXvlbnDs//2/qRS96A5u9zx+tNxw
5KguJG04WhW0+ySMeccfmkrfJnUvzuYzYcvSc78d5nEJbsEzplN3L0tNbzoX
6kjHTAc8KsXEV6Rxu5g/gQ8QWYh8sbrm+wRfAu4Ff9bhiZNHmtw/f+a3HV3o
zeMqerM8V+jnX9qOSnlVQOnLQ3EnvW1Hy7RLJ3Obq4jcy7G2u2Pq7E3EpkIo
EmkUzQNtR589fDypNdK9TEJ6JQag7Qh/qkmooWVh1O8AanMAbjsqaVF7Auug
YqSTa3vnqITIvWz9XFzfbaZ7jZSw2UN5p6RA2xH+rMEIxdckcf9P/UsHYpnN
YmAMHd/86l2++QYZGMQGAtXAIFIEbQAm425pMR4+HBuxCzwUL71kvXQppNEE
psZW7gxH/u3/gt2LpejLoGTq518ajqxXckjFy+qg3eUB08V/g8WL4dyfnM1n
Ig5d7G+XunuJhjx2ScVS05uwe8GPIR0zHQraVKIi2L3Ixo56HZKErmCmI19I
J023cD9SEcfUJPrPPxj2cRdqGN529GCqIvttR/5Q4KG4E3AvxUPlSoc2y6ta
y6LNBLQd4Y8GMd8XYtbkGonBcrSxB2g7qpmYYVqr1AqLZg/cdnSpc9buo+0N
p3IH8AQi5d06uO1oVu0lPQ/tqPTBI5fkQOnLNzdVNme2tW0SYFj0p2ot0Hb0
zn8Lx8bJZ6uhGhga2RwG5pl/6TCslMIgA4PYCCADg0gRtAEYiH9kxJSXt/ji
ixSLXoyHD3u6uiI2i7OGpfng1aVX/x9SA6Pe+WdHVZmQ3Rf1uT3d1VSmS68K
2o04dK7286RDjoxFf3X3lUWfj7yI+KzO4ZJMuxfFo1dt4puk7iUS8mhnr009
fgNwLzPs3YnG7ZJGvowMH0gl8oV00nTDwC7R4l140nRC//zn9ENA2xGLfWDT
th3xNdMFvaVA21GvnJf9Va3CF8IqBQOAeznX36R2MKuFxOEPlHD4QNvR+R6e
1plt20YRbzBcxVMA7iXnDl+kobnPi/QOwJ11AG1H289ONfBM2VlqKmChJ6yH
BsC97D0lnZQm0FVKI5PTrr0fi4DSF1alhqjtaBXIwNDIZjAwK/6lYvLnmGYk
ZGAQGwFkYBApgjYAcwhKJBRjXlbmSjsqKiJms58/uGq8EWBg9J+/7+1vw18u
rFcsfH9Yl/MH8rCXk294uqtXgnZDhnlHQz5pw5Gp9G3vyL1o4LnvR8vuRVf+
F9KsXSL38iSCORfqlK3bYPdinfohEiAZhhKNYkb5PXjMtLjvXYuqOaHrGPAb
RdPfwpEvalVjQudchdY81j6cA5S+8MQXvQHyb0YU//lb3KoGQSFQ+tI2VZzp
tiO5Za5s6FumtR1pXUa47ah6stEdpP+bHXtxFm474ioZl17bLpHtr2klci8H
6zp4Cno6d6jAmTftvTVM5F52XB2sH1dhEfpHBQF3ALU5kH9bBpS+nKtbsnkY
Ouw7liGhE2g7ev2Q6F67CQs9oXuZ5NjsoW8uyAH3ciRvTqX2Uz8hMjA08rf8
zz5uuMmER6YMzPP+BRkYxAYDGRhEiqANQDshjcZWUqJ67TWK4kX+wgumvLyA
UBg2aO23itU7/7y2xGWtgVFt+6P1SkFI9bThxTfWQbHhCD9sJewFJyjj2W4f
JHUvFtZu32TLql8z4tZTcS/AjGkq7gV/WITfkboXHIuqGR4zPd3ztn6hKqFo
3HDYK5dVwpEvC/OscDj5in3SSdOdo7kGK/mU7WVI//lTaTtSmOO7snTh8Nvg
tiP8WVrajppmewD3cpFzc85Mf5qK2mE9198ElL5UC4eY1na0YLadaOkDSl9q
Jma8GEO/+2vsvoImEVD6crZ1hsa2o1XEvQNg4Wh5t+6tM5NE7mV/qWRKwdDS
o1gMFizvewVQ+oI/uy7ajnDu3ddtfVcAtB31sRMuYEMGhkY2uoFZ8S2/+hdk
YBAbDGRgECmCNgBdRMxm59276q1bKYoX/KHZscNVX/8kGPT2txlPfgyku8Qa
GP3n7y+n7EZ9bnd7OcWGI0d1YVj/LLvVN9livvYuqXux3T4YlK1uuIi49Y6B
Qli8wO4Fx73Uoep4D3Yv5vHzYS/5n3bSMdNTj9/Qzl5LaNQRjlrVyGHvBNzL
lPC0z6tO6JyxhMLeKVkl4F4aB/fOq1sTOif8z59K2xHc5ZQi4WiYI+8E2o7K
hr6lpe1IqJMAbUcnOov+f/be/LmJM+/bnT/knKq3Tp06lar31KTqqUneSvJm
TiZPMmGGCRMgCUMSEhISlhAggTGEAWKIw2IYBxPAxAGDwdjGeN9t4U2W5EW2
ZVuybNmWJWvf19bWWuzToIwwsnz33d1aWs59Vf8US90tu6W0Lu7v59MlF6R/
QU4c3iD+YHIQPHbUKuBm9iTj8ODBm4PjAPdyorVPaSMPuMgIeDhSKlBsuda/
lnvZdXtodJFcDqeT1Z8AwjnXRwWStdzLX0+PV3INeCjzq3fA4MGlOw2GN/eI
13Iv245OD0uyQCIRSGewXV9KAEtfCn5SYhidHCRkYDLI+jYwq/0LMjCIdQYy
MAiGoAsgzUSLjXQ7d8KLF9Ubb9iLioIaTVC1ANkrHftkOPX1weUnA0eO2ydp
DBxFMCtM0G605CiuYJogaJ2DcS/EYxi6F+PA8aB7kfSXj9nGZwf2g+N2VeJ8
SlVHBGbTgID/CTjyxWodobTPOGCapv045UyJtd7+Rqec/WNHXHl7+i2HCbMW
Dd0HLH35RfiADWNHw2r54Zb7a7mXA42lnDnxMss+/zkyBWDsiPgR8YBMn+Oa
jC7adt0eAowdlQ8q2TB2FMfKC8CGBcFjR8RPjQ62rN4BMCxx7zghA4wd3Wkw
ZMvY0fl/K8BjR/MMWrORgckgbx7/CqDH07kl38Ak8i/IwCDWGb9LBKU9sOoG
DJF+0AWQNigVG0W7jSx5ebhcvoQHsI46/Vfvw/RKE5vx2OexT4N/ffYP89kP
aQwcBQ2zzsbTpOLFcPYlV2dBXMkRQUA3Zm37Gsa9hN1rfpGHdC8Bx5r2JobP
NU9aMy0fOUY8jNrf1L0gGj0KcC887naGkS9mxxRHeATgXvrGv0vYNA3D6re/
F3dyZXfZP3ZEPCal57AaX9DfKusFuJcfeq6xYexI73Zc5rcD7rdvj/Y5/b9+
a2PJ57/S5jzR2gdY+lLEH2Pt2JHdi4PHjoifauwpTKhmQvQCwEORSq4BMHb0
/gWxcM6V6ZMlx+YMnbiqBI8dqfSggmb20NhiAo8dtTNuzUZt1Blk/RqY/5iW
eNGCDAxiXYHWwCAYgi6AVIN1dlIqNiIeSTzeNzhIPNfLf2TOO7y44fcw4kXz
wevOiuKwzbIcXfRSlqc/8DKNgSOfpNNaspPUvRgL/uzuK44L2n18znKOqebj
NLgXfd8Bv3WK9PcPUzM9O7Afs42T7molOG6fkpwHuJfe7k3y+dtMIl88fpNA
coF20zQMcW//SXVnCffLDI4d+UO+nvkWgHvJ1NiRxDgLHjvqmOvP+NhRMByu
lQj31pesdad9svPhgu2ZfOaMf/578GCpUAxwL9/UP5ozs2tyJwYejlSPqABj
RztuDgzKLZk+TRDEBSCcc316RQoYO7rF0bF/7IigvNW08SvJWu5l8zdSnoil
82txSGewfYekgKUvV39W0Rs7igOtgckg7DEwR48enYQD5nWt5V+QgUGsM5CB
QTAEXQCpYCkQcDc1URIv8ueeMx48SDxr+XEp0oT1cm7CiN2EmznvsE/YTzwx
4nV7+Q2Qi16Mx96aLjm3cuAI492BGTgyXXnbO960FHpmOfpSGPdI64wVm8Hi
RXfrjw5e/lruhdiJS14P4158ZvKw2RTVTMPE7U5Jzvt9Rkq7XUkw5BEv3K/t
2wZomp5SPGAuQ2Jvf5VVXDl4PLNjR1MG0WXudywcO/pF+AA8dmTzZf6bnUin
BI8dtcoSOMbMfv73zi/urmoDjB3Vi1nX0BRjdNG2954QsPSlVKDwBFid8mrD
gkdurDlzRGxHSubUZn+mT5Mcqdy76/QsYOlLcU12jB1hWLjgJyXAvew7JJXO
JG3IERmYDMIeA5PENTCx+qNEkgUZGMS6AhkYBEPQBZBEIi4X1VEjYtPt3Omq
riaeGzJoHaVXNR+8DiletDv/4m6qjGCPF4fjC5OQSS8rB46if/3owJHh7Euk
7sVastM/0xv/qnHMNXrTULaR1L04BwrD3sR9DZA9R9pHn3q0/aR/iHAQ08+V
ktZMm5TVVP/EalU9j7sd4F5Eo0edTkaLNEgjXyCbpmEgLgC339Iu/imzY0dG
t/be6DXw2JHFk5yXDE8oEmqV9R5tz1/LveR2XZEYM68ISMeOrg1wYmNHcWTq
81/rdJ/p4AGWvlzqGXL4WPrd3+7Ff2ieAriXI1Ui1o4dxajkGt75YQIwdtQp
otytk34wb/h8iRrgXg5dlGfL2FE7x7z5H6K13MvGraONLUn+DEQGJoO88e3B
T6tvsWFLmoEB+hdkYBDrDGRgEAxBFwBzouG6hj174K0Lsak3bnTevx82m8M2
i7upUrf7HUjxotr0giX/24B0YvnJohdPb5Xx5CYY8aI/8HLcwJG06WeYdmli
c9QexzXiuBcOWTCtv/Oma/RmetxLJIIb5eVg90KjZprAah0ZHPgC4F4E/E/M
pgFK+4yDNPKlfegA8Rgmh1hJOBJsFBQV934BGDsSKupTPXbEma0HuJfL3O9k
pvgLLw2Qjh21ynqzYuxIYgDVb6X/89+DByvGpsBjR+Na+uvHUk31iGrbDT5g
7KhLyvZvFxMKN2DsiNiK2jSYPwlDLqmmsdcKHjvqHqYcTp4R5uVe8NjR+X8r
bPbkfw4jA5NB1p+BiQkYeEqoDX8jECwCGRgEQ9AFQJtonTSlViNiW3z1VVth
YSxf13RqP6R4IbZYrzRx9IBMCL/oxXz2Qy+/YSn46z8FRvxujHfHdOVt8rCX
/Nfc3VdXB+1ClhwZKzZj4gdL4cSuI+nuxaSslnRvJa2ZDgaoxUqQxu1ye99l
GLdLGvlCo2kajMI8BmiaJrZ28U9uf2ojLCZ0w+Cxo575lvRbDpvPSTp2ZMIy
vzxAYlADxo721pe0ysaDYZLv0Wn+/BeqdICxow/LGh+OT5Oec6YQaxzgsaPr
PXPsHzs6W6UEuJcDxbJpNf3oqrQhlXv3nZ0HLH25WqnDvCy9kFaCYeGrP6sA
7mXH5+Ikjh3FgQxMBkEGBhkYRFaDDAyCIegCoEpQo3GUlGi3b6cqXix5eQGp
lNiDT9hvyf8WMl83Om3krCgOGbTEc8NOi7v1lvHYW/QWvYQsSsiBI/P1ravD
XpahS45MNR97ZS1r/Q6T614IrJp2sHshNuV4HtWaaa9HLZ78Hhy3OzdbzCRu
lzTyhdgm5m8HGRwiDotb1Si6AHAvlYPHdfbUpt0a3drbw4UsHDvqmOsHjx2J
dNI0n9VqLF43eOyI+KnFGx+RnZC0ff5rne5zjwSApS/ET80YSyd37F78Yvs0
eOxIZmB7VRB47Ij4UfMw026dNIB5wwX3tAD3su/s/LzKl+nThKK71woeOyp/
EP9vH8kFGZgM8t/HDu18eJsNGzIwCAQNkIFBMARdAJAEpFJHSYl640ZK4kX1
xhvWixej4gWfn6aUr0s80nb9HPGs6An4JXx78VEY8RJNevEOt8UWvTx++kyv
rfwA7MCRMsFVAVlyZGn+0q8eXOvXGAlijpmy5LqXqd7tYPeiGDtFtWYax+3T
0gKAeyE28eT3TOJ2lyEiX/on89xeLZNDrCQQ8vLnKgDupYT75aS6M1mHSwhr
x45mzPIfeq4Blr60ynp9wQwnkwTD4UbpKGDs6Fh7JXjsKI40fP4T51wxNvVh
WeNa7mV3VRubx44aRBrA2BHxozZxar8mM4d07OhS3aINY2nZ90rAY0fEj9r5
LK3NikOl9h3KmQEsfTlxZi4VY0dxIAOTQdafgSED5cAg1hW/SwSlPaAv4L9x
0AUAYCkQiHZJK158kWrGS3TUaPmJeLFdP6fd+RdKMS/RbiOCoHrGUZZnOPwn
GPFCPMxVWxi2PP1GEHbo3N1XYRqO1ho4iuAYTMkRsdm7TgWtc2v9MiNBzC4t
UTZsIO058hqGYP46DiNvhv9F0t0LTNXRqPBrhnG7MJEvBit53xM8U9oeQNM0
sXFld714ajt9SMeOMtJ2RDp2VDR0nyVjR8faKwFjR43SUaojPKn+/B/XGsFj
RxVjU6wdO5IZXAfLRwFLXwo5smwfO9pxaWx9jB0V3NNmy9hR8W0NeOxoeCRN
3Wo03v7Mv3QgoiADg0BkNWgNDIIh6AJYTXTOiGrAC7Fptmwhnkg8ffk/4gW+
2IjYTKf2e3paojEvYYvu8bQRXMRubNHLylfhk3RCLnpZa+AoGrSrv/Mmk4Lp
ZSruBaZjmsBlHiZ1L/PD32A2aitcIxFcrarn9r4Ljts1Gvoo7TYOt1cLjnxp
4n+W3MgXo1P+UPgdwL3UjuYRj0niEROcA8TYkcOX7n+/Xk9jR3o3ndDR1H3+
mzEv6diR1gk1KpV+PIFQIUcGcC8Hy0fXwdhRrcDE/hsAmLEjqZyl82txgMeO
3nx75E6ZFscjaTsftAYmgyADg0BkNcjAIBiCLoAY3r4+S16e6o03qIoX3c6d
0VYjYif+iWFKo0bRfN1YqfRSMODlN1gv74UUL8Zjb7lbb4WdTxNT4Re9EJui
eFfCgaOAbszWeYx00Qu45Igg5DFYJ35S1L2eLPeC2cbnh79Junsh0Os44Jpp
bu+7i8oqqg1KKwmGPBPztwHupbZvm3jhfhIjX7y4kzN1A+BebvZ8OW+EWnFE
G3/I1zZdDXAvV3nfyy2pjZ1JyPoYOzrccl+kU9Lefyo+/4lzfjg+DR47Eig0
ST9usmgT68BjRw0i9p58FPixI5bfAJCOHREPyPQ5QkE6dpRzQmYwprs1GxmY
DPLfx77+pKqUDRsyMAgEDZCBQTDkN34BhM1mV3W18eDBheefpypeDHv2EM+N
uB6bExriZWW+7vKTmBf4biNisxcfJZ6y8rXAL3ox5r/m6iwIO3Rxf/2lMO6V
tcCEvYBLjgiC7kXzyDmweCE248BxSPfic82TupcZ/hcu8zDVa8BqHREO7U9p
3C7BrLq5vn8HQL8IJBc8/qRlz4YjwbHFFnDT9JC8emhEkKwjJmRUI7jUcxyN
HdGDdOyoViJkOMKT9M//ca3xm/pHgKUvFWNTHpylkSMyg+tIlQg8dmT30new
aYB07GjP9ZmVY0esvQFYT2NH4Lajzf8Q8QT2jJwbMjAZBBkYBCKrQQYGwZDf
5gUQkErtRUWaLVuoWhf5c88ZDx50NzUtBR7/cxUN8RKXrxvSK5wP8iFjXh4v
ejm5CeOUrVz0EsGsGO8O5KIX8/WtnpGa2MBR7K8f9lpdozcNZRsZlhwR+Mwi
48BxGPcScKwZGvPMDl3zirFTpO7FYeRRvQyczhlwzTSxTUsLcJzR7bHBKmof
OgBwLxzhEbNjiskh4lCYx8oE/wQsfWmZKLB7H0+Npe7tr3UuorEj2pCOHV3k
ttAbO4ojiReAw+cv5AoB7uVMB4/NY0fXe+YA7mXvPaFYk4RfeEqBGTuKewoL
bwDW09hRO8cMHjsqvq1J59hRHMjAZBBkYBCIrAYZGARDfjsXQDRW13zyJNVY
3WiXNPFEb9+vASBe/iNL/reUxEtcvm7YacE4ZfAxL9FeaXxhcuUrCswLHLXH
YcRLtOGIeHzc74T46wetc46+PN2tP5K6F2vb14CSIwKPtl/fdyDN7mWqdzsN
9+L3GcE109GqI7d7geqeV+L2avsn88CRL3JtB5NDxGH36lsmCgDupUzwT5X1
adNQKt7+pG1HaOwIAMzY0bA6aaE9yboA6sWzn1Y0A8aOeucXk3KgVEA6dlQ9
osr0OZJAu+2IbTcApGNH1ZwsqMwmmJd79x2SgseOVOoMt2YjA5NB/nT08I4H
ZWzY0mVgEIh1BTIwCIas+wsAl8ujsbry556jKl6027cTz41WGi3/R7yoNr1A
VbwQT4zu4XHMy3Cb7dohSPESnTaKi9iNLnoxXXkb4FtqPv1/otto7n8TD44k
GqnwKbmKih2k4uVx0G5fXsgB+gLlVrapOz5Mv3uxatqhLoIV4Lh9brYY7F5G
hV/b7ZPk+1obP+4Ymy0GR75MKR4kMfIFpml6bLElHHnmW1jS3/5o7IgJ4LEj
YquVCL3BZE7BML8ApAYLeOyoVCjO3rGjH5qnWD52hPnDlMaO4mDPDQDp2NH5
ErXNyermqShsHjuKAxmYDIIMDAKR1SADg2DIurwAIi6Xu6nJfPLk4quvUrUu
ihdfNOXkEE+PBrxEMBfWUWfOO8xEvBAEZELH7ZP6Ay9DihfT6fc8vVUR7zOL
9uEXvcQ+GXJzc+N/OTiGiR/AtEsbyja6x+8CgnaXwrhzrkrV9h5YvCjqXjeP
nAt5oBaewrgXSfdWGu4lFPIoFsrANdMC/idm0wDVPa8kHAnOLNaCI1+GpD/6
8WRONJA2TXOmbiRsmk7i25907OjhRAkWSPcQyroZOzrb06B2JN8RMbkASMeO
TrT2KW1p6tWlit2Lk44djS6me0qOKs3DZqpjR3Gw4QaAdOxo1+lZNHaUCpCB
ySDIwCAQWQ0yMAiGrKcLwC8S2QoLaaS7EJt640brxYvEHqK7Chm0rppSw+Ed
8NYloXgJqmecD/KNx96C7zZy1RaG9IqVrwtm0UtsIx5GPDihgYFslyY2c8Pn
4LAXyIJp4gE2yc9hP9QXGUj3YlJW0ygkUqvqwVVHxE+1GtBLhkFl5DXxPwO4
l+7Rb62uWYZHWYnOPsOkaTopb38s4G6aqgC4lxuC82jsaC1gxo74i8m8ZlZC
+wIAjx0RP+LIFOR7yRANIg1g7GjLtf7qERUeZsvX5IRMqz0HimU0xo7iyPgN
wG9q7Cj9bUdgkIHJIMjAIBBZDTIwCIZk+wUQ1GholxnJn3vOsGeP8/59YifR
vQWkE7br57Q7/0JJvKjffcV6Odc/8bSFJ6RXuBqL4MXL45iX2ycDMuHKl7YU
wn2STnvVERpJL3EGxq8ehGmXJjZ71yncKAH8wiELphebNzlmyiJBDOaPmFL3
YjYNCPifgGumFQtlTGqmHx/FMcURHgFHvqio59UAIG2aLuUdmtL2gHfC/O3P
V3SBx46IB6Cxo7UgHTt6MDmY3LGjOGhcAKRtRzcHx9k8dnSwfDSrx45sWPBS
3SLtsaM4MngD8NsZO9r28cTwCBsXgyEDk0H+lPPPjyrL2bAhA4NA0AAZGARD
svECWAoEvH19lrw81Rtv0FjuEo3VxTo7f90bHqCRrAsSL9D5usRmu3bIO9y2
FHzmn8ZwjdjVes6Y/xr8ope4pJfYJ8PxPe/ADBzp77zpGi4Ku/WAXztkwbSq
7T23sg1QVL0SzDYuHzmWIvdit0+moWba7dUKJBcA7qW+f8eU4kFcAAsTiF0J
FfWkTdOBEPm6fSZvf7ll5obgPGDpS534Hho7WotMjR3FQekCcPj8l3qGwGNH
c2aWTu7YvXghR5btY0e1AhPDsaM4MnIDYHOGzpeofyNjR3fKtOwZO4oDGZgM
ggwMApHV/C4RlPaQjV/AEUkkiy6AgFQazdSlYV2IjXjiyljdsM3ibqo0ndpP
ybokS7xEY15Wlkov/2fayHx9K5N6oyixT4PDf/s/wO7FWLXNI60DCxPIgmnt
o08xdRfkXxOzjc8Pf5M69wJTM+33GanueSXBkGdi/jbAvRDb2GxxciNfFOax
Ut4hwNKXdvFPbr+FfEdPoPf2d/hsDydKAO7ll8FLi/Z5GntmiMQ4m9t1BY0d
wQN5ARDnXC+e/bCsEdB2xOaxo+oR1ToYO9pzfYb52FEc6b8BqOaY0dgRS6Dx
12f+pQMRBRkYBCKrQWtgEAxh+QWAy+XO+/eNBw/SqJAmNtUbb1jy8rx9fUuB
X2+E8PlpZ0Wxbvc7VMWL5oPXbdfPrRQvYYsO45SZTr8HL14SxrwshXDveJOt
/ACkeEm46CVKBMc80jpj1TYYA2Nt+1rcfQ/8+4csmDbwjvjMIsi/aUrdi9u9
QFozPSE6ybBmGiZut38yz+3VMjlKHBa3qlF0AeBeKgeP6+zU4laovv1DkRBX
3n6+K2ct93Kp5/jQYh+lfSYFE2YtGrqPxo6oAnMBkI4dsbntSKxx7L0nzPax
IyZtR2DSeQMwLsN2nJCtj7Gj4tuabBw7ioP469stNB0RMjAMeS0n58OKSjZs
yMAgEDRABgbBEBZeANFoF1NODo0mI2JbeP5548GDxB5i6S4EPmG/9XKu5oPX
qYoX/Vfvu2pKg6qnX9hpiBfD4T85yvLiYl6Wn0wbORtPQ04bgRe9hByLDl6+
7tYfo3YFYGAet0vz8qPt0mv99ZfCuEteT1pyRGymoVzIgunlFLsXv89I6l6Y
10wvP4nbbR3YC3AvHOERgxXWR8EQCHm5srvgpulJdSeNPVN6+8tM4qu87wFL
X9qmq/0hH43TYIIv6G+V9QLcCxo7AgC+AGDGjtjcdnSxfTrbx44quYbkjh3F
kZ4bAJszdOKqEo0dsY0r53hbXukRdNO5hJCBYQgyMAhEVoMMDIIhLLkAwmZz
tD+aXrQLsWm2bLEXFcXKjJaf9BlF54wWN/yeknUhHm/OO4x11IVtT6c5QnoF
VfESzdf1S/jxL9ahc/cVQ3YbEZuleLtnpCbiTxCpsRTGvbIWc8PncZoloYEx
Vmx2j9+N4E/TcVf/9cN+m03yM2nJkaLudYvo35AF08updy/T0gKwe2FeM01g
dc12j34LjtuVazsYHiWOSXUnuGmaP1cBE/mSEMi3v8VjqhAVA9zL7eFCozuZ
C34gEWomAWNHR9vz0dgRmLUuAOKcH45Pg8eOeucX03y2kODhSPWIasu1/qwe
O5pQuD+9Ik362FEcqb4BwINL5a2mN/eI18HYkXQGy/axoxgjfMuuvwv+/HsO
sW17nYsHKL8XkIFhCDIwCERWgwwMgiEZvACWAgGss9OSl6feuJGedYll6kZc
rthuA9IJ+80CenNG1su5K7ukl2llvETFi0/UHf96o9NGd3dDihdjwZ9dnQUh
izLhby/s1ruGiwxlGxNOGMUZGFvnMb96cPVOVv71A445mKBdZcMGu7QEsmCa
wGUeTp17wXH73Gxxb/cmcM20Xsehuuc4PH4TOG63tm9bcuN2CVRWceXgcYB7
aZkosHtB4cmkkL79/SFfz3wLwL1c5n43ZUjmgh9ItC7jFUEpYOnLL8IHaOyI
lIQXwLjWuL+mI0vHjkYXbet+7OhAsYz22FEcKb0BGJa4tx2dXgdjRzZ7sOAn
5ToYOyKwWwKnv56IupfYdq9ITnU/yMAw5P87cuyD+1Vs2JCBQSBogAwMgiHp
vwC8fX3Wixc1W7bQsy6xCulYpu7yk1hdrKPOnHdYtekFquJFt/sdR+lVfH56
5UkG1TPOB/nwddLEptv7B3vxUe9w2+qXjCtHnY2nDWdfgp828s/0rvkLlHOs
bV+D83Vjnwzffr4R0HAU/etDhr0sNm9yzlVBFkwTOIy8Gf4XKXIvoZBHPn+b
1L2oVfUMa6ajcbu1fdsA+kU4czW5cbt2r75logAc+aKyipkfCPz2n9ANX+Z+
B2ia7plvSX/TtC/or5G0A9zLDz3XJMbMLClZCenY0UVuS/rHjuKIuwDMmPfc
IwHAvZzp4LF57OiH5qmsHjvCQ5FKruGvp8cBY0fNw8lcMZKiGwCDBc/5UQFw
L/vOzmfL2FF1nWHj1tF1MHaEByL3iuR/e6ErTr8QG/EfjTpqA6TIwDAEGRgE
IqtBBgbBkPRcAL7BQVthoXb7dnrWhdiI5xJ7WDlktPwkVpfechdiM53a726q
XDlnRBCQCR1leYbDf6IhXuIapQlCFqWrsyAp00YEYa/VPX4Xplp6pYHJzc1d
64+yFMYlPVdgwl60jz51KxOYpbWwatqnerenzr0oFsq4ve8C3AvxU+IxDN0L
TNxu3/h3DiyZFTBe3MmfqwBHvowttiRrsc1ab3+tc/H2cCFg6UuFqNjhy8CX
Wa5i+ASnADB21DHXn34pFAfM2NGwmvI/OqeC2AUAM3YkUGjAe8sU0bEjQNsR
8SP2jx0J51zgsaPCJjXmDyf3oEm/AcCDS8U1BvDYUWNv5henwTA+6drxuRiw
9OXEmTmbnaWLweIY4Vu2vc5d7V5iW/NDau9uZGAYggwMApHVIAODYEjqLgC/
SGQvKqJdHk1s6o0brRcvrmwyWn5c2ezy9LRY8r9Vv/sKjSJp4onE05fwZ2yJ
T9TtuH2SknghNtu1Q15+Q8QbL0zCDh2lUmljwZ/d3VfXmjZafrLoxdZ5DEa8
6O+8GU3ZBRsYyLAXYjMOHIcvOYpEcJOyWtK9NUXuhXiKWlXP424HuJfe7k3y
+duhENMl+kp9TxP/M4B7aR86kNy43XAkOLbYAo584cruevFkrkBY/fbHAu6m
qQqAe7khOC+3UGtcSgqLDm0B7xZg6cvt0Wp3AHaBVuoQ6ZTgsaNaiTCDY0dx
RC8AoUq3u6oNsPSlYmwqe8eOLrZPs3zsyOjAc8sX0jN2FEdybwB4Iufmb6SA
pS8F97SYN8kSKRXY7METZ+YA7mXH5+JsGTtSKzz/2icCuJdDHw1PT1J+LcjA
MAQZGAQiq0EGBsGQ5F4AAanUUVKi27lT/txz9KyL6o03zCdPupuaVka7PNnz
hLOiWP/V+zSWu+h2v0M8N27OaCkY8A632YuP6g+8nBTxEvG7PSM18DEvhrMv
AbqNlp/UGzkHCtdKeonbTDUfe2UtS+Ffv2isZWAgw16oBu2Gg5h+rjR17oVA
q2khdS9zs8U4bqex85UYrCKO8Eia43YV5rEywT8B7qVRdMHiViX3oMvPvv1D
kRBf0XWp5zigaZp4QPpXmLgDWNl4PcC9XOD+PGPO/JISvdtBOnZEPCbTp/kM
nYLBMx088NiR1pl4VV7GMbn8pGNHYg27fuFx4KHI3W49YOzo/QviTlEKV4wk
6wZApQ8cuihfB2NHOB65U6Z98+2RtdzLxq2j5Q90mT5NKDB36MbFWYB72fJq
V2877P/l40AGhiF/PPztP8qq2bAhA4NA0AAZGARDmF8AUeti2LNn4fnn6VkX
xYsvmnJy4vqjl5mluxBPWd1nRBDxur38BnvxUd3ePyRFvCyFcJ+k0151BFK8
EJu1ZCdg2iiCYwnrjRJuj6ul+/JwoyRuJ7n/obe3N/pf4MNeKAXtBgM27fT1
yUdvp8696HUcAf8TcNXRtLTA7zPS2PlKHJiib/w7cNyueOF+kPECm5VY3KpG
0QWAeykT/FNhHkviEVcSe/uTNk3Xie9hgQx8E++SCwBjR8SPiAek/6zi8Abx
WokQ4F7YM3YUw4MHS4VigHth+dhR+aAS0HYUHTvK9GmSwJ92vH9BDFj6UtSm
SfrYURzMbwAwb/hqpQ7gXjZ/I23nszp+J8bwiHPbxxOApS9nzsuzZeyos0EH
HjsqKZwfGqT/10cGhiHIwCAQWQ0yMAiG0LsAcLncef++8eBBxYsv0rMuC88/
vzpQN4pP2G+7fk678y/0lrvYbxYEpBNx+ww7LV5+g+3aIUrWRX/g5bUyXgj8
M72U8nVJp40CujFHX57u1h9h3IuxaptHWreyWjohYb/NMVO22LyJ1L2oOz50
K9tiq2hIwX0G9dTliY63wO5lqne7VdNOz72YTQPCof1g9yKe/N7rUdPY+Uo8
ftOQ9EeAe0lF3K4Xd/ZM30pb5EtCiLe/0a29N3oN4F5+GbykdWagd3jGLL/A
/Rmw9KVsvJ4NY0f8xdnDLfezZewoCkemAIwdfVjWyOaxo0G5ZcfNgaweO1Kb
/UdK5sBjR3J9OlaMMLwDbOy1gseOrlbqsmLsSKX25ZyQAdzLri8l0pnMf9rA
oFZ4cnaNAtzLv/aJorm7TP76yMAwBBkYBCKrQQYGwRD4CyCo0biqq005ObSt
i/y553Q7d9qLigJSafzOVQuumlLjsc9pWJe1lrssP+mSxjhl5rMfUhIvhsN/
SlgnHQXXiF2t54wFf4YVL/mvORtP48o1f8+UInaJzd51KmG1dBx+65R55Jyi
7nVS92LgHfEahkh3+HTPmEolzgeLF2Kb4X9h1yf+HZIC415Eo0fd7gV6+3/6
WnDHxPxtsHvpn8xLbtxuOBIUKuqLe79IZ+TLavwh331eMcC9XOo5PqrJwAoT
dwC7PVoNcC8FvFuLDm36TyyOBZvpbE9Ddo0dzZltJ1r7AEtfzj0SsHbsSGP3
Hq+ZALiXg+WjLB87wvzhojYNwL28f0HcI07fihHad4BSuXff2XmAezl0UT6v
olavkxEwLFx8WwMeO6quozmqk2ZIx462vc5dOXaEDEwGefXw8W33atiwIQOD
QNAAGRgEQ8AXQMy6LL76KpMaI3tRUVyN0fKKTF3NB6/TEC/anX+xXT+3ernL
8pMuaVdtofHkJkrixXjsLeeD/IBMmPBXEbIo3d1X4YuNoqXSPknnUmjNf42F
6ZV+uuilYrN7/G7YS54J4Fa26Xr2wIS9zHQcCbopLG/wOKSKsVOk7mV++BuX
eRh+tyuBdC92+yS9/ccIR4JTigfgqiOO8IjZMcXwQHHMG4dKeYcA7qVloiAV
kS9xDC32ASJfiI0zW+8PpfsLVCgS6pILjrbnA8aOBKpUzWTB4/R7y0Q88NiR
SKfM9Gk+g8PnL+KPAdzL/poOoYqlAReeQKhUoAC4l203+G1ilp58jOZhM3js
6BZHl+qxozho3AHanKEzP6vAY0fdw6z2YDG6e62b/yECLH0p+EmJYVmwhoeg
+aEGMHb0txe67hXJ8cAzdWDIwGQQZGAQiKzmd4mgtAdkYH7jrL4AkmVdbIWF
vsEESzUC0gn7zQJ6mbqqTS9EW6RDhgT/Au6X8Kl2ST8WLyc3uRqLgurE9S6P
xUtfMXyxEbHZ7u72jjetFfOy/CRi18HLh4zYjSa9BHTk3zrDfptdWgIzcBQL
e4F/+2O28fnhb0jdi2LslMcRv8AJErt9ktS9EA8wmwbo7X8lcm0HuOqodWCv
yshjfqCVGJ3y2tE8gHupHDyusoqTe9DVyC0zNwTnwU3TFo8p1aexGolx9oee
a4ClLzWSdl/Qn/4Ti4MzJz7QWLqWe9lbX9IoHQ2G2fWtrV48+2lFM2Ds6OH4
NNvOOUaX1AAeO7rJlXsCGa4gBzOt9hwolgHcy/G782pzBq5tSneAeHCpvNW0
8SsJQL8U1xiyYuxIOoPtOyQFuBfip9kydjQ96Tz00TDM2FEcVO//E37jQAaG
HsjAIBBZDVoDg2BI9AJIinXRbNmyujw6SsigdTdV0svUBaS7LEeTdWlVGplO
v4dxykL6xNMlNMQL8WDPYHnYsea/w0ZwzCOtg4zYJTbikTBJL8tPBo5MQ7mk
4mV12AvM299h5M3wvyB1Lypxvh+juXLDbp8UjR4FuxcB/xOjoY/e/leiNQ+3
Dx0AuJf6/h2z6ubkpq+4/ZZ28U/gyJdJdWcSj5gQi8dUIQKNHV3lfZ+Rpmmt
y1g0dB/gXq4ISonHpP/E4pCZ9Sc7HwKWvlzmt1tWhXVnlnGt8Zv6R4ClL5d6
hswYS0tqZAbXkSoRwL0cr5lYMLP6a7INC56tUgLcy0cFEv50xlaMwN8BDkvc
245OA9xLzo8KlT5BYBrbsNmDBT8pAe5l8z9E7Rxzpk8TCrslcO6oGOBedmzg
jfDjp7NjoDUwGQQZGAQiq0EGBkGbqHWZ/uwzJtZFvXGjJS8P6+xcbV2WmWXq
qt99xZL/raenJYK5Vu857LR4equoJusSm+XiZ8QTiacn/J3QEC+mK2+D83UJ
/OpB+IhdQ9lG50BhyEE+HLQUxt3KNu2jT2Hci2kod3XYC/jtb9W0T/VuB4uX
iY631FOXcR/NMXlI96LXcejt/5mX45pNf9VRIOQdkleDI1/4cxXEw5J40NX4
Q76e+ZbzXTlsa5r2Bf0N048A7iW364pIR3NVVRJx+r3XBjgA93Ky86HEwDQR
OrmYMe+lniGAe/mm/tHD3iQv9EoWdi9+vWcO4F523BwYlK/51ZIN4KFIJdfw
zg8Ta7mXv54ev9utJx6WwZOEuQNU6QM5PyoA7mXb0WmeKLWhVcmi/IFu49ZR
gH65+rMqK8aO8EDkXpH8by90rdk0/UrPgxJF3NhRHMjAZJD//fW/AB/O6dyQ
gUEgaIAMDIISsQ6jpFiXiCuBG2GSqUtsxBOJpxM7SXj+Ib3C3XrLdPo9StZF
t/cPa3VJ/3rOhllXZwEl8WLMf83Veg6Qr7tMK2LXK4dSDSGPwSb5GWbgSNmw
gXgk8fiE+0n49g8HMZOyWtK9FexeJh+9rZu9GQzQDI1Mp3vx+E0CyYU0Vx0R
zOh5pJEvdq8+uQddzahGcJn7HWDpS9NURUaaprmKYUDT9NH2/FZZb8bHjoLh
cKN0dG99yVru5UBjaatsnFUjPMTJPByf/rCsca1b7k8rmuvFs8tsvQFoEGm2
3eCv5V62XOsvH1Ti4UyKC1KEc65Pr0gBS1/OViltWObbpsAXAOYNF9cY3twj
Xsu9ED8qbzXhwaW0nTBthkecOz4XA9zLoZwZlToLooMJBN0mcNP0uaNiu4V8
PRIyMBkEGRgEIqtBBgZBCvPmaFLrEs3UtV7OZZKp6xP2L+EJ7hmWggGfqNtR
lmc89hbVLulopVHCLunl/4gXSuG6hrMvRfN1wb9zahG7VdsgI3YJfGYR5MCR
9tGnpO3ScW//YMCmnyudfPQ22L1Iurca5eXhIM3F/273gnjye7B74XG3q1X1
9DqsV5KRqiMClVX8UPgdwL0QP9XZUz7vs2if/2XwEmjsqOcsO5umfxE+MGFQ
74iUItIpj7VXApa+FA93O/3sGuERqnSApmliK+QKHb5fvRbbbgDEGsfB8lHA
0pcfmqdMrsxnAQEwOvDjd+cB7mXP9ZkJBVtG1QAXQDvfBm6aPvOzyuZkdfxO
FNKm6c3/EHX3Zv7TBga5zA1umj700fD0JOx6JGRgMgh7DMzRo0cn1ybTvycE
gqUgA4NISFKsi2bLFoB1WX6SqeusKKadqRutkE6YqUsQtujozRkZj73lKMtb
q9JomZl4ARQbLVOM2NXfedPRl4cbJTB/0OjAkbrjQxj3Yh4557dCNfjE3v64
z6Cdvj7R8RbYvUz1breoGmmLkXS6l2DIMzF/u7ZvW5qrjixuVaPoAsC9lPIO
TWl7knvQ1WABd534HsC9XOZ+N2UQpf/z3+ZzgpumL3B/njHL03xWq9G7HZf5
7QD3crqrVmZO+folSihtzjMdPNCddlP3nPmZRWvsuQGwe/GL7dMA97L3npDl
TdN4KHKLo/vr6fG13Ms7P0zUCjKQcQ0g4QVA2jS96/TsuIzV8TtRSJumiR/d
KdPiOKvXU0WxWwI/5U2Dm6abH2oo7RMZmAzyv78++d6dBjZsaA0MAkEDZGAQ
MZJlXdZK040Stlmwjjrambr6r953lF5NmKkbxS/hOx/kUy2S/rXSqLZwrUqj
x78fjThF4oVGxK5X1gJenRIj5DFYRP9WNmyAaThyzJSF/RTGgoi3vx9TqcT5
pEG7M/wvrJp2+D3HkWb3QloznYqqI9K43eLeL4bk1WmIfOHK2wGRL8SPeuZb
ok3T6fz89wX9rbJecNN0l1yQ/iyaOLxB/MHkIMC9HGgs5cylvLKKEh48eHNw
HOBePq1o5sgSLPRiww0AHo6UDyq3XOsHNE03iKh9tUw/PWIbuGm6sEmd5qZp
GOIuAJszdL5EDXAvG7+SNGbJcpHGFhO4afrEmTmDMQuigwkelCi2vNIDaJou
KZzH3JQ/OZGBySDIwCAQWQ0yML9xkmJdtNu3g63L8pNMXfvNAt3ud5Keqbu8
YrmLbu8fqIoX89kPMU4ZsYe1zvxxuG731VSIF4KAboxSxK5ruCjshv13c0zd
ZeAdgVn0ouvZQzwYcrdP928bF3UfIHUv88PfOBjICra5l1RUHXlxJ3+uAuBe
iK1d/JPbn/LgUNLIl4cTJQ7fU0eXts9/oWYyt+sKYOlL5WSzO5D5f1UHN00T
W5mI5w0yvVCTS6t0HtA0TWw3B8c9eOILPuM3AINyC7hp+nrPnN3Lrl94HHK9
F9w0TfyUeEymTzMxT5dBQjRNX63UrY+m6V1fSsYnE9+NsI0RvmXX3wWApS85
u0bVCprp8cjAZBBkYBCIrAYZmN8gQY0mKdbFVljoGxwEXADRCmnTqf2LG36f
9ExdAnxh0lVbSGO5C7HZrh0CVBoRhB06z2A51XBdZ+NpGPFCI2LXp+TC/HEf
79xvc8yUqdreIxUvirrXzSPngm7KOR4OI29++BtS96IYO4XZxqnuPIbdPjkh
Ogl2L9zedxULZczdSzgSnFU3g91LKqqOiOMKFfUl3C8B7qV2NC8NkS9yyww4
8uWG4PyifT7uWWn4/F90aK8ISsFN08RjUn0apMjM+tNdtQD3cranYcHGrhES
0qbpE619ShsoFCKDNwALZux4zQTAvRypEskMrP6abMOChU1qgHt5/4K4R0wz
qzw9RC+AYYl7xwkZwL0cuijPlqbpM+flAPeycetodR3N5r40o1Z4/rVPBG6a
FnQz+kRCBiaDIAODQGQ1yMD8dvD29Vny8lRvvMFkwihqXVbuNu4CWMIDXv4j
6+VcehXS0UxdYg8JM3UJIl63l99gLz6qP/AyVetiOPwnx+2T3uG2tSqNHu/f
7/aM1Nju7qYqXvwzvVB/BYoRu5j4AWTELoHfOmUeOQez6EXV9p5zripCPQvX
qmmf4X9B6l6U43k+V/y3dXjMpoFR4dcw7iXE2IdE3UsT/7P0Vx1NaXvAVUdl
gn8qzGPJPehqLB5ThagY4F4u9RwfWuxL+NyUfv67A1jZeD37m6YtXje4afpw
y33+4mymT/MZSJumd1e1CRTkkzsZuQGAaZrmzrJLdsUB0zR9i6PLbNM0DJye
0fXRNI3jkXXTNI25QzcuzgLcy99e6LpXJAc3TcOADEwGeeXgqXdLmtiwIQOD
QNAAGZj1TdhsdlVXGw8eXHj++RTlukQvACYV0qpNL5hO7Xc3Va6Vqbv8H/FC
I1Y3Omfkbr0FCHghWArhPkmnvepIisRL0DqXoojd5Scpuy55PWTKroF3xKPt
J91n7n/o7X38AiMR3KJqnOrdDhYvEx1vqcT5uI/+PxHqdZzBgS/S414IIN2L
x5/kb3MK81jl4HFw3O6kmqQwizlYwM2ZrQe4F2IjHhCNfElIij7/Q5FQx1w/
IPKFJU3T3iBeKxECmqaJjXgAq8aOPHiwYmwK0DRN/Ih4wFpjR3Gk/wagekQF
bpouFSg8AVY37PCnHR8VSABLX47fnTc6WHTNJCTaNP3fX6zpXt7cI77TYMiK
pmmewL7t44n10TTd/FCTlKZpGJCBySDIwCAQWQ0yMOuSgFRqKyxUb9yYIusS
JVohLTv6Bb0Kad3ud+w3CwCZussMxAvMcpdff1fzAmfjaWP+a6kQL2G3/vG0
UdW2VETsEgTdi5Apu8RjiEeGPLBuJPbJcPLEEfXUZdKCaeIB2unrwQDNBfOR
CK5W1Qv4n6TNvSj1Pa0De8HuRSC5kHT3YnTKwVVHxb1fCBX1qY7bDUVCfEXX
pZ7j8JEvCUnF579IJwVHvvwifGDzZf5f1YfV8sMt9wHu5TK/Xe9mV/mOQKEB
N01f6hkyYxSuvXTeAIwu2vbeE2Z107Rc7z1SMgdwL59ekQrnWD05FaWx1wpu
mj5xVZkVTdMGYwDcNL3t4wmewJ7p04RicsS+773BZDVNw4AMTAZBBgaByGqQ
gVk3LAUC7qYmU04OvXQX9caN4OboGPj8NO0K6VimbtgGyhSlLV5Mp98D9xnF
eNwo3XrOWPBnePFCPB5XQl3tERzzyloszV9CiheqEbvLVFJ21R0fupVt8FYn
SuyTYc+H/wPsXiTdW/VzpWHqA01RQiGPYqGMx93ONvfiwBI0vzDB7tWDq46I
jSu768VT7hamDKKrvO8B7uWXwUurI18SktzPf9LIF5Y0TS/YTGd7GgDu5WTn
Q4lBnenTfIY5s+1Eax/AvXxT/0hqoBz1nJ4bAI3dm9sgBjdNjy6yOi+FNPLl
nR8mKrkG9o8djcuwXadnUdM0qzDqfKe/ngA3TXc2rFk3QBtkYDLIywe/23qr
hQ0bMjAIBA2Qgcl2cLncUVKi27kzpdZl+UmZkfVyLr3lLvqv3ndWFOPz0+BD
0BMv+gMv24uPEk8ExOrGeFxs1FcMX2wUazWC/HP4lFx71ynIbiOqEbvLT1J2
7dKSxeZNMO7FPHLOb52C33mUYMBmlJfDGJip3u2j/T/STsHFcfvcbDG3912Y
nqOkuBetebh96ADYvfRP5iXdvXhxJ1d2l7TqyO6loODooXUu3h4uBLiXy9zv
JnTD8DtM1uc/aeTLCU4BV0HhxFKE0+8tHu7OrqZph89fyBWCm6ZbpTRTm1J9
A+AJhG5y5QD3wv6madLIl2jTtA1LZr1aKjBY8BNXleCm6WqOOdOnCUV1nQHc
NH3mvNxmZ/tfhAAPRO4Vyf/2QlfSm6ZhQAYmgyADg0BkNcjAZCnRWN3FV1+l
al0UL75oysnBOjvBE0ZRIpgL66gz5x1WbXqBqnXRfPC69XIuIFP36VFoiZfo
chd8YRLm1xUVL5bi7fAxL7a7u73jTRE/yRBTFEoxLzQidgl8ZpFpKBcyZdcx
Uxb2U/4XYY9DqhLnR+0K2MBM9W63atqX6b79/T7jtLQALF6ITcD/RK/jMO85
Wn7iXjjCI2D30jf+ndlBWViBCYS8QkV9ce8XGa86cvhsdeJ7APdyviuHK28P
RajdqDP//CeNfCG2Gkl7xpumg+Fwo3QUHPnCtqZp4pzrxbPgpulSoRgy8iUh
Kb0BaBPrwE3TN7lylke+COdc4MiXIyVzrG2ajhGNfHlzj3gdNE0Pjzh3fC4G
N01LZ7JgDQ9Bb7sBHPly+usJoy6F8TXIwGQQZGAQiKwGGZgsApfLnffvG/bs
kT/3HI3lLvaiooAUqjckZNC6akoNh3ekokI6xlIw4B1usxcfpSReLBc/A7dI
P/Mb04hdnQXwK16IzXx9q2ewPIJBuRGqMS9UI3aXnyx6cc5VaR99Cpmyi6m7
4HceJRLBrZr26f5PVjqWtQzMDP8Lh5EXey7Vt7/bvSCe/J7UvQiH9ptNA1Rf
SEIMVlFG3Es4EpxUd7Kh6sgf8vXMt5zvygHol6apCiwAJRvjYPj5Txr5UjR0
X+syMjlEUhDplMfaKwHu5SK3Re2gIFTTgFCl21/TAXAvZzp4WiedP/pKUnQD
INY4DpaPAtzL8ZoJjZ3V4kJt9h+/Ow9wLx8VSPjT7IoJSghp5EvOj4qsaJpW
qX3gyJfN/xA1trC6QivG9KTz0EfDAPey6++CyZGUx9cgA5NBkIFBILIaZGBY
Tthsjqa70FjusvD888aDB13V1cROYI4VkE7YbxbQaJGOVkj7hOQNO49fkdMS
XfGi2/sHSiteME4ZpHjxz/RSynghNtOVt93dV0MWJcz+IzjmkdbBx7wQm63z
mFfOgQ9jIR7pVrZBJr1QTdmNgfsMa6XsxhmYaMmRxxFv8ODf/nb7pGj0KKl7
IR5DPJLqC0mI2THVN/4d2L1whEe05uTPtswbh8oE/wS4lxLul2moOiIY1Qgu
c78DuJd7o9eM7jU7yMj3T/fznzTy5YeeaxJj5huc1Q7rRW4LwL0ca68U6ZSZ
Ps1n0Drd5x4JAO5lf02HUJWcUIik3wCYXP4fmqcA7mXX7aFBOeW8mnSC+cNF
bRqAe/nr6fH1Efmy44Tsfl3KHTJzMCx89WcVwL28+fZI8W1NVjRNk0a+bHml
p65sMT0ngwxMBnn5q9Nbfmljw4YMDAJBA2RgWMhSIBAdMqJXZqR64w3iucQe
oI6FB7z8R5b8b9XvvkLJuixu+H20QlrEaYc5UFA94269ZT77IdVRo8fixUL+
ZSHid/sknY7a44azL1FqlIbP1yXwyjn2rlPw4sVU87FHWkdp2sij7TcN5Srq
Xk9dyi6BXd8tHzkGyNeNfTJ8+cn/a5SXr1VyBPP2N5sGhEP7Sd3LlOS8202+
dAqGDLoXlVX8UPgduOpoSF6d6qojgkX7/C+DlwDu5Srve5mJaWIJjc9/mMiX
LrmA6jxU0nH6vWUiHjjypVU2Hgyz6FubBw+WCsXgyJd68WwSzzmJNwCeQKhU
oNhyrR8Q+VI9osLDrBYXtQITOPLlbJVyPUW+sP8OsLrOsHHrKEC/nDgzZzBm
wRoezB0qKZwHRL4Q242LsymKfEkIMjAZBBkYBCKrQQaGPfhFIntRkXb7dnoV
0rqdO5337wc1UJmEYZsF66gzndq/uOH3NMqMVqa7AC6AxwEvw22O2ycNh/+U
KvGCWT0jNfaqI/DWhUa+rl896ODl6++8CRvzUrHZNVwUclD4d6iAY84i+jdk
xC7DlN2p3u3geqOVBiY3NxewQ9BfP4LrdRzSgune7k3T0gK/LzmTJjDupXVg
r2rFIFWyIK2ZJrae6VtpqDqyeEwVomKAe7nUc3xosS8pioPS538WRb60ysYP
NJaCI1+cfhZNwcBEvhTxxxy+JJc1J+sGoEGk2XaDD1j6UsiR2b0sythZjXDO
9ekVKcC9HCiWTauTkCieUjBv+GqlDj7yhc13gDCRL+OTWdD9TdD8UAOOfPnX
PpFake6rCxmYDIIMDAKR1fwuEZT2wOb//7KfWLTLwvPP07AuihdfNJ88CR+r
6xP2266f0+1+h8ackf1mQUA6sXq3qy+AoHrGVVtIdbkLJfESdug8g+XWkp00
xAtkvu5SGPfKOY6+PHjx8jjmhZcf0FFYlR3yGOzSEnXHh5DiJSkpu+Bt8tHb
2unrTAxMKORZVFbBFEzPzRbjeHJm1SHdi1Lfk5TDrcTtt5DWTBMPsLhVST90
HFjA3TZdDXAvxMaZrfeHkpbNCP/5v24iX872NGRd5MuJ1r45c0rKmpnfAIwu
2vbeEwLcy5EqkczA6q/JpJEv718Q94hZXZYdpZpj3viVhFLkCzvvANdT5MsI
37Lr7wKAe9mxgSfozsxrofHXZ/6lAxHlpf1nNhd3sGFDBgaBoAFaA5N+otaF
XrRLdNNu3w4ZqxvBXF7+I3rWhdgMh3e4akpDBlBMRPQCCOkVnt4q27VD+gMv
UxUvxpOb3K23YMRL0DDr7r5qvr6Vkngx5r/mbDztn+ldCpH/E2rYa/XKWmyd
x+BHjaKl0l45h3TnT/8uQcwlr9f17IEUL4q6101DuR4tVNLOMweK4BZVY1zK
7lrb7MB+q6Y92j1Ez8DAF0wrFsqSUjC9DOdemvifybUdSTncSmBqptNTdRSK
hLjy9ks9xwHu5eFEicWT5Bt1mM9/uXWxgHdrHUS+HG65P6yWZ/o0n0Fpc57p
4AHcy+6qNoEihWXNTG4ANHZvboMY4F523BzgzrL6a7INCxY2qcGRL3e79eyP
fBmWuHeckAHcy67Ts+OyBIvT2HYHuJ4iX9QKz7/2iUgjX/BAxq4utAYmgyAD
g0BkNcjApAe/SOQoKaG91iVaZhRNdyFd7hLBXJ6eFuvlXHrWRbXpBXPeYWIP
xH7ABwpbdJ7eKvn5z6kOGT2tk24sCukVJC/nScCLs/E0pWTdaLgufMZL2K3H
xA/MDZ9TEi/E4z3SuggOOzSxFMYxdZdx4DikeCE2fd8Bt7ItEqQ8l+HHVGul
7MZtCVN2qRqYaMF0b/cm0oJpraYlKQXTy9DuZVbdHI4kOXgBpma6cvB4GqqO
CCZ0w+C43V8GLy3a51NxaPDnv83nvD1avQ4iX/bWlzRKR1kV+eLw+W8OjgPc
y4dljQ/Hp1N9zvRuAOxe/HrPHMC9bLnWXz6oZHPkCx6KVHIN6yDyRaUP5Pyo
ALiXzd9IG3vXXPTFnjtAHI+QRr6cOS/PisgXuyVw4+IswL2kP/IlIcjAZBBk
YBCIrAYZmNQRkEodJSW6nTtplEdHt8VXXzWfPOluaiItM1rCAz5hv/1mAT3r
QmyaD163Xs4l7TNaCgZ8om5HWZ7x5CYa1oXYrJf3wowahSxKGnNG0Tppd/fV
oAHqn9Sj4sVU8zEl8fI45mX0JvFcmENE8VunzCPnlA0bIMWLuuND51wVjXqj
5Scpu7MD+2EWvUz1bjfKy8OJ9A68gXE6ZyALpvU6CsuEwGTQvURrpku4XwLc
Synv0JQ2+eNOqyGN273M/W5Cl/zA4Rhrff77gv5WWS848qVh+hEbIl8apaN7
60sA+qV4uDvrIl8KucKkR74khOoNAB6OVI+owJEvF9unWR750iO2vX9BDHAv
e67PrIPIF+JHxTWGWORLQlhyB8gT2Ld9PAFwL/sOSaUzGf60gQEPRB6UKLa8
0sO2yJeEIAOTQf7X/ry//8xhw4YMDAJBA2RgkkvYbHZVV5tycmivdYlWSENm
6gakE86KYsPhHfSsC7Hpdr9D7CGoIumgCekVGKfMenkvPetiPPaWoyzPJ+pe
CpL881O0SNp05W2q4sVSvN0zWA5ZJx32WumJFwcvHzdKYA4RJTptBB/zsti8
yTrxU9BNp0oS9xl0szcl3Vth3It85JgL2AQEY2DMpgFu7z6YgmmrdYTGK0p8
0My5F4IZPY+0ZnpssSUVh44DJm6XK29P9fKShJ//AtUYOPLlF+EDE5b5KJVh
tfxwy31w5MuCjV1TMAKFJlORLwmhdAMwKLfsuj2U1ZEv02rPgWIZOPKleZjk
X0wyDh5cIo18OXFVabCQe7CM3wHOy72HcmbAkS/da6/hYRWCbtOODTyAe9n3
3uDkSHKS05ICMjAZBBkYBCKrQQYmKeByuaOkhF55dKzJCDLaZQkPeHpaLPnf
qja9QM+6ROeM3E2VYZsFfCy/hP94ucuxt5gsdyGdM1oK4dEiaWP+a1TFi638
gGekJgL3bS6CYx5pnaX5S2ripWqba/Rm0DoHc4gYXsOQaSgXPubFPHKOeAql
Q8TAbOOKsVMw4kXSvVU3exP3kS+t6f0PSqVy9U/1Og5MwbR48nunM2kRKJl1
L+ypmYaJ2yUeQDws1WeyvOrzf8YsB0e+XOD+TDwmDScGZsFmOtvTkF2RL3Nm
24nWvgxGviQE8gZAZnAdr5nI6sgXowPPLV8gjXzB/CwaVUsIT+TcdnSaRuRL
QjJ4B2gwBk6cmQNHvtwp0+I4e2fZYkxPOg99NAxwL9te5zY/TPe7mxRkYDII
MjAIRFaDDAwTghoNE/Gi2bLFevGit68P5lhR8WLOO0y1Pzo2ZGQ6td/dVAmO
1Y3yWLzcPkkjU/fXdJfaQmIPpMtdCKLixXD2pdRVGv36itSD9q5Tult/hBcv
luYvMfEDSqNGBGG/zTFTpmp7D9K9GAeOu5VtS2GaS+6tmvYZ/hdUU3ZpQzxd
raonLZgmtiQWTD9+ma7ZDLoX9tRMw8TtVoiKkx63CyD2+W/CrL8IH4AjX7iK
FM5DQWLxuouHuwHu5UBjKQsjXwq5QoB7+bSiuV48m5FzJr0BsHvxi+3TAPey
7Qa/ekTF5sgXzB++xdH99fR4tke+SOXefWfnaUe+JCQjd4CkcbvRyBebne1/
keUnkS/njooB7uVvL3TdK5JnPPIlIcjAZBBkYBCIrAYZGBpEXC7n/fva7dvp
1RhFrQtMf/QyM/Gi+eB1S/63WEcdjHUhwBcmHWV5NGJ1Taffcz7I94m6I14o
JYIrR12t56iueDFf3+rqLAjMC2AOESXs1ruGi4wVm+HFi63zmFfWEvZSXrTs
0fYbeEcgxYv20acueT2NUulfX1cQM8rLYQaOJjreUk9d9mNMe5BDIY9ioYy0
5Ci5BdMEHr9JILmQKfcCWTNt91LTdPTIYNwuAOLz3x3AGqYfAdxLNPLFF0xH
LAkAbxCvlQjBkS9lIh7bIl8ejk9/WNYI0C83B8fTE/mSEMANAB6OlA8qt1zr
B+iX6z1zLI98qRWYwHG7R0rm5HoWXTMJMVjwE1eVAPcCE/mSkDTfAeJ4pPyB
Dhy3eyhnZl7O9r8IAeYO3SuS/+2FLoB+OXdUbLewNzoYGZgM8r/2/bCpqIsN
GzIwCAQNkIGhhF8kMuXkUE3W1WzZYiss9A0OUjpWQDpBY9RI/e4r0QkjSOuy
/KTSyNVYRHXUKBbtMiaArUiO+N2ekRpKGS+Gsy/Zq44Qzwo7yLuqV+KVtVAq
NoqKF/hWo6cvKog556ogF70sNm+ySX6mF/MSBfcZ1FOXJzreInUv0/2fWFSN
CVN2KQHpXnjc7YvKqmQVTBP4ccfYbHGm3Esg5OXPVYCrjhpFF9JQM73Mgrjd
tQhFQne5D09wCtgf+dK7MA2OfLnIbVE7Mn+eKxEoNLur2gDu5UwHT2lL+cor
MGvdAHRJDTtuDgDcy/GaiQUzq5NRhXOuT69IAe6F+Cl/2pHp0ySBNG6X2M78
rIKJfElIOu8Au3ut4Lhd4qc8AYtiUgB0Nui2vc4FuJdDHw1PT2b43U0KMjAZ
BBkYBCKrQQYGhuiiF9Ubb8BbF+LB0fJoysfCXO6mSu3Ov1DKdYlOGJEG6sbh
l/CphuuujnaBuQBwjdjZeBp+2uhxkTTF5S5Rwl6ra7jIULYRxrrobv3R3nXK
K+fQEC8EIY/BIvo3ZL2RgXcEU3fROEoMP6ZSifNhBo6U43lx1dL0iM4c8bjb
01kwTRAMeaYUD2r7tmXEvcBUHaWtZpolcbsJkRhnf+i5BnAvBbxbbIh8kZn1
p7tqAe7lWHulSKfM9Gk+g9RgAUe+7K/pEKqoSekUsfrzX6xxHCwfBbiXvfeE
g3KSCLLMItd7j5TMAdzLOz9M1ApYnVqzDBe3e+iiXMpsuUh67gDHJ137DkkB
7mXj1tHyB7qsiHwZ4Vv2vTcIcC87NvAE3Wy/uqLUl/Yf29AiHaAzcYwMDEOQ
gUEgshpkYMAENRpLXh58sZFu507n/fuk5dEJCRm01su58ItetDv/Yrt+LiCd
oHqgiNeNccrgF70YDv/JUZa3VrQL4AJYCuHe8Sbz9a0UiqT7iiGLpOMIWucc
fXnwGS/0VrxECTjmjAPHYcSLqu09x0wZvVLpGD7XvHzkGEy1tElZHQwkpwZF
r+OQ5r0Ih/YPDtxJyuFizKqbm/ifZcS9ECjMY+Cqo1LeoRk9LxWHjgMLuDmz
9SyJ241j0aEtGroPcC+5XVcEqnQYKjB6t+Myvx0c+cKZE7Mq8kVpc57p4IEj
X1ql6Z41A7Dy89/k8v/QPAWOfGkQsS5NdCU2LFjYpAbH7d7i6NZB3C7xU+Ix
zA+U6jtA0rhdYrv6syorIl/kMnfOrlGAe9nySs+DEgUeyAKPpJM7//1537b/
s4zYjm1oobEHZGAY8uLes29f62bDhgwMAkEDZGDWIqjRmHJyIMWLYc8eV3U1
ZLTLagLSCXPeYXjx4ii9SnW5S5Sw0+J8kK/b+wfIOSNXbSFpk1HCCyDid7v7
io0Ff4YXL5BF0qvxqwetbV9Dthq5x+9SDdddidcwBBn2YhrKpd1tFMPnmocp
OZrhf2HVtDM8VgyzaYC050g0etRun1xO6ttfax5uHdgLcC/1/TtS51509pna
0Tywe5lUd6ahZjoUCfEVXayK243hDmBl4/UA93K0Pb9jrp8NkS8PJgcB7oXY
iAcQD8vsea7E4fMX8ccA7oXYSoViD86ub5rRTwC7F7/JlQPcC7ERD/AE2Jgm
GgUPRSq5BnDcbm75gtHBomsmIaRxuxu/klRzzHhwKSmHS90dIEzc7okzcwYj
e2NSYhh1PnDcLrH9lDfN5siXGB4XXnR4IOpeYpuwQ011P8jAMAQZGAQiq0EG
ZjVBjcZ48CBkrK6rujrictE+ln9i2HB4B4x40e1+h7Z4WX4S9uK4fRLGvRgO
/8n5ID+ohg24iLsAIpjV1VkAM3BkLPgz8Uh6K16i+JRcU83HpOJFf+dNBy8f
N0poH+jxscwiXc8emEUvTCJ2Y/gxFYx7IR6D2cYZHiuG3T45KvyatGDa7X56
ESbl7W92THWPfgtwL7V928QL94PJC5lZCWncbnHvF0JFfRrcC4HMJL7K+55t
cbsEvqC/Y67/aHs+QL+Ujde7AxlO9giGw62y8QONpQD3cpnfrnezKL7Dgwcr
xqbAcbvnHgm0zgysdyJlaGS0fFC57QYf4F5+aJ7S2FmdjNopsr5/QQxwLweK
ZdPqlHz+JBHSuF1iu1qpoxG3CyAVd4Awcbv7DknHJ+nfeqUNzB26cXEWHLf7
r30itYLtVxdBMBCu/nHys/9ZFadfiG3Pf9UQP6W0N2RgGIIMDAKR1SADsxLI
dS8Lzz9vPnkSlzOKOPDyH+l2vwOTrGu7fo62eCEI6RWO2ydhFr3Yi4/6RN1U
9x+7AODdi638gE/SSfsVLUO7F3PD515ZC+2u51+PZRbp+w7AJL14tLChxABw
nwEm74V4DPOGoxhu98KE6CTpuhenM97LMXz7OzAFadWRcOaqH0/J9+Vo3C64
6ogru5uGmmkCrXPx3ug1FsbtEghUY7ldVwDupWjovtaVtPJx2gyr5cfaKwHu
5WTnQ4mB8r/VppRW6Tw4bvdoU/e4NvO/24S0iXX/KOIC3MvB8lGxhkWyazX8
acee6zMA9/JRgaRHnJzRztQBE7d74qqSdtwugKTfAXb3Wjf/Q7QO4nbxQORB
iWLLKz0A97LvvcERPqszkWJwymb3/FfNavcS24gHUNohMjAMQQYGgchqkIGJ
EjabLXl5pCVHmi1bmEwbRQlIJ/RfvU/qXojHeHroTNc+fVFP1r3ATBu5W2+F
nTRvA4gLANK9GPNfc3dfpdpqFIdfPQjjXuxdp4LWOSYHWoZzL4q61y2ifzOp
N4oB414mH72tnytl3nAUw+8zTknOk+a9RGeOVkP77e/HHcKZq2D3IpBccGAk
Q3D0CEeCY4st4LjdRtEFiztpjgsAFnDXie8B3Mv5rpxMxe3KrYsFvFsA9/Jd
x48SI/1lbMlCZtaf7WkAuJfDLfd7F6YzfZrPIFTpvql/BHAvu6vaOLKUXP/M
GV207b0nBLiXHTcH2sSsyApei2m1hzRut5JrwENsD+Wo5pg3fyMFuJd9Z+fH
ZalanJbEO8DxSdeuLyXguN3qOkNWxO2SVh0RPyUek+nThGK8V3dsQwvAvez5
r5rWW5Q/XZGBYciLu8+9/VMvGzZkYBAIGiADE3G5bIWFYPdC/NSUk8Nw0QtB
ULVgOrWftNjIkv8tPs/oy0LYaXGU5cEUG/klfCYHivjds5XkJUfWkp3e8aal
EKN/gMONEkvzl6QDR86BQiZJL1ECjjnSvBdlwwab5GfmA0fLjxf32tRTl9Ps
XnDcLpu5StpzZDYNAHZC4+0fDHnEC/fBVUfdo9+aHVMMXhyIeeMQOG63cvC4
yipO0dFXEoqEuPL28105LIzbNWHWX4QPAO7lBKeAqxgWjgrTf24r0bsd1wY4
APeyt76kViJkVeTLnNkGjtv9sKzx4fg02yJfoiyYseM1EwD3suVaf/mgks2R
L0YHnlu+AHAvxFbUprFhbPz9r6R72AGO2938jZR4TErPISl3gCq1Dxy3++bb
I8W3NRjG9gDkZYiqoyyK250bs5x5nwNwLx/8X/crzo97XHQ+XZGBYQgyMAgE
fdp2Rz98drdRedb8fFvh7pdfXvHh9fLLuwvb5mllA/zGDYyjpATcc0T81Hrx
Ir1uo5WEbRZL/rekA0eO0qsRjNFoc8TrdtUWkua9OG6fDFsY/fvLUgj3DJYb
818Duxd71RFcw/TLbMixaOs8RpKyW7EZEz+gXW8UI+y3mUfOgd3LYvMm51xV
JBkyJBLBjfLyyUdvp9O9hEIe+fzt3u5NAPfC427XasiXYFF6+4cjwVl1c33/
DoB7aR3YqzWnatYGJm53StuToqPHMaoRXOZ+B3Av90avGd3a9JzMStwBrGH6
EcC9EBvxgGjcbgY//51+L2nc7rUBjsXLovgUM+Yt5ArBcbtF/DGHL8NRxgkx
ufwX26fBcbuFHJndyyLZFYcNCxa1acDuJbd8QW1m4+9/JcMS967Ts+C43fJW
U7LidgEw/AQwGAPn/60Ax+2eOS/Pirhd0qqjv73QdePiLOZmr5yMYdZgsaqj
tbaiwwPdbaB/oAGDDAxDkIFBIOjyH/9CxcDMF640L/G8XEjZwvxmDYxvcFC9
cSN43YslL49Jym6UCOay3yxY3PB7gHvRfPC6u6lyCWd6j+HprTIc/hNAvOj2
/sH5IJ+heyHwz/SSdkw7G08zHDgiCHutpB3TxorNXhmjca0oS2HcLi1RNmwg
dS8Mg2Vi2PXdku6tAPcy0fGWdvp6Et1LJIIvKqu4ve8C3AvxU7WqnngkzA7h
3/4qIw9cdRStmWbw4kDYvXqYuN1AKB15oYv2+V8GLwHcy1Xe9zJTOhbhxBGK
hLrkghOcAoB7uT1abfM9DcbJyOd/MBxulI6C43YvclsWbBmoi1oLyLhdpS0d
oUNU8QRCN7nyLdf6Ae4lt0G8YM5wDjMAzB++260HVx0dKJZNKFjk6xIilXsP
XZSD43YL7mltzjR9zaf9CRCtOnrz7RGAezmUMzMvZ3WGcxSYqqPTX08QD8v0
mZLjMPvunh4Fu5cz73OUU4+jeJh8/iMDw5AXvjgP9uFp25CBQWQV80/9C7SB
WfmUNaG2nua3aGDCZjM4bjfqXpiveyHw9LSo330F7F6wjjrmBwrIhKbT7wHc
i/7Ay67aQtphLzGChllryc40uBcC9/hd/Z030+BeCDza/sXmTWlzL5htfHZg
P6l7CQaSmQCpVtXzuNsB7qW3e5NioSxEpXII5u1vsIo4wiPgmukpxYMUlQ15
cSdp3G7P9K30xO06fLaHEyUA93Kp5/jQYl8azmQ1Ip30h55rAPdSwLslt8bn
HaX/85+/OHu45T7AvRxrrxTplGk+KwCP65mk859WNIPjdqUGNgZy4uFI9YgK
XHXE/rjdWoEJXHX0UYGEP83ql7AMV3WU86NCpU/rchEanwA4HrlTpgVXHe34
XDw8wkYbGQdM1VHOrtHpySx4LYCqo9h2bEPLeO/TuztkYDIIMjAIBHXm41wK
jDUBr36hvRLmt2ZgwGNHSXQv+Pw0OG5XtekFV00p83UvYYvOdu0Q+boXxu4l
4ne7OgtIZ46YtEvH8KsHjVXbAO7FULbRI02CuSIIuhfBcbvKhg12aUmy3Avu
M5DWTKvE+cl1L1briID/CTjyZW62GMefFkzEPhlyc3MBewa//f24g7TqaGL+
doqqjggm1Z0sidv1h3yc2XqAeyE24gHEw9JwMnEsOrRXBKUA95LbdUWoSXIU
Mw1kZv3prlqAeznQWMqZy8DaIQBClW5/TQc4brd3Pgk53qmgS2rYcXMAcOO9
6/ZQacdQpk8TBH/a8VGBBOBe3r8grhWwaK1UQmzOUME9Ldi9pDRuFwDVT4Dq
OgO46oj4aWML2/8iy0+qjurKFsFxu7v+LhB0Z8FrCQbCrbemwVVHX71UL2hU
xj0RGZgMggwMAkGRBCqF3MDEPelJ7kvsZ/Ntu5/dJQUH89sxMH6RSLNlC2Dp
i/HgwaBGw/xAS3jAfrMA4F4WN/yeeADDvJco7tZb4MiXpLgXAu94k7Hgz2lw
L2G33t51Cpy16x6/mxQfEgli1omfwD1HxAOSkrW7/GQCSD9XOtHxFti94D5D
Ug4XBcftpFVH09ICvy++7pahgQlHgjOLteC4XYHkgsefqrtTnX2mcvA4OG5X
YR5L0dHjGFrsu9RzHOBeHk6UOHwZaLy1+Zy3R6sB7uVoe37HXD+ghik9n/9q
h/Uyvz274nalBsuJ1j6Ae/m0orlePBsMszFcdHTRdrB8FHDLve0Gv3pEhYcj
rL0BmFC4wTXTfz09frdbj/nZ+PuPgXnDxTUGcM30jhMynihjSyzgL4DuXuu2
jyfAcbt3yrTrpuqo+WES7ifTAGnN9Gf/s6r6x8lgIME7BRmYDMIeA3P06NHJ
Ncj0LwmB+A+rXAmkgXlmzUxiv/Kso4GeRfotGJilQMB68SK4Y9o3OJiUY/mE
/dqdfwHoF0v+t2FbEpQIvjBpPvshuOcopE9Ck2nIogSPHaku/x1XJucacI/f
1d3641ruhfiRc6CQedbur8dStoHHjiyifyfLvSw/GTua7v8E4F7mh7/xuWil
aa+NWlUPjnwRT37vdi8kfC4TA2OwitqHDgDcS9/4dymqmV5+MnYEjnwp5R2a
VHem6OhxyC0zNwTnAe7ll8FLi/Yk/91h8AX9rbLeo+35AP1SI2l3B0jebqn+
/Hf6vWUiHjhut3i4m3hYSk+DElqn+1LPEDhut1QoZmfcrljjIK06usmVx6qO
WHgDINd7j9+dB8ftFjapWV51hAeXyltNG7+SgKuOGnutmT1PmAtgeMQJrpkm
tqs/q2x2Vv9Fogi6TTs28MBxu/eK5FkRtyvsUH/zWhPAvXz0f1fcPT0KqDpC
BiaDvPD5+Y0/ctmwoTUwCHazhnyB8iUrBQxgfQuNaJn1b2B8g4OqN95Yy70o
XnzRVV2dlANFMBe47ch47POgKvG3XWoHetJ2BHAvxpObGHZMx8B4dwBN08aC
P/sknUm5AHCjBDx25OjLY94xHYV07Ij4acAxl5RjEYSDmEqcD3Av0/2fuJLd
/uN2L4wKvwa4F9HoUbsd9M8T9AwM6dgRR3jEYBUl5TWuJhwJji22FPd+AYjb
HZJXpydu1+IxVYiKAe7lMve7UY0gDWeyGq5iGBy3WzR0X+uKXxaVkNR9/nuD
eKN0dG99CThuV+3I8DfQlTh8/iL+GNi9XOoZMmMs8kUxFszYD81T4H/rvNg+
bXI9I45YdQNgw4KX6hZJq46MDhatlUpIO9+2+RspG6qOSAFfANIZ7FDOzPqo
OiKtmSa2n/Km7ZYseC3SAeOxDS3guN0r+3lmTQoNPDIwDEEGBoEgZbV8eXl3
28osGBJdAm1WVq6DgVQw69jARFwucOKu+eRJ5lVHUbCOOkDirnbnX3zC/qQc
yC/hA9qO9AdexjhlSTkQrhED2o4MZ19yd19dCj2+j2V4AURwzMHLB7gXc8Pn
uFGSlBe1FMZtkp/BcbsebXL+UlGsmnZA0zTxI4uqMYmHW34y6ySfvw1wL8Kh
/WYTeYMkVQNDOnbUxP9MqU9h0bPKKi4T/BOw9IUzdSM9cbukkS/nu3K48nbA
aE/qkBhnL3B/BrgX4qczZjn8DlP0+c+ZE4Pjdk92PpQY1Kk4ND08eLBUKAZX
HZ1o7WNn3K7J5f//2Xvz9ybOe++//8fzy3Odq9dz9Vw9J+m3p+nTJKdP05zS
0BJIAgk0B8gCJBCWEOAAoZAANeCwOJjN4ICNwdjGeN+NvMhC8ibLsixbtmzJ
2vddGm2jxf4OmdZxjeae0ejWaEzmfd2/BN2aexSPbmle+nze70KeEsxejtXK
UkYdseQLAJWoo0Mls1OGNGzGcyKh1Lf1uBLAXlbtlBfXWpEQW5qniC4Aqy16
/PQsmL0cPq5cEVFH4yMecMw0HnVk0LD96sKknfSc3sijGHVEKo7A5FAcgeHE
iVT/3CD0lL48/VfKBCYNrrLkmBS9YJ5XAhPq69P8+tdE7MWwZk1ECueH+ITb
aTu6A2D54i27lrnd7sL3pS/e0hMMWL6QOu66K/bFndrF+ZlcABHDoLV8DcDy
BVbU0dO1XJOGzs0AyxeIdrtPl0P06pGjjMVM43K5RgBpR9hDNivVnJ20CAxp
25F87kEsnYiltOQJWVplBWDLF72LIYNWibH/suBrAH6pl99HojlIvDX5bUVD
D8B2uwJN2rVY0Pd/qVl7tKMKwF4Otj7gz03BXTQTxRKJR2NT4KijPbWdYj2E
eDjo8oTQ2wI1mL3sui+W6Aj7MXP+BQBnL2+fkQHYy8dXFOyPOhpTIp+dVbEn
Zpqinr0A3J5YwVUtmL18tl8xNg7n96+sSq0MnPpCBmYvh7dLxkco8YrcSjvp
ubSjD8xejq5uVQxQqn7ExRGYHOpX287/+ZKQDYMjMJxYq0WE8g/48r0oExjq
xTI0EMzzR2Dmo1FnXh4g7chTVARrrbD4CaD0xXpwK5S2owWy0hfH2c3oHBzD
K3Dpi/3K2qhqed8EvQvgaelLXx647QiW5Qtp6Yt96CREyxdMNnUFwHFXPXI0
gkDO30FRj0x6AlD6opy+llbMNEUCMzwiALcd9Y19HQiZMn15BEokY0PqGkDb
UYlg96gOGsQDS+dRfTd4EcBeSocLTb4cpN4EokjVeAvYbrdNyQ/H6NiSQNz/
SaOOdjWUNCkkrLLbbVOoPq1uB9vtYnNYaLcbjMYrBrXrrz8BsJettwfa5STg
KIdfANB4kjRmGnu0ZRhCuGFWpdKHSWOmsQkMx0xT1NILAEESxaXGVWtHwDHT
wv4VwCts5vC5I3Iwe/nsvcERERur2pbJYUSu7BGC2cvelxvEnWlXFXIEJofi
CAwnTqRSFb7yT+FFuKiClSUlMORQJa3JT/WcEZiIVApwfTF/9BGUtKMFMtcX
/bqXkE44Wcng0hfLvlcCbXegLDQfRwM91wClL4ttR8tE4wKIGAYtd1cRsRdb
9aaoGVpIDbj0BXsoZIWZphr0KgCOu+Nda13GDojL4dJpqwGOu4MDn/h80+ke
k5TA4G1HNb3v5artSGUbKhPuB5S+9E7dYabtyBt2P5KVsNDyJRyLdM4+Advt
lo81uMP0/y9B2f9Jo46wUS4Vssput19jBMdMby5vejQ2FURZZy6KJpI1I/pN
N0XgqKOKQS02k/RoufoCQMpe3j4ju9djQeOsDtahwl72X1ArWNyqg18AKJqs
eGhes0ECjpnu4LGdhmHyOKNX86bA7GXrauGKiJl2GJGigwNg9rLzl7V9j9Lo
PF0qjsDkUByB4cSJpqgSmPTKWtIpmHmq54nAeIqKiNjL3IsvwnLcXSArfbF/
tQdK2tECWemL6/IuKG1HmGLWGUDpi6vko6VtR8uU1gUALn0x3/l/sJKmF8hK
X6C3HSViiGnqBjhpOhaFnDjs800DHHf5Pet02mp6RwYTGNK2I5mqNHttR86A
vkn6DYC91EnybD6aXynTUjwZ71W15ncfBli+YBMi8TADJ7NM/frRk91XAOzl
Sn+ZzptpeVKG+78zFCiV9IHZy2VRB6vsdsdMtiPNPSs06qhdbt56ewAcdVTW
r1mMOiIV818AHktdWwomwHa7Re1Glkcd6S1RUvay/dTM8EQOOhbT0tCQpKbe
+s5fpAD2smbD0znsj5lGAvGbF2befKkbzF4eN7Kxo3CZvI7wvVMSMHvZ9vPq
xhsTKWOmKYojMDkUR2A4caIpGgSGAlP5cRIYVK02rl8PKH1JOOD88jKPRl2X
TxKxF8O7r8Jy3J2PRb3leYDSl5CoEcpCC8DAI+zfg4MV4KdTvwBQ2wTA9cXV
/kXcC61HA1z6YunbFwvA7AfxO4YnejYwmXYUjwdnZ4oBbUcy6YlIOI2G7mUi
IjCkaUc9ki+zlzQdjYdEs5XgpOlJUxYLb5Zq0ioFW748kpV4w5CZGxVNO9Rg
u90zvdcnbDNQ1qK9/4di6MPxQXDU0anuOqUDTgIaFM063Kc7hWD2UigQszPq
SDBj3146BLZ8uS1Qe0LpQWkmvwCIpryk7OVivY7lUUdWJ5pfYgCzl01HpjpE
Odg60hKKJmvqrW9uAPm9rFo7crfchCCs68JbJiQQv1+kXv9qL4C9bHpdUF+u
Q6Ns50g4e9ny00owe6n5dhwQM01RHIHJof7j4wt/uiBiw+AIDKcVJoqsJM18
o3SNYJ4DAuOvqZl78UUGSl9i+jnTR38iwi/O818mETjOcujcuO3EOgZKX+JO
ravkI0DpSxIh/+mZ4gXgHy5ixnEXXPqibVwd0FKMaqckcNi0rPMN88ztZBLy
HYHDPgDLcZdIzxIY0rSjhidbs9p2NGnqBbcdiWYrmUmaNvl0pcOFAPby3eBF
nYeSFzrkEyOz2z3OK+hW90OMYaKx/8cSiTbl2L6mMgB7OdpRJTVrYZ1k5jL5
Ahd7h8Ds5XSnUOtmoustXUl07s8rJGD2UshTLouZpnpwRr4AiKa8O29Mk8ZM
qy1sZF+LosJe3jmgqOE52BAzDVZTqx1c94KNa7f0bg+rK5EWvmcv9eW6Ta8L
AOxl/au994vU7GcvQT9a8+34tp9XA9jLlp9W3jslyZy94OIITA7FERhOnGiK
IzAZC5w3bVy/HlbpC6ZAc5Vu9QvZLn15ulDbHfOuXzFQ+hIcrACUviDCuxSP
Q3oBJAIWR+MOQOlLIgStxQBc+mIVHoLruAsufZkZ2MN+x10iLSMwpG1HozPF
2Ws7svnUdZI8AHtpkn7jCTFRLIFEA/Xy+yy0fAlEkfKxBgB7wUbjVBc9u12A
0t3/+XNT4Jhp7FHeLEPBVVTkDUeKRKNg9nKkuYedMdNyo/dYrQzMXs60TKaM
maaobH8BkGkC+4qVYPZyqGSW5ezF7YtfqzKv2ikHsJc1eyfuNrIoZppIHTzH
pg9kYPZyOl9ttbHRN3ipqNS9vPlSd0mhCpuZ65MlEXX24nXAbInlCEwOxREY
TpxoiiMwmSmqUOh++1sGAo+SiN/+1R4GXF8SPqfr8i4GSl+SiMtdsY+e68uz
Al8AIWUrkenuc1z6kj3HXX7POriOu0Ra3Bm++uqv4ulr4LYjlx9OS8uzisZD
AuU9cNuRygbTS5lI8WRcpOlmoeVLOBZpU/LBdrulkho7hXo2GqK+/0vN2hOP
HwHYy76mMlZFHQXRWJlYvrm8aSXGTM85kDMtk2D2cqxWprRmWreZvS8AMk3g
UMksKXvBpmXpBKCICnvBHn1u2Mvh40oVi32DcVFkLzcvzHicbOdIsWii7c7U
zl/Wgi1fig4OwGUvuDgCk0P9x4cX/vRNPxsGR2A4rTCxjMAAJAGqrq4OPCEb
kp87N/t//k9K/DLzn/8pbWqCtZCs7uHcW79OyV40a345cetbWAvJ6+8ad//f
lOwF+3fFg0JYC008rjSf+21K9mLO+7/Kh2fSPSDRBTA6LFTXfUFU+qJ5uE06
2Jvxq/m7xgbbZlu2EOGXmbZPpMN8WGs9Vf8Daec6QtNd/hGJGOpyEsnQUAO/
dytx6cub/SJolyKuxbf/xg9fIGIvtfz/rmnJh7vuUvEGq2737iZiLzd7tjeK
ro1IhrN3AotqG2go6DkBajvqK3gy3MfAmSzTA0HtXzsuAdhL3uOr7QO87J0A
lf2/WdT3VWsV2G73alfTk+HB7J1nWhoeGbnV2ftBeSMoZvpB0x1eDv7ipGoV
iP+nHJRzhI0dt5/U9sB542TjC0BLj2T/DRB4wcbWi6MVbdBXhilhv+TcTel/
fQLqOcIe/erKGP/JaK5PlkQ3vxO/8xcxmL1s2yWurhnJ9ZmSSPhk5PxXwrd+
0wWOOjq2W9DVycQnS4a6dfLxtn+vArOXvA9betoHsnQCpG9/KrcYHIGhJ47A
cOJEU5wTLy3NR6O2zz8n6jxynDiBTYC1lrfsGlHpi2Xvxph+DsoqYNNdx9nN
cQs0X9NAXzGs0pdFpbwAUNuErXoTUeARxNIXTH51g6b+dWZKX5JJFFD6MtGz
AbrjLraiWlWaPcddIsXiQXDpC/ZoBPVm6e0fiDhbZQWA0hfsUWxONpZeJmfQ
fl9ynYWWLxO2mTO91wHs5WT3FalZke3TAF8AloD3+gAPzF6wCc4QW8oYnnrU
KFSfVreD2EtlS4N8BpuZ65NdLip1L7vuiwfVMN84cHcAgyNysmIOzF4+vqIQ
TXkhLgpdSChxt9G6Zu8EuO7lWpXZ7WN7ewuVupf9h6fHxuGY4GVPVOpesHHq
C5lBk612Wojqe6Te+3IDmL1c2tGnnfRk9TQyeftzBCZDcQSGEyeaykoadZoV
MyuNwKBqtf4Pf0jJXjS//nWoL1P30UUl3E7L3o1E+MVzu2AehcN54hYNwHTX
3wSxl8rlvvcpEX4J9BXTPvKzF0Bg7B5R6Yu99gOIgUeJiNsqPMSY60vQq5h6
8iEgbDoRo2+nkFI+3/TgwCdZddxNKb1N2CzaRsReeOJDi21H2Xj7j+pai/mf
ELGXqsFjehcTPiGReJg30wBgLxd7jw3psvL/Hyyd18Sw3S5ARBeALxIqlwrB
7OWCoJVVMdM8pQbMXjaXN1WOTgZR1pmL2v2RCx1TYPayvXSoW2GFvjSsHcDm
RS/W68DsZUvBxGMpiy6YZ4XG5iva7GD2go38EgP72Yuw37N99wSYvWATqmtG
cn2mJKLIXv76mVStZAsKBogl7AUXR2ByqP/44NLqc4NsGByB4bTCRJXALIEq
aREYSk1IK4nABJqb1T/7WbbzpjGFRF36dS8Rme5GZNCKHIL8aiLTXevB36Nz
47AWiqr6bQV/TMleHDc2oMaM7meXXgCJkMvZspsIv/iHi+YT0EweQtYhXcu6
lOxFU/863NIXTJbZMiZLXzBp5soZcNxdpgjqfTKeB0g7mjG0LJ0P9+1v86mr
Bo8RsZcSwe5RXWsiycT9r8TYD06a5s00MG/54g77SiU1pHa7gShkEgjQsxdA
KIY2KSSkMdMTVgNjJ0kqvkq3p7YTbLdbJBr1hiH7GGcuuz9SyFOC2cvW2wON
UiOayEqSS+Y7gMERIWUvG7+RtwxD+4jPhtDYfA3P8c4BBSl7sTrZ4nREJIrs
BZu2wII0TIAospfD2yXjI0zwikxE0e/l9EaeYgB+TSyROAKTQ3EEhhMnmqLc
L5ROY1G6NjArg8DMR6OOEyeIOo/chYXQFkKj7hvnAKa7sPKmk6GAp/gIUekL
9lASXll+oOcaUemLr+lUMpLpQosXQMQwSGS6ay1fgz2a8Uv5u5IxxDFyjqj0
xdy7MxaAVmaDCQ1bZwb2EOEX7Vge9NKXUNAgEX/BjOPuUmktvQ1PthLhlyfj
eRF0efE/rLc/qeNuq6wghDIR9UuaNF0pLXYG7QycyVKFY5HGqa5c2e0CtPQC
oBgzLdJly7eZhvo1RlL2crF3yORj3Y/jVNjLppuiikFtltgLrkx2ALUlRNpz
tPEbeV2/HY2zNwiYIns5fUv/nLEXXOwkMD9C9nLirQ4m2QsujsDkUByB4cSJ
piiDlTQ6i9IGMKmdeNN4Fdn//I0ZjYY1a1Kyl7kXXwwPQruvj1tNRJ1HutUv
BJqroC1E3HkEN286ibhcJR8R5U2HxpqhrIJdAPMJ1Cs8z1jetL79PSL84p64
BWshXC5jx3jXWpYEHmH/jj0KfcUFCqUvWktvyidCefurbENlwv2AtCNm2o6Q
aKB5shLAXm7256udWWFfAMWTcYFm+DivAMBeCoR31C6Y1JG6Fi8A3qwcHDO9
r6kMm8Me+xSx3nykuQfMXk53CmcdMDsZoYg6ewlGs97qQm8HoMJe3j4ju9dj
QSJsuWCeFUX2cvyaVqVnul4uXdFgL7jYRmB+hOzl6OpWcWduSgpp/PUzv+ng
hIsjMJw40RT10pYlMylbxlBzgWF9DQzy+PHciy+mxC/G9euZ6Twyf/o2LNPd
pwsNt1v2vcKA6e7TzqPzrxF1HtEz3U0pmagNYLqLyB/CWgiTd7qcyHTX0Lk5
4pqEuFYihmjH8ohKX9QjR2NRyLdmkbANXPoSCmblWxaN0pdFZfj2J3XcHVLX
RONMZJuKNN0Xe48BkqaxCcw4qyyV1Kxgg90uQNgFINLNHO0ARR3taiipmxCz
J2aaIntRWJmwek5LnhB6o3eWJewFV7o7wPPBXnCv3R85e8HFHgLDsRfmxdXA
5FC/3HrpjTNDbBgcgeG0wpRGc9FSskJc27K0VoYqgGE1gXHm5RF1HmEPQVwI
0HnkunwSlunufCzqe3ieqPMIewibAGWhBbLOo/k4tFshRP7Q+N1/psQvtupN
MdcsrIWSMQRguusYOZeE2goU9ComejakZC+yzjfs2hqIa+GymHkC/rtE+EWt
Kk0m4d/ABiN2QOlLs2ibiczfJpO3P9hxt06SZ/OpaR+cunQe1c3+fEDpS738
PhJluglF7dJd6S9jid0ukaRm7ZHmcrDdbrlU6IswwdCoSGF1nu4UrlD2clug
Xn/9CYC9YI8yyV5wUd8Bnif2Quq1u/+Cmv3shUrOETYBmwY4CBsIzPPEXoJ+
tObbcSp+L2N8c65PliMwuRRHYDhxoima9i4/+ckrKae3f7qEv1AGMCwlMAmH
w7h+PVHnEfL4MbSF3E7rwa0p2Yt+3UvBXmihyQmn2XF2M5HpbmRCBGshZjqP
ni6EIq72L4g6j7zC8xBNd8MOKZHprrZxNWLohrUQLvPMbaLSl2nRJxFED3c5
FPXIpCeYd32Z1tXV9W0ChE3HKDj90nv7U3HcpXHYdIVEA49kJWxLmmah3e6z
UjosFwStKyhmmgp7OdLcI9bn/qZmmSiyF2wONpP506OyA8g0gUMlsyudvbh9
cYrsZUyZy/cmqVA0WVNvzZy94MotgUGjyYclmueJvWz7eTWrvHbB4ghMDsUR
GE6caCodAvPP9S04hVH948ZApfpn+kLZA+apWEhgwoODgM6jmNEIa6GIbNjw
7qsMdB6FpT2AzqOED9rvrYx1HkUMg9byNSnZi+XuqrBWAGshTB5FCWN50xFE
DzDdNc/chl6IkpPSl0DI1CP5ElD6YnVJKR4q3bc/Sxx3n5qrqDvyuw+zKmka
t9sFs5ec2O0ulcHruizqALMXbAJ7Yqa1bt+5rn6OvWRJ4B3guWEv16rMq3bK
wezl8LcalrMXBEnU1Fvf+YsUCnvBlSsC8/zVvaws9oKLIzA51P+3ueCPfxtm
w+AIDKcVprQIzLMMhlhp8Bf2OfG6CwuJOo8cJ07MR6H16XjLrjHQeYTJX1dI
1HmEPQRrlfk4ykzn0XwC9Q8XEZW+OFt2QzTdTUTc5t6dRPjFoyiBtRAugOlu
NvKm4/HglKKAiL0IBe+zufRlUWm9/VniuKu0y68J/wYofWmfqmG47SiejHer
+1lrt4vLGQpcH+CB2cvZ3kalw5LDk1wqrdt3sXcIzF4ONHRx7CUTEe0APyr2
wn6/FwRJ3C03rdkggchecDFPYGzm8M0LM2++1P3jYS+XdvTNjrKuNXKBc+LN
qTgCw4kTTaVJYBaW9SIRiXL/Ea7MN0BYn79Jv9/0/vtEnUeBZnjtM4ifqPNI
t/oFmJ1HPidR55Fl3ysQO48SXjMznUeJgMXRuIMIvwTG7sFaCFPIOqRtXJ2S
veha1kE33dWMfgUw3YWeN+1yjQgF7xPhlylFQTwdDEJR4NKXjqF9Dm/a/1cp
vv1Z4rjrDNorpcUA9lI6XGjyMU052G+364uESiV9YPZyqrtOatbm8CSXSmF1
krKXPbWdfFUuiVZK2f2RG72zK4K94Hp2B3g+2IvVieaXGJ4D9uL2xK7d0q9a
OwKdveBiksAYNMFLX02CwctKYS8OI1KZP0aFvWgn2ftauBqYHIojMJw40VT6
BGZhQdUOrIRJq/oFF0sITEQq1fz61ynxi2HNGlQNzZAzqpARdR6ZPvoTxM6j
yIQI1HnkhPara2Saz0znUVgrsNxdRWS6i9omYC00n0BdsquAziO4pruIewxg
uuvUN0Fca4FC6YvLNQJ3RUyJZAxc+iKfe4DNoXFkKm9/NjjuRuLhXlUrgL1c
FnwtM0MucyIV++12fZHQw/HBXQ0lAPZytKNKpJvJ1RkuExW/F3ayF6MnRJox
zSr2gmvpDiCe9ZOyl43fyOv67Wg8mcNzBgtnL2Dwgg1sDjYz1ycLktUWzb+k
IWUvW3fI6bEXXMwQmPERz6kvZM8He9FOeooODoDBC/vZCy5R90Drya4ZPp1v
zhyByVAcgeHEiaboEJjvpVK1F376ylIS88ornxb+4AuTlthAYLwlJUSdR/bD
hyF2Hvkqi4k6jxx5B5OIH9ZC/qYiBjKP5uOo/3EBUeeRv+0cxM4j30AhUemL
uu6LJAoNicQCOqLOI039677ZalgLYUomUYDp7szAHuimuz7fdL/oQ4ZLX1z+
mY6hfYDSF2wC7YOD3/42n/qR+OucO+5OWqWXBV8D8EuvqjUSZ/SHbIp2u+FY
hMmzWqpQDG1SSPY1lQHYy8HWByU9bbEEKyoZqLCXT6vbWche5hzImZZJMHvB
RiFPySr2ggvfAR5LXTtvTK909qK3RJ8P9qJSh/IvacDgBZwxTV3ZJjD9PfbD
2yXPB3tRDNjyt/aQspeigwPsZy8eg6/hy86//u9vvvxf567+kU5LOEdgMtQv
/vvbVafEbBgcgeHEiYZyS2CSfr91586U7EX9s5/5a6Bl/iYRv/2rPUSdR4Hm
KlgLJXxO54VtRJ1HYWkPtIWIO49s518LT0CLi4p7dUSdR5a7q0LKVohfwALa
dqLOI0Pn5qgXWrb1AuOmu9jR1KpSIvYi4L/rsA9AXA5XIhmTzz0gYi/g0hf+
P6TVagFLEP31scMOqWty7rhrC5hKhwsB7KVSWuwM2rN9GksViCJU7Hbd4az/
zyESFfaCPYrNwWayIYt2zGSjwl7aFCqWwKJFyY3eY7UyKuzF7s8ZiwMIjSdv
1Ei2FEysdPYilPr2X1CDwcuqnfJrVWaWsxfFNHL89Cwpe9l/eHpsHM5PTtnb
AR43mre/1U/KXk59IVMr2ZK5RqT+Ju3R1a1U2IvDyGozZ0yWSVvV7sYv/9e5
pUPRkfbvOByByVAcgeHEaUUrhwQmqlDofvvblPhF/4c/wO08Mv7360SdR6hq
CtpCSrH14O9T4hf7qffiFg2shQCdR87i9yF2HoXUPEDnUdz79NdkKF/A5hOo
Y+QcUecR9hDcziOnvknW+QaR6S7iHoO4FqZAYG5w4BMi/CKTnkBR+D94gUtf
eiRfehHQBbm4M5w8eRIwLeVfX++S59xxNxIPt0/VgNuOlHYmjH8XRcVut2jo
gc5rYvKslooKe9nVUPJwfBCbiT8ltwRGrDcfae5ZiexlUO1c0ewFiSTu9Vg2
fiNf0ewFjc13iNybjkxRYS9uX86aAalobNy///A0KXs5fnpWMQ3zwxT6DoBG
k/Xluq2rhaTs5dJXkzYzq014YtEEr3xm78sNzwd7meHPffduxTL2gg8aZTAc
gclQHIHhxGlFK6WfTFpHoPf563vwQP2zn6XEL7bPP0/6oTUEBZqrdKtfSIlf
7F/tgdh5FGi7Q9R55C3PW4mdR17heaLOI29fHjYBn5n5F7Cod9bQuTkle9E2
rg5o0zOXBisRQ9QjR4lKXzSjX0E33dVpq/k964hKXyxmHtzlFshKX+r6Nk3r
6kgPQo/AhFAfb/Jmzh13Jcb+i73HiNhLfvdhgbqDYXMVqVlxsvsKgL2c6b0+
YcuZm0oskeDNyg+2PgDb7ZZK+nyRf/rz5YrArFz2Ipixf14hIfV7udE7y072
4kZiRe3GP58aW9HsBQkl7jZa3zmgALOXNXsnimutLGcvPXzXZ/sVpOwl/5JG
pYa/90LcAfCA6U2vC8DgZf2rvTcvzHic0PrTsyE85GjnL2vB4GXLTyvvHBtm
P3uRNSqK1t5LyV7SLYPJ/KaDE65f/OXyH76WsGFwBIYTJxpivgZmPhq1Hz5M
1Hnke/CA9pksUxLxO/IOEhm/+GvLoC0UCrgu70rJXsy7fhUSNcJaKO7UOovf
Z6bzyF77AWHnkfqfoEGGX8D86gZN/esp8Yu5d2csANO0we8YJjLdHe9a6zJ2
QFwLUyhokIi/YFXpy5PxvGCEUtMNDQIzbRGWCHbn1nHX5NN9N3gRUPrySFbi
DbuzfRpLpXbpCoR3wFFHAg3TJsCLosherg/wnKEUpf7ME5iVy166FdbtpUNg
9rLppohtXruLMjgiF+t1YPCCjS0FE4+lLtayF6sTLbhvIg05WrN34m6jFQmx
6xJaKgRJ1NRbN30go8JerLZs8QooO4DHGaUSML3pdcH9IjUSYDUQcxiRe6ck
W35aCWYvO39ZW/PtuNfB6hqeiD/Cvz5w/jc3wOwFGzUHWj2G9DpnOQKToTgC
w4nTihbDBAZVq/V/+ENK/KL77W8jUint01immH7O9NGfUrIXw7uvRhUyWAuh
c+NEnUe2E+sgdh6FJx4z1HmkbCXqPLLXfoB3Hi0V7S9gyRhiGzhG1Hnkkl1d
LLPJXMyb7lrMPAH/3ZTshd+zzqBvgLvcAlnpS8OTrVpLL/WjpUVgPCFLk/Qb
IvZSzP+EAcddJBponqwEsJeb/flq53S2T2OpTH7bd+KHAPZypON8m5KfK7td
6uzF4HURHYRJAsNX6fbUdq449oImko1S49bbA6TspWJQG4yy8e5SpgmcrJgj
ZS87b0yLpry5PllCKdSh49e0pEa77xxQVLTZ2cxerLbotVv6NRskYPCyau0I
Ns3toRNyR10Z7gBqZeDSV5Ok7GXramF9uQ6NshTr4aIYcrT35QZe+Uwsyt4L
bOF7o93Wk10n/uUCGLz89X9/0/BlZ7rsBRdHYDIUR2A4cVrRYpLABJqb5158
MSV+se7cCbHzCOmsJ+o8sh7cCrHzCOGVm3f9KiV+8RQfSab6yZiG5uOor+kU
Q51HfXmEnUfC8ymRCL0vYBHXpL79PaLOo5B1KONXs2QtRD/15EOA6S7EtTCh
qEcmPUFU+iIRfxEKGuCuuECh9CWCpndnRJHAjEiGwWHTrbKCQMSZ0WujIJGm
G9B2hD2ETcj2OSyVO+yrGm8B2+1iEwLRnBWfi3QzRzuqwOzlsqgDwF5wMUNg
qLAXbALb2EswGq8Y1JKyF2xCo9SIJth4dyma8pIGTGPj2D2VTMNeN1Sh1PfZ
WRUpe9l6XNkhcqOx+VyfL6HGxv1UjHbXbJDcLTdlm73gor0D8Dus+7cMk5q9
bH+r/3GjGe45QxfFkKOjq1v7m7S5PlkSWSZtNQdaSYteTv/rt10FT0Td9BME
OAKToTgCw4nTihYzBGY+GnWcOEEUOe0toZNkl3ohNOo8/yVR55G37BqshZKh
gPv6fqLOoyAfWmhy3Kl13NjATOeRrXoTxc6jpaLxBcw7XU5U+mLp25eIwGwS
sakrAKa7Qa8C4lqYXK4RotIXbKhVpXAjlhZgl74sigqBsfnUd/kHAI670xYh
/RdGTTqP6mZ/PqD0pV5+H4kyd28YjkUap7qOdJwH2+2a/DbGTmmZqLCXC4JW
pcNC5WjZJjAU2QvbMqbt/khZv2bTTRGYvWwvHWqXm9nJXh5LXR9fUZCyly9v
j6otWXd2oic0Nt/Ed5Ea7WJj/wW1UJqz9DEq6uA5tu+eIGUv7/xFWvHQjCDM
cch0dwAkEH9YoqFitLt/y3B/D6NBdekqFk30PVJTCTnK39qjGMjZnk9RAKPd
peP8b26Ibosj37tUZbL/cwQmQ/1iUyHp/szM4AgMJ040xACBiRmNxvXrU7IX
za9/DbfzyPzp2ynZi37dS2HxE1gLoXPjtqNvpO48OvoG9iishYIjtdazLzPT
eWS+8/9S4hdH445EAHQvltZHcCLitgoPEeEXjwIai1t4+u3IDTDd1Y7lwTXd
jceDU4oCIvYyOPCJzwe/BQZ66cuiwAQmGg+JZisBjru9U3ey7biLRAOPZCUA
9vLd4EWTj7kbcypRRwXCO9OOrJvhEAkue8GVJQITRGNtCtVKZC9Kq/9CxxRp
yNGu++JuhTXXJ5tCSCRRJbCSBkz/+dRYUbvR5mVFHPmzcvvidxuta/ZOkLKX
07f0Kj17vTgQJFHx0PzOX6Sk7GXrDnkHz4GiTNM86heAQRO8mjdF2nCEB0yP
j8A3SYMohxGpzB/b9vNqKiFH2klWv5aIPyK6LaZi9nL1jyWS6n/6fssRmByK
IzCcOK1oZZvAhPr6iDqPTO+/n3A4aK+7TMHeVv26l1LiF8vejQk3tD6IQNsd
os4j9/X9sDqPkpGAt+4YYefR4wJYnUdJFAF0HvkGCknNWKh/BIesQ9rG1SnZ
i65lXcQ1mfGr+UFem3C8ay1jprs+37RQ8D4RfplSFKyU0pdFAQgMOGy6avBY
tsOm48m4SNOd330Y0HY0pOvL6jksU79+FBx1hD0qNkJjs+lKatae6q6Dy15w
Qb8B94YjlaOTH1e2rDj2IpixH6qWkrKXzyskg+qs9+XREG60Sxpy9PYZ2b0e
ixv5e4cL2wiM1YnmlxhIjXbxgGlscq7Pl1B6Qzj/kmbV2hFS9nL4uHJ4JGcF
PFQugP4e+18/k5KCFzxgWq1kbzsbpjG++coeISl42fbz6nunJCw32sXNXk7/
67ek7KV088MZ/tyzR+AITA7FHgJz5MiRcQLl+n8SJ07sVVYJjOvCBaLOI+wh
2isu0zwadd84R9R5hD2ETYCyUMLnJMo8wgbCK4eyCibUKAd0HkWm+bAWirlm
AZ1HYa2AykEotaElUMfIOaLSF9vAsSS8cpREDNHLzwNMd9EwzJ+ek0lUrSol
Yi9Cwfsu1wjE5XBlr/RlUSkJTAj1dcivgsOmE8nsmg+ondPXhH8DlL60T9VE
4sx9752wzXwjuAVgL8d5Bd3qfobDrxcl0s2QshdswoSVpjcRxBtwky9QJBoF
gxdsHGnuEetZ5AsRjMYbpUbSkCNsHKuVyY1sNKqlaPaypWDi2YBp9hCY58Zo
d3jEd/i4khS8rFo7kn9Jozfk+B4fcAEggXjLIyOVhqP1r/aWFKpsZvbyilg0
wSufOfBaMyl72fnL2sYbE0E/e+HewvcNR1W7G0nBy4l/uVBzoNUySdg/xRGY
HOrFjYX/9ddRNgyuBoYTJxr6SSqldYSUO3DC4SDqPJp78UXkMTz3EqvJsncj
UedRSNQFa6HIhIgo8wj7d5idR4MVRJ1HrpKPEl5otx5BRT3tzqOlIv0Ijrgm
DZ2bU7IXTf3rvllonjmYgl4FwHTXpq6AuBamQGBucOATIvwyOZEfjwfhrpjt
0pdFPUtgJk294LBpZwByntQyecPuSmkxgL2UDhcy2XakdumKhh6Ao44ap7py
FXVEpefoVHed1KzNZBUoN+AKq/Ni79CKYy92f+S2QE1q9oKNMy2TSis0B3hY
QiKJun47acPRH74POXosTe3JnHMCg5u9bD2uXOlGuyia7OA5tu6QUzF7Ycxo
l1QpLwCbOXw1b2r9q71UjHZbHhnZHHLkMCJ3jg1TaTja+3JD3yM1m0OOAo4g
xXTps7+40lXwBJsPPiCNt3/mNx2ccHEEhhOnFa1s1MAAOo8Ma9bEjMbMTvkH
hcVPiDqPzJ++HdOnqJmkoflY1F9XSFT64rywDWLnkaf6EFHnUaAHno0wini6
vyLqPPIPF6V1NPBHMMB019C5Oeqdzeyl/JMss2VE7GXqyYfQTXd12mp+z7qU
7EXAf9diJvQupi0GSl8WtZTAgMOmSwS7sx02HU/Ge1WtgLajy4KvZebhrJ7D
UtkRV6mkhjTqyB3OQWsAnjHNAHvBleENuFhvPt7Wt+LYC0WzF2wU8pRzjpwl
XhHJ4IgUNhtIG46wcahkVjwLYkc5JDB6S7TgvomK2QvLjXYRJHG33MRmsxeA
ll0AIyLnqS9kVBqO/vqZFJucq9OmojG+mUrCETZOb+SJO+FHHEKUftRUc6CV
NF0aGwW/K5ZUj8ejlIo2uRqYHIojMJw4rWhBJzCAziPHiRPzUTgNQZi8ZdeI
Oo+c57+E1XkUt2gcZzcT4Rd/U3qwAiDUKLcV/JGo8yiq6oe1EDjzKGIYTPeA
RB/B8aDV3LuTCL84Rs6ROsxQFxq2zgzsIcIvevl5uDYsoaBBIv6CqPRFJj2B
opCd9xgrfVnU4s7wyf5NgLDpyicnQ2h2b20mrdLLgq8BpS+8mQbG2o4CUaR2
ogPMXr4TP8xJ1BHOXg62PgCzlxOPH0FhL7jofQPHTpVKyBE2TncKx0wsyhAZ
VDuP1cpIwcumm6Kyfo0nxLpOBPGs/9g9FZXW/rPV2ikDeQlfTghMz7D38Lca
UvDCfqNdxTSyIsxeAMIvADSafNxo3v5WPyl4efOl7qt5U2xuOAr60bY7U3tf
biAFL1t+Wll0cMCsZuPfBVc8GpdUjxf8rpgUvADMXgDiCEwOxREYTpxWtCAS
GEDnkfpnP/PX1EA65YWE20nUeaRb/QLSWQ9roZCo0bLvFQY6jxDhXaLSF/e9
T5NI6vJvGgqM3QN1HoXoLJTyIzigbScy3cX+PWiClkuFyWXsAJjuem2QA5Et
Zh5R3jS/Z51OC7OpCheTpS+LWtwZ1nzwC6KwaZVtKKv3XyafrnS4EMBe7kuu
O4MMxZWGY5E2JR8cM10gvKN25cAeNhRDmxQSUvZytKNKpJuBu3S6F0AQjT0a
m/q0up2UvRQKxFo3W25t0ESyXW6mYvbCzoBpJJJoGXZQaTh6+4zsDs9s81Jl
R0wSGLcvXtFmf+eAghS8sNxoF0ESTa12Kg1HLDF7Aairc7ikUEWl4WjramF9
uQ4J5MYRi4rMat+dY8NbflpJpeGI5WYvDpWr4ctOKi67uNkLNp/GKhyByaFe
fLfwv46OsmFwBIYTJxqCRWAAnUf6P/whqoDWABISdRF1Hpk++hOqmoKySjIU
8JaeICp9gZl5hLjcFfsIO4/6iqGsgikRcrnav4DVebRUyz6CkzEEYLprFR5K
RNwZv5q/KxFDtGN5RKUv6pGjsSi0tRae9ul7ZNITRKUvEvEXoSDkOuRYPCie
vsZk6QuuEOrbunvNmg9+gY0vvl0NCJvO0v0XEg3Uy+8D2MtlwdeTVmhB9mDF
k3GBZhgcM32m97rUDLnNjYpw9rKvqYx59oKL+gXgQEJlYvnm8iYweMEm3B4c
wyZn42xpyBNCy/o1VMxejtXKWBhyZPOiRe3Gt8/ISNnLx1cUj6WuZUa7pGKG
wIwpkdO39FSKXlhutEu96IVVZi8pxe+wUkw4Orxdgk3O9fmC1N+kpd5whE3O
9fmCJGtUfPduBZWil/O/uSG6LSY1ewGIIzA5FEdgOHFa0cp8AxwdHAR0Htk+
/zzph+NAOI9GXZdPEnUeOfIOJhE4C6Fz47YT61KyF/OuXwX50IocUK2EsPOo
4I/Yo7AWihgGLXdXpWQv1vI1UfNoJgdf+hEccU3qWtYRme56p6HFRWFC3GMT
PRtSshdZ5xvQTXcd9gGi0hdsaOZgvjRcepuwWbSN4dIXTOOGxwDH3arBYzaf
enEy9Puvp7hD3QGwfMEe6lW1MhYtJDaOn+m9Do6ZFmiGmY868kVCD8cHSdnL
qe66YYOa/HB0ReUC0Lp9VEKOPq1ufzQ2FUTZcss550AKecqVa/YinvWfrJij
2HAk09D8TSGrBAYJJSi67GLj8LeanmE2Rk0tpFP0wk6zl6UyaIIUXXbffKmb
5enSDiNSmT9GseHozrFhNjccBRzBznP8s7+4kqWGo5TiCEwOxREYTpxWtDLc
AGNGo/KNN4g6j3wPHsA6T1Q1ZfroT0SdR/7aMlgLBdrumHf9KiV+sZ96L27R
QFuo5xph51HFvmQEzpeW+QTq7csjKn1xtuym13m0VPhHMLaQe+IWM6a7ySRq
nrkNMN2NIDBDecClL4MDn/h80xCXwxSM2J+M5zFf+mLzqeskeUTspZj/iVjT
sCxsGu79F6nlS6W0mLG2I9KY6SMd5ztnnzAfdYSzl10NJcx47YIFvgAUVue5
rn5S9rKntpOn1MQSrKhbwBuOdt0Xr1CzFzcSqxJYP76ioNJwVNRupN5wlFJZ
IjDUXXaxOdhMbH42TiNzUS96YbPZy8I/nF72bxmmUvSy6XXB/SK1x8nSP0os
muh7pKZY9LL35Ya2O1NsbjiiGC2NJxx1nuN7DNCuMY7A5FAvvHPl9f+RsmFw
BIYTJxrKZAP019Qw03nkry3TrX4h251HCZ/TdXkXUeeR7+H5+RicrxNJxOUq
+YgIvyDCu1BWwYTaJohMd7ERGLsHZRXsIzgW0AFMd12yqxBNdyOIflr0CRF+
Mc/chmu6a9A3AEpfZmeK4S6HaVpXV9e3ieHSl2g8JJqtJGIv2GiVFXhCKQLK
Yd1/kVq+3OzPVzshky4iTdhmCoR3wHa7tRMdgSjTZQ/OUKBU0gcGL9i4IGhV
OqimyWcooguAr9Idae4hZS/H2/r6NdDS8TKU0RO60TtLpeFo130xC81eRFPe
s9VaKkUvH19RtAw70m04SinoBKZn2Lv/gppK0cvW48omvoud6dJpFb288xdp
canRamMpr6Be9IKNz94bfNzIotiyZdJOeihGS2Mjf2sPmxOOIv7I0P1RKtHS
2Chae09SDc26cFEcgcmhOALDidOK1k9SifRZSb/ffvgwUecR9hCszqMk4rcd
3cFA5lFkQmQ9+PuU7MWy7xXsUSirYAqNNdvOv5aSvdivrEWNclgLBcbuEbEX
W/Um1DYBa6GJ3iua+tdTshddy7qQdQjWQphs6gpZ5xsp2ctEzwa/A2YeMTjw
SCh43+UagbjcwveOuzzxIeZLX1S2oTLhfiL2gjvuEj038/svJBponqwEsJeL
vcdEmu4MV6GoaYealL2USmrs8MyxKYoie7ks6mCMveBadgF4wxGKRrsXe4cU
Vla4pqCJZLfCeqhaSqXh6GSjXKKD6S6VuWxe9A7PvPEbORX2crJiDpwuna5g
ERiVPkyx6AVPOBpTsq7tC5feEC64ql2zQUKx6KWHz/RmQlFIIN7yyEix6AVv
OJoaZ2kBT9CP8spnDrzWTAW8bPt5NcsbjhQdM1W7G6lES5/+128bvuyk57JL
Rem+/VPecXAEhp44AsOJ04oW7Q3Q9P77z7KXuRdfhJh5FJENG959NSV70a97
KdjbCmWV+VjUX1dIVPrivLAt4YNzmwA23fVUH4LVeZQIuRyNO4jwi7cvL4nC
+e6aiLitwkMA091kDNqX5AiiB+RNq0eOJuCtlUyimrlyfs86IvwypSiIx+mb
1z2rWDwICJvGhnj6WjZKXzwhS6usAFD6IpqtxB13iZTJ/Vc8GRdpui/2HgPg
l/apGiTKhI2A2qUrGnoAZi/YBJ3XxMDJLJXB67o+wCNlL9gcbCbD57aw5AKY
dbgLBWJS8IKNItGoyccKawjqRS/rrz8p5Cmx+bk+5X9Sr9xNMVoaSsNRSmVI
YNy+OHWnl3cOKO42WrGnwDp5iELRZAfP8dl+xXNQ9DI17rv01eSbL3VTYS+b
3+hpeWRkbcLRGN98ZY+QCnjBG4545TOsbThyqFytJ7soOr0U/K546P5oxJ/d
JlmuBiaHeuHtK68flLJhcASGEycaor0BJhwOza9/vRS/GNasQdXQLB89twuI
Sl8sezcm3HCoSNyicZzdTIRfAm13oKyCKarqJyp9sZ59OTgIzTYWYLqL/XtI
zYO1UMg6BDDdDWjbYS2Eu74Qlb5g/+4ydsBaC5PPNz048AmTpS9WlxTguNs2
sAubAHdFTIlkTKxpKOZ/QsRe6iR5Sx13iUT7C5jSLr8m/Bs4adoWYAJ3UGEv
BcI7E7asZAkBpHRYqLCXUklfTtgLruGREb5Kd6ChixS8fFzZUjk66Q0zbZuT
UoIZO8Wil623ByoGtawyezE4IoXNBirxRtjYUjDRMuxAItny2KG9A/QMeynG
G2Fj/wU1a112x8b9+Zc0z03Ry/a3+tMqemEyjpy6qFvs4uPKHuEYn6XNUwFH
cOj+aMHviqmAF2zUHGjVjzL0SwFHYHIojsBw4rSilckGiDx+vIhfHCdOzEfh
/KAT089Z9m4kwi/esmtQVll4GmzdaNn3Skr2Yjv6BjoHp2c2GQn4mk4Rlb44
bmyIWeHc2SVRBGC662jckbnpLq75BOqUXiIqfTH37owFdFAWwhT0KqaefEhU
+jIzsAei6W48HlROXyNiL9kofYmg3v6JbwClL/K5B8ucb6FI75JXDR4jYi8l
gt3jhscUD0XjC5gtYLovuQ5gL9eEf1PaobXjAWTy274TP2Qhexk2qE9111Fh
L85QzopJHEiocnTyg/JGKiFHbQoVG0KO7P5IWb9m6+0BKuzlULW0W2Flj9kL
Gk8+lrr2FSupgBc84Qhuw1FKpbsD4Ba77xxQUAEvq3bKWeuyqzeEi0uNmz6Q
PQdFL+MjHupFL9vf6l9a9MIqApOWxS42DrzWzCuf8TrCuT7x1JI1Ksq311IE
L+d/c4N/fSCTaGka4ghMDsURGE6cVrQy3ACdeXmz//ZvyGOq92uk8lUWE5nu
Gv/79ahCBmWVZCjgLT1BVPriKT6ShHRf87T0hSBvGhu+plOwOo8ihkFr+Zps
m+4ufJ83bejcTIRf3BO3YJnuggOPcNNdKAvhcrlGhIL3mQw8Ups6G55sJWIv
PZIvAyH4P2OFUB9v8iag7Qh7FJtD/YBpfQFDooH2qRpSyxcGkp1NfluppAbM
Xs70XpeaodmJU1EohvJm5Uc7qsDgZVdDycPxwRyylzGT7WLvEJWGo+NtfXwV
NB6biQQz9pONcirgZdNN0Y3eWVY1HE0ZghfrdX8+NUax6KVKYHUjDPEuijtA
WrnSiy672LOyff7pyu2JNbXat++eoAJesPHZfkUHz5Hrs04ttTJw88LMptcF
VMDL+ld7r+ZNPRstzRICk5bFLu70MjvKCh+qZ2WZtDV82Xn6X7+lyF5gRUvT
EEdgciiOwHDitKKV4QY4H41Ku+GYZMatJuvBrUSlL468g0kEzs95YNPdkKgR
yirzcRSQN20r+GNU1Q9lIXDpC0TT3WQMccmuErEXXcu6iGsSykILZKUv2EPY
BFhroahHPv43QOmLWlUKN/AoEDL1SL4EOO7OGFogLreoccPjEsFuIvZSNXhM
70q78oTiFzD2WL7YERcV9iI2wo+NAMgXCdVNiPc1lVFhL9hkJs9tUUE0xlNq
9tR2UjR7mXXk3rF2RRe9IJFEy7CDSq40Nv58auxivU6mYZrLke4AQqmPerfR
mr0T+SUGhZpF+AsXbvNy/PQsRfCyZoOk4KpWb2BjcYXNHL5fpKbYbYSN/VuG
Hzea0Wjq90VuCYzDiLTdmaJosYvHG/U9UseirCN7C993G/GvD1DvNsJmMl/0
skwcgcmhXlh35ff7pWwYHIHhxImGMt8AoXz+AvKm9eteQjrrM19igaz0xXF2
c9yigbIQapQ7bmwgwi/eumPMlL5ANN0FuL5gwzFyDpbpLrj0Rdb5hk0NzTMH
k8XMA4RNS8RfhIIwwygTyRjYcTdLYdN6l/yR+Gsi9lLM/0SsaaDX7kTl7U9q
+VI6XMiA5Ys77KsabwGzl5PdV/r1o9k+k6UyeF1UQo72NZU1KSShWG7cSEy+
wO3BsY8rW6g0HDXIZ9hg9jKodl7omKICXljosivTBM5WaykWveDR0tlzegGL
aAfQW6LXqswUu41wp5cOkZuF0dLDI778S5pVa0fSKnpBUbZwvEV5nNGWR8bP
3hukCF7woheDhuQGPycEBgcvR1e3UrfYrcwfw57F/KmSKh6NS6rHSzc/pAhe
8Hgjy6Qt1yf+VByByaE4AsOJ04pWzglMTD8HyJu27N2ITcjk+IsClL5gw19X
OB+D0KNNUvpy/rXQWHPmqyx8H3jkfnyUiL1Yy9dEDINwFoq47UMnidiLtnG1
nF8EZSFMfscwY64v4LBpAf9dnbYa1lq4rC5px9A+IvbSLNqWDcddZ0APTjvC
HvWE6OcXg9/+Oo+qdLgQwF4uC76etMJ/1ctEkb0INMMMNEAtasJquCzqIGUv
RzuqeLPyXLEXsd58rqufYsNRv8aYk5NcqjkHcqN3lmLRy6774na5ORhlS4yL
wRG5wzNvKZigXvQyZcjlL+ALz+wAeLfR9lMzFMHLpiNTdxutVieLjI5x6Q3h
a7f07/xFShG8bPpAdrfcxEKnFyQQf9xoPrxdQhG8kBa9LBOTBMbrCPPKZ05v
5FEELyy32J3hz9UcaKXebVS1u1HWyGhjLKk4ApNDcQSGE6cVrdwSGIDrCzaw
R2kfeakSPqen+AgRe7Ee/H1kQgRlIVQrAZS+uCv2JRE4XrghZStR4BHc0peA
tl3buBpguhsPWqF8AUvEEL38PGOlLzptNSBsWiY9gaIeiMsFI3ZSx90YVI/f
he8tXwTKewD2Uibcr7INZbgK0V/fFjBVSosB7CW/+7BA3ZFt4uEO+xqnusDs
5TivgGH2ItLNnHj8iJS9nOquw2YydlZLhbvsflrdTgpe3r/XUCQa1brT8A7K
huz+SM2Iftd9MfWiF6U16y61FGXzolUCK8VuI2zsK1Y+lrrQOCuqLPAdAAkl
OkTu49e0FMHLqp3y/BLDmJJ1NQluT6ym3rp1hzytbiPFNOteCBpN8jus547I
qYOXTa8LSgpVNnN6nVMMEJigH+17pE4LvOAWu+zMlVZ0zKQFXvBc6dx2GxGJ
IzA51Atrr/x+3xgbBkdgOHGioVwRmKhCBgg8Mn30J1imu4DAI2x4S09AMd0F
Bx7BLX1xtX/BQOlLLKCzCg8BSl/86gZ8ZuZfwFzGjvGutcyUvpCGTdusfbDW
whSLB+VzD+r6NhGxF574kMsP+S47Gg+Bk6axIZqtxKZlvtazf31v2P1IVgJg
L9iol9/PtuULlbqX47yCztkn4RhDLTOhGNqmHDvY+oCUvVwWdSgd9AuTaCuW
SPRrjBSLXvbUdjbIZ0TDYubPc1FoItmtsFK02GVb0YsbibUMO6hnG719RlbY
bDA4ct/htSgklLj5YJQ6eMHGZ2dVHSI32yx2cX9d6jYv2Didr2ZnrvSIyHnp
q8n1r/ZSBC9vvtR97oi8v8dOb7nsERgcvFza0UcdvOAWu9pJmD+gQFE8Gk8X
vJz9xZXWk10OFRuvsUVxBCaH4ggMJ04rWj9JpbSOkO4OnET8ntsFROwFG9ij
8yiEUt6Ez+m6vIuB0hdw4JGr5KO4U5v5KvMJNDB2z3zn/zFQ+uKdLtfUv06E
X6zCQ4nIDx6bmXwEh/0q9chRZkpfUNQzpShgMmxabepsFm0jYi91fZumdXUQ
l8M1aeotE+4HsJc6SZ7NpyY9zsl/iM/nA6Yt/esj0UDzZCWYvZQOF5p82Q3H
oZJzxDB7cYYCD8cHdzWUUAmYtgTgGwGRirrTCzbOdfUvNhzlyodTonMX8pTr
rz+hyF7YU/SCRBKPpa5j91QUwQs2sMnsKXpZSL/iBRvvHFAU17Ku28hqiza1
2vcfnqYOXrbvnsCe4vbkPlp9mdTKwNW8KYrBRvg49YWMercRkaDvALFoor9J
e2WPcMtPK6mzF3Za7MajcVmjomp344l/uUARvODdRoqO3FQ/pisaf/3Mbzo4
4eIIDCdOK1oM18CERF2Gd19loPQl0HaHidIXxOWpPkTEXqxnX0aEdzNfZeF7
x11b9SYGSl/CDikgbFrXsi5oerLsKfS+gCViiGnqBiBsWjV8AFbpSzKJaubK
AW1HgwOfeDwws28c3kme+BDYcTcYofmbI5H0LnnV4DFw29GkqZfi0RZ3hpMn
TwKm4X/9SDzMm2nI7z4MYC/XhH/LtuWL2qUrGnoAZi9HOs43TnUFogx1DVA0
e9nXVFY3IWY+5AiPNzrS3EMFvGwub7o9OGby/dPOyTCBMXpCtwVqijYv2Nhe
OlQzoveEcn/jj8aTvXI3dX9dbGz8Rn6HZ7Z5c3/yuGiAl1U75adv6Ycncpac
nlJ6Q7im3ko9URq3eSkuNbIw26i/x54ueDm8XdLyyIgE4FSCQdwBxJ2GdMHL
6Y08XvmM18Guvws98FK09t7Q/dGIn0VFbqTiamByqOeewFitY+0lp0+f/nRR
2H+UtI9Zs7EYJ06MizECE7eaAI67EEtf0Llx+6n3GCh9CQ5W2M6/lu3SF7Dj
LsTSF7DjLiDwiMZHsMvYMdGzgYi9jHetxSZk/opw2ax9QsH7jIVNR1Av2PKl
WbRNbxPCWg6XM6Bvkn4DYC942lFabUcUCczgyIBA3QGOmb4s+HpI15dVo5UJ
28yV/jL2sBdfJNSmHDvaUUXRaDeWYPqnW4XVWSQa3VzeRLHhqE2hCqIpfvdn
hsB4Qmij1EjR5gUbm26KbvTOsqToRTzrv1ive/uMjHq3UU5ypYlEA7zg3UZN
fBeruo1U6tDdchN1jxfc5iX/kmZsnBUX0qJwc91zR+RvvtRNHbxsf6u/vlyX
rs0LqTLcAfBWoyt7hNt+Xk0dvJx4q+O5AS/nf3Oj8xyf5d1GROIITA7173++
8truMTaMLBAYa/sS8PKMTpeMwV6QEyfGxQyBATvumj99G0rpSzIU8D08T8Re
IJa+oFqJs/j9bLu+kLYd2SrfgVX64p0uBzjuGjo3h6yElq1pfQQHvYqZgT2A
0he9/HwCUrC1zzcNSDvCw6YDAThJWwvfJ01Pah4CLF+wh6A77oZQH2/yJoC9
YEOgvIdNS/fIpAQmnowP6foudIPYy8XeY9m22xUbx78R3GIPe5lz26mkS+NG
u8MG8nYwuPKGIw3ymT21nRSLXopEowqrE3DArBKYdG1esHGmZVIwY8eemL2z
oijxrL+w2bDxGzlF8PLnU2Nnq7WiqRz0oKUUPfCy9bjybqNVb2FRJJBKHbp2
S7/pAxl18IKNw8eVPXwXq0KlbeZwfbkurVQjbGxdLbxfpCYNlaYtejuAWe1r
uzN14q0O6tQFG0dXt2LPYluoNG3w0nqyiyWh0rTFEZgc6vklMGD88ndxEIbT
Sle2CUxUITN99Cci9qJb/QKswCOEVw4Im4ZV+pLwmgFtRxADj8BtR9jwDxfN
JyBUboQdUlPXx4DSF/fELfBCFD+C0bBVO5YHYC9TTz5E3HC2VBT1TE7kA9gL
v2cd3LBpraW3bWAXoPSlf+IbuG1H0XhoSF0DttttlRU4AzQ7ucAERmYeviz4
Ghx11KtqjcSz9RtlPBnv14+e6b1O6vfSpuQzwF5CMZQ/N0Ul4Qgb1wd4zBvt
9muMhQIxFfCCjQMNXTylJmXRyzJlicAMqp2FPOWmmyKK4OXzCkm73JzzbiMk
ksBbjahXvGDjZMUc9iyW2LwgoUTPsPc5AC9j4/6Cq1rqcdL42H94mm02L2pl
oKRQtf2t/rTAy6bXBVfzprDnZvv00toBxvjme6cke19uSAu8HHitufHGBNvA
i0PlEt0Wl25++CMEL4viCEwO9bwSmLGSfyp2sf7QdPR9VxLHYDg9L8oegUm4
nY68g4C2I9vRHdgc2usuCtx29LT0pTwv89KXZCTgf1xgPfsyYelLwR/DE48z
fzmkbUfOlt1xLwQv01hAZxs4BmAv5t6dEdck6XFIP4JxyxdZ5xtgx10orUCk
li/YmJzIhxg2bXIMdwztA7AXnviQw0v+vzEtjRseg+12H4m/NnumM1mCiMAo
7fKb/flgu13eTEP2oo7CsUi3uv9k9xUwe8EmYNMY8Nq1BLzlUuG+pjJS8LKr
oYR5o10HEno0NkUlVxobH1e2POv0AhZEAoMmkoIZ+4WOKergZXvpUFm/xuhh
2j9nmWxetGXYkZa5LjYOlcw+lrqQCCv6dPSWaA3Psf+COl3wcu6mlFXgRdjv
yb+kWbNBkhZ4OX56toPnYBV4oWHwgo31r/Ze+mpyRAThmxVFke4A9PqMcPBS
8+24WZ3jpPulwiONGr7sPP+bG9SpC54o3XmO/9yAl0VxBCaH+vc/XXlt1xgb
BlQC8wOAKUnp+LKkQOZ0O2cJw2kFKxsEZh6NesuuAdqODO++GhYvN3SloYTT
7L6+H8BeHGc3o3MQHFbBli/Y8LedS0Yyhjwo4pfcBrQdWcvXhJStmb+cZAxx
ya4C2MvSsGlSAT6Ck0nUpq4AJE1jQzP6VSzqJjpCWjIZW8GWLxLxFz5fRlxi
qRzeyR7Jl2DLF7WpE9ZyuDSO0fL+/wHb7U5bIPjMPEtgdB7Vd4MXweylebLS
G4bz13xWgSjSOfvkOK8AzF7O9F4XaIaz2vqES6SbOdvbSKXo5WhHVZtyjEmj
XQcSalOoKFrs4vFGfJWOhh1N5gQmGI13K6xnWiapBxthMwt5SokuW1caRakt
oXs9lp03ptMCL9j8lmGHG8n9zT4amxdKfQX3TZuOTNGueMlVGNZSqdShiofm
w8eVaVGXVWtH8ERpBGEFBFuga/CCV7xc+mqSdqJ0JiK6AOj1GWFj78sNlflj
rAIvi+UuaVEXHLx0FTxZoR4vVMQRmBzq+SQwiwCGGK8sZTTQ1uXEiXFBJzCB
5ipA2hEsx92Ez+l7eN6861dE7MWy75UgH0KbSVTV77ixAcBeXCUfoUZ55gsF
FfXW8jWA0hffQGHmjrvJGOJRlAAsX7DhlF5K6bhLJKKPYI+lZ5L/PjNtRw77
QL/oQwB7wR7F5kBZC5MX0YDtdrNh+WLzqcF2uyWC3WJNQyIJ585uKYFRO6fv
S66D2csjWYktYIKy9LNyh32NU11HOs6D2cs3gltiI8xMq5RyhgJ1E+KDrQ+o
sJfLoo4JqyHbp7QobzjCU2pOdwopgpdPq9srRycdCH00RPsbuCeEtsvNaXm8
YAOb362w5tbmBTd42VIwkRZ4+fiKokpgZUOwkdWJNvFdh7/VpEVdiFqNckVg
3J5YB8+Rf0mTbp8Rbq4r7Pewx+NFrQw8LNGka/CCm+vevDAzNZ5LWLHsAqDX
Z4RXvGBP1E5CK0/NULTLXX4M4GVRHIHJoZ5LArNY4QKqb6FAaThxYr8gEpiI
bNj86dsA9mLZuxFVTWV4wslQwN9UBGAvsNqO4k6tu2IfgL3Yzr8WHKnNcBVM
Ya0AbPniaNyB2iYyXIUKe6HYdrRMz34EI+6xadEnAPYy3rXWrq2B0nbkco2I
h/YA2IuA/65OWw0r7YiUvWTD8sUZ0HfIr4LtdkWzlTTsdgFa3BnWbl8FZi/3
JddNPghtcSk17VCXSmrA4AUbBcI7E7aZLJ3DoihGS2PjYOuDugmxE4bvNxUF
0RhfpTvX1U8RvGCjUCAW682ZL53uN3C7P9IoNR6rlaUFXnbdF2PPyqHNC54l
nVakET62FEzc4ZkNjtwnzA5PBK5VmbceV0IBL4timMCMjfuLS41pBUnj452/
SAuuaodH2FJZYTOHWx4Zzx2Rr3+1N13wsn/LcH25LnvmumkJuwAcRoRenxE2
8rf2sMpcl3a5Czau/rEEe67HwJZrjAFxBCaHei4JDCVxBIbTcyEoBAZVTYGT
pvXrXgo0V2V4qvOxaKDtjmXfKwD2Yj/1XuZtRwmv2dd0CsBerGdfDvRcy7zt
KGoedTTuALAXy91VQUV9hqvMJ1DfbDWYveha1lFvO1qmpR/BiHtMNXwAwF5k
nW+YZ25DSTvyeMalkiMA9oIN5fQ1WJYvVNhLj+RLuJYvVNgLNsETgm/rurgz
/H7rb4jYy3eDF3lDbdCXXviH0S5pyBE2ioYeqF3Z4j+4qEdLY+NsbyNjCUex
RIKv0l3sHaIOXvbUdjbIZ7xhaECA4jdwoydUM6L/vEKSFnjZXjp0W6DOoc2L
G4m1DDtOVsylRV3wVqMqgTXn4MXtizfxXcevaVftlKcLXrafmqFirssAgdEb
wk2t9uOnZ1etHUkXvGz6QHbtll4xzYobfCQQ53dYr+ZNbV0tTJe6vPlS96kv
ZI8bzR4nK1x3gn60v0l759jwp/+RNnXZ9vPqooMD2NOxg+T6dTxVJuUu2Cjd
/PDHBl4WxRGYHOrf37jy2idjbBgME5gfnGC4LiROK1kZboBxq2lm32YAe8GG
6/LJzB13wVFHeNsRNifDVZKIy992DsBesOGpPpTwZvrbcdQ86mzZDWAvUNqO
cPaia1kHYC+a+tfdE7fSajtaJvwjOOxXqUeOAtgLNrRjeWgYArF2uUZI2Qs2
AVbStMM7+WQ8D8xe2gZ2mRzDUJbDRYW91EnyMrTbfVZ4xvQ14d/ABOZmf77S
/rT5Dvr9F95wRGr2go1SSY3Jn0Vjw1AMFelmKBa97GsqK5cKmXHZjSUSeLDR
5vImiuAFz5WedcA3TgFfAHMOpKxfs+u+ON2Kl4pBba7ACxpPimf9Re3GdA1e
8FQjNni8jCmR4lrr9lMz6VKXVTvlx69pm/guq5PqrXGWCAyCJIT9noKr2nRT
pPGxdYf8brlJpc6xRTOuEZGzpFD12XuD6VKXpQYvaJQVPVNjfHNl/tjR1a3p
Uhe8zwh77uwocy7BYM0N6LsKnnz3bgUN6nL+NzcavuxUdMzEo1k3HGOzOAKT
Q/1YCcwPAIYrgeG0okV7A4xbTc7zX4LZi/Xg1szbjkKiRtvRNwDsxbzrV/66
wgzbjp6yF2DUETacxe+j2ky/bVJhL+7HRzNPO/KrG8DsBRv2oZOJSKY3ZaOD
1ZrRr8DsRTV8IOxXZbjQwvd+L+CeI2xgE1yukczXWvg+5wjstZsNu12zZ5qU
vZT3/4/KNgRxUUyReFik6V7MmCYiMNeEf5OZf2BNEO+/1C5d+VgDKXjBRtV4
ix1G5juRpGZt8XDProYSKuzlxONHIt0MDQ9bGhLrzUWi0Y8rW6iDl0KBuF9j
zN7pPXsBoInkoNp5o3d2e+lQWuDl8wpJo9SYK/AyZQje67HsK1amS13ePiO7
WK/LeZz0mBK522hNN8wIH5uOTBXcNwmldH7Eh0tgFNNIxUPzZ/sVNKgLNg4f
V9bUW/WGMMRToifc2uWvn0lpUBeWGLwsSjFgq/l2PH9rz5afVtIAL9gTeeUz
LOkz0o+a+NcH6DUZLZa7/BgMXiiKIzA51I+UwHA+vJyeF9HeAMF2u8b/fj0k
6srw3MLSHnDMNG75kvBl9JMKFfZiK/hjaKw505ejFZCyF0fjjqh5NMOFEEO3
vv09MHux9O0LO6QZLuR3DIN7jnC7Xa8NQjSPxcwbHPgEzF76RR/arH2Zr4VJ
a+kFZ0xjo+HJ1mldHSzn24Xv2QvYaxe32x3VtUJcFBMSDQjUHRd7jy0lLc8S
mGXsBVfm91/xZFxsHKfScHSk43zjVJc7nK27kjm3vVwqpGixi0dLY0/J0sks
lcLqTAu8YONi7xBfpQuiWS/GWLwA5hxIzYg+XYMXbGBPaZQa7f4cNOyoLaG6
fvuxe6o/nxpLF7xsKZgoajfKNAxZ/aRUJtQFG5+dVdXwHBlmSWe+A+DU5fBx
JY0mI7zc5dot/fCIL+fOumplgLa1C9sMXnDqcnojjwZywcbOX9bifUaxaO5z
puYG9Dh1OfEvF7hyF7jiCEwO9W+rrtLb+aEPBgnMD/yFK4DhtNJFewNEOuuJ
kqb9tWUZntXTupcT68DsxVN8JOHMqBso4TWTspenli99xfPxjLqVw1qBvfYD
MHuxVb6DTctklfkEGtC2M8NeXMYOsNcuNiZ6Nti1NRkuFI8HDfoGcM4RNoSC
97FpmdvtJpKxGUNL28AuUvYyqXkIMepI4xitk+SRshexpiEah1khYAuYmicr
87sPP9tntJTAfDd48Vn2giuTL2B2xFU70UGl4ehk95VudX84lpWbdGco0KSQ
ULR5waOlebPyUCy7DgZBNIa3GqUFXk53CnlKDUSbF7A8IbSsc6iQp9x6eyBd
8HKyUd4uNzNvrmvzoi3DjrPV2o3fyNOlLmwweMmQuqzZO5FfYugZ9iIhOLfG
9HaADKnLmg2S0/nqpla71ZZLUxQ0mhwf8dwvUv/1M2m6+dGL47P3Bm9emMl5
n1EsmsA7jGhTF2wcXd2KHSHneUYRf0TRMdN5jk+vw4grd6EujsDkUD8+AvND
/xHHXzg9B/pJKlF87rLkI/26l7xl15KIn/bJzMeiCK8c3HOEDff1/TFDRvYX
qFHurTsG9nvBhq/pVDKzZoeQshWcc4QNa/maDO12kzHEO11O2nNk6vo4ZM2o
dSURQ2zqiomeDVTYS4Y8BEU9alWpgP8uM+wlFg9O6+qaRduosJcICsfuI5GM
TVuE5f3/wzx7IQ2Y3nbxXWwcurGrRwLqsaL3BUxqVnwnfkil4ehKfxk2me6r
BMkXCfHnpk5111EEL9goHu7JdrS0yRdoU6iox0nj40hzD/YsxsCL0uov69ek
a6uLjzMtk90Ka5DZX5PdSKxX7qYRIc0Sg5cMqcvvvrfVLa61KrLgi0JxB8B9
XYpLjfsPT9OjLtj4bL8CO0JubXU9zmh/j/3mhZn9W4bpIRdsbF0tvJo3xe+w
5tZW12FE+pu0905J6Pm64OPT//i7ra7XkcvmL4fKJakeb/iys+B3xbSpC1fu
kq5ofAHI5KaD01L9yAjMEvzC9R9xei6UyQYYEnXBYi8Jp9lfVwj22sWG88K2
qFJMexVMkWm+q+QjKuwlE7vdRMjll9y2lq8hZS+I/OF8gj49iHpnHSPnwDlH
OHsJmp7QXgUTGrYaJi/LOt9ggL14POOTE/lg8IL3HEFhL4GQSTx9ra5vE6nf
y4yhBVbdSwj1jepay4T7GWYvkXgYN9oFB0zjGdM6D7l1T1pfwNxhX5uSf7L7
CkWzF53XlMFrTa1YIjFsUF8f4FEHL6e66/hzU75IFv1JFFZnmVh+oKErLfCC
zW+QzzgQJoxT7P5Iu9x8pmVy001RutRl/fUnFzqmBDN2NMHcr/yZGOou9hnl
yuAFCSUypy40bHVpCLAD6A3hDp6j4Kp26w45PeSCp0jnX9L08F0IkrN+Fry9
6NJXk9vf6qdNXTa9Ljh3RI4dx2bOGamIRROLpi40kqOXNhld2SPE3V0YjiNf
VDwax610y7fXnv3FFdrU5fS/flu1u3Ho/ihX7kJDXA1MDsUeAnPkyJFxyqLz
Uq1jJRx+4fTcKcMN0LJ34/TJLzJhL+jcuKf4CBi84DHTkQkR7VWSkUBwsMJ+
ZW222Qtqm/B0fwUGL1DYC2LotvTtA4MXbOjb38uQvSDuMVKjXSjsBXuuxcyT
iL+gwl6wmZm8KFx6m7Bv7GsweMFzjtSmTljWKzafunfqTjH/EzB7KRPuH9W1
QmQv3rC7faomZcPRsvFIVkKFveCi8gUMd3qhWPRysvtKm5IfiML/mXvCaiiV
9FH018W7jZoUkuzFG9HrM8ITpStHJ00+JhxIJDr3bYE63TCjxSzpG72zg2on
Y+DFjcREU96idiMNQ91FW1283IX5PiOVPtzEd+WXGLYeV2ZCXQ5/q6los2ej
3CWllu4ACJIYHvHdLTcdPq5cs0FCm7rk3FPX44yOiJz3i9SHt0totxctRkjX
l+vUypz5Bc2OOnnlM0UHBw681kwbuWz6PkL60o6+tjtTy5qMmCQw+lHT0P3R
mgOtmRS64NSlfHut6LbYMpnFNL0fgzgCk0P9239d/d1HMjaM7NbALC1+Oc3h
F07PjzLfAOntwMlQIMivJjV7wdlLWNpD+/RQo9zXdAps9pI5e5lPoFQajjJn
L4mI2ztdTmr2grOXgLad3ioLT38sc9vUFVNPPiRlL9LHmbKXUNCgnL5G2nCE
5xxl7rUbjNjlcw9IG46wwRMf0lp6M1xuUSrb0CPx12DwgrOXccNjWMAnnoxP
WqWV0mJS8IKN5slKWyC9shPw21/t0lWNt1BxesFGgfCO2DiOnXBmr3i5LAFv
3YSYor/uYq509ix28T6j4219aVEXbHxa3V4mlmvdWY9HkRu9FYNaGp66iwYv
TEYaqS0h3NeFdocRNo7dU1UJrNihmDlnXG5fXCj1FddmVOiSE+qyKBRNVteM
VDw0n85X04uNXjr2H56+W24aG6f/gw5tLSKXv34m3fS6gDZywcfh7ZKHJZpc
JRlpJz19j9T3TkkycXQBU5elyiqBsUzaJNXjrSe7MnF0wceJf7lQuvkh//qA
fhR+aeWPVhyByaF+DATG+oPzLuf9wul5E/MEBp0b95aeMO/6FSl7cZzdTJu9
PC16Gal13NhACl6sZ1/OhL3EvTqv8Lzl7ipS9mKrfCeoqKfNXkLWIfvQSVLw
gg1z785M6l68NqF2LI8cvHyfc+QydtD+CI7HgyZjK5WiF2xIJUcyzJhOJGNa
Sy9pujQ+sGkmR2rv2XQViDiH1DWkDUd4xvSkqRcWe3EG7e1TNcsSjlIObA5v
pgGJ0vmVNuVf34642pT8M73XqYCXIx3ns9FwNOe2PxwfpO6vu6uhpHi4Z9ig
hnsaixoz2crE8j21nemClwMNXdgTFdaMEt9IlSF12XVffFugZqbcBYkkxLP+
ez2WQyWzNDKMlnrq3uGZsUNl+4SXakyJVLTZT9/SbzoytRKpC4omx8b9NfXW
/Eua7bsnMkQuOaQuNnO4v8eOV7lkjlzefKkbOw52tPGRHJjQKgZssJALReqy
VHAJzNyAXnRbXHOgNXPkwlEXBsQRmBzquScwy4pfOPzC6TkTYwQm4XM+ddml
UPSCDdflXbT9XqKqfopFL7aCPwb6iul57SZRJKiodzTuIAUveMY07ZyjWEDn
nrhFpegFG7aBY7RzjiKI3jB5mdRlFx+a0a/8/wAUND6CHfYBKk4v+JhSFAQC
c/ReFC6rS0rF6QUfQ4pvHd7JTJbDlUjGVLahVlkBKXjBxiPx19jkzBdd+L7o
RWYeLh0upFL0ck34tyFdXyROv9R/6V8/EEX69aMFwjtUwAs2vhHcwubDTTiS
mrXUE6XxcUHQKtLNZCPbCC93udg7lG6fETbOdfVjz81eq1EwGh9UO28L1LSp
y/rrT860TN5qHWQgSFqmCbQMOwqbDR9fUdBGLtjAno4dRDTlRSJMmIrgdi7/
f3v34R7Fnef7fv6H63vvc+/YZ8+dM/OcPXu8u+NZz8xasx6SMbYxxgEHjHHC
lm2wjbENNgM21mCTgwjGJoicQUgERRRRAiGplTNKKKGIJIQQYXxL+olSqSW1
qquru6q73q/n++wzO6Ouqu7fT6WuD79wIrppZVDNnO9KXJxrb0jq0tV115bT
IUa5uLKci1yTp11euKTY86lLeXFnfERDUGCZixOL5HrxL3FihpGHx7o0Xe2y
xded2JCz7r0EFycWySUdZ9vnqQnHyzVsY+TKM3h9fmNBRMn59Rd2zzq6+ckg
1yMXsZrukbmhzDDyDBIYAz08YfNfZ2ebodyRwChXfmHwC3ySuxOYf9y+1X0p
vHXrfDXBS92Hj7bvXqptn6O77XVdSXvUrPQilfRjNy6f1LbH9M3KxPaE5XW7
/qome2mN+qq3MU/L2+lpvV56rCZylprgpeLUxKbLK253Vmk4Ue/NhmuVJ1TN
NgqfkB05pSZ/o/QS5RHU/wlua8spLFivZraR2OSoqvJYb6/2f1Ls7K7Nrzg6
5tbS8kK7uVcO3ujRYe5Jc2d1YvG+oMS5arKX6Pyf6tpc2thr8LyqB72IhXbz
G1zdlPyX/ta/c+9OVl3B7owTKoMX3Qe9iMV1t1+KVb/Gi1RLo46HFduau3WO
OJq6uuPLqgIT0/2PhTuburxz+Oy25MyUiqs3et2y5861jp6YgobA6GJt2xiJ
kl67M7E8o6pVHNNNcxAKa26IyEXzOrpyvbIq94djlVFZLY3tbt/5uqG5Nynr
+p7QhoUbKlwc5WJU6mLL6YiIbtryc/X8hUUuruUi16tvZovNo8s89S5qKm6I
IS7rvsl3ZdMiu5ozPUU64NnjV6Xje+aNiLVzw3YV7lp8KeCV6Dd+e1iXyEU6
jnS0ExtybPF1Nzpc+r1Qfwfo6ei5klodvzX1xIJz26bt0yVvEYu67Jhx6Pz6
CyXxV3rcnwZDiQTGQD6cwCi3nWbwC3yVmxKYf9y+dTMrVuVsI6kaPp/QcXrb
3etOj7S/2153I+1Q8/aZaoIXqVoPfdJTFK/hbYrgRc1sI7HYS8elbXc7651+
Oz2tHeUhapbYlRd7aS86cO+208uW3r7Veq3yRFHy+2qCFzHhqLn69N2RTjTm
n+Dr14uKi7YkJc5UOeglO2upK4u9dHbXFlUFR1z8RE3wIiYcVdbrMPdHbG90
JG2xmuAlKHHuxfITnT06TC1pv9maXBHzU8pKNcHLypiFZ/IPN9/QIWgSwcum
2KBFEatVZi9ipRe9Br00d3fGXyncmByhPnWR6vNzB4/mpOm7vq5YU3dnms3Z
zYzkeUaHM/NLm1p1vCSh9+693KvtJy5XLwvN1bCHkVzSa9dEFMYUNLR12z+s
6ZXAZFd0BqdcW3uqyvXIRU5dpAO6e2mXgvLuiOTWLUfq5q8pn/xBri77Ssxe
Uhzwc/WJ6CYPpC5l5d2x8S1i7VxdhriImvpShphedOnydQ/sYVRe3ClWcVmx
KFevIS7yQBcxvUg6flen23covtHRW5DaGH2g5PBKW8Ar0R/84aQueYtyoIt0
cA0DXRwY7Q7QVnNd7FWk15QiZUkHPLfsfHZoARsYGYsExkA+m8AMrv3Cwrvw
ZfomMGLES9v2RSqDFzHhSHqJsye901zZlbRHffDSuHp8R9gK6VVOneUfd3ud
Cl7EhKPu4nNOv50bDddLj9XFfaAyeJHq2sVl3Q1Oz1vp6apuLD+kPniRqtK2
vMPhiiij/QluupbqVPAi/WR52e6emxpHDt/ouVZSczY6/QuVwUtwwqvpRVta
Okq0nU526053fm3c6axVaoIXMeGoqD7J9cBHzDban7FVTfAi1U8pKzOuprgy
4Ug+b1ZdwQFbiPrgZVnMptDC89c0TfcbLq+h5mhO2tKo404FL5+c3rs7I0F6
rS7X8Et/6pJeXbc3PXfRmVgNqYtUa+MuRhdX6L6XdO7V9tCsq4HRxdo2MFKm
Lt+fzZcOdaXJUcCr7Rt4a9ft7IrOI4kNekUuYl2XwDM1bh3rIvIWsXbuCwsK
dMlbxEAX6YB7Qhsu5XV2dbsrrxBLuEREN+05ULskoFSXVVzsVnTZ8nO1dHx3
D3QpzLkeH9EgVs396OU0vcIW5YouQYFl0incvW10aWazmE8kxre4sj30iPXB
H06unB0rHb8gtdHFgS4O9A2D7N8YWswnOjI3VMfxLXJtfjLoxIJzF/dnsqKL
qZDAGOjh8Zv/OivbDKVrAjOYvxC/wLfpksDcba5zNnjpG/QSHCi90Klz9VZm
dEStVznVSFRL0Ns3Lp+81+PEXIO73S3dxefaYr5ROdVIqvo9k9uTVt9pd24e
0K320ta8n2vPv6M+eKmJnNVRHuLsoJeuVltt4Y/58TPVBy/SDzeWH7p9a+x/
l1f+Ce7tbauvi87PWxkf+5zK4MXFQS/tXRW5Vw6qH/EilfTD5bWRt++4NIxc
LPMSkbtZZfCyPf79uMJdjdd1WOi1+FrumfzDKmcbiUEvtde1zFBT6rzVlX41
R/1UI6mWRK8/knO2vMXVU0tq2luiS3O3pkY7Nc9IjHjZnZGQVVfp+jX80jfc
qCe9uu5wZr7m1MX/WPi25EzpILfv6vagXdzQEZ5b92Nc6RfHslyJXNSnLkoq
v4GLWUXbwq9+EVT6/PfZukQuz3xnk44mVtPVfV2X3tv/sBV3nY5v2RPal7e4
skP0iCUdc8uRuojk1ur6W/peuVBQ1CU2hl65rmL+wqLJ0y7rm7fIkctPO9Ld
F7nkXG6T6mhQxU9rShbOyZj9VJK+eYtylEt8RIP75haVZjYXpDbKYYu+g1vk
+vixkHXvJYjIpb3JLfGRCFuyQwtE2LJjxqG//fdVuuctUm2btu/EgnPJO9Ol
07njjUAXJDAG8skERrn6rgqENPBirtwAe6/kdIbtqlw0VWXqom3Qy72ezm7b
meunv2tc/6T64KVvld3YLU4NeultzOu4tO3ayTdVpi6ims/O7S4+p36Ho7s9
rV01Mc1Z66rOPqc+eKkMfarp8oqeFidWiO292dByNaIi85uc89PUBy/SD1fn
rr7RXqD+RNKf4OvXi8rLdqdfnKc+dZEqLfX9qspjGga93L5zo7ox6WLBBjU7
SitXesku293Z7dK/oN26011Un6Q+eJHqSNri/No46YWunFdsKn0qd//KmIUq
B73svhSYXXfJxc2dazsaY8pT1C+uK2p3xomsugIXT93c3ZlcVbL9UqxTy+qK
+iriyIGspOImp6cBDlfZej2+rGpbcqa2GUaiAiKTQnJL9NpIWsfIRVvqojTi
N/Dsis6orBaxXZFeQ1zsBrroO72ooLz7Ul7nntCGlUE189eUT/04T9+8RRm5
lFXr/Ggshy3rN1fOX1j0wmtZuoctDka56DINrffWvZzLbWLlFhG2uL4/0Wgl
HflvH2VJJ5JOp/sol/amm2JnohMbcrZ9nuq+sEVey2Xfdxlhuwqlk96+pXMI
WZ/fKHYmOr/+wgHprjrj0NLfrHFH2CJPLAr5OpJRLt6FBMZAvpjAOBnAkMDA
m2m+AbbvXupU8NK49Lmu6AMqV3q519PZUxTfEbVezXbSdltLtwcvVr/Sy+2W
0q7coy3hn6kf7jKwtfSxVztt+9Sv9NLdcLEle7NTw11ENSR90VUTozLhuX2r
tb0xqbbwR5Ur69ptb9RyNeLePbVRUmfnlarKY9lZS2PPP+tU8JKUOLO4aIuG
7Y1aOkryK44m2L5Vn7rIs40aWlxadbatuz6nJio4Y7n64GVv0vzE4n3NnS79
+13PnZvZdZecCl7Wxi0OLzzhykovd+7dyWssOZkXsSxmk1PBy6aUvSnVmZ23
tDzIC923e8VmRs5OMpIX1z2ak1bT7tJ0JzHQ5bitMCAyadaB05pTl0VnYvem
59pqG10c7tLW3Su2il4TUejKCrrK+nB/emB0cUxBw9U2V3OMi+kZYqMikbe8
sT5Px7zlif7di344VnkksUE6S+8dHba6Lqu+aSvu2hPaICYT6bJe7vCa+nGe
dHDpFDpGLq1tt5XTiNwatkg1+71csZZLbHyLg1EuTj2CiaRFrNkitiXScZnc
0Uo6xbpv8o8GVUin1mstl6arXQWpjemRNSc25Ig1W3TZCdpxffXUuU3zksTy
udIF6PJGfhmatIR8HbljxqEffr/JfUmLPKvowJyTYvlc1nLxXiQwBvLFBGZw
ChIJDHye5htg96VwVcHLV1M6ggPVbG8kpy7qV3exC15u5kWp2d6otzGv07av
Neor9au7DAYvh1/ouLRNzWyje7e7bjZltRUEqV9WV1nSqzrKQ+72jD0JSIx1
qc5drSF1kYOXEZfYHa6tLafiyoHsrKVOTTISlRg/o7BgvXQENSeStXSUlNSc
TclbpXIvaWVJr3Jxid26tqLk0sMqF9dVbm/k4tbS7TdbL1YlqF/jRdThrO2u
DHq51tWSUp25O+OE+gVe5E2lY8pTWm9qHOBx++7d4qb64Lz0H+JCNaQuYjvp
6NJcVxbXLW1qDcktCUxMn3cyUnPkIpX08p1pNlc2MxJr54bn1ol9ol1ZPtdu
oIt0tL0pFRlVrTduaX8ILay5kV3RuS+2PvBMjY7ziexGuaw9VRWccs3FyKWh
uVfsBL0ntGHJlkp9V24ZXrOXFEtnEWu5tF536TG/oKjLltNx+ty1PQdqA1aW
z19YpOMauaPVC69lSSfavvuqdF6n9oke/ggmYhapokLr9m8r37y8cOGcDN0X
bBmtpBOtWJQrnVe6ABeHuIiYRSzVItXK2bFuHdNiVwvGnxGruKScrnR97dzq
zNorqdUZx3LOr79wbtn5HTMO6b46roNaP2777llHpVNnhxYwxMWXkMAY6OG/
bvrrazYzlDt2owZ8nuYb4L3uTsfBy/Wjq8cMXm43lHTbznSErdCQuiiDF8fL
vNzr7eqpSevI2Nl8dq6zkYscvFxPDbzdUur47dxqL+0oD2nOWufUmroagpcb
7QWN5YcqbcvzYl/SkLpIVX75q9H2NlK6c+dG07XU8rLdGemfORu5KEe8SAdx
fCLZ3Xu3m9rz8yuOXshZ7mzkIka8iOBF8zIvbd31+bVx0fk/qdxOWrnEbk5N
lCvbGxVfyw0vPKFyVyO5dqStzbia0nVLy67Knbe6suoKjuSc/T5uq1Opiwhe
IksvxF1M1HDe5u7OSzXlR3PSvosJ1pa6iJV1pYN039aywmRpU2t0ccXONNuS
sARXIhextEtgYrrmNXWLGzoSS64dSqtcfDJ7zu6LuuQtor44lrUzsVzzQJfs
is7kwvZ9sfVrT1W5KWwR9cn24sAzNSJy0XCdtuIuqQ6FXfNM0iJKOoWYVXQi
ukk6u4bLFjGLGNAitn7+aH6Bu2MWucReRdJ5T5xqkC5Dw3ZFjXU3xWiW1d8k
iUVx3bROi4MSS7hsXl4o8hYNq7iISUNi+yFlzOKZjEVZS6dHiCVcEo6Xl2Zq
+SMiMhaxQotUu2cd3THj0Ppx2z0Ws8i1bdo+Mb6FvMXnkcAYiAQG8Gqu3ACb
17xrF7xc++7ljtPbHAQvA5FL1PrWff4aIhd5Y6Prp7/rKYofbcTL3e6WW3WZ
HRk7W6O+ajgwVVvqItW1k29KB3EQvNzurOqqiWnN+1nbQBd5jZdrF5d1VoY7
CF5utBc0V5+uyd9YkjpPW+Qi1niptC1vq491ELz03GxsupYqBrqo38ZoeKVf
nCcdROVUo/auivLayMyS7ep3MrKrkAuz04u2VDdq3GOos6e5qD4prnDX3qT5
TqUuYpmX9IoQzcFL841rF6sSDmdtVz/PSA5epBe233R6C+M79+4UNZWHFcc7
Xt1FvjM89vpEu6lGiRWX5I2N1H8Bu9J6Lf5K4fZLsV9FHNGWuojhLqcLMqRD
OfWWb/TeLmhoDsktcXE5F+VYF5G61F53IjSQx7e4I28Rc4vWRBSGZl0tbnBi
DENje6+8bMuyQ1e+CCp95jubm8IWqZ7/PlssnxuX26p+LZfq+lvyurhiqZY5
35W4O2axy1u2n2yQLkC6jN7b/1BzzbacDqnEXs9iu2fPjGYZXnPm5on5RGJ8
i5q8pabihnIci5yxeGDSkIPJRGL9FumSpAvrvTX2+CgxiEVemEWqde8lBLwS
vXR6hOczFuXgFukaDq+0ifVb1CyZ21TWciW1WpQIWMR0Ic/MGHIctgRO2yFd
z8X9mdK13XFhfB28EQmMgUhgAK/myg2wM2xX7Zx/ver/h9at87uTQ4ev8fKP
O729lRldSXtcjFxENf34knQc6YDDr+R2S+nNysSOjJ0t4Z+5ErmIXY3aYr7p
Lj53t9t+bvLdntabTVntRQeas9a5ErnIuxq1ZG8ecUfpnq7q9sak+tK9FZnf
OLVz9IhVkjpPOtRoi+tev15UXxddWrI9K2ORhrlFdvOM8vNWSkfr7R1jyHR7
V0V1Y1LulYMJtm81TC9S7mokHUTbjtLNndX5tXFxhbsOpHzpbOoij3hp69ay
1mv7zdbsuktn8g9vTPzWqdRFBC+J5RHOBi+dt7ryGkvCiuM3pexVOcRFmcAs
ili9O+NESnXm8KlGDr6A1Xe2J1eVHMhK0jy9SLm6S1ZdpfrhLpWt11Mqroq1
XFycWKRc10XMMFI51qW4oSOjqvVQWuWPcaU6zicaHrlIp8i92j7m3CIxgUgk
LWJYi+5rtgwvsV3RtvCrZy81SWd3sGORWJ4lKev6ntAGqQJ+rp6/pvyjH8o8
FrOIEhtDS2eXrkG6mIJR1j+prrkpAhYxV0hemMVNuw5pCFsOHa2TLm/4+i1d
nXdEtCL2GBLpynefZS+ck+HW9W+dKnExQYFlYvGW4ZOJxNZCyilCcrrijm2d
NddXT50TYUvoj3nS1Q6fTNTT0SNHK/FbU0W6cmLBOZGuBPyvDQamK3YlXY+Y
SSTCls6mgRFHuizFDC9FAmOgh/9r019fzjJDkcAAGrhyA7zbXHerOF2+A99u
KLlVltKZsP366e9a9/k3rh7vYuQitjSSjtZtO3O3fWDf6rud9bfqMrtyj3Zc
2tZ8dq6LeYtcTaHvdWTs7G3Mk9/dzaasG7UX2gqCrl1c5nre4mC4S1erTc5b
XBnioqy82Jeqc1e3XI1QbifdfaOmrS2nqvKYyFtSkt9yJW9RBC8fVVw5cP36
qAOfenrbm9rzi6qC04u2OLuO7ojDXVLyVpXXRt7ocW4gxN17t+vaitIrQs5l
r3d2hpFc0mu1TTVqvnFNpC5bkv7ubOoigpfkihin1tet7WhMqc7UNsNImcB8
9OXHDs6i/AJW39l+qaY8OC99TeI5Z7eNtiuxi3RyVcn1nrHjjsrW6/IQF9dn
Fcn1zuGzK86nHLcVSgd3sJquGNmSVt58KK0yMLrYTWGLPLHox7jS0Kyr0hml
89pfyZ172RWdUgWnXJNjFn23InJcr6zKFeNbxJSi1q7BAWlirpA8jkVs8ezh
oSzD66MfyhZuqJAuRgxu6eq+29B4S0QrUh06WifSFbHHkFRTX8owMF1R1uRp
l6XrWRJQKka2hJ1rSIpvFrnKqQNVIlcRmwqZKlqxq9lPJcnTiKJDrp7ZeyU7
saEgtTFsV6Gcq4hpQYYPXHFc0rWJpOXAd+kh6y7H7MzNj7miDFXkaUFGzQxS
X5ufDJIuMnJFvHTNIhpyfPslgbEyEhgDkcAAXk37OjBdLZ0J2zui1ldtnXlt
0zTXwxZl6tIevPjG5ZM9ZQliMpFYwqUp9D1dwha5pGNKR+4qPNndmN5ZGd5W
ENR0eUV9wieVoU/pkrfIqUtj6uLrpcc6amO7Wm3XKk+IsKXs0oLsyCm65C2i
Ci+8VZO/seVqRO/Nhs7OKyJsKS/bnZWxyNl9oh1XfOxz0jErrhwQy+ra/Qlu
6SipbbqUX3H0YsEG1/MWuWIzvpaO6exwl7q2opyaqOj8n5xdTVdZQYlz4wp3
lTVedGpH6Tv37lS1lSVXxBzPDlobt1hD6iKV9Fr1a7x03uoqaiqPLL2wI/2o
s6vp2s0ziilPke8My5YtG+2MNe0tBxOiROTyyem9rkQuYmmXranRY66pW9rU
Ksa3BCam65i3KAe6xJdVjTi96GpbtzyNaE1E4eKT2S9uveCmsOVZxfK50hml
84prKK/vzq7oTC/t2BdbL88bemdTgcdiFrmk80pnX3Uwc//5+pOJ1+TFWOT1
WNy0rbOGmuSfO+fb4s9WlS/ZcGXJ2vKVWyq3BV0V04KkmjM3z/BExUHNetP2
2qysBfPzvvwsN2BxwUezL5s5VBmxZo6Pe3tKwmcvJ33xatLcKdFfPBv99fPR
Xz11zvDkxKl687eHPnvshFTfjD+6+LGDq588JNXPLxo8D8iVmOXEgnPn119I
3pl+JbVa84ItJDBWRgJjIBIYwKtpvgH2FMXrmLpc2/R0897ZLcELmk68o3m9
3DGrbud/NRx94VrE3PrI9+oTPqmJnKVj0qKs6ohXqmPfrU7+rDz5Q92TFmXl
J75RlrG0OOf7K6VBuTl/d30mkYMZRtlZS6sqj8ljXTq7a5va82ubLp1P2Zhd
tjvB9u2Z5Hf1yltEXchZXlQVLJ1FZZ9s666vbskVo1yOp3+rOXIRFZyxXDpU
4/Vy9b8UjZ21F6sSwgtP7Ehbqy1ykWpL0t/P5B/Ob8gac1ejzltd5S1VkaUX
dmec0DbQRS7p5SfzIrLqCm7e7hEHH57AXO/pzqqrPF2Qsf1SrIsTi5QzjHZn
JMRfKRxxC2kxuOW4rfBwZr6O84nsIpdtyZkhuSXyQJcbt+7kXm0XE4jEgi1S
uS9mUZZ0osDzJZujS0PS6yKymkTG4rEZQw7q5RW5s1bnL9hW+vHGkvdWFM9b
Xer5+UFqavycnKkf5Er16ry8cTMyxk2/PO45I+cEqa+Jk9MmTEid8HjypL9c
mPRYwhOPxkpleHKivqb8a+TUfwmT6rnfnZLq+f9x8qXfHJPK8NjE2Xr9oX1v
PbRLqg8f/FGqT3+94bNfr//ywdWGZybO1g+/3ySG3Jxbdl6OWcYc0KIBCYyV
kcAYiAQG8Gq/GomaF95uKHEpdfnhT/Wr/1i3/k91W/5c+9Of3RS51O54rGb3
Y9X7/KoO+lUcdUvYIqo8+K8lpx4vPP3XnLPjbWFuCVsGKnpqZuwLyVGTL0Q/
6Y6kxW6gS3HR1tIrB2vqYvMrjkqVkrcqwfZtxMVP9E1a5ApL/fBiwYaSmrNq
Upfu3ut1bUWZVefiCnedzlrlYt4ir+6SXHq4oilT5XCXqrYyEbk4u3O0Xa2M
WXg4a7t0qMZOR/8Qea2rJauuILL0wraLB5fFbHIlcpFKOsIBW8iIq7v8orgz
TPd/2/VZRcr6IS40OC89q65SOcOooKHZVtt43Fa4M83mprBF1JKwBOkUZ/PL
I/Krc6+2n7hcLS/V8uH+dM8kLVJN35z4zs70+Qey39qW+dGOvI9+NmAQywi1
OGvSoqxJX9gmzLdN+Ng2/kPbeH/jc5URarZt3BtZ417N6otZXszoi1meTTc8
RXFc45+42JexTEid+F9JfTHLf3pZzCJnLNP++bSIWUTG8sp/O2R4bKIhYHn/
wZ+VGcuCX683PDNxtrZN2yevzSIvz+L55XBJYKxMQ+trfuiAnYf9Av/6YqYZ
igQG0EDzDfB2c5lzkcuKR+vXPOqxyKXykHsjl7Lgx4tD+iKX3LPjs90ZuaRH
TkqLfCIl8gl3Ry4JCTOSU+YkXZwXn/5pQsZXriyQq77OJL97IWd5fsXRhpYs
B/tHd/Y017UV5dfG6TW+RVvq0thZW95clFgecTw7yNk9o0db3SWu7Jx0zBFP
V9vRmNdYImYVrUr82cW8xS51kTczknTf7i1uqk+uKjldkLE1NXpp1PE5J3fI
d4ZHXpnmYuTy+bmD0mHDim3SWQoamsWwFqlWnE8JiExyU9Ii6t3D5+YHx3xz
NnVNVNbnx9PdPW9otHpmXdLTa5KfWpE6ZfnFJwPSn/jG6KRlaeakr/rClr6k
5dOsvqTlA5vxuYqy3snuy1jkmGVGf8xi5gEtTw4ELBPG9Y9j+cuFJ/4zfiBj
+Xfj85Mx65n+dEUexDL9d8EiYPGicSxyuvLugztEuuK9AYtIV+SJQsqARV4C
1yRIYKyMMTAGIoEBvJrmG+Ct+qy65Y7ylrpVfUNcagP/XLvV/UNcDvyl8vBf
rhx3Y94ihrjknRmfc3a8m8KWzIiJyrwlLnqK+/KW8+efjoqZei522pnY507F
Tj8R94IH8haxnEt60Zb8iqNN7fk9vfbLfTR3Vte1FZU1XkyvCBEjW7RtVDRm
SUeWTuE4dalqK8tvyEosjzictd2VWUXDU5fokhDpyMrVXVpvXi9vqUqsuCTG
t+iVt4iSjiZmGF3rai1uqs9rqDldkCHVxuQIB5OJXElg5obsWXY+ZF1C9Nbk
pE0XLgZEJi06E+vWpEWuN/dHvLQj7PmfIqdvjXl2U7whYcsza/vDlu/TngxI
n/xthuExy8TPbWZMWt6yDWQsr2T2BSwv3c9YnjbjaBa7dEWeKDTZ3INY5IEr
Uj37z2dEuiIPX/GWESxvPhR0P1rZLkcrnzy48bNfr5dq4a9XGR6YqKnV//mj
CFWUA1ekKogoEdFKW80IYxHNjwTGykhgDEQCA3g1zTfAruqE+pWP9s8nelT6
D3Xr/jQwvsVNeUt/2FKzx69qf9/4lr68xT1hS+mpx0v7w5aC0L4pRdnn3Di4
5WJf2DI5yc1TikTYEh777LnYaaEx00/FPu+ZsEWq6PQvUvJW5VccrW261Nnd
N79GjGYRC+SmV4QkFu87nbUqOGO5O2IWu4Eu0rnya+OGr+vSc+dmVVtZdt0l
MbjFxflEI5Z0TOng5c1F0rlqOxpF2BJWHK972CLqb1HrNibv2591bk9GRHBe
2prEcxpWynUqgfng1J55oYffP3709QNHX9130jNhi1Qzdp19aUfYCz9HTN96
ftrmWE8mLVM3Jj6zNunpNSnyyBZjwpYl/fOGFmVNXJA1OHXI8Jjl7fsjWGbe
H8Eir8ryrInGsYyflGYfrfzlwhN/jBtIVx6JMTxCGV5PPxwxYqjy/G8HR63M
+KfDhscmo9UbD+4RcYpyuRVRIlHxiiErS3+zRo5TlCNVpCqJvyJvMO2loYpT
SGCsjATGQA//eeO46RlmKBIYQAPNN8CW0pNuSVp29icte/uSlqoDfUmLmyYT
DSQtoePETKK+9VvcELNk9A9rudSftIiRLQluC1uiY/qSloj+pEWqU7EeDVvO
X150PuPrtMKtmaV7U4p3pZYeSK8Iic7/6XTWKqlc2YpIWwVnLE8s3pdTE1XX
NjDHp6qtTCzbklgeEV0Ssj9jq44jW+zq59TVR7N2nysKDc4LFdOItl08uCR6
ve5hi6hFEWuWRG/59nzQ52E73w/+SZfVWhwnMG8f2/vm0f1vHD4488CxV/cF
eyJsCTo9Y9fZF36OeH5b1PStMdM2xXk+aZHqyYD0vmlEHh7K0p+xTPqyP2OZ
bxv/UV/MMu59I2IWeXKQPHZFqhcuDwxfmWro8JUn749UEcutPJ40PFQxz7or
TymCFOXiKnYr2Ur1ssnilFkP7lVGKHZDU6Sa++AmOUsRZXhg4mA4ilwhX0fK
KYpy1o+o+vxGvb98+QgSGCsjgTEQCQzg1TTfABtro7VlLH0BS3/1ZSz3B7To
HrOIgEWMZumbPXS6L2PpH9Oi8xyiyxGT+jOWSf0Zy8CAFjfNIYqOeSYqZqpU
YTHTFDHL9JOemkN0MuG1U8lzTia9fSplbnDqJ8fTvtgZN/vn2Lc8nK4MryNp
i8NzNp0vDLpcFZldm5xYHiEPaHFfzKKs9QkBa+K/X5+4/pvoFW6KWZT12bmV
88+unXs68MOQLXpFLg4SmLeP7X3r6L7Zhw++fuDozP3HPRa2vNA/mWjaZk+E
Lf2zh5KeXpmqGNNy2RMBy+KBgGVgrtB82/i5/eNYPrSNm+OpjGW2bTBXkYes
yEuveHz1lfGTBoOUge2BFPXEn+LlOGXyI+eNHYUyfCyK3TQfY4MUeWmU4WUX
m9yf2hNoF54YOMdH3s1neCnHnNiVPKmH5MQzSGCsjATGQCQwgFfTfAO81njB
fuDK0GilP13pW55Fx4ClNPhxu2hFlEhX9FqhJSO8b+CKKDlakQexuCNgEcNX
7NKVkP50RapgDw5lkepw3CtSHYx7dW/sTKl2xb7RH7O8aXjMImpHwof7UhYd
Sf9hV8rf9lxcuStVhxVxna8Fy6IWLo1cvDjim0XhAe7OWxacWzH/7NqPz6z/
KDTwg1Nb3zv5szsiF1HvntjZN77lyH75zvC/n3/VXWHLnpC+aUQ7z3kobNlw
4Zm1fTGLGM0yELN8584BLYvth68MzBLq22PIbenKW4o4RaqXFaHK84pQ5Wn3
hCf3d/wZqPEpdvmJXYTSl6L8Qf9xKc8MTUgcpyWjZSa6L5PiIA+RS96jZ3iJ
RWVHK4+lH8qKXBG//8tjo4UhcikjkeFFSOLVSGCsjATGQCQwgFfTPgamPt7p
9WxPDg5NEVUyNEiRl7qVS9uQFTk/kSttYB7QYCVHDcYpukwOkgeoiJJnA8nV
vwbLYHk8V3lZRCtSiWhFqt2xr++Mm22qgEVZm2Pfk2pDzIfrYj5cfX7eyuiP
jQhbPvt71EDesiTib1+FL/sy7O/uzVvC+sa3fHx6g8hb3g92V97y7omg/rDl
wOz+yUQzDwwZ3KJjAjMj6MyMXWdf7F+wRcwhem6L/mu2iOlCfeNYVqcOzBj6
+yW3TBpaPLD0yuDqK1J9cj9a0WvsytvZQ4KUmUNHpyizFFfGqAyduTN8UZTR
YhNnZ/SMloc4iEREvThSMDIQjzx00Klww8HYj9FKXu5V1DePbF/95KERa/O0
g2PmFUfmho4ZVjgo5QIj6qs609EG967jAdzi6ABWRgJjoIf/tGHctMtmKBIY
QAPNN8Ca2qissAnDKzNshABEMaTEPglxXHY5iYPMJGpoAKKy7EISu1IGJsry
2KwfuzoSN0NOUUTtvz9SRS6RqIgyPEJRU1ti52yOfW9jjP+6mA/Xnv9wVfTH
Un0fPd+QsEWqZVELv436aknE376O+HZRxHcemEz08Zn1/fOJtvaNbwne7oak
ZVffNKJje2cfPjjr0OGZB469tv/EmLGJUwnMy7v75g29JKYO/Rzx/I/R/TFL
jO6zhMQ2Q8olWXQIWOSRKsrBKvLmQXKu4kp+ItVrQyOUGUMjFIeDUsY/kTZC
TjLSDJ3R0pIpj0Q/9Uj01P89cgYiL0Iy/XenXvzdCbua8dujM39zUNSs3x16
++Fjot77/fFP/nz8g0cOSf938WMHR6yAKSdXv3xW1LqZZ/cviHa2CqLLNMQO
ono6etzxRxNKPIBbHB3AykhgDEQCA3g17WNg2vJGCyjUV3CsMVGGLiNJxqz9
ca/ZxSN2JSb4jFjbTTkoxdn6MfYdMY5FHsoilchYVkR/YlTGIict8sgWEbZ8
Ef69W4e1fNq/cstHoZs+OLXV/9SP7pg99HZ/zNK3YMvBI31r5KpIWtSOY9l1
tm8ci5gxpF/GMnX9wMwgeXKQPD9IyxShvymyFKm+UMQp9xewHYhTHCxjO9tm
H568Oiw8eX5oeDItfeSERLnDjl/C5MfiRU35Y8xTj0Q/80ikqKmPRj/3aMQL
/3Fuxr+ffvXhU3K9/u8hb/1HX819/PTiZ8NGrCXTw9a+E71jbpQcWcTszB1e
+TEjD5/obLrh4l8Qnr8sjg5gcXQAKyOBMRAJDODVNN8Az9sujJYemK0MDyJ8
rORQRarAmPfXDUYr8wbSlfOfGpuuKGcPyTFL35otEd+5b9mWz86tmn927Sdn
+mIWsUauvnOI3j6+W5mxzDp0qH/q0DFXd3/uX4xlcD2W/pruQsByf7BKf6iy
MnVwyMr9OUFjjFqxS1EWZU38zDYkSJlrsx+aMjwzecM+Mxn/wuXxz6VPeObi
QD2dJtXEKWlPPhYv1ZS/DNbTfnFT/WL68pA/hb36n2GvPx45e1K0qDcnRX/+
1ED0sWJW1Np3oqXa812GqP3LM5Whh0XWuOD5y+LoABZHB7AyEhgDPfzohnHP
pJuhSGAADTTfAFccOWV4FEA5VVti3lWGJyOmKHZZiqkSFbsKiPpSGbDIQ1nc
NJpl/tk188+uFTsQKTMWqXRZj0WeKKQcxKItYHl5T6gIVeTVVwaqf7VbUWPv
5rxhcHSK3fSfKd+nyVlK/0iV+3sGLcmyC0+kUoYnEz61TZiXOf69ITXuzazx
L/UnJC9cnvBCulwTp118YkrK5MlJoqZMvjB1SuK0CXEvPZHw8qQ4ORV564no
eVOjP54W/fWs2JWfX5bzEKmUeYg7bp74hecvy6MDWBwdwMpIYAxEAgN4NRIY
F2tr7MixhrMlFkJRWWvPf6RMSJRleDDiQg0MWRH1TeTXIlcZHL7ihnRFhCqi
5p3ZKKKVj0I2i2hF28ZD7xwfSFREvXlkvwhV3hhY8PbY/WgleIwgZW+IHKQo
V1mRS0wFEmW3tu3Q8MR+OEpfivJDX4oyeelgTfpKHnOSNeHj+zUva+K8TKkm
+GdMeDdz4uzLE99IF/XEKxcnP5f65HMpz05Pfe7ZZLlenpr4+rT4956L+/K1
uJWfX5Zr17d9qUjwBlvMztyM0yXuuJvB83j+sjg6gMXRAayMBMZAJDCAV9N8
A/z+UPhoIYBby8A1WikHpQxP5BJL2ipLmaXoOCdIGaT0byS0UR6pIi9yqz5U
UeYnA6NTju6TR6cMz1LkOMUuMxkciLI93C486Zvpsy3quS2x0wLj5Hp2Y7wY
bTLl+7QpyxX19/4KSJu8JF2qJ74cUhM/ypjonzHJ//Lkdy89NXtovXFp6mup
Us2cceHtF+I/mHVh4acZcolgZN2igbEirbWdGm4gfP22ODqAxdEBLI4OYGUk
MAYigQG8muYb4PKDUYY/9VMB0V+MGH2MVsMjkdFKntEzvL4IW655Cs+I9cmZ
dcrAxG6Oj7LeP/nTO8f3DE9IBkabHN0/GJIcOvTGwcNyvX7wyGt7gwdqzyll
vfhz+PM/Rj2/Va5IZT27Ln7q6oTBWpEw9YeEZ/5+YcqSlKelWpQ2ZcHF4fXU
p2lPz+2rqW/1BSBzPk7/aEGGXMoY5NvPBpKQtPPu3TTWTfj6bXF0AIujA1gc
HcDKSGAM9PAf1o+2f6KHiwQG0EB7AnMgTuWzvFvra7sxFZ6thWHLvwz7Xlst
6F+4dcz69Mw6NTU3ZMuHepR/8I/vndgh1Zxju949sttBvX1w7xt7Ds8MOj5z
17Hh9fKPoTN+PDNavbgx7KUNEXJNXx099ZsLynpreZKjCkh6d/ElZSnzDanc
8ZuCEfH12+LoABZHB7A4OoCVkcAYiAQGMFi4v7iD+YdrebXmG2BecfPb66Ok
emN1mPgP7qi3Vsc4fhh/59vUTxZlhh6uutF5W8v7h8v4AmZltL7F0QEsjg5g
cXQAKyOBMZCpEpicURj9IQHucz9/8XgCI+Pvr8XRAayM1rc4OoDF0QEsjg5g
ZSQwBnr49+vGPXHRDMUYGFhP2WD+QgIDg9ABrIzWtzg6gMXRASyODmBlJDAG
IoEBDDIkfiGBgVHoAFZG61scHcDi6AAWRwewMhIYA5HAAEYoC/T7lR0SGBiC
DmBltL7F0QEsjg5gcXQAKyOBMdDD/7Zu3MQ0MxQJDKyiLNx/WPxCAgOj0AGs
jNa3ODqAxdEBLI4OYGUkMAYigQE8aJTwhQQGxqED+KT4+yorKx38GK1vcXQA
i6MDWBwdwMpIYAxEAgN4xvDwxc8/XLkWDAkMDEEH8EnynWHZsmUOfozWtzg6
gMXRASyODmBlJDAGIoEBPGPoyi996Uvff0sCA6PRAXwSCQzUoANYHB3A4ugA
VkYCY6CHH147blyqGYoEBr5NTmDuhy/9SGBgNDqATyKBgRp0AIujA1gcHcDK
SGAMRAIDeEZZoJ+ff6AifOlHAgOj0QF8EgkM1KADWBwdwOLoAFZGAmMgEhjA
SCQwMBodwCeRwEANOoDF0QEsjg5gZSQwBvrXf1kz/r9SzFAkMLAiEhgYjQ7g
k0hgoAYdwOLoABZHB7AyEhgDkcAARiKBgdHoAD6JBAZq0AEsjg5gcXQAKyOB
MRAJDGAkEhgYjQ7gk0hgoAYdwOLoABZHB7AyEhgDkcAARiKBgdHoAD6JBAZq
0AEsjg5gcXQAKyOBMdC//vOa8Y8lm6FIYGBF+iUwDmQ4FBwc7PgH4NvoAD5p
8Mbi7+/gx2h9i6MDWBwdwOLoAFY2ZuurecQggdGGBAYwEmNgYDQ6gE9iDAzU
oANYHB3A4ugAVuZK65PAuIgEBjASCQyMRgfwSSQwUIMOYHF0AIujA1gZCYyB
/u1/rprw5wtmKBIYWJEJEhhunhZHB/BJJDBQg19/i6MDWBx/AqzMxWcHEhhX
kMAARiKBgdHoAD5p2X3x8fEOfozWtzg6gMXRASyODmBlJDAGIoEBjOSelXid
PYKWE8NX0AGsjNa3ODqAxdEBLI4OYGUaWt/1hw4IJDCAkRgDA6PRAayM1rc4
OoDF0QEsjg5gZYyBMdC//W7lxD8mmKFIYGBFJDAwGh3Aymh9i6MDWBwdwOLo
AFZGAmMgEhjASCQw8JTRGpoO4EuYhAin0AE08KUPzZfeCzQwbQcw7YX5EhIY
A5HAAEZyQwLj7IO2aW+ebr0wFw/uvufcMX/S8Q84+F/pADoe2Wc6gGlb/xcT
dwANL1f5EjU/pq0DeN2v/y8+1AFM++uv5oxGMW3razgCHUAD03YA7v/ya0lg
XPHvv/1h0n/EmaFIYGBFHlyJl7+/Oh6cL2AeYNovYBqOYNoOYNrW/8XEHYBv
4J7hMx3AtL/+as5oFNO2voYj0AE0MG0HsPL9f8QnDtN2IZMjgQGMxBiY0ZHA
aPgBvoB55sg+0wFM2/q/mLgDWPkbuCf5TAcw7a+/mjMaxbStr+EIdAANTNsB
uP/LryWBcQUJDGAkEpjRkcBo+AG+gHnmyD7TAUzb+r+YuAPwDdwzfKYDmPbX
X80ZjWLa1tdwBDqABqbtANz/5deSwLiCBAYwkntmIQEAAACAW+n9aGQJv//N
95N/H2OGIoEBNDD6vgsAAADAiox+EvJKJDCAVzP6vgsAAADAiox+EvJKJDAA
AAAAAADu9sh/X/7kv0WboUhgAAAAAACAryKBAQAAAAAAcDcSGAAAAAAAAHf7
wz8FTHk40gy1aNGinNEZ/TkBAAAAAABoZ54EhjEwAAAAAADAV5HAAAAAAAAA
uBsJDAAAAAAAgLv9x0PLnvqXMDMUCQwAAAAAAPBVJDAA9FQWHujv5/crmZ+f
f3iZ0RcFTygLVLT7CPwC6QiWcr9D+IcbfSUAAABmQQIDQC+jP4Lz8G0B4f4O
Axg6gbUMdgcSGAAAgPseffDbZ/75rBmKBAbwamONgOAxzMeN1QFIYKxEGcfx
qw8AAHAfCQwAHSgfv/0C5XlHZeGK/54HMZ92/5mbZsbQ4VD0CAAAgPtIYAC4
ztGMg8FwhicxH3a/mRnpYnn209H4vQcAALiPBAaAy+RHrpGfte7/zzyc+y4C
GPQb7Ajy+DcSGAAAgPv++P8unfq702YoEhjAW5GwgDlI+GVoEFdGAgMAAGCP
BAaAixj+gCFP3uGK/cj9/PwD2Y7cIoZuP00C41hZWXig4jdF/mXR/bdFlxO5
42qtHNt6V+vb0dZwHnvLXsGoT8PAtqMDmJCVb8JGI4EB4CK7W3j/X1nln1j+
wvo+eRiU38gbIpHO+T77r3IkMKNzvHGYjr8tupzIPVdr3e3Kvav1h9HScB57
y17BuE/DsLajA5iSdW/CZvDH/2fJs/8jxAxFAgN4J+WD1mh/Zv24vfs0+7VX
+ZJlOcP/KY0EZhRqflt0+dB0OZF7rrbMstuVe1frD6Ol4Tz2lr2CcZ+GYW1H
BzAl696ETYIEBvAaJh36K09A8fd39K8c3OFdZtIOoPz3Lbsjl/XNSBo8LxmM
C0zb+qNdHAnMSBz/W7CSi78uupzIPVdbZvdEZp0O4l2tP/yoGhrOY2/ZKxj3
aRjWdnQAU7LuTdg8SGAAL2Haob92Jxky62jIjCT+vrrEtB1gzIuQr5weoJmp
W3/kcIgEZrhhN0vFIklD40oXm0iXE7nlakfolVbpIN7V+mMcVF3DeewtewXD
Pg3j2o4OYEYWvgmbCQkM4A3MPPRXeTMf8Q/o4A9wk9fKzB1gbHIP4AuWJqZu
/VEX4iaBGWZIo4z8yzD0u7HWj06XE+l/tcOeuCzVQbyr9Yf+uMaG89hb9grG
fBqGth0dwHSsfRM2lT/934un/X/BZigSGGAUnh4+2jdwRY2Bf8xQTEEZ7fma
7apdYvIOoOYd8DCumblb38FGaDS6PWVLOrgXuj47X5cT6Xu1o3zvt1AH8a7W
l7nScB57y17B85+G4W1HBzAVbsLmQgIDmJsBw0fVPr7df53853PUC+NZTDsv
6ABOvAk6gHPM3vrqox6n+otvUv2cofxUtfzG6HIi/a52+Pf+vqmqiuNb4q7g
Xa0vfsDFhvPYW/YKnv00TNF2dADT4CZsQiQwgHkZNHy0rG/pThX8B4489uO1
g38ohwNe0gHGxiAoDbyh9Ulg1HPiKUPRhBo+NF1OpOPVDu0kfV/87V5lhS//
3tX6ww+loeE89pa9goc/DTO0HR3APLgJm9Cf/6+vp//TCTMUCQygYPjwUWeM
OQiGB3CneVMHGDOCG3uUFIbwmtYngVHPmW+7Lj2Q6HIiHa92cBkof8W8RYt9
+feu1hdcbDiPvWWv4OFPwwxtRwcwD27CJkQCA5iNKYaPOsfxRi1sheMc7+sA
jltYyzY+1uV9rT/mCWj2PorPe+wboVM/7I4T6Xm10v/sN3zRKGt9+feu1h/8
n11oOI+9Za/g6U/DBG1HBzARbsImRAIDmI0Zho+6cM1DV50IZ36v07ywAwxp
f3Yjd4UXtv5Yl8Ivfh/nPm4XvhrrciL3X621vvx7V+vr8hqPvWWvYI5Pw6Nt
Z463DAf40A1FAgOYjRmGj7pw1aPhBq+WV3aAIYuPjIT8RR2vbP0RkcAM4dyX
XX2ewbWfyP1Xa60v/97V+rq8xmNv2SuY49PwaNuZ4y3DAT50Qz32fy564b8d
NUORwACCCYaPar3wURev4PHbCV7aARytXUL7q+alre/o8Hy3+8XpSV/a0zFd
TuSBq7XUl3/van3Vr3F0Bo+9Za9gkk/Dk21nkrcMByx1EzYfEhjAO3jJ0N/+
zW79Bv/yjvQ0CS28owP0TztSBDG0vz68o/WHIoFR8q5ncPM8yPsG72p91a8h
gVHLJJ8GCQyULHUTNh8SGMA7MPTX4ugAVkbrezvvegY3z4O8b/Cu1lf9GhIY
tUzyaZDAQMlSN2Hz+csDX8546JAZigQGcISvPRZHB7AyWt/bedczuHke5H2D
d7W+6tdwL1LLJJ8GCQyULHUTNh8SGMA78LXH4ugAVkbrez3vWovVPAu6+gbv
an1dXsN4PCVzfBqsxAslPnRDkcAA3oFHMIujA1gZre/1nPqwnWxI/U/k/qu1
1pd/72p9tWfQb00qV96yVzDHp+HRtjPHW4YD1roJm87jD3z+6kMHzFAkMIAj
PIJZHB3Aymh9r+exbap0OZH7r9ZaX/69q/UdUttwZtqXzXjm+DQ82nbmeMtw
wFo3YdMhgQG8A0N/LY4OYGW0vvdz5tN2KRvT5URuv1qLdT7van1dTuCxt+wV
TPFpeLbtTPGW4YDFbsJmQwIDeAeG/locHcDKaH3v58TH7drjiC4ncvvVWuzL
v3e1vspXOT6+x96yVzDFp+HZtjPFW4YDFrsJmw0JDOAdGPprcXQAK6P1fYCi
EVXnaJq+GOtyIndfrdW+/HtX66t7mfrHave+Za9ghk/Dw21nhrcMB6x2EzaZ
8Q98NuuhvWaoRYsW5YzC6A8JMAGG/locHcDKaH1foHzOGP0TV/7Lscbvxbqc
yM1Xa7kv/97V+qoOP9bLPPaWvYIJPg1Pt50J3jIcsNxN2FzMk8AwBgZwhKG/
FkcHsDJa3ycoH0h+5Tdi+4T7K55GNH8t1uVE7r1a6335967WV3N0Z0Je975l
r2D8p+HxtjP+LcMB692ETYUEBvAODP21ODqAldH6vmHIv/aKZ5Ky++FXWdnQ
ZxGXgjFdTuTWq7Xgl3/vav3RONVwHnvLXsHwT8PzbWf4W4YDFrwJm8nEBz59
86EgMxQJDOAIQ38tjg5gZbS+r7B/Ihnd6I2nqon1OJE+BxnzTVin/3lX6495
fjUN58Yr8UIe6wBjvtRjbUcHMC9L3oTNgwQG8A4M/bU4OoCV0fo+ZEgDjcZR
o6h9BHP5RLodxPFxLdUBvav1HR9X5avddSXeyWMdwPFLPdl2dACTsupN2CRI
YADvwNBfi6MDWBmt71PKwh3+u/BYTaL+EczFE+l4EEdvwmJf/r2r9R2dX3XD
uedKvJXHOoCjl3q27egApmTdm7ApkMAA3oGhvxZHB7AyWt/3lElPJf5+yoby
8/MPHAzLRufkI5j2E+l9kFHehBW//HtX649yfucaTvcr8Woe6wCjvNSAtqMD
mI3Fb8JGe+KBj995aIcZigQGcIShvxZHB7AyWh8AAMAnkMAA3oGhvxZHB7Ay
Wh8AAMAnkMAA3oGhvxZHB7AyWh8AAMAnPPnA3Pce/MkMRQIDAAAAAAB8FQkM
AAAAAACAu5HAAAAAAAAAuBsJDAAAAAAAgLs9/cCHHz641QxFAgMAAAAAAHwV
CQwAAAAAAIC7kcAAAAAAAAC429QH/Oc9uNkMRQIDAAAAAAB8FQkMAAAAAACA
u5HAAAAAAAAAuBsJDAAAAAAAgLtNe+C9Tx/caIZyWwLT0GALDwoI8L8vICAo
3NbQ4J6TAQAAAAAADOfjCUyDLWgwehkqIMhGCgMAAAAAADzCpxMYW9Ao6ct9
QTa9TwkAAAAAADDc8w+8+/mD68xQuicwivylf97RwH/dYAsPIIMBAAAAAACe
5LMJTMNgzjJCyjKYzgSEMxcJAAAAAAC4ma8mMIMBzCijXOQMhggGAAAAAAC4
24sPvL3wwdVmKF0TmDEDGCIYAAAAAADgOT6awKiJV4hgAAAAAACAh/hmAqNi
CIzKHwIAAAAAAHCdbyYwqoa3kMAAAAAAAAAPmfF/vPn1r1eYodySwDjMVpiG
BAAAAAAAPMMnExiVo1tIYAAAAAAAgGeQwJDAAAAAAAAAdztrGosWLcpRZ8w3
RQIDAAAAAADgdqwDAwAAAAAA4G7shQQAAAAAAOBucriiKoFhCAwAAAAAAIAG
aqYhMQkJAAAAAADAFSpmGBHAAAAAAAAAuGbMCEYOYFgFBgAAAAAAQKPBCMY/
yDZskMtg/sIIGAAAAAAAAO0GUxZ//4DBFKbBNpjNMAAGAAAAAADARcoMZkTk
LwAAAAAAAC5rsAUFjJK+BIwwOQkAAAAAAACaNDTYwoMCBoOYgIC+8IX0BQAA
AAAAAAAAAAAAAAAAAPAiZeGB/v5+fn6/UpL+fz//wPCyMqOvDgAAAAAAwLuV
BfoPzV1G4ucfTgwDAAAAAACgRVng2OGLzD/c6MsFAAAAAADwOnb5S984F8VA
l7K+iUl+ZDAAAAAAAADaDc1fRg1Xwv2JYAAAAAAAALQZEsA4TFYUGYxfIKvB
AAAAAAAAqKU+gFH+KAkMAAAAAACAU/oWegn09/Mbc26RYhAM05AAAAAAAADc
ggQGAAAAAADAPcrKysLDA/2HbodEAgMAAAAAAKBVf9oS6O/v7yf51ehIYAAA
AAAAAJxWFu7vMHIhgQEAAAAAAHDFkP2QhvPz8/MPDAwvYx0YAAAAAAAAjRS5
ykDi4t8XuISHl5WVjfqTJDAAAAAAAACqDRn/4tc30kXVj5LAAAAAAAAAqOVM
qqIcLEMCAwAAAAAAoJYTE4uGjJYhgQEAAAAAAFBLkcD4BTqYgWS/Wi8JDAAA
AAAAgFrqBrYM3yyJBAYAAAAAAEC1oeGKn39guGIHpLKy8EB/v5G2qnY8YAYA
AAAAAABDDB/gMjK/wPDBnySBAQAAAAAAcM5YIYyfv9iluowIBgAAAAAAwAXD
Zxz5+fn5981KUvyQ6oV7AQAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAEv4/wFSqGtX
    "], {{0, 875}, {1494, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  DefaultBaseStyle->"ImageGraphics",
  ImageSize->Large,
  ImageSizeRaw->{1494, 875},
  PlotRange->{{0, 1494}, {0, 875}}]], "Output",
 CellChangeTimes->{
  3.7670039472590375`*^9, {3.767004133510419*^9, 3.76700414049401*^9}, 
   3.767004394025344*^9, {3.7670046765721254`*^9, 3.7670047239881277`*^9}, {
   3.76700496937527*^9, 3.7670049744089127`*^9}, {3.7670051204966173`*^9, 
   3.767005146198245*^9}, {3.7670051982943273`*^9, 3.7670052269702873`*^9}, 
   3.7670702379066343`*^9, 3.7673473269486847`*^9, {3.7673473687454495`*^9, 
   3.767347376041238*^9}, {3.7673474407354*^9, 3.767347493060093*^9}, 
   3.7673655745926905`*^9, 3.7673656070866795`*^9, 3.7673657711246786`*^9, {
   3.7673661678721113`*^9, 3.767366202294535*^9}, {3.767422203539076*^9, 
   3.7674222317761064`*^9}, {3.767422268805151*^9, 3.767422300409484*^9}, 
   3.7674225290090213`*^9, 3.7674227658286467`*^9, 3.796457540474828*^9, {
   3.7964577834092903`*^9, 3.7964577952698708`*^9}, 3.796457857045063*^9, 
   3.7964607011599493`*^9, 3.7964608191889725`*^9, 3.796460894415992*^9, 
   3.796461569594681*^9, 3.7964618206150265`*^9, {3.7964786295981073`*^9, 
   3.7964786403866806`*^9}, 
   3.7964788168445625`*^9},ExpressionUUID->"2e39dcf0-499a-4afa-843a-\
480431255b13"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
    "The", " ", "initial", " ", "values", " ", "of", " ", "the", " ", 
     "potential"}], " ", "//", " ", 
    RowBox[{"fix", " ", "normalization"}]}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"\[Phi]iv", "=", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"\[Phi]", "[", "a", "]"}], "/.", 
        RowBox[{
         RowBox[{"allsols", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "[", 
         RowBox[{"[", "1", "]"}], "]"}]}], "/.", 
       RowBox[{"a", "\[Rule]", "ap"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "1", ",", 
        RowBox[{"Length", "[", "kv", "]"}]}], "}"}]}], "]"}]}], 
   ";"}]}]], "Input",
 CellChangeTimes->{{3.7674227337281446`*^9, 3.7674227931822314`*^9}, {
  3.796461832169712*^9, 
  3.7964618527819977`*^9}},ExpressionUUID->"b508c1ff-7c91-49bd-bf9b-\
ec77793ce40c"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Rasterize", "[", 
  RowBox[{
   RowBox[{"LogLogPlot", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"Table", "[", 
        RowBox[{
         FractionBox[
          RowBox[{
           RowBox[{"\[Phi]", "[", "a", "]"}], "/.", 
           RowBox[{
            RowBox[{"allsols", "[", 
             RowBox[{"[", "i", "]"}], "]"}], "[", 
            RowBox[{"[", "1", "]"}], "]"}]}], 
          RowBox[{"\[Phi]iv", "[", 
           RowBox[{"[", "i", "]"}], "]"}]], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "1", ",", 
           RowBox[{"Length", "[", "kv", "]"}]}], "}"}]}], "]"}], "//", 
       "Abs"}], "//", "Evaluate"}], ",", 
     RowBox[{"{", 
      RowBox[{"a", ",", "ap", ",", "1"}], "}"}], ",", 
     RowBox[{"Frame", "\[Rule]", "True"}], ",", 
     RowBox[{"FrameStyle", "\[Rule]", "Black"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<a\>\"", ",", 
        "\"\<\[Phi](a)/\!\(\*SubscriptBox[\(\[Phi]\), \(i\)]\)\>\""}], 
       "}"}]}], ",", 
     RowBox[{"BaseStyle", "\[Rule]", 
      RowBox[{"FontSize", "\[Rule]", "12"}]}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", 
      RowBox[{"1", "/", "GoldenRatio"}]}], ",", 
     RowBox[{"GridLines", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"PlotRange", "\[Rule]", "All"}], ",", 
     RowBox[{"Epilog", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Black", ",", "Dashed", ",", 
        RowBox[{"Line", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"aeq", ",", 
              SuperscriptBox["10", 
               RowBox[{"-", "6"}]]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"aeq", ",", 
              SuperscriptBox["10", "6"]}], "}"}]}], "}"}], "//", "Log"}], 
         "]"}]}], "}"}]}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"ColorData", "[", 
         RowBox[{"\"\<Rainbow\>\"", ",", 
          RowBox[{"i", "/", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Length", "[", "kv", "]"}], "-", "1"}], ")"}]}]}], "]"}],
         ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "0", ",", 
          RowBox[{
           RowBox[{"Length", "[", "kv", "]"}], "-", "1"}]}], "}"}]}], "]"}]}],
      ",", 
     RowBox[{"PlotLegends", "\[Rule]", 
      RowBox[{"BarLegend", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"\"\<Rainbow\>\"", ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", "10"}], "}"}]}], "}"}], ",", 
        RowBox[{"LegendLabel", "\[Rule]", "\"\<k [h/Mpc]\>\""}]}], "]"}]}]}], 
    "]"}], ",", 
   RowBox[{"RasterSize", "\[Rule]", "1500"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "Large"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7625168700408125`*^9, 3.7625169021553583`*^9}, {
   3.76251712800572*^9, 3.762517148948697*^9}, {3.7670005014731812`*^9, 
   3.767000615088724*^9}, {3.767000691624955*^9, 3.7670006948145638`*^9}, {
   3.767003428915862*^9, 3.7670034299534373`*^9}, {3.7670036997674494`*^9, 
   3.767003699961059*^9}, {3.767003819508198*^9, 3.7670038414614596`*^9}, {
   3.767003967197608*^9, 3.7670039726001277`*^9}, {3.7670041594627047`*^9, 
   3.767004196915502*^9}, {3.7670047478141136`*^9, 3.7670048820931873`*^9}, 
   3.767004985508012*^9, {3.767005241202551*^9, 3.7670052659092054`*^9}, {
   3.767347501035697*^9, 3.7673475012757664`*^9}, {3.7673656172377315`*^9, 
   3.7673657122845483`*^9}, 3.7673663588692036`*^9, {3.76742260568736*^9, 
   3.7674226170464115`*^9}, {3.7674228028621917`*^9, 3.767422817001612*^9}, 
   3.7964574371099877`*^9, {3.796461622531852*^9, 3.796461636035144*^9}, 
   3.7964786535227766`*^9},ExpressionUUID->"af61a2ec-71cf-4b3e-9fdb-\
6d90c4d37a21"],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzsvXl0E1ee953zvuc55/3z/W+eeWZ5p5fpyXTPJN0znXSH7nQnNATSdMgG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    "], {{0, 875}, {1494, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  DefaultBaseStyle->"ImageGraphics",
  ImageSize->Large,
  ImageSizeRaw->{1494, 875},
  PlotRange->{{0, 1494}, {0, 875}}]], "Output",
 CellChangeTimes->{{3.762516888390074*^9, 3.762516903070718*^9}, 
   3.762516960795412*^9, {3.7625171180054917`*^9, 3.7625171498777895`*^9}, {
   3.7670005204798765`*^9, 3.767000615576269*^9}, {3.7670006532748256`*^9, 
   3.7670006954236865`*^9}, 3.767001170557514*^9, 3.7670012317129374`*^9, 
   3.7670017480307236`*^9, {3.7670034230535975`*^9, 3.7670034530829163`*^9}, 
   3.767003499376281*^9, 3.7670035413111963`*^9, 3.7670035792949905`*^9, {
   3.7670036953937397`*^9, 3.767003700561793*^9}, 3.7670037641230774`*^9, {
   3.76700382285056*^9, 3.7670038419693527`*^9}, {3.7670039565293765`*^9, 
   3.767003973557585*^9}, {3.7670041524422994`*^9, 3.7670041979513736`*^9}, 
   3.7670043983575234`*^9, {3.767004772681619*^9, 3.7670048095278296`*^9}, {
   3.767004870628583*^9, 3.7670048833715267`*^9}, {3.767004980671635*^9, 
   3.7670050009992943`*^9}, {3.7670052425353823`*^9, 3.7670052672290325`*^9}, 
   3.7670702444643064`*^9, 3.767347503540399*^9, 3.7673656356862073`*^9, {
   3.767365666467109*^9, 3.7673657138171115`*^9}, {3.767366354651165*^9, 
   3.7673663601578007`*^9}, 3.7674228321145506`*^9, {3.7964616425243435`*^9, 
   3.7964616491090317`*^9}, 3.7964618659663167`*^9, 3.7964786581546993`*^9, 
   3.796478828795741*^9},ExpressionUUID->"a1b5ed08-b8c9-485e-a552-\
d43a145c7ac6"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"The", " ", "transfer", " ", "function", " ", "T", 
    RowBox[{"(", "k", ")"}]}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"\[Phi]largelate", "=", 
     FractionBox[
      RowBox[{
       RowBox[{
        RowBox[{"\[Phi]", "[", "a", "]"}], "/.", 
        RowBox[{
         RowBox[{"allsols", "[", 
          RowBox[{"[", "1", "]"}], "]"}], "[", 
         RowBox[{"[", "1", "]"}], "]"}]}], "/.", 
       RowBox[{"a", "\[Rule]", "1"}]}], 
      RowBox[{
       RowBox[{
        RowBox[{"\[Phi]", "[", "a", "]"}], "/.", 
        RowBox[{
         RowBox[{"allsols", "[", 
          RowBox[{"[", "1", "]"}], "]"}], "[", 
         RowBox[{"[", "1", "]"}], "]"}]}], "/.", 
       RowBox[{"a", "\[Rule]", "ap"}]}]]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Tk", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          FractionBox[
           RowBox[{"kv", "[", 
            RowBox[{"[", "i", "]"}], "]"}], "3000"], "//", "N"}], ",", 
         RowBox[{"(", 
          FractionBox[
           FractionBox[
            RowBox[{
             RowBox[{
              RowBox[{"\[Phi]", "[", "a", "]"}], "/.", 
              RowBox[{
               RowBox[{"allsols", "[", 
                RowBox[{"[", "i", "]"}], "]"}], "[", 
               RowBox[{"[", "1", "]"}], "]"}]}], "/.", 
             RowBox[{"a", "\[Rule]", "1"}]}], 
            RowBox[{
             RowBox[{
              RowBox[{"\[Phi]", "[", "a", "]"}], "/.", 
              RowBox[{
               RowBox[{"allsols", "[", 
                RowBox[{"[", "i", "]"}], "]"}], "[", 
               RowBox[{"[", "1", "]"}], "]"}]}], "/.", 
             RowBox[{"a", "\[Rule]", "ap"}]}]], "\[Phi]largelate"], ")"}]}], 
        "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", 
         RowBox[{"Length", "[", "kv", "]"}]}], "}"}]}], "]"}]}], 
    ";"}]}]}]], "Input",
 CellChangeTimes->{{3.796462895955121*^9, 3.7964629579590936`*^9}, {
  3.796463070478815*^9, 
  3.7964631383173065`*^9}},ExpressionUUID->"c962bd63-cdc8-4022-ad22-\
a551b42f29dc"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"BBKS", " ", "formula", " ", "of", " ", 
    RowBox[{"Eq", ".", " ", 
     RowBox[{"(", "7.71", ")"}]}], " ", "Dodelson"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"See", " ", "also", " ", 
     RowBox[{"https", ":"}]}], "//", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"arxiv", ".", "org"}], "/", "pdf"}], "/", "astro"}], "-", 
     RowBox[{
      RowBox[{"ph", "/", "0610532."}], "pdf"}]}]}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"T", "[", "q_", "]"}], ":=", 
   RowBox[{
    FractionBox[
     RowBox[{"Log", "[", 
      RowBox[{"1", "+", 
       RowBox[{
        FractionBox["234", "100"], " ", "q"}]}], "]"}], 
     RowBox[{
      FractionBox["234", "100"], " ", "q"}]], "*", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "+", 
       RowBox[{
        FractionBox["389", "100"], " ", "q"}], "+", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FractionBox["162", "10"], "q"}], ")"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FractionBox["547", "100"], " ", "q"}], ")"}], "3"], "+", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FractionBox["671", "100"], "q"}], ")"}], "4"]}], ")"}], 
     RowBox[{
      RowBox[{"-", "1"}], "/", "4"}]]}]}]}]], "Input",
 CellChangeTimes->{{3.7471620480693283`*^9, 3.747162160283125*^9}, {
   3.747209831869811*^9, 3.747209832367145*^9}, 3.7474063779904537`*^9, {
   3.74740642705599*^9, 3.747406431954713*^9}, {3.767423108274251*^9, 
   3.76742311237904*^9}},ExpressionUUID->"a2d02dc6-bc31-4f16-be53-\
bc9f052e7e14"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TBBKSnum", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       FractionBox[
        RowBox[{"kv", "[", 
         RowBox[{"[", "i", "]"}], "]"}], "3000"], ",", 
       RowBox[{"T", "[", 
        FractionBox[
         RowBox[{"kv", "[", 
          RowBox[{"[", "i", "]"}], "]"}], 
         RowBox[{"3000", " ", "om", " ", "h"}]], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       RowBox[{"Length", "[", "kv", "]"}]}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.796463287561076*^9, 
  3.79646340517665*^9}},ExpressionUUID->"e0e99587-94b3-4a84-a196-\
e12826bc39ce"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ListLogLogPlot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"Tk", ",", "TBBKSnum"}], "}"}], ",", 
   RowBox[{"Joined", "\[Rule]", "True"}], ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<k [h/Mpc]\>\"", ",", "\"\<T(k)\>\""}], "}"}]}], ",", 
   RowBox[{"FrameStyle", "\[Rule]", "Black"}], ",", 
   RowBox[{"BaseStyle", "\[Rule]", 
    RowBox[{"FontSize", "\[Rule]", "12"}]}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"Black", ",", 
      RowBox[{"{", 
       RowBox[{"Black", ",", "Dashed"}], "}"}]}], "}"}]}], ",", 
   RowBox[{"PlotLegends", "\[Rule]", 
    RowBox[{"LineLegend", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"Black", ",", 
        RowBox[{"{", 
         RowBox[{"Black", ",", "Dashed"}], "}"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\"\<T(k)\>\"", ",", "\"\<BBKS\>\""}], "}"}]}], "]"}]}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.7964631430104885`*^9, 3.79646315332225*^9}, {
  3.796463221232972*^9, 3.796463239243848*^9}, {3.796463395739915*^9, 
  3.7964634489515924`*^9}, {3.7964638711984377`*^9, 3.796463881387824*^9}, {
  3.7964639258123207`*^9, 
  3.796463944930564*^9}},ExpressionUUID->"1a868f70-a54c-4399-b76e-\
e0f96159e429"],

Cell[BoxData[
 TemplateBox[{GraphicsBox[{{}, {{{}, {}, {
        Hue[0.67, 0.6, 0.6], 
        Directive[
         PointSize[0.012833333333333334`], 
         AbsoluteThickness[1.6], FontSize -> 12, 
         GrayLevel[0]], 
        LineBox[CompressedData["
1:eJxTTMoPSmViYGAQBWIQveB//8yuhdIHGKBArnz65R8nJQ/ExPAnv2kI2T/n
3Tze9E/iB9q996h+3Ze/XyptmftVKbEDkyO0pJ9oNu+fcXdto7OzyIFTN9Zf
Xvh4yn7R0K27N2YLHXipe7qYOWXp/sln9nyVnyJwwO1cQO7aU1v2K3V0xMtU
8h5ImtNiUm98eD/EPs4D5rz5nV/OX4Caz3bgQMGjjObrt6HmMR0Qunz30+29
T/crg/X/3c93/N/ve//fQ9V/219wrODr3OZv+1XA8m/3O5eyvJ/Q9Q9q3939
CtePJ7k/YYb789Mhj7CVvzgOQNTftd+p/VKXMYvvAET9W/tNiRdK3nMKHYCa
b6/4jynj0TORA1D77W8b/jqwa4n4ATGI+xyCtjZkRj2RPAAAhoGpsw==
         "]]}, {
        Hue[0.9060679774997897, 0.6, 0.6], 
        Directive[
         PointSize[0.012833333333333334`], 
         AbsoluteThickness[1.6], FontSize -> 12, 
         GrayLevel[0], 
         Dashing[{Small, Small}]], 
        LineBox[CompressedData["
1:eJxTTMoPSmViYGAQBWIQveB//8yuhdIHHG68zLgrm7Vfrnz65R8nJQ98cnfd
sFandP+cd/N40z+JH5h/q+QpV0Ljfqm0Ze5XpcQOPD/DmF7C1Lt/xt21jc7O
IgcYFxplhp+YsV80dOvujdlCB9zC2XQ5jizdP/nMnq/yUwQOnLRg7mBZuHm/
UkdHvEwl74EzGzSfLX1zEGof54FChxJHGeELUPPZDkx5uHHxjIDbUPOYDljz
pdtOm/50vzJY/9/9iwVuXur7+R6q/tv+y/Mmdbxo+QY1/+1+j72t8lJF/6D8
u/tnz/Ov7zvPfIABCkrjGTQdn3AcUITI27/fnJ1dEsF3AKrf/tTlWaLvuYQO
QM23t/1brhrLJgqV/2vvlcK5YaeExAExiPsc2LlVQ+faSh8AAHQ/ob4=
         "]]}}}, {}, {}, {}, {}}, {
    DisplayFunction -> Identity, GridLines -> {None, None}, DisplayFunction -> 
     Identity, DisplayFunction -> Identity, DisplayFunction -> Identity, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.05]}}, 
     AxesOrigin -> {-7.246547096050271, -7.304457056455679}, 
     PlotRange -> {{-7.099637370064974, 
      2.302585092994046}, {-7.188513293654795, 0.}}, PlotRangeClipping -> 
     True, ImagePadding -> All, DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True}, 
     AxesLabel -> {None, None}, 
     AxesOrigin -> {-7.246547096050271, -7.304457056455679}, 
     BaseStyle -> (FontSize -> 12), DisplayFunction :> Identity, 
     Frame -> {{True, True}, {True, True}}, FrameLabel -> {{
        FormBox["\"T(k)\"", TraditionalForm], None}, {
        FormBox["\"k [h/Mpc]\"", TraditionalForm], None}}, FrameStyle -> 
     GrayLevel[0], FrameTicks -> {{
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}, {
        Charting`ScaledTicks[{Log, Exp}], 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, GridLines -> {None, None}, 
     GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], 
     Method -> {"CoordinatesToolOptions" -> {"DisplayFunction" -> ({
           (Exp[#]& )[
            Part[#, 1]], 
           (Exp[#]& )[
            Part[#, 2]]}& ), "CopiedValueFunction" -> ({
           (Exp[#]& )[
            Part[#, 1]], 
           (Exp[#]& )[
            Part[#, 2]]}& )}}, 
     PlotRange -> {{-7.099637370064974, 
      2.302585092994046}, {-7.188513293654795, 0.}}, PlotRangeClipping -> 
     True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.05]}}, Ticks -> {
       Charting`ScaledTicks[{Log, Exp}], 
       Charting`ScaledTicks[{Log, Exp}]}}],FormBox[
    FormBox[
     TemplateBox[{"\"T(k)\"", "\"BBKS\""}, "LineLegend", 
      DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    AbsoluteThickness[1.6], 
                    GrayLevel[0]], {
                    LineBox[{{0, 10}, {40, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    AbsoluteThickness[1.6], 
                    GrayLevel[0]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {40, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    AbsoluteThickness[1.6], 
                    GrayLevel[0], 
                    Dashing[{Small, Small}]], {
                    LineBox[{{0, 10}, {40, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    AbsoluteThickness[1.6], 
                    GrayLevel[0], 
                    Dashing[{Small, Small}]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {40, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               InterpretationBox[
                ButtonBox[
                 TooltipBox[
                  GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                   "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                   FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                   PlotRangePadding -> None, ImageSize -> 
                   Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], "GrayLevel[0]"], 
                 Appearance -> None, BaseStyle -> {}, BaselinePosition -> 
                 Baseline, DefaultBaseStyle -> {}, ButtonFunction :> 
                 With[{Typeset`box$ = EvaluationBox[]}, 
                   If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                 Automatic, Method -> "Preemptive"], 
                GrayLevel[0], Editable -> False, Selectable -> False], ",", 
               RowBox[{"{", 
                 RowBox[{
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> GrayLevel[0.], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], "GrayLevel[0]"], 
                    Appearance -> None, BaseStyle -> {}, BaselinePosition -> 
                    Baseline, DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    GrayLevel[0], Editable -> False, Selectable -> False], 
                   ",", 
                   RowBox[{"Dashing", "[", 
                    RowBox[{"{", 
                    RowBox[{"Small", ",", "Small"}], "}"}], "]"}]}], "}"}]}], 
             "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2}], "}"}], ",", 
           RowBox[{"Joined", "\[Rule]", 
             RowBox[{"{", 
               RowBox[{"True", ",", "True"}], "}"}]}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", 
             RowBox[{"{", 
               RowBox[{
                 RowBox[{"{", 
                   RowBox[{"False", ",", "Automatic"}], "}"}], ",", 
                 RowBox[{"{", 
                   RowBox[{"False", ",", "Automatic"}], "}"}]}], "}"}]}]}], 
         "]"}]& ), Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.796477678202487*^9, 3.796478667866559*^9, {3.7964788465533543`*^9, 
   3.7964788491491604`*^9}},ExpressionUUID->"0e28bb88-3d7d-4a78-b32c-\
4b51cf906cd5"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ListLogLinearPlot", "[", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       FractionBox[
        RowBox[{"kv", "[", 
         RowBox[{"[", "i", "]"}], "]"}], "3000"], ",", 
       RowBox[{"100", 
        RowBox[{"(", 
         RowBox[{"1", "-", 
          FractionBox[
           RowBox[{"Tk", "[", 
            RowBox[{"[", 
             RowBox[{"i", ",", "2"}], "]"}], "]"}], 
           RowBox[{"T", "[", 
            FractionBox[
             RowBox[{"kv", "[", 
              RowBox[{"[", "i", "]"}], "]"}], 
             RowBox[{"3000", " ", "om", " ", "h"}]], "]"}]]}], ")"}]}]}], 
      "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       RowBox[{"Length", "[", "kv", "]"}]}], "}"}]}], "]"}], ",", 
   RowBox[{"Joined", "\[Rule]", "True"}], ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
     "\"\<k [h/Mpc]\>\"", ",", 
      "\"\<% diff T(k) vs \!\(\*SubscriptBox[\(T\), \(BBKS\)]\)(k)\>\""}], 
     "}"}]}], ",", 
   RowBox[{"FrameStyle", "\[Rule]", "Black"}], ",", 
   RowBox[{"BaseStyle", "\[Rule]", 
    RowBox[{"FontSize", "\[Rule]", "12"}]}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", "Black"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "2"}], ",", "2"}], "}"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7964634621576986`*^9, 
  3.7964635979697404`*^9}},ExpressionUUID->"5843d70c-5137-458c-8417-\
12de12381046"],

Cell[BoxData[
 GraphicsBox[{{}, {{}, {}, 
    {GrayLevel[0], PointSize[0.012833333333333334`], AbsoluteThickness[1.6], 
     StyleBox[LineBox[CompressedData["
1:eJxTTMoPSmViYGAQBmIQveB//8yuhdIHFuy23Lcq/8p+ufLpl3+clDwQ4HRx
D5fCzf1z3s3jTf8kfmBCcO/F+Al390ulLXO/KiV2oGB32PWd1+/tn3F3baOz
s8gBgehjxV6JN/aLhm7dvTFb6MAB1b+l6w5t2j/5zJ6v8lMEDvx4eHPTPIa7
9kodHfEylbwHGn5lzpt16L09xD7OA3WJ59JZwj7ZQ8xnOzDlBdsMdbtX9hDz
mA5UPImpmX3lrr0yWP/f/Q5L97b07dgMVf9tf8PTV6+uqizYDzH/7f4F8Ze/
2fA9htp3dz+XUJxyj/QHewYouFK6MuBbxkd7RYi8/Y3AOy8/+r6Gqn9r33Dw
xZaLP3fsP3s3y0Eq/jNM3wEAXuabnQ==
       "]],
      FontSize->12]}}, {}, {}, {}, {}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{True, True},
  AxesLabel->{None, None},
  AxesOrigin->{-7.246547096050271, 0},
  BaseStyle->(FontSize -> 12),
  DisplayFunction->Identity,
  Frame->{{True, True}, {True, True}},
  FrameLabel->{{
     FormBox[
     "\"% diff T(k) vs \\!\\(\\*SubscriptBox[\\(T\\), \\(BBKS\\)]\\)(k)\"", 
      TraditionalForm], None}, {
     FormBox["\"k [h/Mpc]\"", TraditionalForm], None}},
  FrameStyle->GrayLevel[0],
  FrameTicks->FrontEndValueCache[{{Automatic, Automatic}, {
      Charting`ScaledTicks[{Log, Exp}], 
      Charting`ScaledFrameTicks[{Log, Exp}]}}, {{
     Automatic, Automatic}, {{{-6.907755278982137, 
        FormBox["0.001`", TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {-4.605170185988091, 
        FormBox[
         TagBox[
          InterpretationBox["\"0.010\"", 0.01, AutoDelete -> True], 
          NumberForm[#, {
            DirectedInfinity[1], 3}]& ], TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.3025850929940455`, 
        FormBox[
         TagBox[
          InterpretationBox["\"0.100\"", 0.1, AutoDelete -> True], 
          NumberForm[#, {
            DirectedInfinity[1], 3}]& ], TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {0., 
        FormBox["1", TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {2.302585092994046, 
        FormBox["10", TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {-9.210340371976182, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-8.517193191416238, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-8.111728083308073, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-7.824046010856292, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-7.600902459542082, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-7.418580902748128, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-7.264430222920869, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-7.1308988302963465`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-7.013115794639964, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-6.214608098422191, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-5.809142990314028, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-5.521460917862246, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-5.298317366548036, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-5.115995809754082, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-4.961845129926823, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-4.8283137373023015`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-4.710530701645918, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-3.912023005428146, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-3.506557897319982, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-3.2188758248682006`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.995732273553991, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.8134107167600364`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.659260036932778, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.5257286443082556`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.4079456086518722`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-1.6094379124341003`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-1.2039728043259361`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.916290731874155, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.6931471805599453, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.5108256237659907, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.35667494393873245`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.2231435513142097, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.10536051565782628`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {0.6931471805599453, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.0986122886681098`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.3862943611198906`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.6094379124341003`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.791759469228055, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.9459101490553132`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {2.0794415416798357`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {2.1972245773362196`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {2.995732273553991, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {3.4011973816621555`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {3.6888794541139363`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {3.912023005428146, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.0943445622221, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.248495242049359, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.382026634673881, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.499809670330265, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.605170185988092, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}}, {{-6.907755278982137, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {-4.605170185988091, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.3025850929940455`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {0., 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {2.302585092994046, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {-9.210340371976182, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-8.517193191416238, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-8.111728083308073, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-7.824046010856292, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-7.600902459542082, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-7.418580902748128, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-7.264430222920869, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-7.1308988302963465`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-7.013115794639964, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-6.214608098422191, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-5.809142990314028, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-5.521460917862246, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-5.298317366548036, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-5.115995809754082, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-4.961845129926823, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-4.8283137373023015`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-4.710530701645918, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-3.912023005428146, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-3.506557897319982, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-3.2188758248682006`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.995732273553991, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.8134107167600364`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.659260036932778, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.5257286443082556`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.4079456086518722`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-1.6094379124341003`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-1.2039728043259361`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.916290731874155, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.6931471805599453, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.5108256237659907, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.35667494393873245`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.2231435513142097, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.10536051565782628`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {0.6931471805599453, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.0986122886681098`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.3862943611198906`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.6094379124341003`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.791759469228055, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.9459101490553132`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {2.0794415416798357`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {2.1972245773362196`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {2.995732273553991, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {3.4011973816621555`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {3.6888794541139363`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {3.912023005428146, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.0943445622221, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.248495242049359, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.382026634673881, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.499809670330265, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.605170185988092, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}}}}],
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  ImagePadding->All,
  Method->{"CoordinatesToolOptions" -> {"DisplayFunction" -> ({
        (Exp[#]& )[
         Part[#, 1]], 
        (Identity[#]& )[
         Part[#, 2]]}& ), "CopiedValueFunction" -> ({
        (Exp[#]& )[
         Part[#, 1]], 
        (Identity[#]& )[
         Part[#, 2]]}& )}},
  PlotRange->{{-7.099637370064974, 2.302585092994046}, {-2, 2}},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {0, 0}},
  Ticks->FrontEndValueCache[{
     Charting`ScaledTicks[{Log, Exp}], Automatic}, {{{-6.907755278982137, 
       FormBox["0.001`", TraditionalForm], {0.01, 0.}, {
        AbsoluteThickness[0.1]}}, {-4.605170185988091, 
       FormBox[
        TagBox[
         InterpretationBox["\"0.010\"", 0.01, AutoDelete -> True], 
         NumberForm[#, {
           DirectedInfinity[1], 3}]& ], TraditionalForm], {0.01, 0.}, {
        AbsoluteThickness[0.1]}}, {-2.3025850929940455`, 
       FormBox[
        TagBox[
         InterpretationBox["\"0.100\"", 0.1, AutoDelete -> True], 
         NumberForm[#, {
           DirectedInfinity[1], 3}]& ], TraditionalForm], {0.01, 0.}, {
        AbsoluteThickness[0.1]}}, {0., 
       FormBox["1", TraditionalForm], {0.01, 0.}, {
        AbsoluteThickness[0.1]}}, {2.302585092994046, 
       FormBox["10", TraditionalForm], {0.01, 0.}, {
        AbsoluteThickness[0.1]}}, {-9.210340371976182, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-8.517193191416238, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-8.111728083308073, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-7.824046010856292, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-7.600902459542082, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-7.418580902748128, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-7.264430222920869, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-7.1308988302963465`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-7.013115794639964, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-6.214608098422191, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-5.809142990314028, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-5.521460917862246, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-5.298317366548036, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-5.115995809754082, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-4.961845129926823, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-4.8283137373023015`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-4.710530701645918, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-3.912023005428146, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-3.506557897319982, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-3.2188758248682006`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-2.995732273553991, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-2.8134107167600364`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-2.659260036932778, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-2.5257286443082556`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-2.4079456086518722`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-1.6094379124341003`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-1.2039728043259361`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-0.916290731874155, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-0.6931471805599453, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-0.5108256237659907, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-0.35667494393873245`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-0.2231435513142097, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {-0.10536051565782628`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {0.6931471805599453, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {1.0986122886681098`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {1.3862943611198906`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {1.6094379124341003`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {1.791759469228055, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {1.9459101490553132`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {2.0794415416798357`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {2.1972245773362196`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {2.995732273553991, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {3.4011973816621555`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {3.6888794541139363`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {3.912023005428146, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {4.0943445622221, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {4.248495242049359, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {4.382026634673881, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {4.499809670330265, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}, {4.605170185988092, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
        AbsoluteThickness[0.1]}}}, Automatic}]]], "Output",
 CellChangeTimes->{{3.7964635121935673`*^9, 3.7964635988654003`*^9}, 
   3.7964748496147194`*^9, 3.7964786769482403`*^9, 
   3.796478854330247*^9},ExpressionUUID->"a34ccd26-7afd-41b8-b7b6-\
47398db06aad"]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Delta]\[Zeta]0", "=", 
   RowBox[{
    SqrtBox["23"], 
    SuperscriptBox["10", 
     RowBox[{"-", "5"}]]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"As0", "=", 
   RowBox[{"\[Delta]\[Zeta]0", "^", "2"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ns0", "=", 
   RowBox[{"96", "/", "100"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"kn", "=", 
   RowBox[{
    FractionBox["5", "100"], "*", "3000"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.7673659844458447`*^9, 3.767365984630737*^9}, {
  3.767423227822042*^9, 
  3.7674232288516235`*^9}},ExpressionUUID->"3807af40-d811-4577-956c-\
d05c11b878ee"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Tkint", "=", 
   RowBox[{"Interpolation", "[", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"kv", "[", 
          RowBox[{"[", "i", "]"}], "]"}], ",", 
         RowBox[{"(", 
          FractionBox[
           FractionBox[
            RowBox[{
             RowBox[{
              RowBox[{"\[Phi]", "[", "a", "]"}], "/.", 
              RowBox[{
               RowBox[{"allsols", "[", 
                RowBox[{"[", "i", "]"}], "]"}], "[", 
               RowBox[{"[", "1", "]"}], "]"}]}], "/.", 
             RowBox[{"a", "\[Rule]", "1"}]}], 
            RowBox[{
             RowBox[{
              RowBox[{"\[Phi]", "[", "a", "]"}], "/.", 
              RowBox[{
               RowBox[{"allsols", "[", 
                RowBox[{"[", "i", "]"}], "]"}], "[", 
               RowBox[{"[", "1", "]"}], "]"}]}], "/.", 
             RowBox[{"a", "\[Rule]", "ap"}]}]], "\[Phi]largelate"], ")"}]}], 
        "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", 
         RowBox[{"Length", "[", "kv", "]"}]}], "}"}]}], "]"}], ",", 
     RowBox[{"InterpolationOrder", "\[Rule]", "2"}]}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.7964636455533075`*^9, 3.7964636801138277`*^9}, {
  3.7964788673536253`*^9, 
  3.7964788696594734`*^9}},ExpressionUUID->"c576ae96-4dd7-405e-a96d-\
c124f2f7cf8a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"P\[Psi]", "[", 
   RowBox[{"k_", ",", "As_", ",", "ns_"}], "]"}], ":=", 
  RowBox[{"As", " ", 
   SuperscriptBox[
    RowBox[{"(", 
     FractionBox["k", "kn"], ")"}], 
    RowBox[{"ns", "-", "1"}]]}]}]], "Input",
 CellChangeTimes->{{3.7964636902441597`*^9, 3.7964636949159236`*^9}, {
   3.796475372465112*^9, 3.796475377218424*^9}, {3.7964754094122667`*^9, 
   3.7964754108441386`*^9}, 
   3.796477121941444*^9},ExpressionUUID->"570b440f-da26-4457-9042-\
138d58710465"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dataPk", "=", "\[IndentingNewLine]", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       FractionBox[
        RowBox[{"kv", "[", 
         RowBox[{"[", "i", "]"}], "]"}], "3000"], ",", 
       RowBox[{
        FractionBox["4", "25"], "2", 
        RowBox[{"Pi", "^", "2"}], 
        FractionBox[
         RowBox[{"3000", "^", "3"}], 
         RowBox[{"om", "^", "2"}]], 
        RowBox[{"kv", "[", 
         RowBox[{"[", "i", "]"}], "]"}], 
        RowBox[{"P\[Psi]", "[", 
         RowBox[{
          RowBox[{"kv", "[", 
           RowBox[{"[", "i", "]"}], "]"}], ",", "As0", ",", "ns0"}], "]"}], 
        RowBox[{
         RowBox[{"Tkint", "[", 
          RowBox[{"kv", "[", 
           RowBox[{"[", "i", "]"}], "]"}], "]"}], "^", "2"}]}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", 
       RowBox[{"Length", "[", "kv", "]"}]}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.767422486743251*^9, 3.7674225253840895`*^9}, {
   3.767422855952385*^9, 3.767422863230461*^9}, {3.76742320421427*^9, 
   3.7674232763004*^9}, {3.7674233231092944`*^9, 3.7674233307089033`*^9}, {
   3.7674234287978077`*^9, 3.767423448990245*^9}, {3.796463713721221*^9, 
   3.7964637185232277`*^9}, {3.7964638195919504`*^9, 3.796463828796745*^9}, {
   3.7964747374232783`*^9, 3.7964748079536085`*^9}, 3.79647485512074*^9, {
   3.7964749251570835`*^9, 3.7964749840150895`*^9}, {3.7964753838122787`*^9, 
   3.796475424712181*^9}, {3.796476015583437*^9, 
   3.7964760408655252`*^9}},ExpressionUUID->"f7479ee2-cb76-452b-b126-\
29778f227b10"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"Compare", " ", "with", " ", "Dodelson", " ", "Fig", " ", "7.4"}], 
   " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{"ListLogLogPlot", "[", 
   RowBox[{"dataPk", ",", 
    RowBox[{"Joined", "\[Rule]", "True"}], ",", 
    RowBox[{"Frame", "\[Rule]", "True"}], ",", 
    RowBox[{"FrameLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"\"\<k (h/Mpc)\>\"", ",", "\"\<P(k)\>\""}], "}"}]}], ",", 
    RowBox[{"BaseStyle", "\[Rule]", 
     RowBox[{"FontSize", "\[Rule]", "12"}]}], ",", 
    RowBox[{"FrameStyle", "\[Rule]", "Black"}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", 
     RowBox[{"{", "Black", "}"}]}], ",", 
    RowBox[{"AspectRatio", "\[Rule]", "0.7"}], ",", 
    RowBox[{"PlotRange", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0.001", ",", "10"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", 
         RowBox[{"3", " ", 
          RowBox[{"10", "^", "4"}]}]}], "}"}]}], "}"}]}], ",", 
    RowBox[{"Epilog", "\[Rule]", 
     RowBox[{"Text", "[", 
      RowBox[{"\"\<sCDM\>\"", ",", 
       RowBox[{"Scaled", "[", 
        RowBox[{"{", 
         RowBox[{"0.83", ",", "0.71"}], "}"}], "]"}]}], "]"}]}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.767365858086037*^9, 3.7673658694896836`*^9}, {
   3.7673660720677624`*^9, 3.7673660746249113`*^9}, {3.7673662690740137`*^9, 
   3.7673663321391687`*^9}, {3.767422903679826*^9, 3.7674229173091516`*^9}, {
   3.7674229658281207`*^9, 3.7674229800678663`*^9}, {3.7674234030985403`*^9, 
   3.7674234142678585`*^9}, {3.796463747922839*^9, 3.7964637512754393`*^9}, {
   3.796463802555925*^9, 3.796463805522716*^9}, {3.796475089373351*^9, 
   3.7964751331447954`*^9}, {3.796475165633068*^9, 3.7964751873119097`*^9}, 
   3.796475685708765*^9, {3.796475901928466*^9, 3.7964759787508745`*^9}, {
   3.7964760611664867`*^9, 3.7964763425034094`*^9}, {3.796476433879326*^9, 
   3.7964764658492517`*^9}, {3.796478193230129*^9, 3.796478274337634*^9}, {
   3.796478341934378*^9, 3.7964783468126855`*^9}, {3.796478919752075*^9, 
   3.79647895114255*^9}},ExpressionUUID->"7fca1c09-28f0-42b2-8a79-\
d4650e6008b1"],

Cell[BoxData[
 GraphicsBox[{{}, {{}, {}, 
    {GrayLevel[0], PointSize[0.012833333333333334`], AbsoluteThickness[1.6], 
     StyleBox[LineBox[CompressedData["
1:eJxTTMoPSmViYGAQBWIQveB//8yuhdIHvF60X7mpJOUgVz798o+TkgeOz/g3
998TaYc57+bxpn8SP8A+z+Vp8BJZB6m0Ze5XpcQOaH8tXXouVt5hxt21jc7O
IgdkM8vEPrcqOIiGbt29MVvowDnuJyqZQYoOk8/s+So/ReCA6ovpLxN4lByU
OjriZSp5D6zv15L0WqEEtY/zwGefnbaO4spQ89kOvJ/vfFjJUxlqHtOBs1O/
v7M0VHZQBuv/u7+JIZQx5LQSVP23/X8Mfl07BDf/7f7qc2I9CW8VoPy7+x8I
vRYuZ5B3YICCMstc9lPrpB0UIfL2Eo/Pi6mzSMD021+NlXb6JycCM98+0Xyn
01lNAaj8X/uVR2a/X/SUw0EM4j6H+bfbpfsKmB0A3kaUaA==
       "]],
      FontSize->12]}}, {}, {}, {}, {}},
  AspectRatio->0.7,
  Axes->{True, True},
  AxesLabel->{None, None},
  AxesOrigin->{-6.861703577122256, 0.05154476330322146},
  BaseStyle->(FontSize -> 12),
  DisplayFunction->Identity,
  Epilog->InsetBox[
    FormBox["\"sCDM\"", TraditionalForm], 
    Scaled[{0.83, 0.71}]],
  Frame->{{True, True}, {True, True}},
  FrameLabel->{{
     FormBox["\"P(k)\"", TraditionalForm], None}, {
     FormBox["\"k (h/Mpc)\"", TraditionalForm], None}},
  FrameStyle->GrayLevel[0],
  FrameTicks->FrontEndValueCache[{{
      Charting`ScaledTicks[{Log, Exp}], 
      Charting`ScaledFrameTicks[{Log, Exp}]}, {
      Charting`ScaledTicks[{Log, Exp}], 
      Charting`ScaledFrameTicks[{Log, Exp}]}}, {{{{0., 
        FormBox["1", TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {2.302585092994046, 
        FormBox["10", TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {4.605170185988092, 
        FormBox["100", TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {6.907755278982137, 
        FormBox["1000", TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {9.210340371976184, 
        FormBox[
         TemplateBox[{"10", "4"}, "Superscript", SyntaxForm -> 
          SuperscriptBox], TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.3025850929940455`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-1.6094379124341003`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-1.2039728043259361`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.916290731874155, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.6931471805599453, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.5108256237659907, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.35667494393873245`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.2231435513142097, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.10536051565782628`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {0.6931471805599453, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.0986122886681098`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.3862943611198906`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.6094379124341003`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.791759469228055, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.9459101490553132`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {2.0794415416798357`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {2.1972245773362196`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {2.995732273553991, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {3.4011973816621555`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {3.6888794541139363`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {3.912023005428146, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.0943445622221, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.248495242049359, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.382026634673881, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.499809670330265, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {5.298317366548036, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {5.703782474656201, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {5.991464547107982, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {6.214608098422191, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {6.396929655216146, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {6.551080335043404, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {6.684611727667927, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {6.802394763324311, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {7.600902459542082, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {8.006367567650246, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {8.294049640102028, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {8.517193191416238, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {8.699514748210191, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {8.85366542803745, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {8.987196820661973, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {9.104979856318357, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {9.903487552536127, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {10.308952660644293`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {10.596634733096073`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {10.819778284410283`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {11.002099841204238`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {11.156250521031495`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {11.289781913656018`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {11.407564949312402`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {11.512925464970229`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {12.206072645530174`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {12.611537753638338`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}}, {{0., 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {2.302585092994046, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {4.605170185988092, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {6.907755278982137, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {9.210340371976184, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.3025850929940455`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-1.6094379124341003`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-1.2039728043259361`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.916290731874155, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.6931471805599453, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.5108256237659907, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.35667494393873245`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.2231435513142097, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.10536051565782628`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {0.6931471805599453, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.0986122886681098`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.3862943611198906`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.6094379124341003`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.791759469228055, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.9459101490553132`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {2.0794415416798357`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {2.1972245773362196`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {2.995732273553991, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {3.4011973816621555`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {3.6888794541139363`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {3.912023005428146, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.0943445622221, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.248495242049359, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.382026634673881, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.499809670330265, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {5.298317366548036, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {5.703782474656201, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {5.991464547107982, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {6.214608098422191, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {6.396929655216146, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {6.551080335043404, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {6.684611727667927, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {6.802394763324311, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {7.600902459542082, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {8.006367567650246, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {8.294049640102028, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {8.517193191416238, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {8.699514748210191, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {8.85366542803745, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {8.987196820661973, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {9.104979856318357, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {9.903487552536127, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {10.308952660644293`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {10.596634733096073`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {10.819778284410283`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {11.002099841204238`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {11.156250521031495`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {11.289781913656018`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {11.407564949312402`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {11.512925464970229`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {12.206072645530174`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {12.611537753638338`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}}}, {{{-6.907755278982137, 
        FormBox["0.001`", TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {-4.605170185988091, 
        FormBox[
         TagBox[
          InterpretationBox["\"0.010\"", 0.01, AutoDelete -> True], 
          NumberForm[#, {
            DirectedInfinity[1], 3}]& ], TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.3025850929940455`, 
        FormBox[
         TagBox[
          InterpretationBox["\"0.100\"", 0.1, AutoDelete -> True], 
          NumberForm[#, {
            DirectedInfinity[1], 3}]& ], TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {0., 
        FormBox["1", TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {2.302585092994046, 
        FormBox["10", TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {-9.210340371976182, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-8.517193191416238, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-8.111728083308073, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-7.824046010856292, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-7.600902459542082, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-7.418580902748128, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-7.264430222920869, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-7.1308988302963465`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-7.013115794639964, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-6.214608098422191, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-5.809142990314028, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-5.521460917862246, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-5.298317366548036, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-5.115995809754082, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-4.961845129926823, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-4.8283137373023015`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-4.710530701645918, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-3.912023005428146, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-3.506557897319982, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-3.2188758248682006`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.995732273553991, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.8134107167600364`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.659260036932778, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.5257286443082556`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.4079456086518722`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-1.6094379124341003`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-1.2039728043259361`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.916290731874155, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.6931471805599453, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.5108256237659907, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.35667494393873245`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.2231435513142097, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.10536051565782628`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {0.6931471805599453, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.0986122886681098`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.3862943611198906`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.6094379124341003`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.791759469228055, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.9459101490553132`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {2.0794415416798357`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {2.1972245773362196`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {2.995732273553991, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {3.4011973816621555`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {3.6888794541139363`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {3.912023005428146, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.0943445622221, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.248495242049359, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.382026634673881, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.499809670330265, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.605170185988092, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}}, {{-6.907755278982137, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {-4.605170185988091, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.3025850929940455`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {0., 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {2.302585092994046, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.01, 0.}, {
         AbsoluteThickness[0.1]}}, {-9.210340371976182, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-8.517193191416238, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-8.111728083308073, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-7.824046010856292, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-7.600902459542082, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-7.418580902748128, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-7.264430222920869, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-7.1308988302963465`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-7.013115794639964, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-6.214608098422191, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-5.809142990314028, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-5.521460917862246, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-5.298317366548036, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-5.115995809754082, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-4.961845129926823, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-4.8283137373023015`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-4.710530701645918, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-3.912023005428146, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-3.506557897319982, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-3.2188758248682006`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.995732273553991, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.8134107167600364`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.659260036932778, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.5257286443082556`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-2.4079456086518722`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-1.6094379124341003`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-1.2039728043259361`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.916290731874155, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.6931471805599453, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.5108256237659907, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.35667494393873245`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.2231435513142097, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {-0.10536051565782628`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {0.6931471805599453, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.0986122886681098`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.3862943611198906`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.6094379124341003`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.791759469228055, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {1.9459101490553132`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {2.0794415416798357`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {2.1972245773362196`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {2.995732273553991, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {3.4011973816621555`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {3.6888794541139363`, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {3.912023005428146, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.0943445622221, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.248495242049359, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.382026634673881, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.499809670330265, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}, {4.605170185988092, 
        FormBox[
         TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}, {
         AbsoluteThickness[0.1]}}}}}],
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  ImagePadding->All,
  ImageSize->350,
  ImageSizeRaw->{1502, 1051},
  Method->{"CoordinatesToolOptions" -> {"DisplayFunction" -> ({
        (Exp[#]& )[
         Part[#, 1]], 
        (Exp[#]& )[
         Part[#, 2]]}& ), "CopiedValueFunction" -> ({
        (Exp[#]& )[
         Part[#, 1]], 
        (Exp[#]& )[
         Part[#, 2]]}& )}},
  PlotRange->NCache[{{-6.907755278982137, 
      Log[10]}, {0, 
      Log[30000]}}, {{-6.907755278982137, 2.302585092994046}, {
     0, 10.308952660644293`}}],
  PlotRangeClipping->True,
  PlotRangePadding->{{0, 0}, {0, 0}},
  Ticks->{
    Charting`ScaledTicks[{Log, Exp}], 
    Charting`ScaledTicks[{Log, Exp}]}]], "Output",
 CellChangeTimes->{{3.7673658652565393`*^9, 3.7673658703532877`*^9}, {
   3.767366042441847*^9, 3.767366075051812*^9}, {3.7673662223331013`*^9, 
   3.767366332586464*^9}, 3.767422867310616*^9, {3.767422910529107*^9, 
   3.7674229188240623`*^9}, {3.767423246581141*^9, 3.767423281221401*^9}, {
   3.7674233260036974`*^9, 3.7674233351134553`*^9}, {3.7674234073680105`*^9, 
   3.7674234538302755`*^9}, {3.7964637050571275`*^9, 3.796463752439705*^9}, {
   3.796463806471927*^9, 3.7964638314774237`*^9}, 3.796474829256418*^9, 
   3.7964748645373316`*^9, {3.796474930935253*^9, 3.7964749884538326`*^9}, 
   3.7964750925506835`*^9, 3.7964751342325444`*^9, {3.796475166751314*^9, 
   3.796475188215172*^9}, 3.7964752730709143`*^9, 3.796475430043749*^9, {
   3.7964756791056356`*^9, 3.7964756867144184`*^9}, {3.7964758975334835`*^9, 
   3.796475980008502*^9}, {3.7964760186611347`*^9, 3.796476043463479*^9}, {
   3.7964761264681926`*^9, 3.7964762009493117`*^9}, {3.7964762409140797`*^9, 
   3.796476343791712*^9}, 3.796476435230425*^9, 3.7964764674131737`*^9, 
   3.796476710639167*^9, 3.7964767467804227`*^9, 3.7964770446937823`*^9, 
   3.796477088355996*^9, {3.7964771764930587`*^9, 3.7964771959874516`*^9}, 
   3.7964772506619787`*^9, {3.7964782149388847`*^9, 3.796478275265788*^9}, 
   3.7964783562195883`*^9, 3.796478686101691*^9, 3.7964788773140917`*^9, {
   3.7964789404989176`*^9, 
   3.796478951650893*^9}},ExpressionUUID->"570e9a44-ef15-4e45-aee0-\
5afc24db4bd5"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"(*", " ", "end", " ", "*)"}]], "Input",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.},
 CellChangeTimes->{{3.735625441878965*^9, 
  3.735625464138838*^9}},ExpressionUUID->"0e208829-35d5-47cb-b54a-\
4bb6acb4171f"]
}, Closed]]
},
CellGrouping->Manual,
WindowSize->{819, 736},
WindowMargins->{{-6, Automatic}, {Automatic, 0}},
FrontEndVersion->"11.2 for Microsoft Windows (64-bit) (September 10, 2017)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[558, 20, 158, 3, 43, "Input",ExpressionUUID->"db04ee06-7e83-4916-8dfc-1e570623902e"],
Cell[CellGroupData[{
Cell[741, 27, 488, 11, 43, "Input",ExpressionUUID->"498c69b4-d2ae-45f2-9c4f-86d2f808d835",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[1232, 40, 728, 22, 90, "Input",ExpressionUUID->"a5c31e34-7d0c-406f-8961-05747b8d6600",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[1963, 64, 998, 26, 189, "Input",ExpressionUUID->"c8d3a0bc-cd93-4e23-a8eb-6763f8e81225",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[2964, 92, 1183, 35, 212, "Input",ExpressionUUID->"a4666d80-c540-4409-b18e-84415c6df473",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[4150, 129, 2721, 74, 233, "Input",ExpressionUUID->"919a3c43-bbf1-4a02-96b5-64dc585f085c",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[6874, 205, 248, 6, 43, "Input",ExpressionUUID->"9d05f989-b7e9-4bc7-a032-53bc21e8e4e7"],
Cell[7125, 213, 1744, 49, 193, "Input",ExpressionUUID->"21338be5-35ce-4e6c-926a-18b7ac170390",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[8872, 264, 39070, 697, 387, "Output",ExpressionUUID->"0b1e3621-7972-4050-bd28-4b3b42bc4dc9",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[47945, 963, 1446, 39, 131, "Input",ExpressionUUID->"08f42a6c-dfef-4a69-af6b-cf8b9b64e082",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[49394, 1004, 16836, 339, 372, "Output",ExpressionUUID->"e6624f99-8e03-4b55-93c6-77105a611c98",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[66233, 1345, 1426, 39, 131, "Input",ExpressionUUID->"21f67940-086a-4d0f-84eb-90366fe82074",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[67662, 1386, 18228, 360, 372, "Output",ExpressionUUID->"670b7099-45df-468c-af38-ec6607c2f691",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[85893, 1748, 1439, 38, 131, "Input",ExpressionUUID->"dd904323-0776-4741-a290-a878d65db428",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[87335, 1788, 51649, 906, 345, "Output",ExpressionUUID->"310bb450-bd56-4478-bced-f05fb9863795",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[138987, 2696, 244, 5, 43, "Input",ExpressionUUID->"0e338e55-3212-424f-9561-904e25c89922",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}]
}, Closed]],
Cell[CellGroupData[{
Cell[139268, 2706, 422, 10, 35, "Input",ExpressionUUID->"4d77c56b-40e7-4487-be08-e4ce8255ee8b",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[139693, 2718, 728, 22, 90, "Input",ExpressionUUID->"31065b41-3987-4eed-b984-f19a31348596",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[140424, 2742, 1175, 30, 218, "Input",ExpressionUUID->"0cf06a0a-d8f8-4ddf-8ce7-64d59c2a8f73",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[141602, 2774, 1898, 57, 318, "Input",ExpressionUUID->"795a662f-48fb-4e99-ab6a-2adacc1d85ef",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[143503, 2833, 6096, 166, 585, "Input",ExpressionUUID->"98a8ab14-440c-40e3-a07e-a33c2ab3a583",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[149602, 3001, 218, 5, 43, "Input",ExpressionUUID->"782bdd20-5acf-43fe-ac43-5bff278d2bf3"],
Cell[149823, 3008, 1744, 49, 193, "Input",ExpressionUUID->"b8e631e1-f5f8-4d2b-9f67-7edc15242a64",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[151570, 3059, 38187, 683, 387, "Output",ExpressionUUID->"eccf32bc-a4fd-4f66-ac56-caec39ea2640",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[CellGroupData[{
Cell[189782, 3746, 1785, 43, 189, "Input",ExpressionUUID->"eb9ec060-d0b2-4925-bf7d-83bfffdfa688"],
Cell[191570, 3791, 89487, 1580, 455, "Output",ExpressionUUID->"51708043-2c36-494b-b2ee-5819cf103c2f"]
}, Open  ]],
Cell[CellGroupData[{
Cell[281094, 5376, 1444, 39, 131, "Input",ExpressionUUID->"439b4876-4c8e-47f2-b562-4a4f2f24b447",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[282541, 5417, 16924, 340, 372, "Output",ExpressionUUID->"2b1592d9-b47a-45cf-a12c-c66ef5e4015c",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}]
}, Open  ]],
Cell[299480, 5760, 1398, 38, 131, "Input",ExpressionUUID->"38b88036-0f64-4859-9faa-e4dd24191c06",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[300881, 5800, 18615, 367, 372, "Output",ExpressionUUID->"7dd88b16-f17f-44de-922b-53a6a19a6f3a",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[CellGroupData[{
Cell[319521, 6171, 2164, 59, 213, "Input",ExpressionUUID->"a8597ec2-9743-44d1-96c1-848199c96884",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[321688, 6232, 39326, 721, 357, "Output",ExpressionUUID->"879edab1-d2b3-4796-bff3-9c3a69711f0b"]
}, Open  ]],
Cell[CellGroupData[{
Cell[361051, 6958, 1451, 39, 131, "Input",ExpressionUUID->"66b428f3-0340-4546-9ef8-cf8da7b8b746",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[362505, 6999, 21595, 416, 372, "Output",ExpressionUUID->"00b1706d-6929-469e-b0df-c9b7313bfd0a"]
}, Open  ]],
Cell[384115, 7418, 1409, 37, 131, "Input",ExpressionUUID->"3b4c2933-646a-459c-9374-ea97c5a820ff",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[385527, 7457, 45930, 812, 345, "Output",ExpressionUUID->"39346b79-da29-47da-9ff7-75e7d184cac1",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[431460, 8271, 184, 3, 43, "Input",ExpressionUUID->"63ab574b-e62a-4da6-b6af-34eb35c98fe1",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}]
}, Closed]],
Cell[CellGroupData[{
Cell[431681, 8279, 692, 14, 35, "Input",ExpressionUUID->"b9d6d53d-bcad-4fa6-88a7-34e7adfb795b",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[432376, 8295, 2376, 64, 420, "Input",ExpressionUUID->"20a3f001-faa0-4fdb-b1b7-bc88125104f5",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[434755, 8361, 2490, 62, 291, "Input",ExpressionUUID->"2c08f95f-7b86-41bf-8ffc-b2a9d8f1b9b9",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[437248, 8425, 6993, 179, 426, "Input",ExpressionUUID->"00b28196-73b5-4b8e-9d0c-4337b6e55de1"],
Cell[444244, 8606, 555, 16, 73, "Input",ExpressionUUID->"11b3cfd6-2567-4c43-b205-79b3ea5bdcf5"],
Cell[444802, 8624, 1096, 29, 160, "Input",ExpressionUUID->"cc350f12-0441-4653-ae39-35a174fb0f9f"],
Cell[445901, 8655, 265, 6, 43, "Input",ExpressionUUID->"f7a4f988-3c07-4139-8a55-9a1d3b78a268"],
Cell[CellGroupData[{
Cell[446191, 8665, 2279, 62, 165, "Input",ExpressionUUID->"00468739-5733-4ef0-a34f-d0f2b343e8a8",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[448473, 8729, 35838, 647, 387, "Output",ExpressionUUID->"d23c91c2-717f-42f6-a26e-beabce0f2d5b"]
}, Open  ]],
Cell[CellGroupData[{
Cell[484348, 9381, 2744, 73, 153, "Input",ExpressionUUID->"944ed33a-9f40-44c5-a365-187f001b6345"],
Cell[487095, 9456, 32759, 612, 373, "Output",ExpressionUUID->"17a758cb-c05d-497a-a230-3dd632d3ecd2"]
}, Open  ]],
Cell[519869, 10071, 911, 24, 102, "Input",ExpressionUUID->"32f1c521-22c3-4f5c-a99c-8d08e948855d"],
Cell[CellGroupData[{
Cell[520805, 10099, 2569, 65, 188, "Input",ExpressionUUID->"ea4d4744-627b-4c51-bfb1-7f00a2ed0ab3"],
Cell[523377, 10166, 86080, 1481, 387, "Output",ExpressionUUID->"333a80bf-2b33-4106-94a0-0fac108c40a6"]
}, Open  ]],
Cell[609472, 11650, 244, 5, 43, "Input",ExpressionUUID->"80e1a4f5-996a-440c-8808-e0b2376a8e9f",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}]
}, Closed]],
Cell[CellGroupData[{
Cell[609753, 11660, 924, 18, 65, "Input",ExpressionUUID->"316a5955-1fd9-4466-98a6-6f065a4aa32f",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[610680, 11680, 2430, 65, 420, "Input",ExpressionUUID->"1aecf578-9dca-42d8-b7ce-8b6fbf7c426a",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[613113, 11747, 2495, 62, 291, "Input",ExpressionUUID->"6ce379b4-b15b-41c6-816b-35ecc349a2c4",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}],
Cell[615611, 11811, 6810, 176, 851, "Input",ExpressionUUID->"0fada880-40ca-4085-ba7e-62eb5f4a19ec"],
Cell[622424, 11989, 784, 21, 131, "Input",ExpressionUUID->"bc62daac-1f46-4a36-b26d-1a63f4985057"],
Cell[CellGroupData[{
Cell[623233, 12014, 1057, 29, 102, "Input",ExpressionUUID->"ebf7bcbb-1234-4fed-a007-29b93cd39c78"],
Cell[624293, 12045, 644, 12, 108, "Output",ExpressionUUID->"65e68414-e0ee-4cad-973e-deb3ab599ed9"]
}, Open  ]],
Cell[624952, 12060, 1160, 27, 102, "Input",ExpressionUUID->"c85d671c-c7d5-48a7-8254-6e789fce4441"],
Cell[CellGroupData[{
Cell[626137, 12091, 4022, 96, 498, "Input",ExpressionUUID->"eed1b495-a3de-4448-98e7-8eabe5ee2adf"],
Cell[630162, 12189, 185360, 3042, 532, "Output",ExpressionUUID->"2e39dcf0-499a-4afa-843a-480431255b13"]
}, Open  ]],
Cell[815537, 15234, 945, 28, 131, "Input",ExpressionUUID->"b508c1ff-7c91-49bd-bf9b-ec77793ce40c"],
Cell[CellGroupData[{
Cell[816507, 15266, 3788, 92, 565, "Input",ExpressionUUID->"af61a2ec-71cf-4b3e-9fdb-6d90c4d37a21"],
Cell[820298, 15360, 124621, 2044, 532, "Output",ExpressionUUID->"a1b5ed08-b8c9-485e-a552-d43a145c7ac6"]
}, Open  ]],
Cell[944934, 17407, 2179, 62, 282, "Input",ExpressionUUID->"c962bd63-cdc8-4022-ad22-a551b42f29dc"],
Cell[947116, 17471, 1708, 51, 207, "Input",ExpressionUUID->"a2d02dc6-bc31-4f16-be53-bc9f052e7e14"],
Cell[948827, 17524, 677, 20, 126, "Input",ExpressionUUID->"e0e99587-94b3-4a84-a196-e12826bc39ce"],
Cell[CellGroupData[{
Cell[949529, 17548, 1314, 33, 189, "Input",ExpressionUUID->"1a868f70-a54c-4399-b76e-e0f96159e429"],
Cell[950846, 17583, 12348, 265, 370, "Output",ExpressionUUID->"0e28bb88-3d7d-4a78-b32c-4b51cf906cd5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[963231, 17853, 1541, 44, 304, "Input",ExpressionUUID->"5843d70c-5137-458c-8417-12de12381046"],
Cell[964775, 17899, 26941, 554, 390, "Output",ExpressionUUID->"a34ccd26-7afd-41b8-b7b6-47398db06aad"]
}, Open  ]],
Cell[991731, 18456, 672, 20, 176, "Input",ExpressionUUID->"3807af40-d811-4577-956c-d05c11b878ee"],
Cell[992406, 18478, 1407, 39, 222, "Input",ExpressionUUID->"c576ae96-4dd7-405e-a96d-c124f2f7cf8a"],
Cell[993816, 18519, 505, 13, 80, "Input",ExpressionUUID->"570b440f-da26-4457-9042-138d58710465"],
Cell[994324, 18534, 1611, 38, 200, "Input",ExpressionUUID->"f7479ee2-cb76-452b-b126-29778f227b10"],
Cell[CellGroupData[{
Cell[995960, 18576, 2164, 46, 218, "Input",ExpressionUUID->"7fca1c09-28f0-42b2-8a79-d4650e6008b1"],
Cell[998127, 18624, 37111, 748, 395, "Output",ExpressionUUID->"570e9a44-ef15-4e45-aee0-5afc24db4bd5"]
}, Open  ]],
Cell[1035253, 19375, 244, 5, 43, "Input",ExpressionUUID->"0e208829-35d5-47cb-b54a-4bb6acb4171f",
 CellGroupingRules->{"GroupTogetherGrouping", 10000.}]
}, Closed]]
}
]
*)

